package org.openxma.dsl.generator.helper;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xpand2.XpandExecutionContextImpl.DefinitionOperationAdapter;
import org.openxma.dsl.common.Strings;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.BoolLiteral;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.FalseLiteral;
import org.openxma.dsl.core.model.ReferenceWithParameter;
import org.openxma.dsl.core.model.TrueLiteral;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.core.util.Literals;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.AttributeProperty;
import org.openxma.dsl.dom.model.AttributeValidationProperty;
import org.openxma.dsl.dom.model.PresentableFeature;
import org.openxma.dsl.generator.helper.style.ApplyAlignmentStyle;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.AlignmentProperty;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.Composite;
import org.openxma.dsl.pom.model.ContentAlignment;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.FieldPart;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.HorizontalAlignment;
import org.openxma.dsl.pom.model.LayoutDataProperty;
import org.openxma.dsl.pom.model.ListBox;
import org.openxma.dsl.pom.model.PagingControl;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.SplitPanel;
import org.openxma.dsl.pom.model.StylePropertyOther;
import org.openxma.dsl.pom.model.StylePropertyPaging;
import org.openxma.dsl.pom.model.StylePropertyPagingControls;
import org.openxma.dsl.pom.model.StylePropertyPagingJumpSize;
import org.openxma.dsl.pom.model.StylePropertyPagingPageSize;
import org.openxma.dsl.pom.model.StylePropertyPagingPosition;
import org.openxma.dsl.pom.model.StylePropertyWidth;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableCombo;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.model.TabulatorPosition;
import org.openxma.dsl.pom.model.VerticalAlignment;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.naming.PomNameProvider;
import org.openxma.dsl.pom.util.StyleAccess;
import org.openxma.dsl.pom.util.TextProperties;

import at.spardat.xma.guidesign.BDAttribute;
import at.spardat.xma.guidesign.GuidesignFactory;
import at.spardat.xma.guidesign.HiddenWidget;
import at.spardat.xma.guidesign.NotebookPage;
import at.spardat.xma.guidesign.PageComposite;
import at.spardat.xma.guidesign.Validator;
import at.spardat.xma.guidesign.XMAComposite;
import at.spardat.xma.guidesign.XMAFormAttachment;
import at.spardat.xma.guidesign.XMAFormData;
import at.spardat.xma.guidesign.XMAGroup;
import at.spardat.xma.guidesign.XMALabel;
import at.spardat.xma.guidesign.XMAList;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAPagingControl;
import at.spardat.xma.guidesign.XMASashForm;
import at.spardat.xma.guidesign.XMAScrolledComposite;
import at.spardat.xma.guidesign.XMATabFolder;
import at.spardat.xma.guidesign.XMATable;
import at.spardat.xma.guidesign.XMATableColumn;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.CollapseChain;
import at.spardat.xma.guidesign.flex.FlexFactory;
import at.spardat.xma.guidesign.types.AlignmentType;
import at.spardat.xma.guidesign.types.AttachSideType;
import at.spardat.xma.guidesign.types.NamedFlag;
import at.spardat.xma.guidesign.types.PagingControlStyle;
import at.spardat.xma.guidesign.types.SelectionType;

public class ComplexTypeExtension {

    private final static String PARTITIONING_DATA_PROPERTY_NAME = "partitioning_data";

    public static List<XMAWidget> getGeneratedXMAWidgetsForGuiElement(GuiElement guiElement) {
        return (List<XMAWidget>) EObjectPropertiesAdapter.getProperty(guiElement, "xmaWidgets");
    }

    /**
     * Set the GuiElement as a (temporary) property of the XMAWidget.
     * 
     * @param xmaWidget
     * @param guiElement
     */
    public static void setLayoutDataPropertyFromGuiElement(XMAWidget xmaWidget, GuiElement guiElement) {

        WidgetExtension.setGuiElementProperty(xmaWidget, guiElement);
        List<XMAWidget> generatedWidgets = getGeneratedXMAWidgetsForGuiElement(guiElement);
        if (generatedWidgets == null) {
            generatedWidgets = new ArrayList<XMAWidget>();
            EObjectPropertiesAdapter.setProperty(guiElement, "xmaWidgets", generatedWidgets);
        }
        generatedWidgets.add(xmaWidget);
    }

    /**
     * Create a XMATable from a Table.
     * 
     * @param table
     * @param parentXMAComposite
     * @param businessDataList
     * @return
     */
    public static EList<EObject> createWidgetsForGuiElement(TableCombo table, XMAComposite parentXMAComposite,
            List businessDataList) {
        EList<EObject> objList = new BasicEList<EObject>();

        XMATable tableCombo = createXMATableCombo(table, parentXMAComposite, businessDataList);
        if (table.getTextProperties() != null) {
            String labelText = TextProperties.getLabelText(table.getTextProperties());
            if (labelText != null) {
                XMALabel newLabel = WidgetExtension.createLabel(table,table.getName(),null,false);
                Util.setLabelText(newLabel,labelText+":");
                parentXMAComposite.getControls().add(newLabel);
                objList.add(newLabel);
                WidgetExtension.setFieldPartProperty(newLabel,FieldPart.LABEL);
                WidgetExtension.setFieldPartProperty(tableCombo,FieldPart.CONTROL);
                tableCombo.setLabel(newLabel);
            }
        }       
        objList.add(tableCombo);
        return objList;
    }


    /**
     * Create a XMATable from a Table.
     * 
     * @param table
     * @param parentXMAComposite
     * @param businessDataList
     * @return
     */
    public static EList<EObject> createWidgetsForGuiElement(Table table, XMAComposite parentXMAComposite,
            List businessDataList) {

        EList<EObject> objList = new BasicEList<EObject>();
        if (table.getPageable() instanceof BoolLiteral) {

            /* Get styles */
            Map<Integer, StyleSpecificationProperty> styleSpecificationProperties = StyleAccess
                    .getCombinedStyleProperty(table);
            StylePropertyPagingPosition positionProp = null;
            if (styleSpecificationProperties != null) {
                positionProp = (StylePropertyPagingPosition) styleSpecificationProperties
                        .get(PomPackage.STYLE_PROPERTY_PAGING_POSITION);
            }

            XMAComposite tableComposite = GuidesignFactory.eINSTANCE.createXMAComposite();
            String tableCompositeName = PomNameProvider.getTableCompositeName(table.getName());
            tableComposite.setNamInstance(tableCompositeName);
            tableComposite.setYnGenerated(true);
            parentXMAComposite.getControls().add(tableComposite);

            XMAPagingControl pagingControl = GuidesignFactory.eINSTANCE.createXMAPagingControl();
            pagingControl.setNamInstance(table.getName() + "_pagingControl");
            pagingControl.setYnGenerated(true);
            tableComposite.getControls().add(pagingControl);

            XMATable xmaTable = createXMATable(table, tableComposite, businessDataList);
            xmaTable.setNamInstance(table.getName());
            xmaTable.setYnGenerated(true);

            pagingControl.setTable(xmaTable);
            tableComposite.getControls().add(xmaTable);

            XMAFormData pagingFormData = GuidesignFactory.eINSTANCE.createXMAFormData();
            if (positionProp == null || positionProp.isTop()) {
                XMAFormAttachment pagingTop = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                pagingFormData.setTopAttach(pagingTop);
            }
            XMAFormAttachment pagingLeft = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            pagingFormData.setLeftAttach(pagingLeft);
            XMAFormAttachment pagingRight = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            pagingRight.setQntNominator(100);
            pagingFormData.setRightAttach(pagingRight);
            if (positionProp != null && positionProp.isBottom()) {
                XMAFormAttachment pagingBottom = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                pagingBottom.setQntNominator(100);
                pagingFormData.setBottomAttach(pagingBottom);
            }
            pagingControl.setFormData(pagingFormData);

            XMAFormData tableFormData = GuidesignFactory.eINSTANCE.createXMAFormData();
            XMAFormAttachment tableTop = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            if (positionProp == null || positionProp.isTop()) {
                tableTop.setCodAttachSide(AttachSideType.BOTTOM_LITERAL);
                tableTop.setQntOffset(0);
                tableTop.setAttachElement(pagingControl);
            }
            tableFormData.setTopAttach(tableTop);
            XMAFormAttachment tableLeft = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            tableFormData.setLeftAttach(tableLeft);
            XMAFormAttachment tableRight = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            tableRight.setQntNominator(100);
            tableFormData.setRightAttach(tableRight);
            XMAFormAttachment tableBottom = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            if (positionProp == null || positionProp.isTop()) {
                tableBottom.setQntNominator(100);
            } else {
                tableBottom.setCodAttachSide(AttachSideType.TOP_LITERAL);
                tableBottom.setQntOffset(0);
                tableBottom.setAttachElement(pagingControl);
            }
            tableFormData.setBottomAttach(tableBottom);

            xmaTable.setFormData(tableFormData);

            /* Apply styles */
            if (styleSpecificationProperties != null) {
                StylePropertyPagingPageSize pageSizeProp = (StylePropertyPagingPageSize) styleSpecificationProperties
                        .get(PomPackage.STYLE_PROPERTY_PAGING_PAGE_SIZE);
                if (pageSizeProp != null) {
                    pagingControl.setPageSize((short) pageSizeProp.getSize());
                    if (pageSizeProp.getMaxSize() > 0) {
                        pagingControl.setMaxPageSize((short) pageSizeProp.getMaxSize());
                    }
                }
                StylePropertyPagingJumpSize jumpSizeProp = (StylePropertyPagingJumpSize) styleSpecificationProperties
                        .get(PomPackage.STYLE_PROPERTY_PAGING_JUMP_SIZE);
                if (jumpSizeProp != null) {
                    pagingControl.setJumpSize((short) jumpSizeProp.getSize());
                }
                StylePropertyPaging pagingStyleProp = (StylePropertyPaging) styleSpecificationProperties
                        .get(PomPackage.STYLE_PROPERTY_PAGING);
                if (pagingStyleProp != null) {
                    if ("custom".equals(pagingStyleProp.getStyle())) {
                        pagingControl.setPagingControlStyle(PagingControlStyle.CUSTOM);
                    } else if ("scroll".equals(pagingStyleProp.getStyle())) {
                        pagingControl.setPagingControlStyle(PagingControlStyle.SCROLLNAV);
                    } else if ("jump".equals(pagingStyleProp.getStyle())) {
                        pagingControl.setPagingControlStyle(PagingControlStyle.JUMPNAV);
                    } else if (pagingStyleProp.isDefault()) {
                        pagingControl.setPagingControlStyle(PagingControlStyle.DEFAULT);
                    }
                }
                StylePropertyPagingControls pagingControlsProp = (StylePropertyPagingControls) styleSpecificationProperties
                        .get(PomPackage.STYLE_PROPERTY_PAGING_CONTROLS);
                if (pagingControlsProp != null) {
                    for (PagingControl pc : pagingControlsProp.getControls()) {
                        if ("start".equals(pc.getControl())) {
                            setNamedFlag(pagingControl, "SHOW_START");
                            continue;
                        }
                        if ("back".equals(pc.getControl())) {
                            setNamedFlag(pagingControl, "SHOW_BACK");
                            continue;
                        }
                        if ("fastback".equals(pc.getControl())) {
                            setNamedFlag(pagingControl, "SHOW_FASTBACK");
                            continue;
                        }
                        if ("reload".equals(pc.getControl())) {
                            setNamedFlag(pagingControl, "SHOW_RELOAD");
                            continue;
                        }
                        if ("jump".equals(pc.getControl())) {
                            setNamedFlag(pagingControl, "SHOW_JUMP");
                            continue;
                        }
                        if ("fastnext".equals(pc.getControl())) {
                            setNamedFlag(pagingControl, "SHOW_FASTNEXT");
                            continue;
                        }
                        if ("info".equals(pc.getControl())) {
                            setNamedFlag(pagingControl, "SHOW_INFO");
                            continue;
                        }
                        if ("pagesize".equals(pc.getControl())) {
                            setNamedFlag(pagingControl, "SHOW_PAGESIZE");
                            continue;
                        }
                        if (pc.isEnd()) {
                            setNamedFlag(pagingControl, "SHOW_END");
                            continue;
                        }
                        if (pc.isNext()) {
                            setNamedFlag(pagingControl, "SHOW_NEXT");
                            continue;
                        }
                        if (pc.isCustomize()) {
                            setNamedFlag(pagingControl, "SHOW_CUSTOMIZE");
                            xmaTable.setYnSortIndicator(false); // do not use the sortindicator when the customize
                                                                // option is set
                            continue;
                        }
                        if (pc.isExport()) {
                            setNamedFlag(pagingControl, "SHOW_EXPORT");
                            continue;
                        }
                    }
                }
            }

            HiddenWidget hiddenWidget = GuidesignFactory.eINSTANCE.createHiddenWidget();
            hiddenWidget.setNamInstance(xmaTable.getNamInstance() + "_customizerComponentId");
            hiddenWidget.setYnGenerated(true);
            Validator bcdValidator = GuidesignFactory.eINSTANCE.createBcdValidator();
            hiddenWidget.setBdValidator(bcdValidator);
            parentXMAComposite.getPageComposite(parentXMAComposite).getControls().add(0, hiddenWidget);

            setLayoutDataPropertyFromGuiElement(tableComposite, table);
            setLayoutDataPropertyFromGuiElement(xmaTable, table);
            setLayoutDataPropertyFromGuiElement(pagingControl, table);

            objList.add(tableComposite);
        } else {
            objList.add(createXMATable(table, parentXMAComposite, businessDataList));
        }
        return objList;
    }

    private static void setNamedFlag(XMAPagingControl pagingControl, String flagName) {
        for (Object styleObject : pagingControl.getCustomStyles().getStyles()) {
            NamedFlag namedFlag = (NamedFlag) styleObject;
            if (flagName.equals(namedFlag.getName())) {
                namedFlag.setValue(true);
                break;
            }
        }
    }
    private static XMATable createXMATableCombo(TableCombo table, XMAComposite parentXMAComposite, List businessDataList) {
//        setTabableForIField(combo,f,true);                
//        
//        if (f instanceof CustomizeableField) {
//            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(combo,(CustomizeableField)f);
//        }            
//        setGuiElementProperty(combo,f);
        XMATable xmaTable = GuidesignFactory.eINSTANCE.createXMATable();
        xmaTable.setYnBorder(true);
        xmaTable.setYnGenerated(true);
        xmaTable.setNamInstance(table.getName());
        xmaTable.setYnOneway(true);
        xmaTable.setCodSelection(SelectionType.SINGLE_LITERAL);
        xmaTable.setYnCombo(true);
        // Create all columns
        // Layout on GUI
        for (TableColumn column : table.getColumns()) {
            createTableColumn(xmaTable, table.getObject(), businessDataList, column);
        }
        setLayoutDataPropertyFromGuiElement(xmaTable, table);


        parentXMAComposite.getControls().add(xmaTable);
        // Key
        BDAttribute newDataAttribute = Util.findBDAttributeForAttribute(parentXMAComposite.getComponent(),
                businessDataList, table.getKey(), table.getObject());
        xmaTable.setDataAttribute(newDataAttribute);
        return xmaTable;
    }

    public static XMATable createXMATable(Table table, XMAComposite parentXMAComposite, List businessDataList) {
        XMATable newTable = GuidesignFactory.eINSTANCE.createXMATable();
        parentXMAComposite.getControls().add(newTable);
        newTable.setYnGenerated(true);
        newTable.setNamInstance(table.getName());
        if (table.getMultiselectionFlag() != null
                && table.getMultiselectionFlag().getMultiSelection() instanceof TrueLiteral) {
            newTable.setCodSelection(SelectionType.MULTI_LITERAL);
        }
        if (table.getCheckSelection() instanceof TrueLiteral) {
            newTable.setCodSelection(SelectionType.CHECK_LITERAL);
        }
        if (table.getSortindicator() instanceof TrueLiteral) {
            newTable.setYnSortIndicator(true);
        }

        WidgetExtension.setTabable(newTable, table);

        Boolean isOneWay = Literals.booleanForBoolLiteral(table.getOneway());
        if (isOneWay != null) {
            newTable.setYnOneway(isOneWay);
        }

        Boolean isGridVisible = Literals.booleanForBoolLiteral(table.getGridvisible());
        if (isGridVisible != null) {
            newTable.setYnGrid(isGridVisible);
        }

        // Create all columns
        // Layout on GUI
        for (TableColumn column : table.getColumns()) {
            createTableColumn(newTable, table.getObject(), businessDataList, column);
        }
        setLayoutDataPropertyFromGuiElement(newTable, table);
        // Key
        BDAttribute newDataAttribute = Util.findBDAttributeForAttribute(parentXMAComposite.getComponent(),
                businessDataList, table.getKey(), table.getObject());
        newTable.setDataAttribute(newDataAttribute);
        // ColumnMinWidth
        int columnMinWidth = table.getColumnMinWidth();
        if (columnMinWidth > 0) {
            newTable.setQntColumnMinWidthLimit(columnMinWidth);
        }

        return newTable;
    }

    public static EList<EObject> createWidgetsForGuiElement(ListBox listBox, XMAComposite parentXMAComposite,
            List businessDataList) {

        EList<EObject> objList = new BasicEList<EObject>();

        XMAList newList = GuidesignFactory.eINSTANCE.createXMAList();
        parentXMAComposite.getControls().add(newList);
        newList.setYnGenerated(true);
        newList.setNamInstance(listBox.getName());

        if (listBox.getMultiselectionFlag() != null
                && listBox.getMultiselectionFlag().getMultiSelection() instanceof TrueLiteral) {
            newList.setYnMultiSelection(true);
        }

        WidgetExtension.setTabable(newList, listBox);

        setLayoutDataPropertyFromGuiElement(newList, listBox);
        // Key
        // BDAttribute newDataAttribute =
        // Util.findBDAttributeForAttribute(parentXMAComposite.getComponent(),businessDataList,listBox.getKeyAsAttribute(),table.getObject());
        // newTable.setDataAttribute(newDataAttribute);

        objList.add(newList);
        return objList;
    }

    /**
     * Create XMATableColumn for a XMATable for a Column of a Table.
     * 
     * @param xmaTable
     * @param table
     * @param businessDataList
     */
    private static void createTableColumn(XMATable xmaTable, DataObjectVariable dataObjectVariable,
            List businessDataList, TableColumn column) {
        XMATableColumn xmaTableColumn = GuidesignFactory.eINSTANCE.createXMATableColumn();
        String xmaTableColumnName = WidgetExtension.composeDefaultTableColumnName(column);
        WidgetExtension.setSourceModelElementProperty(xmaTableColumn, column);
        xmaTableColumn.setNamInstance(xmaTableColumnName);
        String columnLabel = column.getLabelText();
        if (columnLabel == null) {
            if (column.getColumnName() != null) {
                columnLabel = column.getColumnName();
            } else {
                columnLabel = xmaTableColumnName;
            }
        }
        xmaTableColumn.setRscColName(columnLabel);
        if (column.getVisible() instanceof FalseLiteral) {
            xmaTableColumn.setYnHiden(true);
        } else {
            xmaTableColumn.setYnHiden(false);
        }
        xmaTableColumn.setYnAutoPack(false);
        xmaTableColumn.setYnSortable(true);
        xmaTableColumn.setYnResizeable(true);
        if (column.getFeature() != null) {
            DataObjectVariable dav = column.getObject();
            if (dav == null)
                dav = dataObjectVariable;
            BDAttribute bdAttribute = Util.findBDAttributeForFeature(xmaTable.getComponent(), businessDataList,
                    column.getFeature(), dav);
            if (bdAttribute != null) {
                xmaTableColumn.setDataAttribute(bdAttribute);
            }
        }

        Map<Integer, StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(column);
        ApplyAlignmentStyle.applyAlignment(xmaTableColumn, null, styleProperties);

        if (column.getAlign() != null) {
            switch (column.getAlign().getValue()) {
            case ContentAlignment.LEFT_VALUE:
                xmaTableColumn.setCodAlignment(AlignmentType.LEFT_LITERAL);
                break;
            case ContentAlignment.RIGHT_VALUE:
                xmaTableColumn.setCodAlignment(AlignmentType.RIGHT_LITERAL);
                break;
            case ContentAlignment.CENTER_VALUE:
                xmaTableColumn.setCodAlignment(AlignmentType.CENTER_LITERAL);
                break;
            }
        }
        if (column.getWidth() > 0) {
            if (column.isRelativeWidth()) {
                xmaTableColumn.setQntPercent(column.getWidth());
                xmaTableColumn.setQntWidth(-1);
            } else {
                xmaTableColumn.setQntWidth(column.getWidth());
                xmaTableColumn.setQntPercent(-1);
            }
        } else {
            /* Style-property of column not defined, apply width from style if available */
            StylePropertyWidth stylePropertyWidth = styleProperties != null ? (StylePropertyWidth) styleProperties
                    .get(PomPackage.STYLE_PROPERTY_WIDTH) : null;
            if (stylePropertyWidth != null) {
                xmaTableColumn.setQntWidth(stylePropertyWidth.getWidth());
                xmaTableColumn.setQntPercent(-1);
            } else {
                xmaTableColumn.setQntWidth(-1);
            }
        }
        if (column.getMinWidth() > 0) {
            xmaTableColumn.setQntMinWidth(column.getMinWidth());
        } else {
            xmaTableColumn.setQntMinWidth(-1);
        }
        if (column.getMaxWidth() > 0) {
            xmaTableColumn.setQntMaxWidth(column.getMaxWidth());
        } else {
            xmaTableColumn.setQntMaxWidth(-1);
        }
        xmaTable.getColumn().add(xmaTableColumn);
    }

    public static void createFormaterForTable(XMATable xmaTable, XMAPage xmaPage) {

        EList<XMATableColumn> columns = xmaTable.getColumn();
        for (XMATableColumn column : columns) {
            createFormaterForColumn(column, xmaPage);
        }
    }

    static void createFormaterForColumn(XMATableColumn xmaColumn, XMAPage xmaPage) {

        TableColumn column = (TableColumn) WidgetExtension.getSourceModelElementProperty(xmaColumn);
        if (column.getFeature() == null) {
            return;
        }
        List<ReferenceWithParameter> definitionStack = null;// attribute.getType().getDefinitionStack();
        PresentableFeature presentableFeature = column.getFeature().getAttributeHolder();
        ValidatorReference validatorReference = null;
        // Get the validator defined in the format-property
        for (AttributeProperty attributeProperty : presentableFeature.getAttributProperties()) {
            if (attributeProperty instanceof AttributeValidationProperty) {
                AttributeValidationProperty attrValProp = (AttributeValidationProperty) attributeProperty;
                validatorReference = attrValProp.getFormat();
                if (validatorReference != null) { // in case a formatter is defined determine the definition stack
                    definitionStack = validatorReference.getDefinitionStack();
                }
                break;
            }
        }
        // As default get the validator based on the type definition
        if (validatorReference == null && presentableFeature.getAttribute() != null) {
            Attribute att = presentableFeature.getAttribute();
            definitionStack = att.getType().getDefinitionStack();
            validatorReference = att.getType().getValidatorReferenceDefinition();
            // If no explicit ValidatorReference is defined and the attribute is of type "Enum" we can create
            // a default ValidatorRefence for the Enum.
            if (validatorReference == null && att.getType() != null && att.getType().getDataType() instanceof EnumType) {
                validatorReference = WidgetExtension.getDefaultValidatorReferenceForEnum((EnumType) att.getType()
                        .getDataType());
                definitionStack = validatorReference.getDefinitionStack();
            }
        }

        // Create and set the validator
        if (validatorReference != null) {
            Map<Integer, StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(column);
            Validator xmaValidator = ValidatorExtension.createXMAValidator(validatorReference, definitionStack,
                    styleProperties, true);
            xmaColumn.setBdValidator(xmaValidator);
        }
    }

    /**
     * 
     * @param composite
     * @return
     */
    public static XMAComposite createXMAComposite(XmadslComposite composite) {

        String name = composite.getName();
        if (name == null) {
            name = PomNameProvider.composeDefaultNameForGuiElement(composite);
        }

        XMAComposite xmaComposite = null;
        if (composite.getLabelText() == null) {
            xmaComposite = GuidesignFactory.eINSTANCE.createXMAComposite();
        } else {
            XMAGroup xmaGroup = GuidesignFactory.eINSTANCE.createXMAGroup();
            xmaGroup.setLabelName(composite.getLabelText().getString());
            xmaComposite = xmaGroup;
        }

        XMAScrolledComposite scrolledComposite = null;
        if (composite.getScrollable() instanceof TrueLiteral) {
            scrolledComposite = GuidesignFactory.eINSTANCE.createXMAScrolledComposite();
            scrolledComposite.setInnerComp(xmaComposite);
            scrolledComposite.setNamInstance(PomNameProvider.getScrolledName(name));
            scrolledComposite.setYnGenerated(true);
            scrolledComposite.setYnExpandChild(true);
            scrolledComposite.setYnBorder(false);
            setLayoutDataPropertyFromGuiElement(scrolledComposite, composite);
            WidgetExtension.setTabable(scrolledComposite, composite);
        } else {
            WidgetExtension.setTabable(xmaComposite, composite);
        }

        xmaComposite.setYnGenerated(true);
        xmaComposite.setNamInstance(name);
        setLayoutDataPropertyFromGuiElement(xmaComposite, composite);

        return xmaComposite;
    }

    public static XMASashForm createXMASashForm(SplitPanel splitPanel) {
        XMASashForm newSashForm = GuidesignFactory.eINSTANCE.createXMASashForm();
        newSashForm.setYnGenerated(true);
        newSashForm.setNamInstance(splitPanel.getName());

        WidgetExtension.setTabable(newSashForm, splitPanel);

        setLayoutDataPropertyFromGuiElement(newSashForm, splitPanel);
        return newSashForm;
    }

    public static XMATabFolder createXMATabFolder(TabFolder tabFolder) {

        String name = tabFolder.getName();
        if (name == null) {
            name = PomNameProvider.composeDefaultNameForGuiElement(tabFolder);
        }

        XMATabFolder xmaTabFolder = GuidesignFactory.eINSTANCE.createXMATabFolder();

        xmaTabFolder.setYnGenerated(true);
        xmaTabFolder.setNamInstance(name);

        WidgetExtension.setTabable(xmaTabFolder, tabFolder);

        setLayoutDataPropertyFromGuiElement(xmaTabFolder, tabFolder);

        return xmaTabFolder;
    }

    public static NotebookPage createNotebookPage(TabPage tabPage) {
        NotebookPage notebookPage = GuidesignFactory.eINSTANCE.createNotebookPage();

        Map<Integer, StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(tabPage);
        StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);

        String name = tabPage.getName();

        notebookPage.setYnGenerated(true);
        notebookPage.setNamClass(Strings.capitalize(name));
        notebookPage.setNamInstance(Strings.uncapitalize(name));

        String labelString = WidgetExtension.getLabelString(tabPage.getLabelText());
        notebookPage.setLabelName(labelString);

        if (stylePropertyOther != null && stylePropertyOther.getDynamicTabPage() instanceof TrueLiteral) {
            notebookPage.setYnDynamic(true);
        }

        return notebookPage;
    }

    /**
     * 
     * @param notebookPage
     * @param tabPage
     * @return
     */
    public static XMAComposite createAndAddPageComposite(NotebookPage notebookPage, TabPage tabPage) {

        XMAComposite xmaCompositeToWhichControlsCanBeAdded = null;
        PageComposite pageComposite = GuidesignFactory.eINSTANCE.createPageComposite();
        pageComposite.setNamInstance("pageComposite" + tabPage.getName());
        pageComposite.setYnGenerated(true);
        setLayoutDataPropertyFromGuiElement(pageComposite, tabPage);

        if (tabPage.getScrollable() instanceof TrueLiteral) {
            XmadslComposite xmadslComposite = PomFactory.eINSTANCE.createXmadslComposite();
            xmadslComposite.setName("inner" + pageComposite.getNamInstance());
            xmadslComposite.setScrollable(tabPage.getScrollable());
            XMAComposite xmaComposite = createXMAComposite(xmadslComposite);
            // Add the parent of the inner XMAComposite to the PageComposite
            if (xmaComposite.eContainer() != null) {
                XMAScrolledComposite scrolledComposite = (XMAScrolledComposite) xmaComposite.eContainer();
                XMAFormData formData = GuidesignFactory.eINSTANCE.createXMAFormData();
                XMAFormAttachment leftAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                XMAFormAttachment rightAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                XMAFormAttachment topAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                XMAFormAttachment bottomAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();

                topAttachment.setQntNominator(0);
                leftAttachment.setQntNominator(0);
                rightAttachment.setQntNominator(100);
                bottomAttachment.setQntNominator(100);

                formData.setTopAttach(topAttachment);
                formData.setLeftAttach(leftAttachment);
                formData.setRightAttach(rightAttachment);
                formData.setBottomAttach(bottomAttachment);

                scrolledComposite.setFormData(formData);
                pageComposite.getControls().add(scrolledComposite);
            } else {
                throw new RuntimeException("Missing parent of Type 'ScrollableComposite'");
            }
            xmaCompositeToWhichControlsCanBeAdded = xmaComposite;
        } else {
            xmaCompositeToWhichControlsCanBeAdded = pageComposite;
        }

        notebookPage.setComposite(pageComposite);

        return xmaCompositeToWhichControlsCanBeAdded;
    }

    public static Composite setPartitioningDataProperty(Composite widget, PartitioningData obj) {
        if (obj != null) {
            EObjectPropertiesAdapter.setProperty(widget, PARTITIONING_DATA_PROPERTY_NAME, obj);
        }
        return widget;
    }

    public static PartitioningData getPartitioningDataProperty(Composite widget) {
        Object value = EObjectPropertiesAdapter.getProperty(widget, PARTITIONING_DATA_PROPERTY_NAME);
        if (value instanceof PartitioningData || value == null) {
            return (PartitioningData) value;
        }
        throw new RuntimeException("The property '" + PARTITIONING_DATA_PROPERTY_NAME
                + "' is not of type 'PartitioningData'");
    }

    static class PartitioningData {
        int currentIndex = -1;
        int cellCount;
        int margin;
        int padding;

        int columnWidthsCount = 0;
        int rowHeightsCount = 0;

        // boolean floating=false;
        boolean floatingPassed = false;
        boolean fillingPassed = false;
        boolean filling = false;

        double percentCorrecturForCell;
        double sumPercent;
        double sumOffset;
        int absoluteWidth = 0;

        public PartitioningData(int margin, int padding, int cellCount, int columnWidthsCount, int rowHeightsCount) {

            if (columnWidthsCount > 0 && rowHeightsCount > 0) {
                throw new IllegalArgumentException("Only columns or rows can have partitioning information");
            }

            this.margin = margin;
            this.padding = padding;
            this.cellCount = cellCount;
            this.columnWidthsCount = columnWidthsCount;
            this.rowHeightsCount = rowHeightsCount;

            int percentCorrecturTotal = (cellCount - 1) * padding;
            percentCorrecturForCell = percentCorrecturTotal * 1. / cellCount;
        }

        void nextPos(TabulatorPosition pos) {
            currentIndex++;
            filling = false;
            absoluteWidth = 0;
            if (pos.isFloatingPosition()) {
                floatingPassed = true;
                return;
            }
            if (pos.isFillingPosition()) {
                filling = true;
                fillingPassed = true;
                return;
            }
            if (pos.isRelative()) {
                if (pos.isDeltaSpecification()) {
                } else {
                    sumPercent += pos.getWidth();
                    if (currentIndex < cellCount - 1) {
                        sumOffset = 0 - padding / 2.;
                    } else {
                        // sumOffset=0;
                    }
                }
                if (pos.getOffset() != null) {
                    if (pos.getOffset().isNegative()) {
                        sumOffset -= pos.getOffset().getValue();
                    } else {
                        sumOffset += pos.getOffset().getValue();
                    }
                }
            } else {
                if (pos.isDeltaSpecification()) {
                } else {
                    if (floatingPassed == false && fillingPassed == false) {
                        sumOffset += pos.getWidth();
                    } else {
                        absoluteWidth = pos.getWidth();
                        sumOffset = 0.;
                        sumPercent = 0.;
                    }
                    // sumPercent=0;
                }
            }

        }

        boolean isBasicAlignmentDirection() {
            return filling || (fillingPassed == false);
        }

        int getPercentAsInt() {
            return (int) Math.round(sumPercent);
        }

        int getOffsetAsInt() {
            return (int) Math.round(sumOffset);
        }

        int getAbsoluteWidth() {
            if (columnWidthsCount > 0) {
                return absoluteWidth;
            } else {
                return 0;
            }
        }

        int getAbsoluteHeight() {
            if (rowHeightsCount > 0) {
                return absoluteWidth;
            } else {
                return 0;
            }
        }

        XMAFormAttachment getTopAttachment(int index, XMAWidget prevMainWidget) {
            if (rowHeightsCount > index) {
                if (floatingPassed == false && fillingPassed == false) {
                    XMAFormAttachment attachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                    attachment.setQntNominator(getPercentAsInt());
                    attachment.setQntOffset(getOffsetAsInt());
                    return attachment;
                }
            }
            return null;
        }

        XMAFormAttachment getLeftAttachment(int index, XMAWidget prevMainWidget) {
            if (columnWidthsCount > index) {
                if (floatingPassed == false && fillingPassed == false) {
                    XMAFormAttachment rightAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                    rightAttachment.setQntNominator(getPercentAsInt());
                    rightAttachment.setQntOffset(getOffsetAsInt());
                    return rightAttachment;
                }
            }
            return null;
        }

        XMAFormAttachment getRightAttachment(int index, XMAWidget nextMainWidget) {
            if (fillingPassed) {
                XMAFormAttachment attachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                if (index == cellCount - 1) {
                    attachment.setQntNominator(100);
                } else {
                    attachment.setAttachElement(nextMainWidget);
                    attachment.setCodAttachSide(AttachSideType.LEFT_LITERAL);
                    attachment.setQntOffset(-padding);
                }
                return attachment;
            }
            if (columnWidthsCount > index) {
                if (floatingPassed == false) {
                    XMAFormAttachment rightAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                    rightAttachment.setQntNominator(getPercentAsInt());
                    rightAttachment.setQntOffset(getOffsetAsInt());
                    return rightAttachment;
                }
            }
            return null;
        }

        XMAFormAttachment getBottomAttachment(int index, XMAWidget nextMainWidget) {
            if (fillingPassed) {
                XMAFormAttachment attachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                if (index == cellCount - 1) {
                    attachment.setQntNominator(100);
                } else {
                    attachment.setAttachElement(nextMainWidget);
                    attachment.setCodAttachSide(AttachSideType.TOP_LITERAL);
                    attachment.setQntOffset(-padding);
                }
                return attachment;
            }
            if (rowHeightsCount > index) {
                if (floatingPassed == false) {
                    XMAFormAttachment attachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
                    attachment.setQntNominator(getPercentAsInt());
                    attachment.setQntOffset(getOffsetAsInt());
                    return attachment;
                }
            }
            return null;
        }

    }

    public static List<XMAComposite> createMainWidgetList(Integer cellCount) {
        List<XMAComposite> mainWidgetList = new ArrayList<XMAComposite>();
        for (int i = 0; i < cellCount; i++) {
            mainWidgetList.add(GuidesignFactory.eINSTANCE.createXMAComposite());
        }
        return mainWidgetList;
    }

    public static boolean isCollapsingSupported(Composite composite) {
        ComposeData composeData = composite.getContent().getComposeLayout();
        if (composeData != null) {
            for (TabulatorPosition pos : composeData.getCellHeights()) {
                if (pos.isFillingPosition())
                    return false;
            }
            for (TabulatorPosition pos : composeData.getCellWidths()) {
                if (pos.isFillingPosition())
                    return false;
            }
        }
        return true;
    }

    public static XMAComposite createCompositeForCollapseGroup(SetOfGuiElements setOfSimpleElements,
            CollapseChain collapseChain, Composite parentXmadslComposite, XMAComposite parentXMAComposite,
            List<XMAComposite> mainWidgetList, Integer padding, Integer margin, Integer cellCount) {

        int index = collapseChain.getCollapseGroups().size();

        XMAWidget prevMainWidget = null;
        if (index > 0) {
            prevMainWidget = mainWidgetList.get(index - 1);
        }
        XMAComposite compositeForCollapseGroup = mainWidgetList.get(index);
        XMAWidget nextMainWidget = null;
        if (index < mainWidgetList.size() - 1) {
            nextMainWidget = mainWidgetList.get(index + 1);
        }

        // XMAComposite compositeForCollapseGroup = GuidesignFactory.eINSTANCE.createXMAComposite();
        XMAFormData formDataOfCompositeForCollapseGroup = GuidesignFactory.eINSTANCE.createXMAFormData();
        ComposeData composeData = parentXmadslComposite.getContent().getComposeLayout();

        PanelOrientationFlags pof = new PanelOrientationFlags(parentXmadslComposite);

        // Get the PartitioningData
        PartitioningData pd = getPartitioningDataProperty(parentXmadslComposite);
        if (pd == null) {
            int columns = (composeData != null) ? composeData.getCellWidths().size() : 0;
            int rows = (composeData != null) ? composeData.getCellHeights().size() : 0;
            pd = new PartitioningData(margin, padding, cellCount, columns, rows);
            setPartitioningDataProperty(parentXmadslComposite, pd);
        }

        // Set the next position

        // int index = collapseChain.getCollapseGroups().size();
        TabulatorPosition pos = null;
        if (composeData != null) {
            if (composeData.getCellWidths().size() > index) {
                pos = composeData.getCellWidths().get(index);
            }
            if (composeData.getCellHeights().size() > index) {
                pos = composeData.getCellHeights().get(index);
            }
        }
        if (pos != null) {
            pd.nextPos(pos);
        }

        // TOP
        if (pof.isHorizontal() || (pof.isTopAligned() && pd.isBasicAlignmentDirection())) {
            XMAFormAttachment topAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            if (pof.isHorizontal() == false && pof.isTopAligned() && prevMainWidget != null) {
                topAttachment.setCodAttachSide(AttachSideType.BOTTOM_LITERAL);
                topAttachment.setQntOffset(padding);
                topAttachment.setAttachElement(prevMainWidget);
            }
            formDataOfCompositeForCollapseGroup.setTopAttach(topAttachment);
        } else {
            if (composeData != null) {
                XMAFormAttachment topAttachment = pd.getTopAttachment(index, prevMainWidget);
                if (topAttachment != null) {
                    formDataOfCompositeForCollapseGroup.setTopAttach(topAttachment);
                }
            }
        }

        // LEFT
        if (pof.isVertical() || (pof.isLeftAligned() && pd.isBasicAlignmentDirection())) {
            XMAFormAttachment leftAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            if (pof.isVertical() == false && pof.isLeftAligned() && prevMainWidget != null) {
                leftAttachment.setCodAttachSide(AttachSideType.RIGHT_LITERAL);
                leftAttachment.setQntOffset(padding);
                leftAttachment.setAttachElement(prevMainWidget);

            }
            formDataOfCompositeForCollapseGroup.setLeftAttach(leftAttachment);
        } else {
            if (composeData != null) {
                XMAFormAttachment leftAttachment = pd.getLeftAttachment(index, prevMainWidget);
                if (leftAttachment != null) {
                    formDataOfCompositeForCollapseGroup.setLeftAttach(leftAttachment);
                }
            }
        }
        // if (prevMainWidget != null) {
        // pd.sumOffset+=padding;
        // }

        // RIGHT
        if (pof.isVertical() || pof.isRightAligned()) {
            XMAFormAttachment rightAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            if (pof.isRightAligned() && prevMainWidget != null) {
                rightAttachment.setCodAttachSide(AttachSideType.LEFT_LITERAL);
                rightAttachment.setQntOffset(-padding);
                rightAttachment.setAttachElement(prevMainWidget);
            } else {
                rightAttachment.setQntNominator(100);
            }
            formDataOfCompositeForCollapseGroup.setRightAttach(rightAttachment);
        } else {
            if (composeData != null) {
                XMAFormAttachment rightAttachment = pd.getRightAttachment(index, nextMainWidget);
                if (rightAttachment != null) {
                    formDataOfCompositeForCollapseGroup.setRightAttach(rightAttachment);
                }
            }
        }

        // BOTTOM
        if (pof.isHorizontal() || pof.isBottomAligned()) {
            XMAFormAttachment bottomAttachment = GuidesignFactory.eINSTANCE.createXMAFormAttachment();
            if (pof.isBottomAligned() && prevMainWidget != null) {
                bottomAttachment.setCodAttachSide(AttachSideType.TOP_LITERAL);
                bottomAttachment.setQntOffset(-padding);
                bottomAttachment.setAttachElement(prevMainWidget);
            } else {
                bottomAttachment.setQntNominator(100);
            }
            formDataOfCompositeForCollapseGroup.setBottomAttach(bottomAttachment);
        } else {
            if (composeData != null) {
                XMAFormAttachment bottomAttachment = pd.getBottomAttachment(index, nextMainWidget);
                if (bottomAttachment != null) {
                    formDataOfCompositeForCollapseGroup.setBottomAttach(bottomAttachment);
                }
            }
        }

        formDataOfCompositeForCollapseGroup.setQntHeight(pd.getAbsoluteHeight());
        formDataOfCompositeForCollapseGroup.setQntWidth(pd.getAbsoluteWidth());

        compositeForCollapseGroup.setFormData(formDataOfCompositeForCollapseGroup);
        compositeForCollapseGroup.setQntMarginHeight(0);
        compositeForCollapseGroup.setQntMarginWidth(0);

        compositeForCollapseGroup.setYnGenerated(true);
        compositeForCollapseGroup.setNamInstance(parentXMAComposite.getNamInstance() + "_set"
                + collapseChain.getCollapseGroups().size());
        WidgetExtension.setSourceModelElementProperty(compositeForCollapseGroup, setOfSimpleElements);

        WidgetExtension.setGuiElementProperty(compositeForCollapseGroup, parentXmadslComposite);

        return compositeForCollapseGroup;
    }

    static class PanelOrientationFlags {

        private boolean isHorizontal = false;
        private boolean isRightAligned = false;
        private boolean isBottomAligned = false;

        public PanelOrientationFlags(Composite compos) {
            if (compos instanceof XmadslComposite) {
                isHorizontal = ((XmadslComposite) compos).isHorizontalPartitioning();
                EList<LayoutDataProperty> properties = ((XmadslComposite) compos).getLayoutProperties();
                for (LayoutDataProperty prop : properties) {
                    if (prop instanceof AlignmentProperty) {
                        AlignmentProperty ap = (AlignmentProperty) prop;
                        if (ap.getHAlignment().getValue() == HorizontalAlignment.RIGHT_VALUE) {
                            isRightAligned = true;
                            break;
                        }
                        if (ap.getVAlignment().getValue() == VerticalAlignment.BOTTOM_VALUE) {
                            isBottomAligned = true;
                            break;
                        }
                    }
                }
            }

        }

        public boolean isHorizontal() {
            return isHorizontal;
        }

        public boolean isVertical() {
            return isHorizontal == false;
        }

        public boolean isRightAligned() {
            return isRightAligned;
        }

        public boolean isBottomAligned() {
            return isBottomAligned;
        }

        public boolean isTopAligned() {
            return isVertical() && isBottomAligned == false;
        }

        public boolean isLeftAligned() {
            return isHorizontal() && isRightAligned == false;
        }
    }

    public static CollapseChain createCollapseChain(Composite compos) {

        PanelOrientationFlags pof = new PanelOrientationFlags(compos);

        CollapseChain collapseChain = FlexFactory.eINSTANCE.createCollapseChain();
        if (pof.isHorizontal()) {
            if (pof.isRightAligned() == false) {
                collapseChain.setCodDircection(3); // left to right
            } else {
                collapseChain.setCodDircection(4); // right to left
            }
        } else {
            if (pof.isBottomAligned() == false) {
                collapseChain.setCodDircection(1); // top to down
            } else {
                collapseChain.setCodDircection(2); // bottom to top
            }
        }
        return collapseChain;
    }

}
