package org.openxma.dsl.generator.helper;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.openxma.dsl.core.model.BooleanParameterValue;
import org.openxma.dsl.core.model.BooleanValue;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.IntegerParameterValue;
import org.openxma.dsl.core.model.ParameterDefinition;
import org.openxma.dsl.core.model.ParameterValue;
import org.openxma.dsl.core.model.ReferenceWithParameter;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.StringParameterValue;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.core.model.ValueModelType;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.generator.impl.DomPageLogicAttachmentStrategy;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.StylePropertyEnumerationDisplayType;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.Text;

import at.spardat.xma.guidesign.BcdValidator;
import at.spardat.xma.guidesign.BooleanValidator;
import at.spardat.xma.guidesign.CustomValidator;
import at.spardat.xma.guidesign.DateValidator;
import at.spardat.xma.guidesign.DomainValidator;
import at.spardat.xma.guidesign.GuidesignFactory;
import at.spardat.xma.guidesign.StringValidator;
import at.spardat.xma.guidesign.TimeStampValidator;
import at.spardat.xma.guidesign.ValidInState;
import at.spardat.xma.guidesign.Validator;
import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAText;
import at.spardat.xma.guidesign.flex.CompoundValidator;
import at.spardat.xma.guidesign.flex.Constraint;
import at.spardat.xma.guidesign.flex.FlexFactory;
import at.spardat.xma.guidesign.flex.LiteralExpression;
import at.spardat.xma.guidesign.flex.UserDefinedValidator;
import at.spardat.xma.guidesign.flex.type.WidgetModelType;
import at.spardat.xma.guidesign.types.DateFormatType;
import at.spardat.xma.guidesign.types.DomainFormatType;
import at.spardat.xma.guidesign.types.EmptyBoolean;
import at.spardat.xma.guidesign.types.PropertyType;
import at.spardat.xma.guidesign.types.TimeFormatType;

public class ValidatorExtension {
    /**
     * @param dataTypeAndTypeParameter
     * @return true or false whether the given dataType has an associated date
     *         or timestamp validator
     */
    public static boolean hasDateOrTimeStampValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        return hasDateValidator(dataTypeAndTypeParameter) || hasTimeStampValidator(dataTypeAndTypeParameter);
    }
    /**
     * @param dataTypeAndTypeParameter
     * @return true or false whether the given dataType has an associated custom validator
     */
    public static boolean hasCustomValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return null != validatorReference ? validatorReference.isCustomValidator() : false;
    }
    /**
     * @param dataTypeAndTypeParameter
     * @return true or false whether the given dataType has an associated date validator
     */
    public static boolean hasDateValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return null != validatorReference ? validatorReference.isDateValidator() : false;
    }
    /**
     * @param dataTypeAndTypeParameter
     * @return true or false whether the given dataType has an associated timestamp validator
     */
    public static boolean hasTimeStampValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return null != validatorReference ? validatorReference.isTimeStampValidator() : false;
    }
    /**
     *
     * @param dataTypeAndTypeParameter
     * @return true or false whether the given dataType has an associated string validator
     */
    public static boolean hasStringValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return null!=validatorReference ? isStringValidator(validatorReference) : false;
    }
    /**
     *
     * @param dataTypeAndTypeParameter
     * @return true or false whether the given dataType has an associated string validator
     */
    public static boolean hasNumberValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return null!=validatorReference ? isNumberValidator(validatorReference) : false;
    }
    /**
     *
     * @param dataTypeAndTypeParameter
     * @return true or false whether the given dataType has an associated boolean validator
     */
    public static boolean hasBooleanValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return null!=validatorReference ? isBooleanValidator(validatorReference) : false;
    }
    /**
     *
     * @param dataTypeAndTypeParameter
     * @return true or false whether the given dataType has an associated boolean validator
     */
    public static boolean hasI18nValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return null!=validatorReference ? isI18nValidator(validatorReference) : false;
    }
    /**
     * @param simpleType
     * @return true or false whether the given simpleType has an associated string validator
     */
    public static boolean hasStringValidator(SimpleType simpleType) {
        ValidatorReference validatorReference = simpleType.getValidatorReference();
        return null!=validatorReference ? isStringValidator(validatorReference) : false;
    }
    /**
     * @param validatorReference
     * @return true or false whether the given validatorReference is a string validator
     */
    public static boolean isStringValidator(ValidatorReference validatorReference) {
        return validatorReference.isStringValidator();

    }
    /**
     * @param validatorReference
     * @return true or false whether the given validatorReference is a boolean validator
     */
    public static boolean isBooleanValidator(ValidatorReference validatorReference) {
        return validatorReference.isBooleanValidator();
    }
    /**
     * @param validatorReference
     * @return true or false whether the given validatorReference is a I18n validator
     */
    public static boolean isI18nValidator(ValidatorReference validatorReference) {
        return validatorReference.isI18nValidator();
    }
    /**
     * @param validatorReference
     * @return true or false whether the given validatorReference is a number validator
     */
    public static boolean isNumberValidator(ValidatorReference validatorReference) {
        return validatorReference.isNumberValidator();
    }
    /**
     * @param validatorReference
     * @return true or false whether the given validatorReference is a custom validator
     */
    public static boolean isCustomValidator(ValidatorReference validatorReference) {
        return validatorReference.isCustomValidator();

    }
    /**
     * @param validatorReference
     * @return true or false whether the given validatorReference is a user defined validator
     */
    public static boolean isUserDefinedValidator(ValidatorReference validatorReference) {
        return validatorReference.isUserDefinedValidator();
    }
    /**
     * @param validatorReference
     * @return true or false whether the given validatorReference is a date defined validator
     */
    public static boolean isDateValidator(ValidatorReference validatorReference) {
        return validatorReference.isDateValidator();
    }
    /**
     * @param validatorReference
     * @return true or false whether the given validatorReference is a timestamp defined validator
     */
    public static boolean isTimestampValidator(ValidatorReference validatorReference) {
        return validatorReference.isTimeStampValidator();
    }       
    
    /**
     * @see #createXMAValidator(ValidatorReference, List)
     */
    public static Validator createXMAValidator(ValidatorReference validatorReference) {
        return createXMAValidator(validatorReference,validatorReference.getDefinitionStack());
    }
    
    public static Validator createXMAValidator(ValidatorReference validatorReference, List<ReferenceWithParameter> definitionStack) {
        return createXMAValidator(validatorReference,definitionStack,null,false);
    }

    /**
     * Create a xma validator from a xmadsl validator reference including the
     * validator parameter defined in the validator reference.
     *
     * @param validatorReference
     * @param definitionStack
     * @return
     */
    public static Validator createXMAValidator(ValidatorReference validatorReference, List<ReferenceWithParameter> definitionStack, Map<Integer,StyleSpecificationProperty> styleProperties, Boolean handleEnumerationValidator) {
        ValidatorReference rootValidatorReference = validatorReference.getRootValidatorReference();

        if (definitionStack==null) {
            definitionStack = validatorReference.getDefinitionStack();
        }

        List<String> parameterValues = getParameterValuesAsStrings(rootValidatorReference,definitionStack);

        return createXMAValidatorFromRootValidatorReference(rootValidatorReference, parameterValues, styleProperties, (handleEnumerationValidator != null && handleEnumerationValidator));
    }

    public static Validator createXMAValidatorFromRootValidatorReference(ValidatorReference rootValidatorReference, List<String> parameterValues, Map<Integer,StyleSpecificationProperty> styleProperties, boolean handleEnumerationValidator) {

        String validatorName = BasicTypeHelper.determineRootValidatorName(rootValidatorReference);
        if (ValidatorReference.STRING_VALIDATOR.equals(validatorName)) {
            return createStringValidator(parameterValues);
        }
        if (ValidatorReference.NUMBER_VALIDATOR.equals(validatorName)) {
            return createBcdValidator(parameterValues);
        }
        if (ValidatorReference.BOOLEAN_VALIDATOR.equals(validatorName)) {
            return createBooleanValidator(parameterValues);
        }
        if (ValidatorReference.DATE_VALIDATOR.equals(validatorName)) {
            return createDateValidator(parameterValues);
        }
        if (ValidatorReference.TIME_STAMP_VALIDATOR.equals(validatorName)) {
            return createTimeStampValidator(rootValidatorReference,parameterValues);
        }
        if (ValidatorReference.CUSTOM_VALIDATOR.equals(validatorName)) {
            return createCustomValidator(rootValidatorReference,parameterValues);
        }
        if (ValidatorReference.I18N_ENUMERATION_VALIDATOR.equals(validatorName)) {
            if (handleEnumerationValidator) {
                DomainValidator domainValidator = createDomainValidator(parameterValues);  
                StylePropertyEnumerationDisplayType displayTypeStyle = PresentationModelExtension.getEnumerationDisplayTypeStyle(styleProperties);
                if (displayTypeStyle != null) {                    
                    domainValidator.setCodFormattype(WidgetExtension.mapToDomainFormatType(displayTypeStyle));   
                }                  
                return domainValidator;
            } else {
                return null; // do nothing (datasource for domain is handled somewhere else)
            }
        }
        return createUserDefinedValidator(rootValidatorReference,parameterValues);
    }

    /**
     * Override the formatter of a CompoundValidator by a Formatter created
     * from a given ValidatorReference.
     *
     * @param cpv
     * @param validatorReference
     * @return
     */
    public static Validator overrideFormatter(CompoundValidator cpv, ValidatorReference validatorReference) {

        Validator xmaValidator = createXMAValidator(validatorReference,null,null,false);
        Constraint xmaConstraint = FlexFactory.eINSTANCE.createConstraint();

        xmaConstraint.setValidator(xmaValidator);
        xmaConstraint.setCondition(createBooleanLiteral(false));

        cpv.getConstraints().add(xmaConstraint);
        cpv.setFormatter(xmaValidator);

        return xmaValidator;
    }

    /**
     * Create a Boolean literal expression from a boolean value.
     *
     * @param isTrue
     * @return
     */
    public static LiteralExpression createBooleanLiteral(Boolean isTrue) {
        if (isTrue == null) {
            throw new IllegalArgumentException("Boolean parameter must not be null");
        }
        LiteralExpression exp = FlexFactory.eINSTANCE.createLiteralExpression();
        exp.setTypLiteral(PropertyType.BOOLEAN_LITERAL);
        if (isTrue) {
            exp.setTxtLiteral("true");
        } else {
            exp.setTxtLiteral("false");
        }
        return exp;
    }

    /**
     * Get a list containing the string values for all parameter values for a
     * validator reference.
     *
     * In case that some parameters are referenced in a definition stack the
     * according parameter value for the referenced parameter is determined from the
     * definition stack.
     * @param validatorReference
     * @param definitionStack
     * @return
     */
    private static List<String> getParameterValuesAsStrings(ValidatorReference validatorReference, List<ReferenceWithParameter> definitionStack) {
        List<ParameterValue> pvList = validatorReference.getParameterValues();
        List<String> parameterValues = new ArrayList<String>();
        if (definitionStack != null) {
            for (ParameterValue pv : pvList) {
                String spv = getParameterValue(pv,definitionStack,definitionStack.size()-1);
                parameterValues.add(spv);
            }
        } else {
            for (ParameterValue pv : pvList) {
                String spv = getParameterValue(pv,null,-1);
                parameterValues.add(spv);
            }
        }
        return parameterValues;
    }

    /**
     * Get the parameter value as string for a given parameter within a given
     * level of the definition stack.
     * @param pv
     * @param definitionStack
     * @param stackIndex
     * @return
     */
    private static String getParameterValue(ParameterValue pv, List<ReferenceWithParameter> definitionStack, Integer stackIndex) {

        if (pv.getReferencedParameter() == null) {
            return getValueOfParameterAsString(pv);
        } else {
            if (definitionStack == null) {
                throw new RuntimeException("The definition stack must not be null in case of referenced parameters");
            }
            ParameterDefinition pd = pv.getReferencedParameter();
            return getValueOfReferencedParameterAsString(pd,definitionStack,stackIndex);
        }
    }

    /**
     * Get a parameter value as a string.
     * @param pv
     * @return
     */
    private static String getValueOfParameterAsString(ParameterValue pv) {

        if (pv == null) return null;

        if (pv instanceof IntegerParameterValue) {
            return Integer.toString(((IntegerParameterValue)pv).getIntValue());
        } else if (pv instanceof StringParameterValue) {
            return ((StringParameterValue)pv).getStringValue();
        } else if (pv instanceof BooleanParameterValue) {
            return ((BooleanParameterValue)pv).getBooleanValue().toString();
        }
        return null;
    }

    /**
     * Get the string value for a referenced parameter.
     * @param pd
     * @param definitionStack
     * @param stackIndex
     * @return
     */
    private static String getValueOfReferencedParameterAsString(ParameterDefinition pd, List<ReferenceWithParameter> definitionStack, Integer stackIndex) {

        ReferenceWithParameter currentDefinition = definitionStack.get(stackIndex);

        List<ParameterValue> parameters = currentDefinition.getParameterValues();
        List<ParameterDefinition> parameterDefs = currentDefinition.getParameterDefinitions();

        ParameterDefinition def = null;
        for (ParameterDefinition parDef : parameterDefs) {
            if (parDef.getName().equals(pd.getName())) {
                def = parDef;
                break;
            }
        }

        if (def != null) {
            int index = parameterDefs.indexOf(def);
            String stringValue = getParameterValueForIndex(parameters,index,definitionStack,stackIndex-1);
            /* Use default value when there is no value specified */
            if (stringValue == null) {
                switch(pd.getType()) {
                case INTEGER: if (pd.getDefaultInteger() != 0) stringValue = Integer.toString(pd.getDefaultInteger()); break;
                case STRING: if (pd.getDefaultString() != null) stringValue = pd.getDefaultString(); break;
                case BOOLEAN: if (pd.getDefaultBoolean() != BooleanValue.NULL) stringValue = pd.getDefaultBoolean().toString(); break;
                default: throw new RuntimeException("Type for default value is not supported: "+pd.getType());
                }
            }
            return stringValue;
        } else {
            return getValueOfReferencedParameterAsString(pd,definitionStack,stackIndex-1);
        }
    }

    /**
     * Get the parameter value for a given index in a parameter list within a
     * given level in the definition stack.
     * @param parameters
     * @param index
     * @param definitionStack
     * @param stackIndex
     * @return
     */
    private static String getParameterValueForIndex(List<ParameterValue> parameters, Integer index,List<ReferenceWithParameter> definitionStack, Integer stackIndex) {
        if (index > -1 && parameters.size() > index) {
            ParameterValue pv = parameters.get(index);
            return getParameterValue(pv,definitionStack,stackIndex);
        }
        return null;
    }

    /**
     * Create a string validator.
     * @param validatorReference
     * @param parameters
     * @return
     */
    public static StringValidator createStringValidator(List<String> parameters) {

        StringValidator val = GuidesignFactory.eINSTANCE.createStringValidator();
        String minLength = getValueForIndex(parameters,0);
        String maxLength = getValueForIndex(parameters,1);
        String txtFormatrange = getValueForIndex(parameters,2);

        if (minLength.length()>0) val.setCntMinLength(minLength);
        if (maxLength.length()>0) val.setCntMaxLength(maxLength);
        if (txtFormatrange != null && txtFormatrange.length()>0) val.setTxtFormatrange(txtFormatrange);
        return val;
    }

    /**
     * Create a number validator.
     * @param validatorReference
     * @param parameters
     * @return
     */
    private static BcdValidator createBcdValidator(List<String> parameters) {

        BcdValidator val = GuidesignFactory.eINSTANCE.createBcdValidator();
        String precision = getValueForIndex(parameters,0);
        String scale = getValueForIndex(parameters,1);
        String minValue = getValueForIndex(parameters,2);
        String maxValue = getValueForIndex(parameters,3);
        String hasThousendSeperator = getValueForIndex(parameters,4);        
        String allowNegativValues = getValueForIndex(parameters,5);        
        String suppressZero = getValueForIndex(parameters,6);

        /* precision and scale */
        if (precision.length()>0 && scale.length()>0) {
            val.setCntBeforeComma(Util.calculateBeforeComma(precision,scale));
        } else {
            if (precision.length()>0) val.setCntBeforeComma(precision);
        }
        if (scale.length()>0) val.setCntAfterComma(scale);

        /* minValue and maxValue */
        if (minValue.length()>0) val.setQntMinVal(minValue);
        if (maxValue.length()>0) val.setQntMaxVal(maxValue);
        
        if ("true".equals(hasThousendSeperator)) {
            val.setYnThousandSep(EmptyBoolean.TRUE_LITERAL);
        } else if ("false".equals(hasThousendSeperator)) {
            val.setYnThousandSep(EmptyBoolean.FALSE_LITERAL);            
        } else {
            val.setYnThousandSep(EmptyBoolean.NOT_SET_LITERAL);
        }
        
        if ("true".equals(allowNegativValues)) {
            val.setYnNegative(EmptyBoolean.TRUE_LITERAL);
        } else if ("false".equals(allowNegativValues)) {
            val.setYnNegative(EmptyBoolean.FALSE_LITERAL);            
        } else {
            val.setYnNegative(EmptyBoolean.NOT_SET_LITERAL);
        }
        
        if ("true".equals(suppressZero)) {
            val.setYnSuppressZero(true);
        } else if ("false".equals(suppressZero)) {
            val.setYnSuppressZero(false);            
        }            
        
        return val;
    }

    /**
     * Create a boolean validator
     * @param validatorReference
     * @param parameters
     * @return
     */
    private static BooleanValidator createBooleanValidator(List<String> parameters) {

        BooleanValidator val = GuidesignFactory.eINSTANCE.createBooleanValidator();

        return val;
    }
    
    /**
     * Create a DomainValidator.
     */
    private static DomainValidator createDomainValidator(List<String> parameters) {
        DomainValidator val = GuidesignFactory.eINSTANCE.createDomainValidator(); 
        StringBuilder dataSource = new StringBuilder();
        if (parameters.get(0)!=null) {
            dataSource.append(parameters.get(0)); // first param: datasourceprefix
        }
        if (parameters.size() > 1) {
            if (parameters.get(1)!=null) { // append second parameter: datasourcename (
                dataSource.append(parameters.get(1));
            }            
        }
        // do not append third parameter (maxLength)
        val.setUriDataSource(dataSource.toString());
        return val;        
    }

    /**
     * Create a date xma validator.
     * @param validatorReference
     * @param parameters
     * @return
     */
    private static DateValidator createDateValidator(List<String> parameters) {

        DateValidator val = GuidesignFactory.eINSTANCE.createDateValidator();
        String codFormat = getValueForIndex(parameters,0);

        val.setCodFormat(getDateFormatType(codFormat));
        return val;
    }

    /**
     * Mapping from xmadsl date format to xma date format
     * @param code
     * @return
     */
    private static DateFormatType getDateFormatType(String code) {

        String lc = code.toLowerCase();

        if ("short".equals(lc)) return DateFormatType.SHORT_LITERAL;
        if ("medium".equals(lc)) return DateFormatType.MEDIUM_LITERAL;
        if ("long".equals(lc)) return DateFormatType.LONG_LITERAL;
        if ("full".equals(lc)) return DateFormatType.FULL_LITERAL;
        return DateFormatType.MEDIUM_LITERAL;
    }

    /**
     * Create a timestamp xma validator
     * @param validatorReference
     * @param parameters
     * @return
     */
    private static TimeStampValidator createTimeStampValidator(ValidatorReference validatorReference, List<String> parameters) {

        TimeStampValidator val = GuidesignFactory.eINSTANCE.createTimeStampValidator();
        String codFormat = getValueForIndex(parameters,0);
        val.setCodFormat(getTimeFormatType(codFormat));
        return val;
    }

    /**
     * Mapping from xmadsl time format to xma time format.
     * @param code
     * @return
     */
    private static TimeFormatType getTimeFormatType(String code) {

        String lc = code.toLowerCase();
        if ("short".equals(lc)) return TimeFormatType.SHORT_LITERAL;
        if ("medium".equals(lc)) return TimeFormatType.MEDIUM_LITERAL;
        if ("long".equals(lc)) return TimeFormatType.LONG_LITERAL;
        if ("full".equals(lc)) return TimeFormatType.FULL_LITERAL;
        return TimeFormatType.MEDIUM_LITERAL;
    }

    /**
     * Create a custom xma validator.
     * @param validatorReference
     * @param parameters
     * @return
     */
    private static CustomValidator createCustomValidator(ValidatorReference validatorReference, List<String> parameters) {

        CustomValidator val = GuidesignFactory.eINSTANCE.createCustomValidator();
        String txtFormat = getValueForIndex(parameters,0);

        val.setTxtFormatingString(txtFormat);
        return val;
    }

    /**
     * Create a user defined xma validator.
     * @param validatorReference
     * @param parameters
     * @return
     */
    private static UserDefinedValidator createUserDefinedValidator(ValidatorReference validatorReference, List<String> parameters) {
        UserDefinedValidator val = FlexFactory.eINSTANCE.createUserDefinedValidator();
        String namClass = validatorReference.getValidator().getInstanceClass();
        String txtParameter = getValueForIndex(parameters,0);
        val.setNamClass(DomPageLogicAttachmentStrategy.unEscape(namClass));
        val.setTxtParameterString(txtParameter);
        val.setTypModel(getWidgetModelType(validatorReference.getValidator()));
        return val;
    }

    /**
     * Mapping from xmadsl value model type to xma widget model type.
     * @param validator
     * @return
     */
    private static WidgetModelType getWidgetModelType(org.openxma.dsl.core.model.Validator validator) {

        switch (validator.getValueModelType().getValue()) {
            case ValueModelType.STRING_VALUE: return WidgetModelType.STRING_LITERAL;
            case ValueModelType.NUMBER_VALUE: return WidgetModelType.BCD_LITERAL;
            case ValueModelType.BOOLEAN_VALUE: return WidgetModelType.BOOLEAN_LITERAL;
            case ValueModelType.DATE_VALUE: return WidgetModelType.DATE_LITERAL;
            case ValueModelType.TIMESTAMP_VALUE: return WidgetModelType.TIMESTAMP_LITERAL;
            case ValueModelType.ENUMERATION_VALUE: return WidgetModelType.DOM_LITERAL;
            default: return WidgetModelType.STRING_LITERAL;
        }
    }

    /**
     * Get a string value in a list of strings with a given index
     * @param stringList
     * @param index
     * @return
     */
    private static String getValueForIndex(List<String> stringList, Integer index) {
        if (stringList != null && stringList.size()>index && stringList.get(index) != null) {
            return stringList.get(index);
        }
        return "";
    }


    private static ValidatorReference getValidatorReferenceDefinition(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = dataTypeAndTypeParameter.getDataType() instanceof SimpleType ? dataTypeAndTypeParameter
                .getValidatorReferenceDefinition()
                : null;
        return validatorReference;
    }

    public static Integer getMaxLength(DataTypeAndTypeParameter dataType) {
        if (dataType == null) {
            return null;
        }
        ValidatorReference rootValidatorReference = getValidatorReferenceDefinition(dataType);
        List<ReferenceWithParameter> definitionStack = dataType.getDefinitionStack();
        if (rootValidatorReference == null || definitionStack == null) {
            return null;
        }
        List<String> parameterValues = getParameterValuesAsStrings(rootValidatorReference,definitionStack);
        if (rootValidatorReference.isStringValidator()) {
            if (parameterValues.size()>1 && parameterValues.get(1) != null && parameterValues.get(1).length()>0) {
                return Integer.parseInt(parameterValues.get(1));
            }
        }
        if (rootValidatorReference.isI18nValidator()) {
            if (parameterValues.size()>2 && parameterValues.get(2) != null && parameterValues.get(2).length()>0) {
                return Integer.parseInt(parameterValues.get(2));
            }
        }
        return null;
    }
    
    /**
     * Creates and attaches a formatter for a text widget. 
     * 
     * In case that a formatter or constraints are defined in the presentation
     * model a CompoundValidator is created. Otherwise (this means for text
     * widgets with no vallidation information) a simple String formatter is
     * created.
     * 
     * @param xmaText
     * @param xmaComponent
     * @param xmaPage
     * @return
     */
    public static CompoundValidator createAndAttachTextFormatter(XMAText xmaText, XMAComponent xmaComponent, XMAPage xmaPage) {    	
    	GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaText);    	
    	if (guiElement instanceof Text) {
    		Text text = (Text)guiElement;
    		ValidatorReference formatter = text.getFormat();
    		if (formatter == null && text.getConstraints().size() == 0 && text.getType() == null) {
    	    	StringValidator stringValidator = createStringValidator(null); 
    	    	xmaText.setBdValidator(stringValidator);
    	    	return null;  
    		} else {
    			ValidInState validInState = GuidesignFactory.eINSTANCE.createValidInState();
    			CompoundValidator compoundValidator = Util.attachCompoundValidatorToWidget(xmaText,validInState);    			
    			if (text.getType() != null) { // Type is defined for text field 
                    Validator xmaValidator = createXMAValidatorForType(text.getType());    
                    setBasicValidatorForCompoundValidator(xmaValidator, compoundValidator);
                    if (formatter != null) { // Formatter is defined additional to type
                       overrideFormatter(compoundValidator,formatter);
                    }
    			} else if (formatter != null) { // Formatter is defined for text field (but no type is defined) 
    			    Validator xmaValidator = createXMAValidator(formatter,null,null,false);    			    
	    			setBasicValidatorForCompoundValidator(xmaValidator, compoundValidator);
    			}
    			return compoundValidator;
    		}
    	}
    	return null;
    }
    private static void setBasicValidatorForCompoundValidator(Validator xmaValidator, CompoundValidator compoundValidator) {
    
        compoundValidator.setFormatter(xmaValidator);	    			
        Constraint constraint = FlexFactory.eINSTANCE.createConstraint();
        constraint.setValidator(xmaValidator);    			
        compoundValidator.getConstraints().add(constraint);
    }
    
    private static Validator createXMAValidatorForType(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = dataTypeAndTypeParameter.getValidatorReferenceDefinition();
        List<ReferenceWithParameter> definitionStack = dataTypeAndTypeParameter.getDefinitionStack();
        Validator xmaValidator = createXMAValidator(validatorReference,definitionStack);
        return xmaValidator;       
    }
    
    /**
     * Returns the constraints of the according text element in the presentation model.
     * 
     * @param xmaText
     * @return
     */
    public static List<org.openxma.dsl.core.model.Constraint> getConstraintsForTextField(XMAText xmaText) {
    	GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaText);    	
    	if (guiElement instanceof Text) {
    		Text text = (Text)guiElement;
    		return text.getConstraints();
    	}
    	return null;
    }
    
    public static boolean isJsr303ValidationEnabled() {
        return GeneratorConfiguration.INSTANCE.isJsr303ValidationEnabled();
    }
}
