package org.openxma.dsl.generator.impl;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.MapExtensions;
import org.eclipse.xtext.xbase.lib.Pair;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure2;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.generator.impl.AutomationTestCaseGenerator;
import org.openxma.dsl.generator.impl.JsfNames;
import org.openxma.dsl.generator.impl.Types;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.XmadslPage;

@SuppressWarnings("all")
public class ArquillianTestCaseGenerator implements AutomationTestCaseGenerator {
  private String ARQUILLIAN_TEST_PATH = "arquillian";
  
  private String PAGE_OBJECT_PATH = "pageobjects";
  
  @Inject
  private Types types;
  
  @Inject
  private JsfNames jsfNames;
  
  protected String _packageFolderPath(final EObject eObject) {
    Model _model = this.types.model(eObject);
    String _name = _model==null?(String)null:_model.getName();
    String _replaceAll = _name.replaceAll("\\.", "/");
    return _replaceAll;
  }
  
  public String getITCaseName(final XmadslPage dslPage) {
    String _packageFolderPath = this.packageFolderPath(dslPage);
    String _plus = (_packageFolderPath + "/");
    String _plus_1 = (_plus + this.ARQUILLIAN_TEST_PATH);
    String _plus_2 = (_plus_1 + "/");
    String _name = dslPage.getName();
    String _plus_3 = (_plus_2 + _name);
    return (_plus_3 + "ITCase.java");
  }
  
  public CharSequence generateTestCase(final XmadslPage dslPage, final String namespace) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".");
    _builder.append(this.ARQUILLIAN_TEST_PATH, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import ");
    _builder.append(namespace, "");
    _builder.append(".");
    _builder.append(this.ARQUILLIAN_TEST_PATH, "");
    _builder.append(".");
    _builder.append(this.PAGE_OBJECT_PATH, "");
    _builder.append(".");
    String _name = dslPage.getName();
    _builder.append(_name, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("import org.jboss.arquillian.drone.api.annotation.Drone;");
    _builder.newLine();
    _builder.append("import org.jboss.arquillian.graphene.spi.annotations.Page;");
    _builder.newLine();
    _builder.append("import org.jboss.arquillian.junit.Arquillian;");
    _builder.newLine();
    _builder.append("import org.junit.Test;");
    _builder.newLine();
    _builder.append("import org.junit.runner.RunWith; ");
    _builder.newLine();
    _builder.append("import org.openqa.selenium.WebDriver;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("@RunWith(Arquillian.class)");
    _builder.newLine();
    _builder.append("public class ");
    String _name_1 = dslPage.getName();
    _builder.append(_name_1, "");
    _builder.append("ITCase {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@Drone");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("WebDriver driver;");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@Page");
    _builder.newLine();
    _builder.append("\t");
    String _name_2 = dslPage.getName();
    _builder.append(_name_2, "	");
    _builder.append(" ");
    CharSequence _bean = this.jsfNames.getBean(dslPage);
    _builder.append(_bean, "	");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@Test");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public void testSample() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("//implement test case");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getPageObject(final XmadslPage dslPage, final String namespace) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".");
    _builder.append(this.ARQUILLIAN_TEST_PATH, "");
    _builder.append(".");
    _builder.append(this.PAGE_OBJECT_PATH, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("public class ");
    String _name = dslPage.getName();
    _builder.append(_name, "");
    _builder.append(" extends ");
    String _name_1 = dslPage.getName();
    _builder.append(_name_1, "");
    _builder.append("Gen {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public String getPageObjectName(final XmadslPage dslPage, final boolean generated) {
    String _packageFolderPath = this.packageFolderPath(dslPage);
    String _plus = (_packageFolderPath + "/");
    String _plus_1 = (_plus + this.ARQUILLIAN_TEST_PATH);
    String _plus_2 = (_plus_1 + "/");
    String _plus_3 = (_plus_2 + this.PAGE_OBJECT_PATH);
    String _plus_4 = (_plus_3 + "/");
    String _name = dslPage.getName();
    String _plus_5 = (_plus_4 + _name);
    String _xifexpression = null;
    if (generated) {
      _xifexpression = "Gen";
    } else {
      _xifexpression = "";
    }
    String _plus_6 = (_plus_5 + _xifexpression);
    return (_plus_6 + ".java");
  }
  
  public CharSequence getAbstractPageObject(final XmadslPage dslPage, final String namespace, final Map<CharSequence,CharSequence> fieldNameMap) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".");
    _builder.append(this.ARQUILLIAN_TEST_PATH, "");
    _builder.append(".");
    _builder.append(this.PAGE_OBJECT_PATH, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import org.jboss.arquillian.drone.api.annotation.Drone;");
    _builder.newLine();
    _builder.append("import org.jboss.arquillian.graphene.context.GrapheneContext;");
    _builder.newLine();
    _builder.append("import org.openqa.selenium.WebDriver;");
    _builder.newLine();
    _builder.append("import org.openqa.selenium.WebElement;");
    _builder.newLine();
    _builder.append("import org.openqa.selenium.support.FindBy;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("import at.spardat.properties.XProperties;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("public abstract class ");
    String _name = dslPage.getName();
    _builder.append(_name, "");
    _builder.append("Gen {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@Drone");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("protected WebDriver driver;");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _createFields = this.createFields(fieldNameMap);
    _builder.append(_createFields, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public String getPageSource() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return GrapheneContext.getProxy().getPageSource();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public void open() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("GrapheneContext.getProxy().get(getBaseUrl() + \"/");
    String _faceletPageName = this.jsfNames.getFaceletPageName(dslPage, false);
    _builder.append(_faceletPageName, "		");
    _builder.append("\");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("private String getBaseUrl()  {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return XProperties.getNodeOfPackage(\"\").get(\"arquillian.app.baseUrl\", \"http://localhost:9090/dsl-reference-jsf\");");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createFields(final Map<CharSequence,CharSequence> fieldMap) {
    StringConcatenation _builder = new StringConcatenation();
    final Procedure2<CharSequence,CharSequence> _function = new Procedure2<CharSequence,CharSequence>() {
        public void apply(final CharSequence key, final CharSequence value) {
          ArquillianTestCaseGenerator.this.createField(key, value);
        }
      };
    MapExtensions.<CharSequence, CharSequence>forEach(fieldMap, _function);
    _builder.newLineIfNotEmpty();
    {
      Set<Entry<CharSequence,CharSequence>> _entrySet = fieldMap.entrySet();
      for(final Entry<CharSequence,CharSequence> entry : _entrySet) {
        CharSequence _key = entry.getKey();
        CharSequence _value = entry.getValue();
        CharSequence _createField = this.createField(_key, _value);
        _builder.append(_createField, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createField(final CharSequence key, final CharSequence value) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@FindBy(id=\"");
    _builder.append(key, "");
    _builder.append("\")");
    _builder.newLineIfNotEmpty();
    _builder.append("protected WebElement ");
    _builder.append(value, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    return _builder;
  }
  
  public Map getFields(final Content content) {
    LinkedHashMap<String,String> _linkedHashMap = new LinkedHashMap<String,String>();
    LinkedHashMap<String,String> fields = _linkedHashMap;
    EList<EObject> _eContents = content.eContents();
    for (final EObject item : _eContents) {
      {
        Pair<String,String> field = this.getField(item);
        String _key = field.getKey();
        String _value = field.getValue();
        fields.put(_key, _value);
      }
    }
    return fields;
  }
  
  public Pair<String,String> getField(final EObject eObject) {
    Pair<String,String> pair = Pair.<String, String>of("", "");
    boolean _matched = false;
    if (!_matched) {
      if (eObject instanceof CustomizeableField) {
        final CustomizeableField _customizeableField = (CustomizeableField)eObject;
        _matched=true;
        Attribute _attribute = _customizeableField.getAttribute();
        String _name = _attribute.getName();
        Attribute _attribute_1 = _customizeableField.getAttribute();
        String _name_1 = _attribute_1.getName();
        String _plus = (_name_1 + "1");
        Pair<String,String> _mappedTo = Pair.<String, String>of(_name, _plus);
        pair = _mappedTo;
      }
    }
    return pair;
  }
  
  public String packageFolderPath(final EObject eObject) {
    {
      return _packageFolderPath(eObject);
    }
  }
  
  public CharSequence createFields(final Map<CharSequence,CharSequence> fieldMap) {
    {
      return _createFields(fieldMap);
    }
  }
  
  public CharSequence createField(final CharSequence key, final CharSequence value) {
    {
      return _createField(key, value);
    }
  }
}
