package org.openxma.dsl.generator.impl;

import at.spardat.xma.guidesign.XMAComponent;
import com.google.common.base.CaseFormat;
import com.google.common.base.Objects;
import com.google.inject.Singleton;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.IteratorExtensions;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.ConditionsBlock;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.dom.model.Dependant;
import org.openxma.dsl.generator.impl.BackingBean;
import org.openxma.dsl.generator.impl.ExpressionCompiler;
import org.openxma.dsl.generator.impl.JsfData;
import org.openxma.dsl.generator.impl.JsfEvents;
import org.openxma.dsl.generator.impl.JsfNames;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.lib.ImportAdapter;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.ListBox;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableCombo;
import org.openxma.dsl.pom.model.Text;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.model.XmadslPage;

@Singleton
@SuppressWarnings("all")
public class BackingBeanImpl implements BackingBean {
  @Inject
  private Names _names;
  
  @Inject
  private JsfNames _jsfNames;
  
  @Inject
  private JsfEvents _jsfEvents;
  
  @Inject
  private JsfData _jsfData;
  
  @Inject
  private ExpressionCompiler _expressionCompiler;
  
  public CharSequence toComponentBackingBean(final String namespace, final Component it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".backingbean;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("/** ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Backing Bean for Component ");
    XMAComponent _xmaComponent = it.getXmaComponent();
    String _namClass = _xmaComponent.getNamClass();
    _builder.append(_namClass, " ");
    _builder.append(" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("@");
    String _addImport = this._names.addImport(it, "org.springframework.stereotype.Component");
    _builder.append(_addImport, "");
    _builder.append("(\"");
    XMAComponent _xmaComponent_1 = it.getXmaComponent();
    String _namClass_1 = _xmaComponent_1.getNamClass();
    String _firstLower = StringExtensions.toFirstLower(_namClass_1);
    _builder.append(_firstLower, "");
    _builder.append("\")");
    _builder.newLineIfNotEmpty();
    _builder.append("@");
    String _addImport_1 = this._names.addImport(it, "org.springframework.context.annotation.Scope");
    _builder.append(_addImport_1, "");
    _builder.append("(\"session\")");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    XMAComponent _xmaComponent_2 = it.getXmaComponent();
    String _namClass_2 = _xmaComponent_2.getNamClass();
    _builder.append(_namClass_2, "");
    _builder.append(" extends ");
    XMAComponent _xmaComponent_3 = it.getXmaComponent();
    String _namClass_3 = _xmaComponent_3.getNamClass();
    _builder.append(_namClass_3, "");
    _builder.append("Gen {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toBackingBean(final String namespace, final XmadslPage it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".backingbean;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("/** ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Backing Bean for Page ");
    String _name = it.getName();
    _builder.append(_name, " ");
    _builder.append(" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("@");
    String _addImport = this._names.addImport(it, "org.springframework.stereotype.Component");
    _builder.append(_addImport, "");
    _builder.append("(\"");
    String _name_1 = it.getName();
    String _firstLower = StringExtensions.toFirstLower(_name_1);
    _builder.append(_firstLower, "");
    _builder.append("\")");
    _builder.newLineIfNotEmpty();
    _builder.append("@");
    String _addImport_1 = this._names.addImport(it, "org.springframework.context.annotation.Scope");
    _builder.append(_addImport_1, "");
    _builder.append("(\"access\")");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    String _name_2 = it.getName();
    _builder.append(_name_2, "");
    _builder.append(" extends ");
    String _name_3 = it.getName();
    _builder.append(_name_3, "");
    _builder.append("Gen {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toAbstractComponentBackingBean(final String namespace, final Component it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".backingbean;");
    _builder.newLineIfNotEmpty();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.append("/** ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Abstract Backing Bean Base for Component ");
    XMAComponent _xmaComponent = it.getXmaComponent();
    String _namClass = _xmaComponent.getNamClass();
    _builder.append(_namClass, " ");
    _builder.append(" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("public abstract class ");
    XMAComponent _xmaComponent_1 = it.getXmaComponent();
    String _namClass_1 = _xmaComponent_1.getNamClass();
    _builder.append(_namClass_1, "");
    _builder.append("Gen extends ");
    String _addImport = this._names.addImport(it, "org.openxma.dsl.platform.jsf.beans.ComponentBackingBean");
    _builder.append(_addImport, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("// resource bundle");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("protected ");
    String _addImport_1 = this._names.addImport(it, "java.util.ResourceBundle");
    _builder.append(_addImport_1, "	");
    _builder.append(" bundle = ResourceBundle.getBundle(\"");
    XMAComponent _xmaComponent_2 = it.getXmaComponent();
    String _namPackageAsPath = _xmaComponent_2.getNamPackageAsPath();
    String _plus = (_namPackageAsPath + "/");
    XMAComponent _xmaComponent_3 = it.getXmaComponent();
    String _namClass_2 = _xmaComponent_3.getNamClass();
    String _plus_1 = (_plus + _namClass_2);
    _builder.append(_plus_1, "	");
    _builder.append("\", ");
    String _addImport_2 = this._names.addImport(it, "javax.faces.context.FacesContext");
    _builder.append(_addImport_2, "	");
    _builder.append(".getCurrentInstance().getViewRoot().getLocale());");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    EList<Dependant> _dependencies = it.getDependencies();
    CharSequence _injectedDependencies = this._jsfData.toInjectedDependencies(_dependencies);
    _builder.append(_injectedDependencies, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    EList<ObjectProperty> _componentProperties = it.getComponentProperties();
    CharSequence _objectPropertyVars = this._jsfData.toObjectPropertyVars(_componentProperties);
    _builder.append(_objectPropertyVars, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    EList<DataObjectVariable> _dataobjects = it.getDataobjects();
    CharSequence _dataObjectVars = this._jsfData.toDataObjectVars(_dataobjects);
    _builder.append(_dataObjectVars, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _xifexpression = null;
    EventMappingList _events = it.getEvents();
    boolean _notEquals = (!Objects.equal(_events, null));
    if (_notEquals) {
      EventMappingList _events_1 = it.getEvents();
      CharSequence _lifecycleMethods = this._jsfEvents.toLifecycleMethods(_events_1);
      _xifexpression = _lifecycleMethods;
    } else {
      EventMappingList _createEventMappingList = PomFactory.eINSTANCE.createEventMappingList();
      CharSequence _lifecycleMethods_1 = this._jsfEvents.toLifecycleMethods(_createEventMappingList);
      _xifexpression = _lifecycleMethods_1;
    }
    _builder.append(_xifexpression, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// commands");
    _builder.newLine();
    _builder.append("\t");
    EList<Command> _commands = it.getCommands();
    CharSequence _abstractCommands = this.toAbstractCommands(_commands);
    _builder.append(_abstractCommands, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    {
      EList<ObjectProperty> _componentProperties_1 = it.getComponentProperties();
      for(final ObjectProperty compProperty : _componentProperties_1) {
        _builder.append("    ");
        CharSequence _getterAndSetter = this._jsfData.toGetterAndSetter(compProperty);
        _builder.append(_getterAndSetter, "    ");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    {
      EList<DataObjectVariable> _dataobjects_1 = it.getDataobjects();
      for(final DataObjectVariable dataObject : _dataobjects_1) {
        _builder.append("\t");
        CharSequence _getterAndSetter_1 = this._jsfData.toGetterAndSetter(dataObject);
        _builder.append(_getterAndSetter_1, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toAbstractBackingBean(final String namespace, final XmadslPage it, final Component comp) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    _builder.append(namespace, "");
    _builder.append(".backingbean;");
    _builder.newLineIfNotEmpty();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.append("/** ");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Abstract Backing Bean Base for Page ");
    String _name = it.getName();
    _builder.append(_name, " ");
    _builder.append(" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("public abstract class ");
    String _name_1 = it.getName();
    _builder.append(_name_1, "");
    _builder.append("Gen extends ");
    String _addImport = this._names.addImport(it, "org.openxma.dsl.platform.jsf.beans.PageBackingBean");
    _builder.append(_addImport, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// client id constants");
    _builder.newLine();
    _builder.append("\t");
    Content _content = it.getContent();
    CharSequence _faceletIdConstants = _content==null?(CharSequence)null:this.toFaceletIdConstants(_content);
    _builder.append(_faceletIdConstants, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@");
    String _addImport_1 = this._names.addImport(it, "org.springframework.beans.factory.annotation.Autowired");
    _builder.append(_addImport_1, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("protected ");
    XMAComponent _xmaComponent = comp.getXmaComponent();
    String _namClass = _xmaComponent.getNamClass();
    _builder.append(_namClass, "	");
    _builder.append(" component;");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public ");
    XMAComponent _xmaComponent_1 = comp.getXmaComponent();
    String _namClass_1 = _xmaComponent_1.getNamClass();
    _builder.append(_namClass_1, "	");
    _builder.append(" getTypedComponent() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return component;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    EList<Dependant> _dependencies = it.getDependencies();
    CharSequence _injectedDependencies = this._jsfData.toInjectedDependencies(_dependencies);
    _builder.append(_injectedDependencies, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    EList<ObjectProperty> _pageProperties = it.getPageProperties();
    CharSequence _objectPropertyVars = this._jsfData.toObjectPropertyVars(_pageProperties);
    _builder.append(_objectPropertyVars, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// data objects");
    _builder.newLine();
    _builder.append("    ");
    EList<DataObjectVariable> _dataobjects = it.getDataobjects();
    CharSequence _dataObjectVars = this._jsfData.toDataObjectVars(_dataobjects);
    _builder.append(_dataObjectVars, "    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// jsf data models");
    _builder.newLine();
    _builder.append("    ");
    Content _content_1 = it.getContent();
    CharSequence _jsfDataModel = _content_1==null?(CharSequence)null:this._jsfData.toJsfDataModel(_content_1);
    _builder.append(_jsfDataModel, "    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// init data model");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("protected void dataModelInit() {");
    _builder.newLine();
    _builder.append("\t    ");
    Content _content_2 = it.getContent();
    CharSequence _initCode = _content_2==null?(CharSequence)null:this._jsfData.toInitCode(_content_2);
    _builder.append(_initCode, "	    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// lifecycle events");
    _builder.newLine();
    _builder.append("    ");
    CharSequence _lifecycleMethods = this._jsfEvents.toLifecycleMethods(it);
    _builder.append(_lifecycleMethods, "    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    CharSequence _xifexpression = null;
    EventMappingList _events = it.getEvents();
    boolean _notEquals = (!Objects.equal(_events, null));
    if (_notEquals) {
      EventMappingList _events_1 = it.getEvents();
      CharSequence _lifecycleMethods_1 = this._jsfEvents.toLifecycleMethods(_events_1);
      _xifexpression = _lifecycleMethods_1;
    } else {
      EventMappingList _createEventMappingList = PomFactory.eINSTANCE.createEventMappingList();
      CharSequence _lifecycleMethods_2 = this._jsfEvents.toLifecycleMethods(_createEventMappingList);
      _xifexpression = _lifecycleMethods_2;
    }
    _builder.append(_xifexpression, "    ");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// commands");
    _builder.newLine();
    _builder.append("\t");
    EList<Command> _commands = it.getCommands();
    CharSequence _abstractCommands = this.toAbstractCommands(_commands);
    _builder.append(_abstractCommands, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.newLine();
    _builder.append("    ");
    _builder.append("// conditions");
    _builder.newLine();
    _builder.append("    ");
    ConditionsBlock _conditionsBlock = it.getConditionsBlock();
    CharSequence _conditions = _conditionsBlock==null?(CharSequence)null:this.toConditions(_conditionsBlock);
    _builder.append(_conditions, "    ");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    {
      EList<ObjectProperty> _pageProperties_1 = it.getPageProperties();
      for(final ObjectProperty pageProperty : _pageProperties_1) {
        _builder.append("    ");
        CharSequence _getterAndSetter = this._jsfData.toGetterAndSetter(pageProperty);
        _builder.append(_getterAndSetter, "    ");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    {
      EList<DataObjectVariable> _dataobjects_1 = it.getDataobjects();
      for(final DataObjectVariable dataObject : _dataobjects_1) {
        _builder.append("\t");
        CharSequence _getterAndSetter_1 = this._jsfData.toGetterAndSetter(dataObject);
        _builder.append(_getterAndSetter_1, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public void handlePreRenderView(");
    String _addImport_2 = this._names.addImport(it, "javax.faces.event.ComponentSystemEvent");
    _builder.append(_addImport_2, "	");
    _builder.append(" event){");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("processConditions();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (!");
    String _addImport_3 = this._names.addImport(it, "javax.faces.context.FacesContext");
    _builder.append(_addImport_3, "		");
    _builder.append(".getCurrentInstance().isPostback()) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t");
    _builder.append("customizeView(FacesContext.getCurrentInstance().getViewRoot());");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("protected void customizeView(");
    String _addImport_4 = this._names.addImport(it, "javax.faces.component.UIViewRoot");
    _builder.append(_addImport_4, "	");
    _builder.append(" viewRoot) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("//by default do nothing, override to add behaviour");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("protected ");
    String _name_2 = it.getName();
    _builder.append(_name_2, "	");
    _builder.append("Gen getContext() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return this;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("protected void processConditions() {");
    _builder.newLine();
    _builder.append("\t\t");
    ConditionsBlock _conditionsBlock_1 = it.getConditionsBlock();
    CharSequence _conditionsProcessing = _conditionsBlock_1==null?(CharSequence)null:this.toConditionsProcessing(_conditionsBlock_1);
    _builder.append(_conditionsProcessing, "		");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toConditions(final ConditionsBlock conditionsBlock) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<StatusFlag> _statusFlags = conditionsBlock==null?(EList<StatusFlag>)null:conditionsBlock.getStatusFlags();
      for(final StatusFlag statusFlag : _statusFlags) {
        _builder.append("private boolean ");
        String _name = statusFlag.getName();
        _builder.append(_name, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        CharSequence _getterAndSetter = this._jsfData.toGetterAndSetter(statusFlag);
        _builder.append(_getterAndSetter, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toConditionsProcessing(final ConditionsBlock conditionsBlock) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<StatusFlag> _statusFlags = conditionsBlock==null?(EList<StatusFlag>)null:conditionsBlock.getStatusFlags();
      for(final StatusFlag statusFlag : _statusFlags) {
        _builder.append("set");
        String _name = statusFlag.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        _builder.append(_firstUpper, "");
        _builder.append("(");
        EqualityExpr _expression = statusFlag.getExpression();
        String _compile = this._expressionCompiler.compile(_expression);
        _builder.append(_compile, "");
        _builder.append(");   ");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toFaceletIdConstants(final EObject object) {
    CharSequence _xblockexpression = null;
    {
      TreeIterator<EObject> _allContents = EcoreUtil.<EObject>getAllContents(object, false);
      final Iterable<EObject> allElements = IteratorExtensions.<EObject>toIterable(_allContents);
      StringConcatenation _builder = new StringConcatenation();
      CharSequence _faceletIdConstant = this.toFaceletIdConstant(object);
      _builder.append(_faceletIdConstant, "");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      {
        for(final EObject element : allElements) {
          CharSequence _faceletIdConstant_1 = this.toFaceletIdConstant(((EObject) element));
          _builder.append(_faceletIdConstant_1, "");
          _builder.newLineIfNotEmpty();
        }
      }
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence toFaceletIdConstant(final EObject object) {
    CharSequence _xblockexpression = null;
    {
      final CharSequence id = this._jsfNames.toFaceletId(object);
      CharSequence _xifexpression = null;
      boolean _notEquals = (!Objects.equal(id, null));
      if (_notEquals) {
        CharSequence _xblockexpression_1 = null;
        {
          final String name = this.name(object);
          CharSequence _xifexpression_1 = null;
          boolean _notEquals_1 = (!Objects.equal(name, null));
          if (_notEquals_1) {
            StringConcatenation _builder = new StringConcatenation();
            _builder.append("public static String ");
            CharSequence _facetletIdConstantName = this.facetletIdConstantName(name);
            _builder.append(_facetletIdConstantName, "");
            _builder.append(" = \"");
            XmadslPage _parentPage = this._jsfNames.getParentPage(object);
            CharSequence _faceletFormId = this._jsfNames.toFaceletFormId(_parentPage);
            _builder.append(_faceletFormId, "");
            _builder.append(":");
            _builder.append(id, "");
            _builder.append("\";");
            _xifexpression_1 = _builder;
          } else {
            StringConcatenation _builder_1 = new StringConcatenation();
            _xifexpression_1 = _builder_1;
          }
          _xblockexpression_1 = (_xifexpression_1);
        }
        _xifexpression = _xblockexpression_1;
      } else {
        StringConcatenation _builder = new StringConcatenation();
        _xifexpression = _builder;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  public CharSequence facetletIdConstantName(final String name) {
    String _to = CaseFormat.LOWER_CAMEL.to(CaseFormat.UPPER_UNDERSCORE, name);
    String _plus = (_to + "_ID");
    return _plus;
  }
  
  public String name(final EObject object) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (object instanceof Text) {
        final Text _text = (Text)object;
        _matched=true;
        String _name = ((Text) _text).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof ListBox) {
        final ListBox _listBox = (ListBox)object;
        _matched=true;
        String _name = ((ListBox) _listBox).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof TableCombo) {
        final TableCombo _tableCombo = (TableCombo)object;
        _matched=true;
        String _name = ((TableCombo) _tableCombo).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof Table) {
        final Table _table = (Table)object;
        _matched=true;
        String _name = ((Table) _table).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof Combo) {
        final Combo _combo = (Combo)object;
        _matched=true;
        String _name = ((Combo) _combo).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof Button) {
        final Button _button = (Button)object;
        _matched=true;
        String _name = ((Button) _button).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof XmadslComposite) {
        final XmadslComposite _xmadslComposite = (XmadslComposite)object;
        _matched=true;
        String _name = ((XmadslComposite) _xmadslComposite).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof ReferencedXMAComposite) {
        final ReferencedXMAComposite _referencedXMAComposite = (ReferencedXMAComposite)object;
        _matched=true;
        String _name = this.name(((ReferencedXMAComposite) _referencedXMAComposite));
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof TabFolder) {
        final TabFolder _tabFolder = (TabFolder)object;
        _matched=true;
        String _name = ((TabFolder) _tabFolder).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof TabPage) {
        final TabPage _tabPage = (TabPage)object;
        _matched=true;
        String _name = ((TabPage) _tabPage).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof CheckBox) {
        final CheckBox _checkBox = (CheckBox)object;
        _matched=true;
        String _name = ((CheckBox) _checkBox).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (object instanceof ListBox) {
        final ListBox _listBox = (ListBox)object;
        _matched=true;
        String _name = ((ListBox) _listBox).getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      _switchResult = null;
    }
    return _switchResult;
  }
  
  public CharSequence toAbstractCommands(final EList<Command> commands) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final Command command : commands) {
        {
          boolean _isClientOnly = command.isClientOnly();
          boolean _not = (!_isClientOnly);
          if (_not) {
            _builder.append("public abstract void ");
            String _name = command.getName();
            _builder.append(_name, "");
            _builder.append("();");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
}
