package org.openxma.dsl.generator.impl;

import at.spardat.xma.guidesign.XMAComponent;
import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.inject.Singleton;
import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.inject.Inject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Functions.Function0;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.BoolLiteral;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.FalseLiteral;
import org.openxma.dsl.core.model.TrueLiteral;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.PresentableFeature;
import org.openxma.dsl.dom.model.ReadOnlyFlag;
import org.openxma.dsl.dom.model.RequiredFlag;
import org.openxma.dsl.generator.helper.GeneratorConfigurationExtension;
import org.openxma.dsl.generator.impl.ExpressionLanguageImpl;
import org.openxma.dsl.generator.impl.Facelet;
import org.openxma.dsl.generator.impl.FaceletDataImpl;
import org.openxma.dsl.generator.impl.FaceletExpressionCompiler;
import org.openxma.dsl.generator.impl.FaceletLayoutImpl;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.JsfNames;
import org.openxma.dsl.pom.model.AttachmentProperty;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.CommandImplementation;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.ComposedElement;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.EnabledFlag;
import org.openxma.dsl.pom.model.EventFunction;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.FieldFeature;
import org.openxma.dsl.pom.model.FieldFlag;
import org.openxma.dsl.pom.model.FieldPart;
import org.openxma.dsl.pom.model.FieldPartSpecification;
import org.openxma.dsl.pom.model.FieldReference;
import org.openxma.dsl.pom.model.GuiElementEventMapping;
import org.openxma.dsl.pom.model.IComposite;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.IGuiElementWithEvent;
import org.openxma.dsl.pom.model.IReferenceableByStatementVariable;
import org.openxma.dsl.pom.model.InvokeStatement;
import org.openxma.dsl.pom.model.Invokeable;
import org.openxma.dsl.pom.model.JoinDirection;
import org.openxma.dsl.pom.model.Label;
import org.openxma.dsl.pom.model.LabelText;
import org.openxma.dsl.pom.model.ListBox;
import org.openxma.dsl.pom.model.MultiselectionFlag;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.PredefinedCommand;
import org.openxma.dsl.pom.model.PropertyJoin;
import org.openxma.dsl.pom.model.RadioButton;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.Seperator;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.Statement;
import org.openxma.dsl.pom.model.StatementVariable;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.model.TableCombo;
import org.openxma.dsl.pom.model.TabulatorPosition;
import org.openxma.dsl.pom.model.Text;
import org.openxma.dsl.pom.model.Tree;
import org.openxma.dsl.pom.model.VisibleFlag;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.model.XmadslPage;

@Singleton
@SuppressWarnings("all")
public class FaceletImpl implements Facelet {
  private final Log logger = new Function0<Log>() {
    public Log apply() {
      Class<? extends FaceletImpl> _class = FaceletImpl.this.getClass();
      String _name = _class.getName();
      Log _log = LogFactory.getLog(_name);
      return _log;
    }
  }.apply();
  
  @Inject
  private Files _files;
  
  @Inject
  private JsfNames _jsfNames;
  
  @Inject
  private ExpressionLanguageImpl _expressionLanguageImpl;
  
  @Inject
  private FaceletDataImpl _faceletDataImpl;
  
  @Inject
  private FaceletLayoutImpl _faceletLayoutImpl;
  
  @Inject
  private FaceletExpressionCompiler _faceletExpressionCompiler;
  
  private int itemIndex = 0;
  
  private final Boolean createIdentifiers = Boolean.valueOf(false);
  
  private String resourceBundleName = "msgs";
  
  private String formName = "";
  
  private Map<CharSequence,CharSequence> resourceProperties = new Function0<Map<CharSequence,CharSequence>>() {
    public Map<CharSequence,CharSequence> apply() {
      LinkedHashMap<CharSequence,CharSequence> _linkedHashMap = new LinkedHashMap<CharSequence,CharSequence>();
      return _linkedHashMap;
    }
  }.apply();
  
  private Map<CharSequence,CharSequence> clientIds = new Function0<Map<CharSequence,CharSequence>>() {
    public Map<CharSequence,CharSequence> apply() {
      LinkedHashMap<CharSequence,CharSequence> _linkedHashMap = new LinkedHashMap<CharSequence,CharSequence>();
      return _linkedHashMap;
    }
  }.apply();
  
  public Map<CharSequence,CharSequence> getPageResourceProperties(final EObject composite) {
    return this.resourceProperties;
  }
  
  public Map<CharSequence,CharSequence> getFieldClientIds(final EObject composite) {
    return this.clientIds;
  }
  
  protected CharSequence _createContents(final XmadslPage page) {
    CharSequence _xblockexpression = null;
    {
      this.clientIds.clear();
      Component _component = this._jsfNames.component(page);
      XMAComponent _xmaComponent = _component.getXmaComponent();
      String _namClass = _xmaComponent.getNamClass();
      String _lowerCase = _namClass.toLowerCase();
      String _plus = (_lowerCase + "msgs");
      this.resourceBundleName = _plus;
      String _name = page.getName();
      String _plus_1 = (_name + "Form");
      this.formName = _plus_1;
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?> ");
      _builder.newLine();
      _builder.append("<!DOCTYPE html");
      _builder.newLine();
      _builder.append("        ");
      _builder.append("PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"");
      _builder.newLine();
      _builder.append("        ");
      _builder.append("\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
      _builder.newLine();
      _builder.append("<html xmlns=\"http://www.w3.org/1999/xhtml\"");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("xmlns:f=\"http://java.sun.com/jsf/core\"");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("xmlns:h=\"http://java.sun.com/jsf/html\"");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("xmlns:ui=\"http://java.sun.com/jsf/facelets\"");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("xmlns:p=\"http://primefaces.org/ui\"");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("xmlns:fn=\"http://java.sun.com/jsp/jstl/functions\"");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("xmlns:pe=\"http://primefaces.org/ui/extensions\">");
      _builder.newLine();
      _builder.append("<head>");
      _builder.newLine();
      _builder.append("</head>");
      _builder.newLine();
      _builder.append("<body>");
      _builder.newLine();
      _builder.append("<ui:composition template=\"${template}\">");
      _builder.newLine();
      _builder.append("\t\t");
      _builder.append("<ui:param name=\"title\" value=\"");
      LabelText _labelText = page.getLabelText();
      String _label = _labelText==null?(String)null:_labelText.getLabel();
      String _processResourceBundleProperty = this.processResourceBundleProperty(page, _label);
      _builder.append(_processResourceBundleProperty, "		");
      _builder.append("\" />");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("<ui:param name=\"dialogId\" value=\"");
      String _name_1 = page.getName();
      String _firstLower = StringExtensions.toFirstLower(_name_1);
      _builder.append(_firstLower, "		");
      _builder.append("\" />");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("<ui:param name=\"dialogWidgetVar\" value=\"");
      CharSequence _dialogWidgetVar = this._jsfNames.getDialogWidgetVar(page);
      _builder.append(_dialogWidgetVar, "		");
      _builder.append("\" />");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("<ui:define name=\"dialogs\">");
      _builder.newLine();
      {
        Iterable<XmadslPage> _invokablePages = this.getInvokablePages(page);
        for(final XmadslPage invokeablePage : _invokablePages) {
          _builder.append("\t\t\t");
          _builder.append("<ui:include src=\"/");
          String _faceletPageFileName = this._files.faceletPageFileName(invokeablePage, false);
          _builder.append(_faceletPageFileName, "			");
          _builder.append("\">");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t\t");
          _builder.append("\t");
          _builder.append("<ui:param name=\"template\" value=\"/templates/dialogTemplate.xhtml\" />");
          _builder.newLine();
          _builder.append("\t\t\t");
          _builder.append("</ui:include>");
          _builder.newLine();
        }
      }
      _builder.append("\t\t");
      _builder.append("</ui:define>");
      _builder.newLine();
      _builder.append("\t\t");
      _builder.append("<ui:define name=\"genscripts\">");
      _builder.newLine();
      _builder.append("\t\t\t");
      _builder.append("<script type=\"text/javascript\">");
      _builder.newLine();
      _builder.append("\t\t\t\t");
      _builder.append("if (!App.");
      Component _component_1 = this._jsfNames.component(page);
      CharSequence _jSClass = this._jsfNames.getJSClass(_component_1);
      _builder.append(_jSClass, "				");
      _builder.append(") {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t\t");
      _builder.append("App.");
      Component _component_2 = this._jsfNames.component(page);
      CharSequence _jSClass_1 = this._jsfNames.getJSClass(_component_2);
      _builder.append(_jSClass_1, "					");
      _builder.append(" = Xma.Component.extend( {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t\t\t");
      Component _component_3 = this._jsfNames.component(page);
      EList<Command> _commands = _component_3.getCommands();
      CharSequence _uiCommandPrototypes = _commands==null?(CharSequence)null:this.toUiCommandPrototypes(_commands);
      _builder.append(_uiCommandPrototypes, "						");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t\t\t");
      _builder.append("getMainPage : function() { return App.");
      Component _component_4 = this._jsfNames.component(page);
      XmadslPage _mainPage = this._jsfNames.mainPage(_component_4);
      CharSequence _widgetVar = this._jsfNames.getWidgetVar(_mainPage);
      _builder.append(_widgetVar, "						");
      _builder.append("; }");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t\t");
      _builder.append("});");
      _builder.newLine();
      _builder.append("\t\t\t\t\t");
      _builder.append("App.");
      Component _component_5 = this._jsfNames.component(page);
      CharSequence _widgetVar_1 = this._jsfNames.getWidgetVar(_component_5);
      _builder.append(_widgetVar_1, "					");
      _builder.append(" = new App.");
      Component _component_6 = this._jsfNames.component(page);
      CharSequence _jSClass_2 = this._jsfNames.getJSClass(_component_6);
      _builder.append(_jSClass_2, "					");
      _builder.append("();");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("\t\t\t\t");
      _builder.append("App.");
      CharSequence _jSClass_3 = this._jsfNames.getJSClass(page);
      _builder.append(_jSClass_3, "				");
      _builder.append(" = Xma.Page.extend({");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t\t");
      EList<Command> _commands_1 = page.getCommands();
      CharSequence _uiCommandPrototypes_1 = _commands_1==null?(CharSequence)null:this.toUiCommandPrototypes(_commands_1);
      _builder.append(_uiCommandPrototypes_1, "					");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t\t");
      _builder.append("invoke : function(callback) { this.callback = callback; ");
      String _name_2 = page.getName();
      _builder.append(_name_2, "					");
      _builder.append("_invokeCmd(); },");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t\t");
      _builder.append("getDialogWidget : \tfunction() { return ");
      CharSequence _dialogWidgetVar_1 = this._jsfNames.getDialogWidgetVar(page);
      _builder.append(_dialogWidgetVar_1, "					");
      _builder.append(";}");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t");
      _builder.append("});\t");
      _builder.newLine();
      _builder.append("\t\t\t\t");
      _builder.append("App.");
      CharSequence _widgetVar_2 = this._jsfNames.getWidgetVar(page);
      _builder.append(_widgetVar_2, "				");
      _builder.append(" = new App.");
      CharSequence _jSClass_4 = this._jsfNames.getJSClass(page);
      _builder.append(_jSClass_4, "				");
      _builder.append("();");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t");
      _builder.append("</script>\t");
      _builder.newLine();
      _builder.append("\t\t");
      _builder.append("</ui:define>");
      _builder.newLine();
      _builder.append("\t\t");
      _builder.append("<ui:define name=\"center\">");
      _builder.newLine();
      _builder.append("\t\t\t");
      _builder.append("<f:loadBundle basename=\"");
      Component _component_7 = this._jsfNames.component(page);
      XMAComponent _xmaComponent_1 = _component_7.getXmaComponent();
      String _namPackage = _xmaComponent_1.getNamPackage();
      _builder.append(_namPackage, "			");
      _builder.append(".");
      Component _component_8 = this._jsfNames.component(page);
      XMAComponent _xmaComponent_2 = _component_8.getXmaComponent();
      String _namClass_1 = _xmaComponent_2.getNamClass();
      _builder.append(_namClass_1, "			");
      _builder.append("\" var=\"");
      _builder.append(this.resourceBundleName, "			");
      _builder.append("\"/>");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t");
      _builder.append("<h:form id=\"");
      CharSequence _faceletFormId = this._jsfNames.toFaceletFormId(page);
      _builder.append(_faceletFormId, "			");
      _builder.append("\">");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t");
      _builder.append("<f:event listener=\"#{");
      CharSequence _bean = this._jsfNames.getBean(page);
      _builder.append(_bean, "				");
      _builder.append(".handlePreRenderView}\" type=\"preRenderView\" />");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t");
      Content _content = page.getContent();
      CharSequence _createContents = _content==null?(CharSequence)null:this.createContents(_content);
      _builder.append(_createContents, "				");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t");
      _builder.append("<p:remoteCommand action=\"#{");
      CharSequence _bean_1 = this._jsfNames.getBean(page);
      _builder.append(_bean_1, "				");
      _builder.append(".invoke}\" process=\"");
      CharSequence _processedInputsList = this.getProcessedInputsList(page);
      _builder.append(_processedInputsList, "				");
      _builder.append("\" update=\"@form\" id=\"invoke\" name=\"");
      String _name_3 = page.getName();
      _builder.append(_name_3, "				");
      _builder.append("_invokeCmd\" oncomplete=\"App.");
      CharSequence _widgetVar_3 = this._jsfNames.getWidgetVar(page);
      _builder.append(_widgetVar_3, "				");
      _builder.append(".showDialog();\">");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t\t");
      _builder.append("<p:resetInput target=\"");
      String _name_4 = page.getName();
      _builder.append(_name_4, "					");
      _builder.append("Form\" />");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t");
      _builder.append("</p:remoteCommand>");
      _builder.newLine();
      _builder.append("\t\t\t\t");
      EList<Command> _commands_2 = page.getCommands();
      CharSequence _remoteCommands = _commands_2==null?(CharSequence)null:this.toRemoteCommands(_commands_2);
      _builder.append(_remoteCommands, "				");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t");
      EList<ObjectProperty> _pageProperties = page.getPageProperties();
      CharSequence _remoteProperties = _pageProperties==null?(CharSequence)null:this.toRemoteProperties(_pageProperties);
      _builder.append(_remoteProperties, "				");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t\t");
      _builder.append("<ui:insert name=\"additionalWidgets\" />");
      _builder.newLine();
      {
        boolean _useTooltipForValidationMessage = GeneratorConfigurationExtension.useTooltipForValidationMessage();
        if (_useTooltipForValidationMessage) {
          _builder.append("\t\t\t\t");
          _builder.append("<pe:tooltip global=\"true\" myPosition=\"left center\" atPosition=\"right center\" forSelector=\".ui-state-error\"/>");
          _builder.newLine();
        }
      }
      _builder.append("\t\t\t");
      _builder.append("</h:form>");
      _builder.newLine();
      _builder.append("\t\t");
      _builder.append("</ui:define>");
      _builder.newLine();
      _builder.newLine();
      _builder.append("\t");
      _builder.append("</ui:composition>");
      _builder.newLine();
      _builder.append("</body>");
      _builder.newLine();
      _builder.append("</html>");
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence addClientIdForField(final CharSequence name, final CharSequence clientId) {
    CharSequence _put = this.clientIds.put(clientId, name);
    return _put;
  }
  
  public CharSequence addClientIdForField(final CharSequence name) {
    CharSequence _put = this.clientIds.put(name, name);
    return _put;
  }
  
  public CharSequence getProcessedInputsList(final XmadslPage page) {
    CharSequence _xblockexpression = null;
    {
      boolean _or = false;
      EList<ObjectProperty> _pageProperties = page.getPageProperties();
      boolean _equals = Objects.equal(_pageProperties, null);
      if (_equals) {
        _or = true;
      } else {
        EList<ObjectProperty> _pageProperties_1 = page.getPageProperties();
        boolean _isEmpty = _pageProperties_1.isEmpty();
        _or = (_equals || _isEmpty);
      }
      if (_or) {
        return "@none";
      }
      StringConcatenation _builder = new StringConcatenation();
      {
        EList<ObjectProperty> _pageProperties_2 = page.getPageProperties();
        boolean _hasElements = false;
        for(final ObjectProperty prop : _pageProperties_2) {
          if (!_hasElements) {
            _hasElements = true;
          } else {
            _builder.appendImmediate(",", "");
          }
          String _name = prop.getName();
          _builder.append(_name, "");
        }
      }
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public Iterable<XmadslPage> getInvokablePages(final XmadslPage page) {
    Iterable<XmadslPage> _xblockexpression = null;
    {
      EList<Command> _commands = page.getCommands();
      final Function1<Command,CommandImplementation> _function = new Function1<Command,CommandImplementation>() {
          public CommandImplementation apply(final Command it) {
            CommandImplementation _implementation = it.getImplementation();
            return _implementation;
          }
        };
      List<CommandImplementation> _map = ListExtensions.<Command, CommandImplementation>map(_commands, _function);
      List<CommandImplementation> _list = IterableExtensions.<CommandImplementation>toList(_map);
      final Iterable<CommandImplementation> implementations = IterableExtensions.<CommandImplementation>filterNull(_list);
      final Function1<CommandImplementation,Iterable<InvokeStatement>> _function_1 = new Function1<CommandImplementation,Iterable<InvokeStatement>>() {
          public Iterable<InvokeStatement> apply(final CommandImplementation it) {
            EList<Statement> _statements = it.getStatements();
            Iterable<InvokeStatement> _filter = Iterables.<InvokeStatement>filter(_statements, InvokeStatement.class);
            return _filter;
          }
        };
      Iterable<Iterable<InvokeStatement>> _map_1 = IterableExtensions.<CommandImplementation, Iterable<InvokeStatement>>map(implementations, _function_1);
      final Iterable<InvokeStatement> statements = Iterables.<InvokeStatement>concat(_map_1);
      final Function1<InvokeStatement,Invokeable> _function_2 = new Function1<InvokeStatement,Invokeable>() {
          public Invokeable apply(final InvokeStatement it) {
            Invokeable _invokeAble = it.getInvokeAble();
            return _invokeAble;
          }
        };
      Iterable<Invokeable> _map_2 = IterableExtensions.<InvokeStatement, Invokeable>map(statements, _function_2);
      final Set<Invokeable> invokeables = IterableExtensions.<Invokeable>toSet(_map_2);
      Iterable<XmadslPage> _filter = Iterables.<XmadslPage>filter(invokeables, XmadslPage.class);
      _xblockexpression = (_filter);
    }
    return _xblockexpression;
  }
  
  public CharSequence toRemoteProperties(final EList<ObjectProperty> properties) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final ObjectProperty property : properties) {
        _builder.append("<p:inputText value=\"#{");
        XmadslPage _parentPage = this._jsfNames.getParentPage(property);
        CharSequence _bean = this._jsfNames.getBean(_parentPage);
        _builder.append(_bean, "");
        _builder.append(".");
        String _name = property.getName();
        _builder.append(_name, "");
        _builder.append("}\" style=\"display:none;\" id=\"");
        String _name_1 = property.getName();
        _builder.append(_name_1, "");
        _builder.append("\" widgetVar=\"");
        CharSequence _widgetVar = this._jsfNames.getWidgetVar(property);
        _builder.append(_widgetVar, "");
        _builder.append("\" />");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toRemoteCommands(final EList<Command> commands) {
    StringConcatenation _builder = new StringConcatenation();
    {
      final Function1<Command,Boolean> _function = new Function1<Command,Boolean>() {
          public Boolean apply(final Command c) {
            String _name = c.getName();
            boolean _equals = _name.equals("init");
            boolean _not = (!_equals);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Command> _filter = IterableExtensions.<Command>filter(commands, _function);
      for(final Command command : _filter) {
        {
          boolean _and = false;
          boolean _isClientOnly = command.isClientOnly();
          boolean _not = (!_isClientOnly);
          if (!_not) {
            _and = false;
          } else {
            boolean _hasEventMappings = this.hasEventMappings(command);
            boolean _not_1 = (!_hasEventMappings);
            _and = (_not && _not_1);
          }
          if (_and) {
            _builder.append("<p:remoteCommand action=\"#{");
            XmadslPage _parentPage = this._jsfNames.getParentPage(command);
            CharSequence _bean = this._jsfNames.getBean(_parentPage);
            _builder.append(_bean, "");
            _builder.append(".");
            String _name = command.getName();
            _builder.append(_name, "");
            _builder.append("}\" update=\"@form\" id=\"");
            String _name_1 = command.getName();
            _builder.append(_name_1, "");
            _builder.append("\" name=\"");
            CharSequence _widgetVar = this._jsfNames.getWidgetVar(command);
            _builder.append(_widgetVar, "");
            _builder.append("\" />");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final Table table) {
    CharSequence _xblockexpression = null;
    {
      String _name = table.getName();
      String _plus = (this.formName + ":");
      String _name_1 = table.getName();
      String _plus_1 = (_plus + _name_1);
      this.addClientIdForField(_name, _plus_1);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<p:dataTable id=\"");
      CharSequence _faceletId = this._jsfNames.toFaceletId(table);
      _builder.append(_faceletId, "");
      _builder.append("\"");
      _builder.newLineIfNotEmpty();
      {
        boolean _or = false;
        MultiselectionFlag _multiselectionFlag = table.getMultiselectionFlag();
        BoolLiteral _multiSelection = _multiselectionFlag==null?(BoolLiteral)null:_multiselectionFlag.getMultiSelection();
        if ((_multiSelection instanceof TrueLiteral)) {
          _or = true;
        } else {
          BoolLiteral _checkSelection = table.getCheckSelection();
          _or = ((_multiSelection instanceof TrueLiteral) || (_checkSelection instanceof TrueLiteral));
        }
        if (_or) {
          _builder.append("selectionMode=\"multiple\" selection=\"#{");
          CharSequence _modelProperty = this._faceletDataImpl.getModelProperty(table);
          _builder.append(_modelProperty, "");
          _builder.append(".selectedItems}\"");
          _builder.newLineIfNotEmpty();
        } else {
          _builder.append("selectionMode=\"single\" selection=\"#{");
          CharSequence _modelProperty_1 = this._faceletDataImpl.getModelProperty(table);
          _builder.append(_modelProperty_1, "");
          _builder.append(".selectedItem}\"");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append(" \t");
      _builder.append("widgetVar=\"");
      CharSequence _widgetVar = this._jsfNames.getWidgetVar(table);
      _builder.append(_widgetVar, " 	");
      _builder.append("\"");
      _builder.newLineIfNotEmpty();
      _builder.append("value=\"#{");
      CharSequence _modelProperty_2 = this._faceletDataImpl.getModelProperty(table);
      _builder.append(_modelProperty_2, "");
      _builder.append("}\" var=\"item\"");
      _builder.newLineIfNotEmpty();
      _builder.append("rowKey=\"#{item.oid}\"");
      _builder.newLine();
      {
        BoolLiteral _pageable = table.getPageable();
        if ((_pageable instanceof TrueLiteral)) {
          _builder.append("\t\t\t    ");
          _builder.append("paginator=\"true\" rows=\"10\" paginatorPosition=\"top\" paginatorTemplate=\"{CurrentPageReport}  {FirstPageLink} {PreviousPageLink} {PageLinks} {NextPageLink} {LastPageLink}\"");
          _builder.newLine();
        }
      }
      _builder.append("\t\t\t    ");
      _builder.append("resizableColumns=\"true\" >");
      _builder.newLine();
      _builder.append("\t\t\t    ");
      CharSequence _createEventMappingsForControl = this.createEventMappingsForControl(table);
      _builder.append(_createEventMappingsForControl, "			    ");
      _builder.newLineIfNotEmpty();
      {
        EList<TableColumn> _columns = table.getColumns();
        for(final TableColumn column : _columns) {
          CharSequence _createContents = this.createContents(column);
          _builder.append(_createContents, "");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("</p:dataTable>");
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:tree id=\"");
    CharSequence _faceletId = this._jsfNames.toFaceletId(tree);
    _builder.append(_faceletId, "");
    _builder.append("\" ");
    _builder.newLineIfNotEmpty();
    _builder.append(" \t");
    _builder.append("selectionMode=\"single\" widgetVar=\"");
    String _name = tree.getName();
    _builder.append(_name, " 	");
    _builder.append("WidgetVar\"");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t    ");
    _builder.append("value=\"#{");
    CharSequence _rootNodeProperty = this._faceletDataImpl.getRootNodeProperty(tree);
    _builder.append(_rootNodeProperty, "			    ");
    _builder.append("}\" var=\"node\"");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t    ");
    _builder.append("selection=\"#{");
    CharSequence _selectedNodeProperty = this._faceletDataImpl.getSelectedNodeProperty(tree);
    _builder.append(_selectedNodeProperty, "			    ");
    _builder.append("}\" ");
    CharSequence _disabledFlag = this.disabledFlag(tree);
    _builder.append(_disabledFlag, "			    ");
    _builder.append(">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t    ");
    CharSequence _createEventMappingsForControl = this.createEventMappingsForControl(tree);
    _builder.append(_createEventMappingsForControl, "			    ");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t    ");
    _builder.append("<p:treeNode>");
    _builder.newLine();
    _builder.append("\t\t\t    \t");
    _builder.append("<h:outputText value=\"#{node}\" />");
    _builder.newLine();
    _builder.append("\t\t\t    ");
    _builder.append("</p:treeNode>");
    _builder.newLine();
    _builder.append("</p:tree>");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createEventMapping(final IGuiElementWithEvent control, final GuiElementEventMapping mapping) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<!-- error not control event mapping for control type ");
    Class<? extends IGuiElementWithEvent> _class = control.getClass();
    String _name = _class.getName();
    _builder.append(_name, "");
    _builder.append(" -->");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createEventMapping(final Table table, final GuiElementEventMapping mapping) {
    CharSequence _switchResult = null;
    EventType _event = mapping.getEvent();
    final EventType _switchValue = _event;
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(_switchValue,EventType.ON_SELECTION)) {
        _matched=true;
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("<p:ajax event=\"rowSelect\" ");
        CharSequence _addFunctionCommands = this.addFunctionCommands(table, "listener", "onstart");
        _builder.append(_addFunctionCommands, "");
        _builder.append(" update=\"@form\" />");
        _switchResult = _builder;
      }
    }
    if (!_matched) {
      if (Objects.equal(_switchValue,EventType.ON_DOUBLE_CLICK)) {
        _matched=true;
        StringConcatenation _builder_1 = new StringConcatenation();
        _builder_1.append("<p:ajax event=\"rowDblselect\" ");
        CharSequence _addFunctionCommands_1 = this.addFunctionCommands(table, "listener", "onstart");
        _builder_1.append(_addFunctionCommands_1, "");
        _builder_1.append(" update=\"@form\" />");
        _switchResult = _builder_1;
      }
    }
    if (!_matched) {
      StringConcatenation _builder_2 = new StringConcatenation();
      _switchResult = _builder_2;
    }
    return _switchResult;
  }
  
  public CharSequence createEventMappingsForControl(final IGuiElementWithEvent control) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Iterable<GuiElementEventMapping> _eventMappingsForControl = this.getEventMappingsForControl(control);
      for(final GuiElementEventMapping mapping : _eventMappingsForControl) {
        CharSequence _createEventMapping = this.createEventMapping(control, mapping);
        _builder.append(_createEventMapping, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public Iterable<GuiElementEventMapping> getEventMappingsForControl(final IGuiElementWithEvent control) {
    XmadslPage _parentPage = this._jsfNames.getParentPage(control);
    EventMappingList _events = _parentPage.getEvents();
    EList<EventMapping> _mapping = _events.getMapping();
    Iterable<GuiElementEventMapping> _filter = Iterables.<GuiElementEventMapping>filter(_mapping, GuiElementEventMapping.class);
    final Function1<GuiElementEventMapping,Boolean> _function = new Function1<GuiElementEventMapping,Boolean>() {
        public Boolean apply(final GuiElementEventMapping m) {
          IGuiElementWithEvent _control = m.getControl();
          boolean _equals = Objects.equal(_control, control);
          return Boolean.valueOf(_equals);
        }
      };
    Iterable<GuiElementEventMapping> _filter_1 = IterableExtensions.<GuiElementEventMapping>filter(_filter, _function);
    return _filter_1;
  }
  
  public String processResourceBundleProperty(final EObject eObject, final CharSequence text) {
    String _string = new String();
    String s = _string;
    boolean _matched = false;
    if (!_matched) {
      if (eObject instanceof CustomizeableField) {
        final CustomizeableField _customizeableField = (CustomizeableField)eObject;
        _matched=true;
        String _labelForCustomField = this.getLabelForCustomField(_customizeableField);
        s = _labelForCustomField;
      }
    }
    if (!_matched) {
      if (eObject instanceof Button) {
        final Button _button = (Button)eObject;
        _matched=true;
        XmadslPage _parentPage = this._jsfNames.getParentPage(_button);
        String _name = _parentPage.getName();
        String _plus = (_name + ".");
        String _name_1 = _button.getName();
        String _plus_1 = (_plus + _name_1);
        s = _plus_1;
      }
    }
    if (!_matched) {
      if (eObject instanceof TableColumn) {
        final TableColumn _tableColumn = (TableColumn)eObject;
        _matched=true;
        EObject _eContainer = _tableColumn.eContainer();
        if ((_eContainer instanceof Table)) {
          XmadslPage _parentPage = this._jsfNames.getParentPage(_tableColumn);
          String _name = _parentPage.getName();
          String _plus = (_name + ".");
          EObject _eContainer_1 = _tableColumn.eContainer();
          String _name_1 = ((Table) _eContainer_1).getName();
          String _plus_1 = (_plus + _name_1);
          String _plus_2 = (_plus_1 + "_");
          String _elvis = null;
          String _columnName = _tableColumn.getColumnName();
          if (_columnName != null) {
            _elvis = _columnName;
          } else {
            Attribute _attribute = _tableColumn==null?(Attribute)null:_tableColumn.getAttribute();
            String _name_2 = _attribute==null?(String)null:_attribute.getName();
            _elvis = ObjectExtensions.<String>operator_elvis(_columnName, _name_2);
          }
          String _plus_3 = (_plus_2 + _elvis);
          s = _plus_3;
        } else {
          XmadslPage _parentPage_1 = this._jsfNames.getParentPage(_tableColumn);
          String _name_3 = _parentPage_1.getName();
          String _plus_4 = (_name_3 + ".");
          EObject _eContainer_2 = _tableColumn.eContainer();
          String _name_4 = ((TableCombo) _eContainer_2).getName();
          String _plus_5 = (_plus_4 + _name_4);
          String _plus_6 = (_plus_5 + "_");
          String _elvis_1 = null;
          String _columnName_1 = _tableColumn.getColumnName();
          if (_columnName_1 != null) {
            _elvis_1 = _columnName_1;
          } else {
            Attribute _attribute_1 = _tableColumn==null?(Attribute)null:_tableColumn.getAttribute();
            String _name_5 = _attribute_1==null?(String)null:_attribute_1.getName();
            _elvis_1 = ObjectExtensions.<String>operator_elvis(_columnName_1, _name_5);
          }
          String _plus_7 = (_plus_6 + _elvis_1);
          s = _plus_7;
        }
      }
    }
    if (!_matched) {
      if (eObject instanceof Label) {
        final Label _label = (Label)eObject;
        _matched=true;
        XmadslPage _parentPage = this._jsfNames.getParentPage(_label);
        String _name = _parentPage.getName();
        String _plus = (_name + ".");
        String _elvis = null;
        String _name_1 = _label.getName();
        if (_name_1 != null) {
          _elvis = _name_1;
        } else {
          String _plus_1 = ("label" + "_");
          String _plus_2 = (_plus_1 + Integer.valueOf(this.itemIndex));
          _elvis = ObjectExtensions.<String>operator_elvis(_name_1, _plus_2);
        }
        String _plus_3 = (_plus + _elvis);
        s = _plus_3;
      }
    }
    if (!_matched) {
      if (eObject instanceof CheckBox) {
        final CheckBox _checkBox = (CheckBox)eObject;
        _matched=true;
        XmadslPage _parentPage = this._jsfNames.getParentPage(_checkBox);
        String _name = _parentPage.getName();
        String _plus = (_name + ".");
        String _elvis = null;
        String _name_1 = _checkBox.getName();
        if (_name_1 != null) {
          _elvis = _name_1;
        } else {
          String _plus_1 = ("checkbox" + "_");
          String _plus_2 = (_plus_1 + Integer.valueOf(this.itemIndex));
          _elvis = ObjectExtensions.<String>operator_elvis(_name_1, _plus_2);
        }
        String _plus_3 = (_plus + _elvis);
        s = _plus_3;
      }
    }
    if (!_matched) {
      if (eObject instanceof TabPage) {
        final TabPage _tabPage = (TabPage)eObject;
        _matched=true;
        XmadslPage _parentPage = this._jsfNames.getParentPage(_tabPage);
        String _name = _parentPage.getName();
        String _plus = (_name + ".");
        String _name_1 = _tabPage.getName();
        String _plus_1 = (_plus + _name_1);
        s = _plus_1;
      }
    }
    if (!_matched) {
      if (eObject instanceof XmadslComposite) {
        final XmadslComposite _xmadslComposite = (XmadslComposite)eObject;
        _matched=true;
        XmadslPage _parentPage = this._jsfNames.getParentPage(_xmadslComposite);
        String _name = _parentPage.getName();
        String _plus = (_name + ".");
        String _name_1 = _xmadslComposite.getName();
        String _plus_1 = (_plus + _name_1);
        s = _plus_1;
      }
    }
    if (!_matched) {
      if (eObject instanceof XmadslPage) {
        final XmadslPage _xmadslPage = (XmadslPage)eObject;
        _matched=true;
        String _name = ((XmadslPage) _xmadslPage).getName();
        String _plus = (_name + ".");
        String _plus_1 = (_plus + "title");
        s = _plus_1;
      }
    }
    this.resourceProperties.put(s, text);
    String _plus = ("${" + this.resourceBundleName);
    String _plus_1 = (_plus + "[\'");
    String _plus_2 = (_plus_1 + s);
    return (_plus_2 + "\']}");
  }
  
  public String getLabelForCustomField(final CustomizeableField field) {
    String _xifexpression = null;
    EObject _eContainer = field.eContainer();
    EObject _eContainer_1 = _eContainer.eContainer();
    if ((_eContainer_1 instanceof TabPage)) {
      String _xblockexpression = null;
      {
        EObject _eContainer_2 = field.eContainer();
        EObject _eContainer_3 = _eContainer_2.eContainer();
        TabPage contatiner = ((TabPage) _eContainer_3);
        String _name = contatiner.getName();
        char _charAt = _name.charAt(0);
        char _upperCase = Character.toUpperCase(_charAt);
        String _name_1 = contatiner.getName();
        String _substring = _name_1.substring(1);
        String _plus = (Character.valueOf(_upperCase) + _substring);
        String _plus_1 = (_plus + ".");
        DataObjectVariable _object = field.getObject();
        String _name_2 = _object.getName();
        String _plus_2 = (_plus_1 + _name_2);
        String _plus_3 = (_plus_2 + "_");
        Attribute _attribute = field.getAttribute();
        String _name_3 = _attribute.getName();
        String _plus_4 = (_plus_3 + _name_3);
        _xblockexpression = (_plus_4);
      }
      _xifexpression = _xblockexpression;
    } else {
      XmadslPage _parentPage = this._jsfNames.getParentPage(field);
      String _name = _parentPage.getName();
      String _plus = (_name + ".");
      DataObjectVariable _object = field.getObject();
      String _name_1 = _object.getName();
      String _plus_1 = (_plus + _name_1);
      String _plus_2 = (_plus_1 + "_");
      Attribute _attribute = field.getAttribute();
      String _name_2 = _attribute.getName();
      String _plus_3 = (_plus_2 + _name_2);
      _xifexpression = _plus_3;
    }
    return _xifexpression;
  }
  
  protected CharSequence _createContents(final TableColumn column) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:column headerText=\"");
    String _labelText = column.getLabelText();
    CharSequence _encodedHtml = this.toEncodedHtml(_labelText);
    String _processResourceBundleProperty = this.processResourceBundleProperty(column, _encodedHtml);
    _builder.append(_processResourceBundleProperty, "");
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      boolean _and = false;
      boolean _equals = "styling".equals("styling");
      if (!_equals) {
        _and = false;
      } else {
        Attribute _attribute = column.getAttribute();
        boolean _notEquals = (!Objects.equal(_attribute, null));
        _and = (_equals && _notEquals);
      }
      if (_and) {
        _builder.append("styleClass=\"");
        Attribute _attribute_1 = column.getAttribute();
        String _name = _attribute_1.getName();
        _builder.append(_name, "	");
        _builder.append("\"");
      }
    }
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t    ");
    {
      boolean _and_1 = false;
      FieldFeature _feature = column.getFeature();
      boolean _notEquals_1 = (!Objects.equal(_feature, null));
      if (!_notEquals_1) {
        _and_1 = false;
      } else {
        FieldFeature _feature_1 = column.getFeature();
        PresentableFeature _attributeHolder = _feature_1.getAttributeHolder();
        boolean _notEquals_2 = (!Objects.equal(_attributeHolder, null));
        _and_1 = (_notEquals_1 && _notEquals_2);
      }
      if (_and_1) {
        _builder.append("sortBy=\"#{");
        CharSequence _columnValueExpression = this.columnValueExpression(column);
        _builder.append(_columnValueExpression, "				    ");
        _builder.append("}\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t\t\t    ");
        _builder.append("<h:outputText value=\"#{");
        CharSequence _columnValueExpression_1 = this.columnValueExpression(column);
        _builder.append(_columnValueExpression_1, "				    ");
        _builder.append("}\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t\t\t    ");
        _builder.append("\t");
        FieldFeature _feature_2 = column.getFeature();
        PresentableFeature _attributeHolder_1 = _feature_2.getAttributeHolder();
        Attribute _attribute_2 = _attributeHolder_1.getAttribute();
        DataTypeAndTypeParameter _type = _attribute_2.getType();
        CharSequence _createConverter = this._faceletDataImpl.createConverter(_type);
        _builder.append(_createConverter, "				    	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t\t\t    ");
        _builder.append("</h:outputText>");
        _builder.newLine();
      } else {
        _builder.append("\t\t\t\t    ");
        _builder.append(">");
        _builder.newLine();
        _builder.append("\t\t\t\t    ");
        _builder.append("<h:outputText value=\"hardcodedtext\" />");
        _builder.newLine();
      }
    }
    _builder.append("</p:column>");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createContents(final Text text) {
    CharSequence _xblockexpression = null;
    {
      String _name = text.getName();
      String _plus = (this.formName + ":");
      CharSequence _faceletId = this._jsfNames.toFaceletId(text);
      String _plus_1 = (_plus + _faceletId);
      this.addClientIdForField(_name, _plus_1);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<p:inputText id=\"");
      CharSequence _faceletId_1 = this._jsfNames.toFaceletId(text);
      _builder.append(_faceletId_1, "");
      _builder.append("\" value=\"#{");
      XmadslPage _parentPage = this._jsfNames.getParentPage(text);
      CharSequence _bean = this._jsfNames.getBean(_parentPage);
      _builder.append(_bean, "");
      _builder.append(".");
      String _name_1 = text.getName();
      _builder.append(_name_1, "");
      _builder.append("Text}\" ");
      CharSequence _disabledFlag = this.disabledFlag(text);
      _builder.append(_disabledFlag, "");
      _builder.append(" />");
      _builder.newLineIfNotEmpty();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final Combo combo) {
    CharSequence _xblockexpression = null;
    {
      String _name = combo.getName();
      String _plus = (this.formName + ":");
      CharSequence _faceletId = this._jsfNames.toFaceletId(combo);
      String _plus_1 = (_plus + _faceletId);
      this.addClientIdForField(_name, _plus_1);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<p:selectOneMenu id=\"");
      CharSequence _faceletId_1 = this._jsfNames.toFaceletId(combo);
      _builder.append(_faceletId_1, "");
      _builder.append("\" value=\"#{");
      XmadslPage _parentPage = this._jsfNames.getParentPage(combo);
      CharSequence _bean = this._jsfNames.getBean(_parentPage);
      _builder.append(_bean, "");
      _builder.append(".");
      String _name_1 = combo.getName();
      _builder.append(_name_1, "");
      _builder.append("Combo}\" ");
      CharSequence _disabledFlag = this.disabledFlag(combo);
      _builder.append(_disabledFlag, "");
      _builder.append(">");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("<f:selectItem itemLabel=\"Select value\" itemValue=\"\" />");
      _builder.newLine();
      _builder.append("\t\t        ");
      _builder.append("<f:selectItems value=\"#{");
      CharSequence _selectItemsProperty = this._faceletDataImpl.getSelectItemsProperty(combo);
      _builder.append(_selectItemsProperty, "		        ");
      _builder.append("}\" var=\"item\" itemLabel=\"#{item.name}\" itemValue=\"#{item}\" />");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t    ");
      _builder.append("</p:selectOneMenu>");
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final TableCombo combo) {
    CharSequence _xblockexpression = null;
    {
      String _name = combo.getName();
      String _plus = (this.formName + ":");
      CharSequence _faceletId = this._jsfNames.toFaceletId(combo);
      String _plus_1 = (_plus + _faceletId);
      this.addClientIdForField(_name, _plus_1);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<p:selectOneMenu id=\"");
      CharSequence _faceletId_1 = this._jsfNames.toFaceletId(combo);
      _builder.append(_faceletId_1, "");
      _builder.append("\" value=\"#{");
      XmadslPage _parentPage = this._jsfNames.getParentPage(combo);
      CharSequence _bean = this._jsfNames.getBean(_parentPage);
      _builder.append(_bean, "");
      _builder.append(".");
      String _name_1 = combo.getName();
      _builder.append(_name_1, "");
      _builder.append("TableCombo}\" var=\"item\" ");
      CharSequence _disabledFlag = this.disabledFlag(combo);
      _builder.append(_disabledFlag, "");
      _builder.append(">");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("<f:selectItem itemLabel=\"Select value\" itemValue=\"\" />");
      _builder.newLine();
      _builder.append("\t\t        ");
      _builder.append("<f:selectItems value=\"#{");
      CharSequence _listProperty = this._faceletDataImpl.getListProperty(combo);
      _builder.append(_listProperty, "		        ");
      _builder.append("}\" var=\"item\" itemLabel=\"#{item.name}\" itemValue=\"#{item}\" />");
      _builder.newLineIfNotEmpty();
      {
        EList<TableColumn> _columns = combo.getColumns();
        for(final TableColumn col : _columns) {
          _builder.append("\t\t        ");
          CharSequence _createContents = this.createContents(col);
          _builder.append(_createContents, "		        ");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("\t\t    ");
      _builder.append("</p:selectOneMenu>");
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final ListBox listbox) {
    CharSequence _xblockexpression = null;
    {
      String _name = listbox.getName();
      String _plus = (this.formName + ":");
      CharSequence _faceletId = this._jsfNames.toFaceletId(listbox);
      String _plus_1 = (_plus + _faceletId);
      this.addClientIdForField(_name, _plus_1);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<p:selectOneListbox id=\"");
      CharSequence _faceletId_1 = this._jsfNames.toFaceletId(listbox);
      _builder.append(_faceletId_1, "");
      _builder.append("\" value=\"#{");
      XmadslPage _parentPage = this._jsfNames.getParentPage(listbox);
      CharSequence _bean = this._jsfNames.getBean(_parentPage);
      _builder.append(_bean, "");
      _builder.append(".");
      String _name_1 = listbox.getName();
      _builder.append(_name_1, "");
      _builder.append("ListBox}\" ");
      CharSequence _disabledFlag = this.disabledFlag(listbox);
      _builder.append(_disabledFlag, "");
      _builder.append(">");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("<f:selectItem itemLabel=\"Select value\" itemValue=\"\" />");
      _builder.newLine();
      _builder.append("\t\t        ");
      _builder.append("<f:selectItems value=\"#{");
      CharSequence _selectItemsProperty = this._faceletDataImpl.getSelectItemsProperty(listbox);
      _builder.append(_selectItemsProperty, "		        ");
      _builder.append("}\" var=\"item\" itemLabel=\"#{item.name}\" itemValue=\"#{item}\" />");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t    ");
      _builder.append("</p:selectOneListbox>");
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final RadioButton radio) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<ui:remove>Not supported: Radiobutton! </ui:remove>");
    return _builder;
  }
  
  public CharSequence uniqueIdentifierAtribute(final XmadslComposite composite) {
    String _xblockexpression = null;
    {
      boolean _and = false;
      String _name = composite.getName();
      boolean _notEquals = (!Objects.equal(_name, null));
      if (!_notEquals) {
        _and = false;
      } else {
        _and = (_notEquals && (this.createIdentifiers).booleanValue());
      }
      if (_and) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("id=\"");
        CharSequence _faceletId = this._jsfNames.toFaceletId(composite);
        _builder.append(_faceletId, "");
        _builder.append("\"");
        return _builder;
      }
      _xblockexpression = ("");
    }
    return _xblockexpression;
  }
  
  public CharSequence uniqueIdentifierAtribute(final TabFolder folder) {
    String _xblockexpression = null;
    {
      boolean _and = false;
      String _name = folder.getName();
      boolean _notEquals = (!Objects.equal(_name, null));
      if (!_notEquals) {
        _and = false;
      } else {
        _and = (_notEquals && (this.createIdentifiers).booleanValue());
      }
      if (_and) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("id=\"");
        CharSequence _faceletId = this._jsfNames.toFaceletId(folder);
        _builder.append(_faceletId, "");
        _builder.append("\"");
        return _builder;
      }
      _xblockexpression = ("");
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final TabFolder tabFolder) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:tabView ");
    CharSequence _uniqueIdentifierAtribute = this.uniqueIdentifierAtribute(tabFolder);
    _builder.append(_uniqueIdentifierAtribute, "");
    _builder.append(">");
    _builder.newLineIfNotEmpty();
    {
      EList<EObject> _eContents = tabFolder.eContents();
      for(final EObject item : _eContents) {
        CharSequence _createContents = this.createContents(item);
        _builder.append(_createContents, "");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("</p:tabView>");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createContents(final TabPage tabPage) {
    CharSequence _xblockexpression = null;
    {
      String _name = tabPage.getName();
      this.addClientIdForField(_name);
      StringConcatenation _builder = new StringConcatenation();
      {
        boolean _isCustomizable = this._faceletLayoutImpl.isCustomizable(tabPage);
        if (_isCustomizable) {
          _builder.append("<ui:insert name=\"");
          String _itemName = this.getItemName(tabPage);
          _builder.append(_itemName, "");
          _builder.append("\">");
        }
      }
      _builder.newLineIfNotEmpty();
      _builder.append("<p:tab id=\"");
      CharSequence _faceletId = this._jsfNames.toFaceletId(tabPage);
      _builder.append(_faceletId, "");
      _builder.append("\" title=\"");
      LabelText _labelText = tabPage.getLabelText();
      String _string = _labelText==null?(String)null:_labelText.getString();
      String _processResourceBundleProperty = this.processResourceBundleProperty(tabPage, _string);
      _builder.append(_processResourceBundleProperty, "");
      _builder.append("\">");
      _builder.newLineIfNotEmpty();
      {
        EList<EObject> _eContents = tabPage.eContents();
        for(final EObject item : _eContents) {
          _builder.append("\t");
          CharSequence _beforeChildItem = this.beforeChildItem(false, false, 0, item);
          _builder.append(_beforeChildItem, "	");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          CharSequence _createContents = this.createContents(item);
          _builder.append(_createContents, "	");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          CharSequence _afterChildItem = this.afterChildItem(false, false, 0, item);
          _builder.append(_afterChildItem, "	");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("</p:tab>");
      _builder.newLine();
      {
        boolean _isCustomizable_1 = this._faceletLayoutImpl.isCustomizable(tabPage);
        if (_isCustomizable_1) {
          _builder.append("</ui:insert>");
        }
      }
      _builder.newLineIfNotEmpty();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final Button button) {
    CharSequence _xblockexpression = null;
    {
      String _name = button.getName();
      String _plus = (this.formName + ":");
      CharSequence _faceletId = this._jsfNames.toFaceletId(button);
      String _plus_1 = (_plus + _faceletId);
      this.addClientIdForField(_name, _plus_1);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<p:commandButton id=\"");
      CharSequence _faceletId_1 = this._jsfNames.toFaceletId(button);
      _builder.append(_faceletId_1, "");
      _builder.append("\" value=\"");
      LabelText _labelText = button.getLabelText();
      String _label = _labelText.getLabel();
      String _processResourceBundleProperty = this.processResourceBundleProperty(button, _label);
      _builder.append(_processResourceBundleProperty, "");
      _builder.append("\" ");
      CharSequence _addFunctionCommands = this.addFunctionCommands(button, "actionListener", "onclick");
      _builder.append(_addFunctionCommands, "");
      _builder.append(" ");
      {
        boolean _hasEventMappings = this.hasEventMappings(button, false);
        boolean _not = (!_hasEventMappings);
        if (_not) {
          _builder.append("type=\"button\"");
        } else {
          _builder.append("update=\"@form\" ");
        }
      }
      _builder.append(" ");
      CharSequence _buttonStyle = this._faceletLayoutImpl.buttonStyle(button);
      _builder.append(_buttonStyle, "");
      _builder.append("  ");
      CharSequence _disabledFlag = this.disabledFlag(button);
      _builder.append(_disabledFlag, "");
      _builder.append("/>");
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final Seperator seperator) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p:separator />");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _createContents(final FieldReference field) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<h:field attribute=\"");
    Attribute _attribute = field.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createContents(final Label label) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<h:outputLabel value=\"");
    LabelText _labelText = label.getLabelText();
    String _label = _labelText.getLabel();
    CharSequence _encodedHtml = this.toEncodedHtml(_label);
    String _processResourceBundleProperty = this.processResourceBundleProperty(label, _encodedHtml);
    _builder.append(_processResourceBundleProperty, "");
    _builder.append("\" />");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createContents(final CheckBox checkBox) {
    CharSequence _xblockexpression = null;
    {
      String _name = checkBox.getName();
      String _plus = (this.formName + ":");
      CharSequence _faceletId = this._jsfNames.toFaceletId(checkBox);
      String _plus_1 = (_plus + _faceletId);
      this.addClientIdForField(_name, _plus_1);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("<p:selectBooleanCheckbox id=\"");
      CharSequence _faceletId_1 = this._jsfNames.toFaceletId(checkBox);
      _builder.append(_faceletId_1, "");
      _builder.append("\" value=\"#{");
      XmadslPage _parentPage = this._jsfNames.getParentPage(checkBox);
      CharSequence _bean = this._jsfNames.getBean(_parentPage);
      _builder.append(_bean, "");
      _builder.append(".");
      String _name_1 = checkBox.getName();
      _builder.append(_name_1, "");
      _builder.append("Selected}\" ");
      CharSequence _disabledFlag = this.disabledFlag(checkBox);
      _builder.append(_disabledFlag, "");
      _builder.append(" />");
      _builder.newLineIfNotEmpty();
      _builder.append("<p:outputLabel value=\"");
      LabelText _labelText = checkBox.getLabelText();
      String _label = _labelText.getLabel();
      CharSequence _encodedHtml = this.toEncodedHtml(_label);
      String _processResourceBundleProperty = this.processResourceBundleProperty(checkBox, _encodedHtml);
      _builder.append(_processResourceBundleProperty, "");
      _builder.append("\" for=\"");
      CharSequence _faceletId_2 = this._jsfNames.toFaceletId(checkBox);
      _builder.append(_faceletId_2, "");
      _builder.append("\" />");
      _builder.newLineIfNotEmpty();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final LabelText label) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _and = false;
      EObject _eContainer = label.eContainer();
      if (!(_eContainer instanceof TabPage)) {
        _and = false;
      } else {
        EObject _eContainer_1 = label.eContainer();
        LabelText _labelText = ((TabPage) _eContainer_1).getLabelText();
        boolean _equals = Objects.equal(_labelText, label);
        _and = ((_eContainer instanceof TabPage) && _equals);
      }
      if (_and) {
      } else {
        _builder.append("<h:outputLabel value=\"");
        String _label = label.getLabel();
        CharSequence _encodedHtml = this.toEncodedHtml(_label);
        _builder.append(_encodedHtml, "");
        _builder.append("\" />");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final CustomizeableField field) {
    CharSequence _xblockexpression = null;
    {
      Attribute _attribute = field.getAttribute();
      String _name = _attribute.getName();
      String _plus = (this.formName + ":");
      CharSequence _faceletId = this._jsfNames.toFaceletId(field);
      String _plus_1 = (_plus + _faceletId);
      this.addClientIdForField(_name, _plus_1);
      StringConcatenation _builder = new StringConcatenation();
      {
        boolean _or = false;
        boolean _or_1 = false;
        EList<FieldPartSpecification> _parts = field.getParts();
        boolean _equals = Objects.equal(_parts, null);
        if (_equals) {
          _or_1 = true;
        } else {
          EList<FieldPartSpecification> _parts_1 = field.getParts();
          int _size = _parts_1.size();
          boolean _equals_1 = (_size == 0);
          _or_1 = (_equals || _equals_1);
        }
        if (_or_1) {
          _or = true;
        } else {
          EList<FieldPartSpecification> _parts_2 = field.getParts();
          final Function1<FieldPartSpecification,Boolean> _function = new Function1<FieldPartSpecification,Boolean>() {
              public Boolean apply(final FieldPartSpecification fps) {
                FieldPart _part = fps.getPart();
                boolean _equals = Objects.equal(_part, FieldPart.LABEL);
                return Boolean.valueOf(_equals);
              }
            };
          boolean _exists = IterableExtensions.<FieldPartSpecification>exists(_parts_2, _function);
          _or = (_or_1 || _exists);
        }
        if (_or) {
          CharSequence _renderStartColumn = this.renderStartColumn(field);
          _builder.append(_renderStartColumn, "");
          _builder.newLineIfNotEmpty();
          _builder.append("<p:outputLabel value=\"");
          String _labelText = field.getLabelText();
          CharSequence _encodedHtml = this.toEncodedHtml(_labelText);
          String _processResourceBundleProperty = this.processResourceBundleProperty(field, _encodedHtml);
          _builder.append(_processResourceBundleProperty, "");
          _builder.append("\" for=\"");
          CharSequence _faceletId_1 = this._jsfNames.toFaceletId(field);
          _builder.append(_faceletId_1, "");
          _builder.append("\"/>");
          _builder.newLineIfNotEmpty();
          CharSequence _renderEndColumn = this.renderEndColumn(field);
          _builder.append(_renderEndColumn, "");
          _builder.newLineIfNotEmpty();
        }
      }
      {
        boolean _or_2 = false;
        boolean _or_3 = false;
        EList<FieldPartSpecification> _parts_3 = field.getParts();
        boolean _equals_2 = Objects.equal(_parts_3, null);
        if (_equals_2) {
          _or_3 = true;
        } else {
          EList<FieldPartSpecification> _parts_4 = field.getParts();
          int _size_1 = _parts_4.size();
          boolean _equals_3 = (_size_1 == 0);
          _or_3 = (_equals_2 || _equals_3);
        }
        if (_or_3) {
          _or_2 = true;
        } else {
          EList<FieldPartSpecification> _parts_5 = field.getParts();
          final Function1<FieldPartSpecification,Boolean> _function_1 = new Function1<FieldPartSpecification,Boolean>() {
              public Boolean apply(final FieldPartSpecification fps) {
                FieldPart _part = fps.getPart();
                boolean _equals = Objects.equal(_part, FieldPart.CONTROL);
                return Boolean.valueOf(_equals);
              }
            };
          boolean _exists_1 = IterableExtensions.<FieldPartSpecification>exists(_parts_5, _function_1);
          _or_2 = (_or_3 || _exists_1);
        }
        if (_or_2) {
          CharSequence _renderStartColumn_1 = this.renderStartColumn(field);
          _builder.append(_renderStartColumn_1, "");
          _builder.newLineIfNotEmpty();
          {
            EList<FieldFlag> _fieldFlags = field.getFieldFlags();
            final Function1<FieldFlag,Boolean> _function_2 = new Function1<FieldFlag,Boolean>() {
                public Boolean apply(final FieldFlag f) {
                  boolean _and = false;
                  if (!(f instanceof VisibleFlag)) {
                    _and = false;
                  } else {
                    EqualityExpr _expression = f.getExpression();
                    _and = ((f instanceof VisibleFlag) && (_expression instanceof FalseLiteral));
                  }
                  return Boolean.valueOf(_and);
                }
              };
            boolean _exists_2 = IterableExtensions.<FieldFlag>exists(_fieldFlags, _function_2);
            if (_exists_2) {
              _builder.append("\t\t\t    ");
              _builder.append("<p:inputText id=\"");
              CharSequence _faceletId_2 = this._jsfNames.toFaceletId(field);
              _builder.append(_faceletId_2, "			    ");
              _builder.append("\" value=\"#{");
              CharSequence _fieldValueExpression = this.fieldValueExpression(field);
              _builder.append(_fieldValueExpression, "			    ");
              _builder.append("}\" style=\"display:none;\" />");
              _builder.newLineIfNotEmpty();
            } else {
              Attribute _attribute_1 = field.getAttribute();
              DataTypeAndTypeParameter _type = _attribute_1.getType();
              Type _dataType = _type.getDataType();
              if ((_dataType instanceof EnumType)) {
                _builder.append("\t\t\t    ");
                _builder.append("<p:selectOneMenu id=\"");
                CharSequence _faceletId_3 = this._jsfNames.toFaceletId(field);
                _builder.append(_faceletId_3, "			    ");
                _builder.append("\" value=\"#{");
                CharSequence _fieldValueExpression_1 = this.fieldValueExpression(field);
                _builder.append(_fieldValueExpression_1, "			    ");
                _builder.append("}\" ");
                CharSequence _disabledFlag = this.disabledFlag(field);
                _builder.append(_disabledFlag, "			    ");
                _builder.append(" ");
                CharSequence _requiredFlag = this.requiredFlag(field);
                _builder.append(_requiredFlag, "			    ");
                _builder.append(" ");
                {
                  boolean _useTooltipForValidationMessage = GeneratorConfigurationExtension.useTooltipForValidationMessage();
                  if (_useTooltipForValidationMessage) {
                    _builder.append("title=\"#{component.valid ? \'\' : facesContext.getMessageList(component.getClientId())[0].summary}\"");
                  }
                }
                _builder.append(">");
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t\t    ");
                _builder.append("\t");
                _builder.append("<f:selectItem itemLabel=\"Select value\" itemValue=\"\" />");
                _builder.newLine();
                _builder.append("\t\t\t    ");
                _builder.append("    ");
                _builder.append("<f:selectItems value=\"#{");
                CharSequence _selectItemsProperty = this._faceletDataImpl.getSelectItemsProperty(field);
                _builder.append(_selectItemsProperty, "			        ");
                _builder.append("}\" var=\"item\" itemLabel=\"#{item.name}\" itemValue=\"#{item}\" />");
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t\t    ");
                _builder.append("</p:selectOneMenu>");
                _builder.newLine();
              } else {
                Attribute _attribute_2 = field.getAttribute();
                DataTypeAndTypeParameter _type_1 = _attribute_2.getType();
                ValidatorReference _validatorReferenceDefinition = _type_1.getValidatorReferenceDefinition();
                boolean _isI18nValidator = _validatorReferenceDefinition.isI18nValidator();
                if (_isI18nValidator) {
                  _builder.append("\t\t\t    ");
                  _builder.append("<p:selectOneMenu id=\"");
                  CharSequence _faceletId_4 = this._jsfNames.toFaceletId(field);
                  _builder.append(_faceletId_4, "			    ");
                  _builder.append("\" value=\"#{");
                  CharSequence _fieldValueExpression_2 = this.fieldValueExpression(field);
                  _builder.append(_fieldValueExpression_2, "			    ");
                  _builder.append("}\" ");
                  CharSequence _disabledFlag_1 = this.disabledFlag(field);
                  _builder.append(_disabledFlag_1, "			    ");
                  _builder.append(" ");
                  CharSequence _requiredFlag_1 = this.requiredFlag(field);
                  _builder.append(_requiredFlag_1, "			    ");
                  _builder.append(" ");
                  {
                    boolean _useTooltipForValidationMessage_1 = GeneratorConfigurationExtension.useTooltipForValidationMessage();
                    if (_useTooltipForValidationMessage_1) {
                      _builder.append("title=\"#{component.valid ? \'\' : facesContext.getMessageList(component.getClientId())[0].summary}\"");
                    }
                  }
                  _builder.append(">");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t\t\t    ");
                  _builder.append("\t");
                  _builder.append("<f:selectItem itemLabel=\"Select value\" itemValue=\"\" />");
                  _builder.newLine();
                  _builder.append("\t\t\t    ");
                  _builder.append("    ");
                  _builder.append("<f:selectItems value=\"#{");
                  CharSequence _selectItemsProperty_1 = this._faceletDataImpl.getSelectItemsProperty(field);
                  _builder.append(_selectItemsProperty_1, "			        ");
                  _builder.append("}\" var=\"item\" itemLabel=\"#{item.name}\" itemValue=\"#{item}\" />");
                  _builder.newLineIfNotEmpty();
                  _builder.append("\t\t\t    ");
                  _builder.append("</p:selectOneMenu>");
                  _builder.newLine();
                } else {
                  Attribute _attribute_3 = field.getAttribute();
                  DataTypeAndTypeParameter _type_2 = _attribute_3.getType();
                  ValidatorReference _validatorReferenceDefinition_1 = _type_2.getValidatorReferenceDefinition();
                  boolean _isBooleanValidator = _validatorReferenceDefinition_1.isBooleanValidator();
                  if (_isBooleanValidator) {
                    _builder.append("\t\t\t    ");
                    _builder.append("<p:selectBooleanCheckbox id=\"");
                    CharSequence _faceletId_5 = this._jsfNames.toFaceletId(field);
                    _builder.append(_faceletId_5, "			    ");
                    _builder.append("\" value=\"#{");
                    CharSequence _fieldValueExpression_3 = this.fieldValueExpression(field);
                    _builder.append(_fieldValueExpression_3, "			    ");
                    _builder.append("}\" ");
                    CharSequence _disabledFlag_2 = this.disabledFlag(field);
                    _builder.append(_disabledFlag_2, "			    ");
                    _builder.append(" ");
                    CharSequence _requiredFlag_2 = this.requiredFlag(field);
                    _builder.append(_requiredFlag_2, "			    ");
                    _builder.append(" ");
                    {
                      boolean _useTooltipForValidationMessage_2 = GeneratorConfigurationExtension.useTooltipForValidationMessage();
                      if (_useTooltipForValidationMessage_2) {
                        _builder.append("title=\"#{component.valid ? \'\' : facesContext.getMessageList(component.getClientId())[0].summary}\"");
                      }
                    }
                    _builder.append("/>");
                    _builder.newLineIfNotEmpty();
                  } else {
                    Attribute _attribute_4 = field.getAttribute();
                    DataTypeAndTypeParameter _type_3 = _attribute_4.getType();
                    ValidatorReference _validatorReferenceDefinition_2 = _type_3.getValidatorReferenceDefinition();
                    boolean _isDateValidator = _validatorReferenceDefinition_2.isDateValidator();
                    if (_isDateValidator) {
                      _builder.append("\t\t\t    ");
                      _builder.append("<p:calendar id=\"");
                      CharSequence _faceletId_6 = this._jsfNames.toFaceletId(field);
                      _builder.append(_faceletId_6, "			    ");
                      _builder.append("\" value=\"#{");
                      CharSequence _fieldValueExpression_4 = this.fieldValueExpression(field);
                      _builder.append(_fieldValueExpression_4, "			    ");
                      _builder.append("}\" ");
                      CharSequence _disabledFlag_3 = this.disabledFlag(field);
                      _builder.append(_disabledFlag_3, "			    ");
                      _builder.append(" ");
                      CharSequence _requiredFlag_3 = this.requiredFlag(field);
                      _builder.append(_requiredFlag_3, "			    ");
                      _builder.append(" ");
                      {
                        boolean _useTooltipForValidationMessage_3 = GeneratorConfigurationExtension.useTooltipForValidationMessage();
                        if (_useTooltipForValidationMessage_3) {
                          _builder.append("title=\"#{component.valid ? \'\' : facesContext.getMessageList(component.getClientId())[0].summary}\"");
                        }
                      }
                      _builder.append("/>");
                      _builder.newLineIfNotEmpty();
                    } else {
                      _builder.append("\t\t\t    ");
                      _builder.append("<p:inputText id=\"");
                      CharSequence _faceletId_7 = this._jsfNames.toFaceletId(field);
                      _builder.append(_faceletId_7, "			    ");
                      _builder.append("\" value=\"#{");
                      CharSequence _fieldValueExpression_5 = this.fieldValueExpression(field);
                      _builder.append(_fieldValueExpression_5, "			    ");
                      _builder.append("}\" ");
                      {
                        boolean _useTooltipForValidationMessage_4 = GeneratorConfigurationExtension.useTooltipForValidationMessage();
                        if (_useTooltipForValidationMessage_4) {
                          _builder.append("title=\"#{component.valid ? \'\' : facesContext.getMessageList(component.getClientId())[0].summary}\"");
                        }
                      }
                      _builder.append(" ");
                      CharSequence _disabledFlag_4 = this.disabledFlag(field);
                      _builder.append(_disabledFlag_4, "			    ");
                      _builder.append(" ");
                      CharSequence _requiredFlag_4 = this.requiredFlag(field);
                      _builder.append(_requiredFlag_4, "			    ");
                      _builder.append(">");
                      _builder.newLineIfNotEmpty();
                      _builder.append("\t\t\t\t    ");
                      Attribute _attribute_5 = field.getAttribute();
                      DataTypeAndTypeParameter _type_4 = _attribute_5.getType();
                      CharSequence _createConverter = this._faceletDataImpl.createConverter(_type_4);
                      _builder.append(_createConverter, "				    ");
                      _builder.newLineIfNotEmpty();
                      _builder.append("\t\t\t    ");
                      _builder.append("</p:inputText>");
                      _builder.newLine();
                    }
                  }
                }
              }
            }
          }
          _builder.append("\t\t\t    ");
          CharSequence _renderEndColumn_1 = this.renderEndColumn(field);
          _builder.append(_renderEndColumn_1, "			    ");
          _builder.newLineIfNotEmpty();
        }
      }
      {
        boolean _useTooltipForValidationMessage_5 = GeneratorConfigurationExtension.useTooltipForValidationMessage();
        boolean _not = (!_useTooltipForValidationMessage_5);
        if (_not) {
          CharSequence _renderStartColumn_2 = this.renderStartColumn(field);
          _builder.append(_renderStartColumn_2, "");
          _builder.newLineIfNotEmpty();
          _builder.append("<p:message for=\"");
          CharSequence _faceletId_8 = this._jsfNames.toFaceletId(field);
          _builder.append(_faceletId_8, "");
          _builder.append("\" display=\"icon\" />");
          _builder.newLineIfNotEmpty();
          CharSequence _renderEndColumn_2 = this.renderEndColumn(field);
          _builder.append(_renderEndColumn_2, "");
          _builder.newLineIfNotEmpty();
        }
      }
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence renderStartColumn(final EObject field) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isRenderingInTabPage = this.isRenderingInTabPage(field);
      boolean _not = (!_isRenderingInTabPage);
      if (_not) {
        _builder.append("<p:column>");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence renderEndColumn(final EObject field) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isRenderingInTabPage = this.isRenderingInTabPage(field);
      boolean _not = (!_isRenderingInTabPage);
      if (_not) {
        _builder.append("</p:column>");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence requiredFlag(final CustomizeableField field) {
    EList<FieldFlag> _fieldFlags = field.getFieldFlags();
    CharSequence _requiredFlag = this.requiredFlag(_fieldFlags);
    return _requiredFlag;
  }
  
  public CharSequence requriedFlag(final IElementWithLayoutData widget) {
    EList<FieldFlag> _fieldFlags = widget.getFieldFlags();
    CharSequence _requiredFlag = this.requiredFlag(_fieldFlags);
    return _requiredFlag;
  }
  
  public CharSequence readonlyFlag(final CustomizeableField field) {
    EList<FieldFlag> _fieldFlags = field.getFieldFlags();
    CharSequence _readonlyFlag = this.readonlyFlag(_fieldFlags);
    return _readonlyFlag;
  }
  
  public CharSequence readonlyFlag(final IElementWithLayoutData widget) {
    EList<FieldFlag> _fieldFlags = widget.getFieldFlags();
    CharSequence _readonlyFlag = this.readonlyFlag(_fieldFlags);
    return _readonlyFlag;
  }
  
  public CharSequence disabledFlag(final CustomizeableField field) {
    EList<FieldFlag> _fieldFlags = field.getFieldFlags();
    CharSequence _disabledFlag = this.disabledFlag(_fieldFlags);
    return _disabledFlag;
  }
  
  public CharSequence disabledFlag(final IElementWithLayoutData widget) {
    EList<FieldFlag> _fieldFlags = widget.getFieldFlags();
    CharSequence _disabledFlag = this.disabledFlag(_fieldFlags);
    return _disabledFlag;
  }
  
  public CharSequence disabledFlag(final EList<FieldFlag> flags) {
    CharSequence _xblockexpression = null;
    {
      final Function1<FieldFlag,Boolean> _function = new Function1<FieldFlag,Boolean>() {
          public Boolean apply(final FieldFlag f) {
            return Boolean.valueOf((f instanceof EnabledFlag));
          }
        };
      final FieldFlag flag = IterableExtensions.<FieldFlag>findFirst(flags, _function);
      CharSequence _xifexpression = null;
      boolean _notEquals = (!Objects.equal(flag, null));
      if (_notEquals) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("disabled=\"#{!(");
        EqualityExpr _expression = ((EnabledFlag) flag).getExpression();
        String _compile = this._faceletExpressionCompiler.compile(_expression);
        _builder.append(_compile, "");
        _builder.append(")}\"");
        _xifexpression = _builder;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  public CharSequence readonlyFlag(final EList<FieldFlag> flags) {
    CharSequence _xblockexpression = null;
    {
      final Function1<FieldFlag,Boolean> _function = new Function1<FieldFlag,Boolean>() {
          public Boolean apply(final FieldFlag f) {
            return Boolean.valueOf((f instanceof ReadOnlyFlag));
          }
        };
      final FieldFlag flag = IterableExtensions.<FieldFlag>findFirst(flags, _function);
      CharSequence _xifexpression = null;
      boolean _notEquals = (!Objects.equal(flag, null));
      if (_notEquals) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("readonly=\"#{");
        EqualityExpr _expression = ((ReadOnlyFlag) flag).getExpression();
        String _compile = this._faceletExpressionCompiler.compile(_expression);
        _builder.append(_compile, "");
        _builder.append("}\"");
        _xifexpression = _builder;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  public CharSequence requiredFlag(final EList<FieldFlag> flags) {
    CharSequence _xblockexpression = null;
    {
      final Function1<FieldFlag,Boolean> _function = new Function1<FieldFlag,Boolean>() {
          public Boolean apply(final FieldFlag f) {
            return Boolean.valueOf((f instanceof RequiredFlag));
          }
        };
      final FieldFlag flag = IterableExtensions.<FieldFlag>findFirst(flags, _function);
      CharSequence _xifexpression = null;
      boolean _notEquals = (!Objects.equal(flag, null));
      if (_notEquals) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("required=\"#{");
        EqualityExpr _expression = ((RequiredFlag) flag).getExpression();
        String _compile = this._faceletExpressionCompiler.compile(_expression);
        _builder.append(_compile, "");
        _builder.append("}\"");
        _xifexpression = _builder;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createContents(final Content content) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = content.eContents();
      for(final EObject item : _eContents) {
        CharSequence _beforeChildItem = this.beforeChildItem(false, false, 0, item);
        _builder.append(_beforeChildItem, "");
        _builder.newLineIfNotEmpty();
        CharSequence _createContents = this.createContents(item);
        _builder.append(_createContents, "");
        _builder.newLineIfNotEmpty();
        CharSequence _afterChildItem = this.afterChildItem(false, false, 0, item);
        _builder.append(_afterChildItem, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final SetOfGuiElements s) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<ComposedElement> _elements = s.getElements();
      for(final ComposedElement item : _elements) {
        final boolean isCustomizeableField = this.isCustomizeableField(item);
        _builder.newLineIfNotEmpty();
        EObject _eContainer = s.eContainer();
        EObject _eContainer_1 = _eContainer.eContainer();
        boolean _isVerticalPartitioning = ((XmadslComposite) _eContainer_1).isVerticalPartitioning();
        boolean _not = (!isCustomizeableField);
        CharSequence _beforeChildItem = this.beforeChildItem(_isVerticalPartitioning, _not, 1, item);
        _builder.append(_beforeChildItem, "");
        _builder.newLineIfNotEmpty();
        CharSequence _createContents = this.createContents(item);
        _builder.append(_createContents, "");
        _builder.newLineIfNotEmpty();
        EObject _eContainer_2 = s.eContainer();
        EObject _eContainer_3 = _eContainer_2.eContainer();
        boolean _isVerticalPartitioning_1 = ((XmadslComposite) _eContainer_3).isVerticalPartitioning();
        boolean _not_1 = (!isCustomizeableField);
        CharSequence _afterChildItem = this.afterChildItem(_isVerticalPartitioning_1, _not_1, 1, item);
        _builder.append(_afterChildItem, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final ReferencedXMAComposite composite) {
    CharSequence _createChildContents = this.createChildContents(composite);
    return _createChildContents;
  }
  
  protected CharSequence _createContents(final XmadslComposite composite) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _and = false;
      boolean _isHorizontalPartitioning = composite.isHorizontalPartitioning();
      boolean _not = (!_isHorizontalPartitioning);
      if (!_not) {
        _and = false;
      } else {
        boolean _isVerticalPartitioning = composite.isVerticalPartitioning();
        boolean _not_1 = (!_isVerticalPartitioning);
        _and = (_not && _not_1);
      }
      if (_and) {
        {
          CharSequence _compositeLabel = this.getCompositeLabel(composite);
          boolean _notEquals = (!Objects.equal(_compositeLabel, null));
          if (_notEquals) {
            _builder.append("<p:panel ");
            CharSequence _uniqueIdentifierAtribute = this.uniqueIdentifierAtribute(composite);
            _builder.append(_uniqueIdentifierAtribute, "");
            _builder.append(">");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("<f:facet name=\"header\">");
            CharSequence _compositeLabel_1 = this.getCompositeLabel(composite);
            String _processResourceBundleProperty = this.processResourceBundleProperty(composite, _compositeLabel_1);
            _builder.append(_processResourceBundleProperty, "	");
            _builder.append("</f:facet>");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            CharSequence _createChildContents = this.createChildContents(composite);
            _builder.append(_createChildContents, "	");
            _builder.newLineIfNotEmpty();
            _builder.append("</p:panel>");
            _builder.newLine();
          } else {
            _builder.append("<h:panelGroup ");
            CharSequence _uniqueIdentifierAtribute_1 = this.uniqueIdentifierAtribute(composite);
            _builder.append(_uniqueIdentifierAtribute_1, "");
            _builder.append(">");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            CharSequence _createChildContents_1 = this.createChildContents(composite);
            _builder.append(_createChildContents_1, "	");
            _builder.newLineIfNotEmpty();
            _builder.append("</h:panelGroup>\t\t\t");
            _builder.newLine();
          }
        }
      } else {
        boolean _isHorizontalPartitioning_1 = composite.isHorizontalPartitioning();
        if (_isHorizontalPartitioning_1) {
          _builder.append("<p:panelGrid columns=\"");
          Integer _columns = this._faceletLayoutImpl.getColumns(composite);
          _builder.append(_columns, "");
          _builder.append("\" ");
          CharSequence _uniqueIdentifierAtribute_2 = this.uniqueIdentifierAtribute(composite);
          _builder.append(_uniqueIdentifierAtribute_2, "");
          _builder.append(">");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          {
            CharSequence _compositeLabel_2 = this.getCompositeLabel(composite);
            boolean _notEquals_1 = (!Objects.equal(_compositeLabel_2, null));
            if (_notEquals_1) {
              _builder.append("<f:facet name=\"header\">");
              CharSequence _compositeLabel_3 = this.getCompositeLabel(composite);
              String _processResourceBundleProperty_1 = this.processResourceBundleProperty(composite, _compositeLabel_3);
              _builder.append(_processResourceBundleProperty_1, "	");
              _builder.append("</f:facet>");
            }
          }
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          CharSequence _createChildContents_2 = this.createChildContents(composite);
          _builder.append(_createChildContents_2, "	");
          _builder.append(" ");
          _builder.newLineIfNotEmpty();
          _builder.append("</p:panelGrid>");
          _builder.newLine();
        } else {
          boolean _isVerticalPartitioning_1 = composite.isVerticalPartitioning();
          if (_isVerticalPartitioning_1) {
            _builder.append("<p:panelGrid ");
            CharSequence _uniqueIdentifierAtribute_3 = this.uniqueIdentifierAtribute(composite);
            _builder.append(_uniqueIdentifierAtribute_3, "");
            _builder.append(">");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            {
              CharSequence _compositeLabel_4 = this.getCompositeLabel(composite);
              boolean _notEquals_2 = (!Objects.equal(_compositeLabel_4, null));
              if (_notEquals_2) {
                _builder.append("<f:facet name=\"header\">");
                CharSequence _compositeLabel_5 = this.getCompositeLabel(composite);
                String _processResourceBundleProperty_2 = this.processResourceBundleProperty(composite, _compositeLabel_5);
                _builder.append(_processResourceBundleProperty_2, "	");
                _builder.append("<p:row><p:column colspan=\"");
                Integer _columns_1 = this._faceletLayoutImpl.getColumns(composite);
                _builder.append(_columns_1, "	");
                _builder.append("\">");
                CharSequence _compositeLabel_6 = this.getCompositeLabel(composite);
                _builder.append(_compositeLabel_6, "	");
                _builder.append("</p:column></p:row></f:facet>");
              }
            }
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            CharSequence _createChildContents_3 = this.createChildContents(composite);
            _builder.append(_createChildContents_3, "	");
            _builder.append(" ");
            _builder.newLineIfNotEmpty();
            _builder.append("</p:panelGrid>");
            _builder.newLine();
          } else {
            CharSequence _createChildContents_4 = this.createChildContents(composite);
            _builder.append(_createChildContents_4, "");
            _builder.append("\t\t\t\t");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final ComposeData data) {
    return null;
  }
  
  protected CharSequence _createContents(final AttachmentProperty property) {
    return null;
  }
  
  protected CharSequence _createContents(final EObject eObject) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<!-- !!! No implementation for type \'");
    EClass _eClass = eObject.eClass();
    String _name = _eClass.getName();
    _builder.append(_name, "");
    _builder.append("\' !!! -->");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _createContents(final EventMappingList eObject) {
    return null;
  }
  
  public CharSequence beforeChildContents(final boolean vertical, final int tabulators, final CharSequence label) {
    return null;
  }
  
  public CharSequence afterChildContents(final boolean vertical, final int tabulators) {
    return null;
  }
  
  public boolean isRenderingInTabPage(final EObject item) {
    EObject _eContainer = item.eContainer();
    EObject _eContainer_1 = _eContainer.eContainer();
    return (_eContainer_1 instanceof TabPage);
  }
  
  public String getItemName(final EObject item) {
    try {
      String name = "";
      Class<? extends EObject> _class = item.getClass();
      BeanInfo _beanInfo = Introspector.getBeanInfo(_class);
      PropertyDescriptor[] _propertyDescriptors = _beanInfo.getPropertyDescriptors();
      for (final PropertyDescriptor pd : _propertyDescriptors) {
        boolean _and = false;
        Method _readMethod = pd.getReadMethod();
        boolean _notEquals = (!Objects.equal(_readMethod, null));
        if (!_notEquals) {
          _and = false;
        } else {
          String _name = pd.getName();
          boolean _equals = "name".equals(_name);
          _and = (_notEquals && _equals);
        }
        if (_and) {
          Method _readMethod_1 = pd.getReadMethod();
          Object _invoke = _readMethod_1.invoke(item);
          String _string = _invoke==null?(String)null:_invoke.toString();
          name = _string;
        }
      }
      boolean _equals_1 = Objects.equal(name, "");
      if (_equals_1) {
        Class<? extends EObject> _class_1 = item.getClass();
        String _simpleName = _class_1.getSimpleName();
        String _lowerCase = _simpleName.toLowerCase();
        String _plus = (_lowerCase + Integer.valueOf(this.itemIndex));
        name = _plus;
      }
      return name;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public CharSequence beforeChildItem(final boolean vertical, final boolean generateColumn, final int numberOfColumns, final EObject item) {
    StringConcatenation _builder = new StringConcatenation();
    String content = _builder.toString();
    boolean _isCustomizable = this._faceletLayoutImpl.isCustomizable(item);
    if (_isCustomizable) {
      StringConcatenation _builder_1 = new StringConcatenation();
      _builder_1.append("<ui:insert name=\"");
      String _itemName = this.getItemName(item);
      _builder_1.append(_itemName, "");
      _builder_1.append("\">");
      String _plus = (content + _builder_1);
      content = _plus;
    }
    int _plus_1 = (this.itemIndex + 1);
    this.itemIndex = _plus_1;
    boolean _isRenderingInTabPage = this.isRenderingInTabPage(item);
    boolean _not = (!_isRenderingInTabPage);
    if (_not) {
      if (vertical) {
        StringConcatenation _builder_2 = new StringConcatenation();
        {
          EObject _eContainer = item.eContainer();
          boolean _not_1 = (!(_eContainer instanceof SetOfGuiElements));
          if (_not_1) {
            _builder_2.newLineIfNotEmpty();
            _builder_2.append("<p:row>");
            _builder_2.newLine();
          }
        }
        {
          if (generateColumn) {
            _builder_2.append("<p:column colspan=\"");
            _builder_2.append(numberOfColumns, "");
            _builder_2.append("\">");
            _builder_2.newLineIfNotEmpty();
          }
        }
        String _plus_2 = (content + _builder_2);
        content = _plus_2;
      }
    }
    return content;
  }
  
  public CharSequence afterChildItem(final boolean vertical, final boolean generateColumn, final int numberOfColumns, final EObject item) {
    StringConcatenation _builder = new StringConcatenation();
    String content = _builder.toString();
    boolean _isCustomizable = this._faceletLayoutImpl.isCustomizable(item);
    if (_isCustomizable) {
      StringConcatenation _builder_1 = new StringConcatenation();
      _builder_1.append("</ui:insert>");
      String _plus = (content + _builder_1);
      content = _plus;
    }
    boolean _isRenderingInTabPage = this.isRenderingInTabPage(item);
    boolean _not = (!_isRenderingInTabPage);
    if (_not) {
      if (vertical) {
        StringConcatenation _builder_2 = new StringConcatenation();
        {
          if (generateColumn) {
            _builder_2.append("</p:column>");
            _builder_2.newLine();
          }
        }
        {
          EObject _eContainer = item.eContainer();
          boolean _not_1 = (!(_eContainer instanceof SetOfGuiElements));
          if (_not_1) {
            _builder_2.append("</p:row>");
            _builder_2.newLine();
          }
        }
        String _plus_1 = (content + _builder_2);
        content = _plus_1;
      }
    }
    return content;
  }
  
  protected CharSequence _getCompositeLabel(final ReferencedXMAComposite composite) {
    return null;
  }
  
  protected CharSequence _getCompositeLabel(final XmadslComposite composite) {
    LabelText _labelText = composite.getLabelText();
    String _label = _labelText==null?(String)null:_labelText.getLabel();
    return _label;
  }
  
  protected CharSequence _createChildContents(final ReferencedXMAComposite composite) {
    CharSequence _xblockexpression = null;
    {
      Content _content = composite.getContent();
      boolean _equals = Objects.equal(_content, null);
      if (_equals) {
        StringConcatenation _builder = new StringConcatenation();
        return _builder;
      }
      int tabulators = 1;
      boolean _and = false;
      Content _content_1 = composite.getContent();
      ComposeData _composeLayout = _content_1.getComposeLayout();
      boolean _notEquals = (!Objects.equal(_composeLayout, null));
      if (!_notEquals) {
        _and = false;
      } else {
        Content _content_2 = composite.getContent();
        ComposeData _composeLayout_1 = _content_2.getComposeLayout();
        EList<TabulatorPosition> _tabulators = _composeLayout_1.getTabulators();
        boolean _notEquals_1 = (!Objects.equal(_tabulators, null));
        _and = (_notEquals && _notEquals_1);
      }
      if (_and) {
        Content _content_3 = composite.getContent();
        ComposeData _composeLayout_2 = _content_3.getComposeLayout();
        EList<TabulatorPosition> _tabulators_1 = _composeLayout_2.getTabulators();
        int _size = _tabulators_1.size();
        tabulators = _size;
      }
      StringConcatenation _builder_1 = new StringConcatenation();
      boolean _isVerticalPartitioning = composite.isVerticalPartitioning();
      CharSequence _beforeChildContents = this.beforeChildContents(_isVerticalPartitioning, tabulators, null);
      _builder_1.append(_beforeChildContents, "");
      _builder_1.newLineIfNotEmpty();
      {
        Content _content_4 = composite.getContent();
        EList<EObject> _eContents = _content_4.eContents();
        for(final EObject item : _eContents) {
          boolean _isVerticalPartitioning_1 = composite.isVerticalPartitioning();
          CharSequence _beforeChildItem = this.beforeChildItem(_isVerticalPartitioning_1, false, tabulators, item);
          _builder_1.append(_beforeChildItem, "");
          _builder_1.newLineIfNotEmpty();
          CharSequence _createContents = this.createContents(item);
          _builder_1.append(_createContents, "");
          _builder_1.newLineIfNotEmpty();
          boolean _isVerticalPartitioning_2 = composite.isVerticalPartitioning();
          CharSequence _afterChildItem = this.afterChildItem(_isVerticalPartitioning_2, false, tabulators, item);
          _builder_1.append(_afterChildItem, "");
          _builder_1.newLineIfNotEmpty();
        }
      }
      boolean _isVerticalPartitioning_3 = composite.isVerticalPartitioning();
      CharSequence _afterChildContents = this.afterChildContents(_isVerticalPartitioning_3, tabulators);
      _builder_1.append(_afterChildContents, "");
      _builder_1.newLineIfNotEmpty();
      _xblockexpression = (_builder_1);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _createChildContents(final XmadslComposite composite) {
    CharSequence _xblockexpression = null;
    {
      Content _content = composite.getContent();
      boolean _equals = Objects.equal(_content, null);
      if (_equals) {
        StringConcatenation _builder = new StringConcatenation();
        return _builder;
      }
      final Integer numberOfColumns = this._faceletLayoutImpl.getColumns(composite);
      int tabulators = 1;
      int _minus = (-1);
      this.itemIndex = _minus;
      boolean _and = false;
      Content _content_1 = composite.getContent();
      ComposeData _composeLayout = _content_1.getComposeLayout();
      boolean _notEquals = (!Objects.equal(_composeLayout, null));
      if (!_notEquals) {
        _and = false;
      } else {
        Content _content_2 = composite.getContent();
        ComposeData _composeLayout_1 = _content_2.getComposeLayout();
        EList<TabulatorPosition> _tabulators = _composeLayout_1.getTabulators();
        boolean _notEquals_1 = (!Objects.equal(_tabulators, null));
        _and = (_notEquals && _notEquals_1);
      }
      if (_and) {
        Content _content_3 = composite.getContent();
        ComposeData _composeLayout_2 = _content_3.getComposeLayout();
        EList<TabulatorPosition> _tabulators_1 = _composeLayout_2.getTabulators();
        int _size = _tabulators_1.size();
        tabulators = _size;
      }
      StringConcatenation _builder_1 = new StringConcatenation();
      boolean _isVerticalPartitioning = composite.isVerticalPartitioning();
      CharSequence _compositeLabel = this.getCompositeLabel(composite);
      CharSequence _beforeChildContents = this.beforeChildContents(_isVerticalPartitioning, tabulators, _compositeLabel);
      _builder_1.append(_beforeChildContents, "");
      _builder_1.newLineIfNotEmpty();
      {
        Content _content_4 = composite.getContent();
        EList<EObject> _eContents = _content_4.eContents();
        for(final EObject item : _eContents) {
          boolean _or = false;
          boolean _isCustomizeableField = this.isCustomizeableField(item);
          if (_isCustomizeableField) {
            _or = true;
          } else {
            _or = (_isCustomizeableField || (item instanceof SetOfGuiElements));
          }
          final boolean generateColumn = (!_or);
          _builder_1.newLineIfNotEmpty();
          boolean _isVerticalPartitioning_1 = composite.isVerticalPartitioning();
          CharSequence _beforeChildItem = this.beforeChildItem(_isVerticalPartitioning_1, generateColumn, (numberOfColumns).intValue(), item);
          _builder_1.append(_beforeChildItem, "");
          _builder_1.newLineIfNotEmpty();
          CharSequence _createContents = this.createContents(item);
          _builder_1.append(_createContents, "");
          _builder_1.newLineIfNotEmpty();
          boolean _isVerticalPartitioning_2 = composite.isVerticalPartitioning();
          CharSequence _afterChildItem = this.afterChildItem(_isVerticalPartitioning_2, generateColumn, (numberOfColumns).intValue(), item);
          _builder_1.append(_afterChildItem, "");
          _builder_1.newLineIfNotEmpty();
        }
      }
      boolean _isVerticalPartitioning_3 = composite.isVerticalPartitioning();
      CharSequence _afterChildContents = this.afterChildContents(_isVerticalPartitioning_3, tabulators);
      _builder_1.append(_afterChildContents, "");
      _builder_1.newLineIfNotEmpty();
      _xblockexpression = (_builder_1);
    }
    return _xblockexpression;
  }
  
  /**
   * Checks whether the item is a customizeable field
   */
  public boolean isCustomizeableField(final EObject item) {
    return (item instanceof CustomizeableField);
  }
  
  public boolean hasEventMappings(final Command command) {
    boolean _xblockexpression = false;
    {
      EObject _eContainer = command.eContainer();
      if ((_eContainer instanceof Component)) {
        EObject _eContainer_1 = command.eContainer();
        Component commandComponent = ((Component) _eContainer_1);
        EList<Page> _pages = commandComponent.getPages();
        for (final Page page : _pages) {
          boolean _hasEventMappingForCommandInPage = this.hasEventMappingForCommandInPage(command, page);
          if (_hasEventMappingForCommandInPage) {
            return true;
          }
        }
      } else {
        XmadslPage _parentPage = command==null?(XmadslPage)null:this._jsfNames.getParentPage(command);
        boolean _notEquals = (!Objects.equal(_parentPage, null));
        if (_notEquals) {
          XmadslPage _parentPage_1 = this._jsfNames.getParentPage(command);
          return this.hasEventMappingForCommandInPage(command, _parentPage_1);
        }
      }
      _xblockexpression = (false);
    }
    return _xblockexpression;
  }
  
  public boolean hasEventMappingForCommandInPage(final Command command, final Page page) {
    boolean _xblockexpression = false;
    {
      boolean _notEquals = (!Objects.equal(page, null));
      if (_notEquals) {
        boolean _and = false;
        EventMappingList _events = page.getEvents();
        boolean _notEquals_1 = (!Objects.equal(_events, null));
        if (!_notEquals_1) {
          _and = false;
        } else {
          EventMappingList _events_1 = page.getEvents();
          EList<EventMapping> _mapping = _events_1.getMapping();
          boolean _notEquals_2 = (!Objects.equal(_mapping, null));
          _and = (_notEquals_1 && _notEquals_2);
        }
        if (_and) {
          EventMappingList _events_2 = page.getEvents();
          EList<EventMapping> _mapping_1 = _events_2==null?(EList<EventMapping>)null:_events_2.getMapping();
          Iterable<GuiElementEventMapping> _filter = _mapping_1==null?(Iterable<GuiElementEventMapping>)null:Iterables.<GuiElementEventMapping>filter(_mapping_1, GuiElementEventMapping.class);
          for (final GuiElementEventMapping mapping : _filter) {
            EList<EventFunction> _eventFunctions = mapping.getEventFunctions();
            for (final EventFunction function : _eventFunctions) {
              boolean _and_1 = false;
              Command _command = function.getCommand();
              boolean _notEquals_3 = (!Objects.equal(_command, null));
              if (!_notEquals_3) {
                _and_1 = false;
              } else {
                Command _command_1 = function.getCommand();
                boolean _equals = Objects.equal(_command_1, command);
                _and_1 = (_notEquals_3 && _equals);
              }
              if (_and_1) {
                return true;
              }
            }
          }
        }
      }
      _xblockexpression = (false);
    }
    return _xblockexpression;
  }
  
  public boolean hasEventMappings(final IGuiElementWithEvent control, final boolean client) {
    boolean _xblockexpression = false;
    {
      XmadslPage _parentPage = this._jsfNames.getParentPage(control);
      EventMappingList _events = _parentPage.getEvents();
      boolean _equals = Objects.equal(_events, null);
      if (_equals) {
        return false;
      }
      XmadslPage _parentPage_1 = this._jsfNames.getParentPage(control);
      EventMappingList _events_1 = _parentPage_1.getEvents();
      EList<EventMapping> _mapping = _events_1.getMapping();
      Iterable<GuiElementEventMapping> _filter = Iterables.<GuiElementEventMapping>filter(_mapping, GuiElementEventMapping.class);
      for (final GuiElementEventMapping mapping : _filter) {
        EList<EventFunction> _eventFunctions = mapping.getEventFunctions();
        for (final EventFunction function : _eventFunctions) {
          IGuiElementWithEvent _control = mapping.getControl();
          boolean _equals_1 = Objects.equal(control, _control);
          if (_equals_1) {
            boolean _and = false;
            Command _command = function.getCommand();
            boolean _notEquals = (!Objects.equal(_command, null));
            if (!_notEquals) {
              _and = false;
            } else {
              Command _command_1 = function.getCommand();
              boolean _isClientOnly = _command_1.isClientOnly();
              boolean _equals_2 = (_isClientOnly == client);
              _and = (_notEquals && _equals_2);
            }
            if (_and) {
              return true;
            }
            boolean _and_1 = false;
            boolean _and_2 = false;
            PredefinedCommand _predefinedCommand = function.getPredefinedCommand();
            boolean _notEquals_1 = (!Objects.equal(_predefinedCommand, null));
            if (!_notEquals_1) {
              _and_2 = false;
            } else {
              Command _command_2 = function.getCommand();
              boolean _equals_3 = Objects.equal(_command_2, null);
              _and_2 = (_notEquals_1 && _equals_3);
            }
            if (!_and_2) {
              _and_1 = false;
            } else {
              _and_1 = (_and_2 && client);
            }
            if (_and_1) {
              return true;
            }
          }
        }
      }
      _xblockexpression = (false);
    }
    return _xblockexpression;
  }
  
  /**
   * This function creates handlers of button events. It assumes that there is only one remotecommand.
   * All uicommands associated with the button before a remotecommand are mapped to onclick event and
   * all uicommands associated with the button after a remotecommand are mapped to oncomplete event.
   */
  public CharSequence addFunctionCommands(final IGuiElementWithEvent control, final CharSequence listenerAttribute, final CharSequence startAttribute) {
    CharSequence _xblockexpression = null;
    {
      ArrayList<String> _arrayList = new ArrayList<String>();
      final ArrayList<String> calls = _arrayList;
      XmadslPage _parentPage = this._jsfNames.getParentPage(control);
      EventMappingList _events = _parentPage.getEvents();
      boolean _equals = Objects.equal(_events, null);
      if (_equals) {
        return "";
      }
      XmadslPage _parentPage_1 = this._jsfNames.getParentPage(control);
      EventMappingList _events_1 = _parentPage_1.getEvents();
      EList<EventMapping> _mapping = _events_1==null?(EList<EventMapping>)null:_events_1.getMapping();
      Iterable<GuiElementEventMapping> _filter = _mapping==null?(Iterable<GuiElementEventMapping>)null:Iterables.<GuiElementEventMapping>filter(_mapping, GuiElementEventMapping.class);
      for (final GuiElementEventMapping mapping : _filter) {
        {
          boolean remoteCommandAdded = false;
          ArrayList<String> _arrayList_1 = new ArrayList<String>();
          ArrayList<String> beforeCommands = _arrayList_1;
          ArrayList<String> _arrayList_2 = new ArrayList<String>();
          ArrayList<String> afterCommands = _arrayList_2;
          String remoteCommand = "";
          IGuiElementWithEvent _control = mapping.getControl();
          boolean _equals_1 = Objects.equal(control, _control);
          if (_equals_1) {
            EList<EventFunction> _eventFunctions = mapping.getEventFunctions();
            for (final EventFunction function : _eventFunctions) {
              boolean _and = false;
              Command _command = function.getCommand();
              boolean _notEquals = (!Objects.equal(_command, null));
              if (!_notEquals) {
                _and = false;
              } else {
                Command _command_1 = function.getCommand();
                boolean _isClientOnly = _command_1.isClientOnly();
                _and = (_notEquals && _isClientOnly);
              }
              if (_and) {
                boolean _not = (!remoteCommandAdded);
                if (_not) {
                  StringConcatenation _builder = new StringConcatenation();
                  _builder.append("App.");
                  Command _command_2 = function.getCommand();
                  EObject _eContainer = _command_2.eContainer();
                  CharSequence _widgetVar = this._jsfNames.getWidgetVar(_eContainer);
                  _builder.append(_widgetVar, "");
                  _builder.append(".");
                  Command _command_3 = function.getCommand();
                  String _name = _command_3.getName();
                  _builder.append(_name, "");
                  _builder.append("();");
                  beforeCommands.add(_builder.toString());
                } else {
                  StringConcatenation _builder_1 = new StringConcatenation();
                  _builder_1.append("App.");
                  Command _command_4 = function.getCommand();
                  EObject _eContainer_1 = _command_4.eContainer();
                  CharSequence _widgetVar_1 = this._jsfNames.getWidgetVar(_eContainer_1);
                  _builder_1.append(_widgetVar_1, "");
                  _builder_1.append(".");
                  Command _command_5 = function.getCommand();
                  String _name_1 = _command_5.getName();
                  _builder_1.append(_name_1, "");
                  _builder_1.append("();");
                  afterCommands.add(_builder_1.toString());
                }
              } else {
                boolean _and_1 = false;
                Command _command_6 = function.getCommand();
                boolean _notEquals_1 = (!Objects.equal(_command_6, null));
                if (!_notEquals_1) {
                  _and_1 = false;
                } else {
                  Command _command_7 = function.getCommand();
                  boolean _isClientOnly_1 = _command_7.isClientOnly();
                  boolean _not_1 = (!_isClientOnly_1);
                  _and_1 = (_notEquals_1 && _not_1);
                }
                if (_and_1) {
                  StringConcatenation _builder_2 = new StringConcatenation();
                  _builder_2.append(" ");
                  _builder_2.append(listenerAttribute, " ");
                  _builder_2.append("=\"#{");
                  XmadslPage _parentPage_2 = this._jsfNames.getParentPage(control);
                  CharSequence _bean = this._jsfNames.getBean(_parentPage_2);
                  _builder_2.append(_bean, " ");
                  _builder_2.append(".");
                  Command _command_8 = function.getCommand();
                  String _name_2 = _command_8.getName();
                  _builder_2.append(_name_2, " ");
                  _builder_2.append("}\"");
                  remoteCommand = _builder_2.toString();
                  remoteCommandAdded = true;
                } else {
                  PredefinedCommand _predefinedCommand = function.getPredefinedCommand();
                  boolean _notEquals_2 = (!Objects.equal(_predefinedCommand, null));
                  if (_notEquals_2) {
                    boolean _not_2 = (!remoteCommandAdded);
                    if (_not_2) {
                      StringConcatenation _builder_3 = new StringConcatenation();
                      _builder_3.append("App.");
                      XmadslPage _parentPage_3 = this._jsfNames.getParentPage(control);
                      CharSequence _widgetVar_2 = this._jsfNames.getWidgetVar(_parentPage_3);
                      _builder_3.append(_widgetVar_2, "");
                      _builder_3.append(".");
                      PredefinedCommand _predefinedCommand_1 = function.getPredefinedCommand();
                      String _literal = _predefinedCommand_1.getLiteral();
                      _builder_3.append(_literal, "");
                      _builder_3.append("();");
                      beforeCommands.add(_builder_3.toString());
                    } else {
                      StringConcatenation _builder_4 = new StringConcatenation();
                      _builder_4.append("App.");
                      XmadslPage _parentPage_4 = this._jsfNames.getParentPage(control);
                      CharSequence _widgetVar_3 = this._jsfNames.getWidgetVar(_parentPage_4);
                      _builder_4.append(_widgetVar_3, "");
                      _builder_4.append(".");
                      PredefinedCommand _predefinedCommand_2 = function.getPredefinedCommand();
                      String _literal_1 = _predefinedCommand_2.getLiteral();
                      _builder_4.append(_literal_1, "");
                      _builder_4.append("();");
                      afterCommands.add(_builder_4.toString());
                    }
                  }
                }
              }
            }
            int _size = beforeCommands.size();
            boolean _greaterThan = (_size > 0);
            if (_greaterThan) {
              StringConcatenation _builder_5 = new StringConcatenation();
              _builder_5.append(" ");
              _builder_5.append(startAttribute, " ");
              _builder_5.append("=\"");
              calls.add(_builder_5.toString());
              calls.addAll(beforeCommands);
              boolean _not_3 = (!remoteCommandAdded);
              if (_not_3) {
                StringConcatenation _builder_6 = new StringConcatenation();
                _builder_6.append("return false");
                calls.add(_builder_6.toString());
              }
              StringConcatenation _builder_7 = new StringConcatenation();
              _builder_7.append("\"");
              calls.add(_builder_7.toString());
            }
            calls.add(remoteCommand);
            int _size_1 = afterCommands.size();
            boolean _greaterThan_1 = (_size_1 > 0);
            if (_greaterThan_1) {
              StringConcatenation _builder_8 = new StringConcatenation();
              _builder_8.append(" ");
              _builder_8.append("oncomplete=\"");
              calls.add(_builder_8.toString());
              for (final String commandName : afterCommands) {
                calls.addAll(afterCommands);
              }
              StringConcatenation _builder_9 = new StringConcatenation();
              _builder_9.append("\"");
              calls.add(_builder_9.toString());
            }
          }
        }
      }
      StringConcatenation _builder = new StringConcatenation();
      String _join = IterableExtensions.join(calls);
      _builder.append(_join, "");
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  public CharSequence columnValueExpression(final TableColumn column) {
    CharSequence _xifexpression = null;
    FieldFeature _feature = column.getFeature();
    boolean _notEquals = (!Objects.equal(_feature, null));
    if (_notEquals) {
      CharSequence _xifexpression_1 = null;
      boolean _and = false;
      FieldFeature _feature_1 = column.getFeature();
      PresentableFeature _feature_2 = _feature_1.getFeature();
      boolean _notEquals_1 = (!Objects.equal(_feature_2, null));
      if (!_notEquals_1) {
        _and = false;
      } else {
        FieldFeature _feature_3 = column.getFeature();
        PresentableFeature _feature_4 = _feature_3.getFeature();
        String _name = _feature_4.getName();
        boolean _notEquals_2 = (!Objects.equal(_name, null));
        _and = (_notEquals_1 && _notEquals_2);
      }
      if (_and) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("item.");
        FieldFeature _feature_5 = column.getFeature();
        PresentableFeature _feature_6 = _feature_5.getFeature();
        String _name_1 = _feature_6.getName();
        _builder.append(_name_1, "");
        _xifexpression_1 = _builder;
      } else {
        CharSequence _xifexpression_2 = null;
        boolean _and_1 = false;
        boolean _and_2 = false;
        FieldFeature _feature_7 = column.getFeature();
        PresentableFeature _feature_8 = _feature_7.getFeature();
        boolean _notEquals_3 = (!Objects.equal(_feature_8, null));
        if (!_notEquals_3) {
          _and_2 = false;
        } else {
          FieldFeature _feature_9 = column.getFeature();
          PresentableFeature _feature_10 = _feature_9.getFeature();
          Attribute _attribute = _feature_10.getAttribute();
          boolean _notEquals_4 = (!Objects.equal(_attribute, null));
          _and_2 = (_notEquals_3 && _notEquals_4);
        }
        if (!_and_2) {
          _and_1 = false;
        } else {
          FieldFeature _feature_11 = column.getFeature();
          PresentableFeature _feature_12 = _feature_11.getFeature();
          Attribute _attribute_1 = _feature_12.getAttribute();
          String _name_2 = _attribute_1.getName();
          boolean _notEquals_5 = (!Objects.equal(_name_2, null));
          _and_1 = (_and_2 && _notEquals_5);
        }
        if (_and_1) {
          StringConcatenation _builder_1 = new StringConcatenation();
          _builder_1.append("item.");
          FieldFeature _feature_13 = column.getFeature();
          PresentableFeature _feature_14 = _feature_13.getFeature();
          Attribute _attribute_2 = _feature_14.getAttribute();
          String _name_3 = _attribute_2.getName();
          _builder_1.append(_name_3, "");
          _xifexpression_2 = _builder_1;
        }
        _xifexpression_1 = _xifexpression_2;
      }
      _xifexpression = _xifexpression_1;
    }
    return _xifexpression;
  }
  
  public CharSequence fieldValueExpression(final CustomizeableField field) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _and = false;
      FieldFeature _feature = field.getFeature();
      boolean _notEquals = (!Objects.equal(_feature, null));
      if (!_notEquals) {
        _and = false;
      } else {
        FieldFeature _feature_1 = field.getFeature();
        PresentableFeature _attributeHolder = _feature_1.getAttributeHolder();
        boolean _notEquals_1 = (!Objects.equal(_attributeHolder, null));
        _and = (_notEquals && _notEquals_1);
      }
      if (_and) {
        DataObjectVariable _object = field.getObject();
        EObject _eContainer = _object.eContainer();
        CharSequence _referencedBean = this._jsfNames.referencedBean(_eContainer);
        _builder.append(_referencedBean, "");
        _builder.append(".");
        DataObjectVariable _object_1 = field.getObject();
        String _name = _object_1.getName();
        String _faceletName = this._jsfNames.getFaceletName(_name);
        _builder.append(_faceletName, "");
        _builder.append(".");
        FieldFeature _feature_2 = field.getFeature();
        PresentableFeature _attributeHolder_1 = _feature_2.getAttributeHolder();
        String _name_1 = _attributeHolder_1.getName();
        String _faceletName_1 = this._jsfNames.getFaceletName(_name_1);
        _builder.append(_faceletName_1, "");
      }
    }
    return _builder;
  }
  
  public CharSequence toUiCommandPrototype(final Command command) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _or = false;
      boolean _isClientOnly = command.isClientOnly();
      if (_isClientOnly) {
        _or = true;
      } else {
        boolean _hasEventMappings = this.hasEventMappings(command);
        boolean _not = (!_hasEventMappings);
        _or = (_isClientOnly || _not);
      }
      if (_or) {
        String _name = command.getName();
        _builder.append(_name, "");
        _builder.append(" : function() {");
        _builder.newLineIfNotEmpty();
        {
          boolean _isClientOnly_1 = command.isClientOnly();
          if (_isClientOnly_1) {
            {
              boolean _and = false;
              CommandImplementation _implementation = command.getImplementation();
              boolean _notEquals = (!Objects.equal(_implementation, null));
              if (!_notEquals) {
                _and = false;
              } else {
                CommandImplementation _implementation_1 = command.getImplementation();
                EList<Statement> _statements = _implementation_1.getStatements();
                int _size = _statements==null?0:_statements.size();
                boolean _greaterThan = (_size > 0);
                _and = (_notEquals && _greaterThan);
              }
              if (_and) {
                {
                  CommandImplementation _implementation_2 = command.getImplementation();
                  EList<Statement> _statements_1 = _implementation_2.getStatements();
                  for(final Statement statement : _statements_1) {
                    _builder.append("\t");
                    CharSequence _createContents = this.createContents(statement);
                    _builder.append(_createContents, "	");
                    _builder.newLineIfNotEmpty();
                  }
                }
              } else {
                _builder.append("\t");
                _builder.append("alert(\'uicommand \"");
                String _name_1 = command.getName();
                _builder.append(_name_1, "	");
                _builder.append("\" is not implemented. Please define it in custom .xhtml page \');");
                _builder.newLineIfNotEmpty();
              }
            }
          } else {
            CharSequence _widgetVar = this._jsfNames.getWidgetVar(command);
            _builder.append(_widgetVar, "");
            _builder.append("();");
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append("\t    ");
        _builder.append("},");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence toUiCommandPrototypes(final EList<Command> commands) {
    StringConcatenation _builder = new StringConcatenation();
    {
      final Function1<Command,Boolean> _function = new Function1<Command,Boolean>() {
          public Boolean apply(final Command c) {
            String _name = c.getName();
            boolean _equals = _name.equals("init");
            boolean _not = (!_equals);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Command> _filter = IterableExtensions.<Command>filter(commands, _function);
      for(final Command command : _filter) {
        CharSequence _uiCommandPrototype = this.toUiCommandPrototype(command);
        _builder.append(_uiCommandPrototype, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _createContents(final Statement stmt) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Other statement:");
    _builder.append(stmt, "");
    return _builder;
  }
  
  protected CharSequence _createContents(final InvokeStatement stmt) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<PropertyJoin> _propertyJoin = stmt.getPropertyJoin();
      for(final PropertyJoin propertyJoin : _propertyJoin) {
        {
          JoinDirection _joinDirection = propertyJoin.getJoinDirection();
          boolean _equals = Objects.equal(_joinDirection, JoinDirection.IN);
          if (_equals) {
            ObjectProperty _property = propertyJoin.getProperty();
            CharSequence _widgetVar = this._jsfNames.getWidgetVar(_property);
            _builder.append(_widgetVar, "");
            _builder.append(".jq.val(");
            _builder.newLineIfNotEmpty();
            {
              StatementVariable _variable = propertyJoin.getVariable();
              boolean _notEquals = (!Objects.equal(_variable, null));
              if (_notEquals) {
                _builder.append("\t");
                StatementVariable _variable_1 = propertyJoin.getVariable();
                IReferenceableByStatementVariable _reference = _variable_1.getReference();
                CharSequence _widgetVar_1 = this._jsfNames.getWidgetVar(_reference);
                _builder.append(_widgetVar_1, "	");
                _builder.append(".");
                StatementVariable _variable_2 = propertyJoin.getVariable();
                IReferenceableByStatementVariable _reference_1 = _variable_2.getReference();
                CharSequence _clientValue = this.clientValue(_reference_1);
                _builder.append(_clientValue, "	");
                _builder.newLineIfNotEmpty();
              } else {
                String _stringLiteral = propertyJoin.getStringLiteral();
                boolean _notEquals_1 = (!Objects.equal(_stringLiteral, null));
                if (_notEquals_1) {
                  _builder.append("\t");
                  _builder.append("\'");
                  String _stringLiteral_1 = propertyJoin.getStringLiteral();
                  _builder.append(_stringLiteral_1, "	");
                  _builder.append("\'");
                  _builder.newLineIfNotEmpty();
                } else {
                  int _integerLiteral = propertyJoin.getIntegerLiteral();
                  boolean _notEquals_2 = (!Objects.equal(Integer.valueOf(_integerLiteral), null));
                  if (_notEquals_2) {
                    _builder.append("\t");
                    _builder.append("\'");
                    int _integerLiteral_1 = propertyJoin.getIntegerLiteral();
                    _builder.append(_integerLiteral_1, "	");
                    _builder.append("\'");
                    _builder.newLineIfNotEmpty();
                  }
                }
              }
            }
            _builder.append("); ");
            _builder.newLine();
          } else {
            _builder.append("// other direction: ");
            _builder.newLine();
          }
        }
      }
    }
    _builder.append("App.");
    Invokeable _invokeAble = stmt.getInvokeAble();
    CharSequence _widgetVar_2 = this._jsfNames.getWidgetVar(_invokeAble);
    _builder.append(_widgetVar_2, "");
    _builder.append(".invoke();");
    return _builder;
  }
  
  protected CharSequence _clientValue(final Table variable) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("selection[0]");
    return _builder;
  }
  
  protected CharSequence _clientValue(final IReferenceableByStatementVariable variable) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(" ");
    _builder.append("no client value: ");
    _builder.append(variable, " ");
    _builder.append(" ");
    return _builder;
  }
  
  /**
   * TODO: Remove method after introducing i18n internationlization (texts are outsourced to language files!)
   */
  public CharSequence toEncodedHtml(final String seq) {
    return seq.replaceAll("&", "&amp;");
  }
  
  public CharSequence createContents(final EObject composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _createContents((ReferencedXMAComposite)composite);
    } else if (composite instanceof TabFolder) {
      return _createContents((TabFolder)composite);
    } else if (composite instanceof TabPage) {
      return _createContents((TabPage)composite);
    } else if (composite instanceof XmadslComposite) {
      return _createContents((XmadslComposite)composite);
    } else if (composite instanceof Button) {
      return _createContents((Button)composite);
    } else if (composite instanceof CheckBox) {
      return _createContents((CheckBox)composite);
    } else if (composite instanceof Combo) {
      return _createContents((Combo)composite);
    } else if (composite instanceof CustomizeableField) {
      return _createContents((CustomizeableField)composite);
    } else if (composite instanceof Label) {
      return _createContents((Label)composite);
    } else if (composite instanceof ListBox) {
      return _createContents((ListBox)composite);
    } else if (composite instanceof RadioButton) {
      return _createContents((RadioButton)composite);
    } else if (composite instanceof Seperator) {
      return _createContents((Seperator)composite);
    } else if (composite instanceof Table) {
      return _createContents((Table)composite);
    } else if (composite instanceof TableCombo) {
      return _createContents((TableCombo)composite);
    } else if (composite instanceof Text) {
      return _createContents((Text)composite);
    } else if (composite instanceof Tree) {
      return _createContents((Tree)composite);
    } else if (composite instanceof XmadslPage) {
      return _createContents((XmadslPage)composite);
    } else if (composite instanceof AttachmentProperty) {
      return _createContents((AttachmentProperty)composite);
    } else if (composite instanceof FieldReference) {
      return _createContents((FieldReference)composite);
    } else if (composite instanceof InvokeStatement) {
      return _createContents((InvokeStatement)composite);
    } else if (composite instanceof ComposeData) {
      return _createContents((ComposeData)composite);
    } else if (composite instanceof Content) {
      return _createContents((Content)composite);
    } else if (composite instanceof EventMappingList) {
      return _createContents((EventMappingList)composite);
    } else if (composite instanceof LabelText) {
      return _createContents((LabelText)composite);
    } else if (composite instanceof SetOfGuiElements) {
      return _createContents((SetOfGuiElements)composite);
    } else if (composite instanceof Statement) {
      return _createContents((Statement)composite);
    } else if (composite instanceof TableColumn) {
      return _createContents((TableColumn)composite);
    } else if (composite != null) {
      return _createContents(composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
  
  public CharSequence createEventMapping(final IGuiElementWithEvent table, final GuiElementEventMapping mapping) {
    if (table instanceof Table) {
      return _createEventMapping((Table)table, mapping);
    } else if (table != null) {
      return _createEventMapping(table, mapping);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(table, mapping).toString());
    }
  }
  
  public CharSequence getCompositeLabel(final IComposite composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _getCompositeLabel((ReferencedXMAComposite)composite);
    } else if (composite instanceof XmadslComposite) {
      return _getCompositeLabel((XmadslComposite)composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
  
  public CharSequence createChildContents(final IComposite composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _createChildContents((ReferencedXMAComposite)composite);
    } else if (composite instanceof XmadslComposite) {
      return _createChildContents((XmadslComposite)composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
  
  public CharSequence clientValue(final IReferenceableByStatementVariable variable) {
    if (variable instanceof Table) {
      return _clientValue((Table)variable);
    } else if (variable != null) {
      return _clientValue(variable);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(variable).toString());
    }
  }
}
