package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import java.util.Arrays;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.openxma.dsl.core.model.Style;
import org.openxma.dsl.generator.helper.GeneratorConfigurationExtension;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.IComposite;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.LayoutDataProperty;
import org.openxma.dsl.pom.model.PredefinedWidgetStyle;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.StyleProperty;
import org.openxma.dsl.pom.model.StylePropertyOther;
import org.openxma.dsl.pom.model.StylePropertyWidgetStyle;
import org.openxma.dsl.pom.model.StyleSpecification;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.XmadslComposite;

@SuppressWarnings("all")
public class FaceletLayoutImpl {
  protected boolean _hasVerticalPartitioning(final XmadslComposite composite) {
    return composite.isVerticalPartitioning();
  }
  
  protected boolean _hasVerticalPartitioning(final ReferencedXMAComposite composite) {
    return composite.isVerticalPartitioning();
  }
  
  /**
   * This methods returns the number of columns a PanelGrid should have.
   * 
   * In case of vertical panel, PanelGrid should have 3 columns if there is a customizeable field and 1 if there is no customizeable field
   * In case of horizontal panel, PanelGrid should have 3 columns for each customizeable field plus one column of every other field
   */
  public Integer getColumns(final XmadslComposite composite) {
    int columns = 0;
    boolean _isHorizontalPartitioning = composite.isHorizontalPartitioning();
    if (_isHorizontalPartitioning) {
      boolean _and = false;
      Content _content = composite.getContent();
      boolean _notEquals = (!Objects.equal(_content, null));
      if (!_notEquals) {
        _and = false;
      } else {
        Content _content_1 = composite.getContent();
        EList<EObject> _eContents = _content_1.eContents();
        boolean _notEquals_1 = (!Objects.equal(_eContents, null));
        _and = (_notEquals && _notEquals_1);
      }
      if (_and) {
        Content _content_2 = composite.getContent();
        EList<EObject> _eContents_1 = _content_2.eContents();
        for (final EObject item : _eContents_1) {
          Integer _numberOfColumnsForItem = this.numberOfColumnsForItem(item);
          int _plus = (columns + (_numberOfColumnsForItem).intValue());
          columns = _plus;
        }
      }
    } else {
      boolean _isVerticalPartitioning = composite.isVerticalPartitioning();
      if (_isVerticalPartitioning) {
        boolean _and_1 = false;
        boolean _and_2 = false;
        boolean _notEquals_2 = (!Objects.equal(composite, null));
        if (!_notEquals_2) {
          _and_2 = false;
        } else {
          Content _content_3 = composite.getContent();
          boolean _notEquals_3 = (!Objects.equal(_content_3, null));
          _and_2 = (_notEquals_2 && _notEquals_3);
        }
        if (!_and_2) {
          _and_1 = false;
        } else {
          Content _content_4 = composite.getContent();
          EList<EObject> _eContents_2 = _content_4.eContents();
          boolean _notEquals_4 = (!Objects.equal(_eContents_2, null));
          _and_1 = (_and_2 && _notEquals_4);
        }
        if (_and_1) {
          Content _content_5 = composite.getContent();
          EList<EObject> _eContents_3 = _content_5.eContents();
          for (final EObject item_1 : _eContents_3) {
            {
              Integer numberOfColumnsForItem = this.numberOfColumnsForItem(item_1);
              boolean _lessThan = (columns < (numberOfColumnsForItem).intValue());
              if (_lessThan) {
                columns = (numberOfColumnsForItem).intValue();
              }
            }
          }
        }
      }
    }
    return Integer.valueOf(columns);
  }
  
  public Integer numberOfColumnsForItem(final EObject item) {
    int _switchResult = (int) 0;
    boolean _matched = false;
    if (!_matched) {
      if (item instanceof CustomizeableField) {
        final CustomizeableField _customizeableField = (CustomizeableField)item;
        _matched=true;
        int _xifexpression = (int) 0;
        boolean _useTooltipForValidationMessage = GeneratorConfigurationExtension.useTooltipForValidationMessage();
        if (_useTooltipForValidationMessage) {
          _xifexpression = 2;
        } else {
          _xifexpression = 3;
        }
        _switchResult = _xifexpression;
      }
    }
    if (!_matched) {
      if (item instanceof SetOfGuiElements) {
        final SetOfGuiElements _setOfGuiElements = (SetOfGuiElements)item;
        _matched=true;
        Integer _calculateNumberOfColumns = this.calculateNumberOfColumns(_setOfGuiElements);
        _switchResult = _calculateNumberOfColumns;
      }
    }
    if (!_matched) {
      _switchResult = 1;
    }
    return Integer.valueOf(_switchResult);
  }
  
  public Integer calculateNumberOfColumns(final EObject item) {
    int _xblockexpression = (int) 0;
    {
      int columns = 0;
      EList<EObject> _eContents = item.eContents();
      for (final EObject child : _eContents) {
        Integer _numberOfColumnsForItem = this.numberOfColumnsForItem(child);
        int _plus = (columns + (_numberOfColumnsForItem).intValue());
        columns = _plus;
      }
      _xblockexpression = (columns);
    }
    return Integer.valueOf(_xblockexpression);
  }
  
  /**
   * This method returns the number of customizeable fields in the composite.
   */
  public Integer numberOfSetofGuiElements(final XmadslComposite composite) {
    int _xifexpression = (int) 0;
    boolean _and = false;
    boolean _and_1 = false;
    boolean _notEquals = (!Objects.equal(composite, null));
    if (!_notEquals) {
      _and_1 = false;
    } else {
      Content _content = composite.getContent();
      boolean _notEquals_1 = (!Objects.equal(_content, null));
      _and_1 = (_notEquals && _notEquals_1);
    }
    if (!_and_1) {
      _and = false;
    } else {
      Content _content_1 = composite.getContent();
      EList<EObject> _eContents = _content_1.eContents();
      boolean _notEquals_2 = (!Objects.equal(_eContents, null));
      _and = (_and_1 && _notEquals_2);
    }
    if (_and) {
      Content _content_2 = composite.getContent();
      EList<EObject> _eContents_1 = _content_2.eContents();
      final Function1<EObject,Boolean> _function = new Function1<EObject,Boolean>() {
          public Boolean apply(final EObject item) {
            boolean _xifexpression = false;
            EList<EObject> _eContents = item.eContents();
            boolean _notEquals = (!Objects.equal(_eContents, null));
            if (_notEquals) {
              EList<EObject> _eContents_1 = item.eContents();
              Iterable<SetOfGuiElements> _filter = Iterables.<SetOfGuiElements>filter(_eContents_1, SetOfGuiElements.class);
              int _size = IterableExtensions.size(_filter);
              boolean _greaterThan = (_size > 0);
              _xifexpression = _greaterThan;
            } else {
              _xifexpression = false;
            }
            return Boolean.valueOf(_xifexpression);
          }
        };
      Iterable<EObject> _filter = IterableExtensions.<EObject>filter(_eContents_1, _function);
      int _size = IterableExtensions.size(_filter);
      _xifexpression = _size;
    } else {
      _xifexpression = 0;
    }
    return Integer.valueOf(_xifexpression);
  }
  
  /**
   * This method returns the number of customizeable fields in the composite.
   */
  public Integer numberOfCustomizableFields(final XmadslComposite composite) {
    int _xifexpression = (int) 0;
    boolean _and = false;
    boolean _and_1 = false;
    boolean _notEquals = (!Objects.equal(composite, null));
    if (!_notEquals) {
      _and_1 = false;
    } else {
      Content _content = composite.getContent();
      boolean _notEquals_1 = (!Objects.equal(_content, null));
      _and_1 = (_notEquals && _notEquals_1);
    }
    if (!_and_1) {
      _and = false;
    } else {
      Content _content_1 = composite.getContent();
      EList<EObject> _eContents = _content_1.eContents();
      boolean _notEquals_2 = (!Objects.equal(_eContents, null));
      _and = (_and_1 && _notEquals_2);
    }
    if (_and) {
      Content _content_2 = composite.getContent();
      EList<EObject> _eContents_1 = _content_2.eContents();
      final Function1<EObject,Boolean> _function = new Function1<EObject,Boolean>() {
          public Boolean apply(final EObject item) {
            boolean _xifexpression = false;
            EList<EObject> _eContents = item.eContents();
            boolean _notEquals = (!Objects.equal(_eContents, null));
            if (_notEquals) {
              EList<EObject> _eContents_1 = item.eContents();
              Iterable<CustomizeableField> _filter = Iterables.<CustomizeableField>filter(_eContents_1, CustomizeableField.class);
              int _size = IterableExtensions.size(_filter);
              boolean _greaterThan = (_size > 0);
              _xifexpression = _greaterThan;
            } else {
              _xifexpression = false;
            }
            return Boolean.valueOf(_xifexpression);
          }
        };
      Iterable<EObject> _filter = IterableExtensions.<EObject>filter(_eContents_1, _function);
      int _size = IterableExtensions.size(_filter);
      _xifexpression = _size;
    } else {
      _xifexpression = 0;
    }
    return Integer.valueOf(_xifexpression);
  }
  
  public boolean isCustomizable(final EObject object) {
    boolean customizable = false;
    if ((object instanceof TabPage)) {
      StyleProperty styleProperty = ((TabPage) object).getStyleProperty();
      boolean _notEquals = (!Objects.equal(styleProperty, null));
      if (_notEquals) {
        EList<Style> _styles = styleProperty.getStyles();
        final Function1<Style,Boolean> _function = new Function1<Style,Boolean>() {
            public Boolean apply(final Style styleSpec) {
              boolean _isCustomizableStyle = FaceletLayoutImpl.this.isCustomizableStyle(((StyleSpecification) styleSpec));
              return Boolean.valueOf(_isCustomizableStyle);
            }
          };
        boolean _exists = _styles==null?false:IterableExtensions.<Style>exists(_styles, _function);
        customizable = _exists;
      }
    } else {
      if ((object instanceof IElementWithLayoutData)) {
        EList<LayoutDataProperty> _layoutProperties = ((IElementWithLayoutData) object).getLayoutProperties();
        Iterable<StyleProperty> _filter = _layoutProperties==null?(Iterable<StyleProperty>)null:Iterables.<StyleProperty>filter(_layoutProperties, StyleProperty.class);
        final Function1<StyleProperty,Boolean> _function_1 = new Function1<StyleProperty,Boolean>() {
            public Boolean apply(final StyleProperty layoutProperty) {
              EList<Style> _styles = ((StyleProperty) layoutProperty).getStyles();
              final Function1<Style,Boolean> _function = new Function1<Style,Boolean>() {
                  public Boolean apply(final Style styleSpec) {
                    boolean _isCustomizableStyle = FaceletLayoutImpl.this.isCustomizableStyle(((StyleSpecification) styleSpec));
                    return Boolean.valueOf(_isCustomizableStyle);
                  }
                };
              boolean _exists = IterableExtensions.<Style>exists(_styles, _function);
              return Boolean.valueOf(_exists);
            }
          };
        boolean _exists_1 = IterableExtensions.<StyleProperty>exists(_filter, _function_1);
        customizable = _exists_1;
      }
    }
    return customizable;
  }
  
  public boolean isCustomizableStyle(final StyleSpecification styleSpecification) {
    EList<StyleSpecificationProperty> _styleProperty = styleSpecification.getStyleProperty();
    Iterable<StylePropertyOther> _filter = Iterables.<StylePropertyOther>filter(_styleProperty, StylePropertyOther.class);
    final Function1<StylePropertyOther,Boolean> _function = new Function1<StylePropertyOther,Boolean>() {
        public Boolean apply(final StylePropertyOther property) {
          String _customization = ((StylePropertyOther) property)==null?(String)null:((StylePropertyOther) property).getCustomization();
          boolean _equals = _customization==null?false:_customization.equals("facelet");
          return Boolean.valueOf(_equals);
        }
      };
    boolean _exists = IterableExtensions.<StylePropertyOther>exists(_filter, _function);
    return _exists;
  }
  
  public CharSequence buttonStyle(final Button b) {
    CharSequence _xblockexpression = null;
    {
      final CharSequence style = this.getStylePropertyWidgetStyle(b);
      boolean _notEquals = (!Objects.equal(style, null));
      if (_notEquals) {
        StringConcatenation _builder = new StringConcatenation();
        _builder.append("styleClass=\"");
        _builder.append(style, "");
        _builder.append("\"");
        return _builder;
      }
      StringConcatenation _builder_1 = new StringConcatenation();
      _xblockexpression = (_builder_1);
    }
    return _xblockexpression;
  }
  
  public CharSequence getStylePropertyWidgetProp(final StyleSpecification styleSpec) {
    EList<StyleSpecificationProperty> _styleProperty = styleSpec.getStyleProperty();
    Iterable<StylePropertyWidgetStyle> _filter = Iterables.<StylePropertyWidgetStyle>filter(_styleProperty, StylePropertyWidgetStyle.class);
    StylePropertyWidgetStyle _head = IterableExtensions.<StylePropertyWidgetStyle>head(_filter);
    PredefinedWidgetStyle _predefinedWidgetStyle = _head==null?(PredefinedWidgetStyle)null:_head.getPredefinedWidgetStyle();
    String _literal = _predefinedWidgetStyle==null?(String)null:_predefinedWidgetStyle.getLiteral();
    return _literal;
  }
  
  public CharSequence getStylePropertyWidgetStyle(final Button b) {
    CharSequence _xblockexpression = null;
    {
      EList<LayoutDataProperty> _layoutProperties = b.getLayoutProperties();
      Iterable<StyleProperty> _filter = _layoutProperties==null?(Iterable<StyleProperty>)null:Iterables.<StyleProperty>filter(_layoutProperties, StyleProperty.class);
      for (final StyleProperty layoutProp : _filter) {
        EList<Style> _styles = ((StyleProperty) layoutProp).getStyles();
        Iterable<StyleSpecification> _filter_1 = Iterables.<StyleSpecification>filter(_styles, StyleSpecification.class);
        for (final StyleSpecification styleSpec : _filter_1) {
          {
            final CharSequence p = this.getStylePropertyWidgetProp(styleSpec);
            boolean _notEquals = (!Objects.equal(p, null));
            if (_notEquals) {
              return p;
            }
          }
        }
      }
      _xblockexpression = (null);
    }
    return _xblockexpression;
  }
  
  public boolean hasVerticalPartitioning(final IComposite composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _hasVerticalPartitioning((ReferencedXMAComposite)composite);
    } else if (composite instanceof XmadslComposite) {
      return _hasVerticalPartitioning((XmadslComposite)composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
}
