package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import java.util.Arrays;
import java.util.List;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.util.SimpleAttributeResolver;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.common.Strings;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.Function;
import org.openxma.dsl.core.model.IDocumentable;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.AttributeGroup;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.Dao;
import org.openxma.dsl.dom.model.DaoOperation;
import org.openxma.dsl.dom.model.DataBaseConstraint;
import org.openxma.dsl.dom.model.DataBaseConstraintType;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.DelegateOperation;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.FeatureReference;
import org.openxma.dsl.dom.model.Operation;
import org.openxma.dsl.dom.model.Parameter;
import org.openxma.dsl.dom.model.Property;
import org.openxma.dsl.dom.model.QueryOperation;
import org.openxma.dsl.dom.model.QueryParameter;
import org.openxma.dsl.dom.model.ValueObject;
import org.openxma.dsl.generator.FeatureExtensions;
import org.openxma.dsl.generator.helper.EntityExtension;
import org.openxma.dsl.generator.helper.JavaUtilExtension;
import org.openxma.dsl.generator.helper.SimpleTypeExtension;
import org.openxma.dsl.generator.impl.Names;

@SuppressWarnings("all")
public class Features {
  @Inject
  private Names _names;
  
  @Inject
  private FeatureExtensions _featureExtensions;
  
  public String getResolvedName(final Attribute it) {
    String _xifexpression = null;
    String _fQNProperty = this.getFQNProperty(it);
    boolean _notEquals = (!Objects.equal(_fQNProperty, null));
    if (_notEquals) {
      String _fQNProperty_1 = this.getFQNProperty(it);
      String _string = _fQNProperty_1.toString();
      String _firstLower = StringExtensions.toFirstLower(_string);
      String _name = it.getName();
      String _firstUpper = StringExtensions.toFirstUpper(_name);
      String _plus = (_firstLower + _firstUpper);
      _xifexpression = _plus;
    } else {
      String _name_1 = it.getName();
      _xifexpression = _name_1;
    }
    return _xifexpression;
  }
  
  public String getFQNProperty(final Attribute it) {
    Object _property = EObjectPropertiesAdapter.getProperty(it, "fqn");
    String _string = _property==null?(String)null:_property.toString();
    return _string;
  }
  
  protected String _toType(final Attribute it) {
    String _xblockexpression = null;
    {
      DataTypeAndTypeParameter _type = it.getType();
      final String qualifiedName = this._names.getQualifiedName(_type);
      String _xifexpression = null;
      boolean _isMany = it.isMany();
      if (_isMany) {
        String _xifexpression_1 = null;
        EObject _eContainer = it.eContainer();
        if ((_eContainer instanceof Entity)) {
          String _addImport = this._names.addImport(it, "java.util.Set");
          String _plus = (_addImport + "<");
          String _plus_1 = (_plus + qualifiedName);
          String _plus_2 = (_plus_1 + ">");
          _xifexpression_1 = _plus_2;
        } else {
          String _addImport_1 = this._names.addImport(it, "java.util.List");
          String _plus_3 = (_addImport_1 + "<");
          String _plus_4 = (_plus_3 + qualifiedName);
          String _plus_5 = (_plus_4 + ">");
          _xifexpression_1 = _plus_5;
        }
        _xifexpression = _xifexpression_1;
      } else {
        _xifexpression = qualifiedName;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  protected String _toType(final Operation it) {
    String _xifexpression = null;
    Type _type = it.getType();
    boolean _equals = Objects.equal(_type, null);
    if (_equals) {
      return "void";
    } else {
      Type _type_1 = it.getType();
      boolean _isMany = it.isMany();
      String _type_2 = this.toType(_type_1, _isMany);
      _xifexpression = _type_2;
    }
    return _xifexpression;
  }
  
  protected String _toType(final DelegateOperation it) {
    String _xifexpression = null;
    DaoOperation _operation = it.getOperation();
    Type _type = _operation.getType();
    boolean _equals = Objects.equal(_type, null);
    if (_equals) {
      return "void";
    } else {
      String _xifexpression_1 = null;
      DataView _view = it.getView();
      boolean _notEquals = (!Objects.equal(_view, null));
      if (_notEquals) {
        DataView _view_1 = it.getView();
        DaoOperation _operation_1 = it.getOperation();
        boolean _isMany = _operation_1.isMany();
        String _type_1 = this.toType(_view_1, _isMany);
        _xifexpression_1 = _type_1;
      } else {
        DaoOperation _operation_2 = it.getOperation();
        Type _type_2 = _operation_2.getType();
        DaoOperation _operation_3 = it.getOperation();
        boolean _isMany_1 = _operation_3.isMany();
        String _type_3 = this.toType(_type_2, _isMany_1);
        _xifexpression_1 = _type_3;
      }
      _xifexpression = _xifexpression_1;
    }
    return _xifexpression;
  }
  
  protected String _toType(final QueryParameter it) {
    Attribute _attribute = it.getAttribute();
    Type _dataType = _attribute.getDataType();
    Attribute _attribute_1 = it.getAttribute();
    boolean _isMany = _attribute_1.isMany();
    String _type = this.toType(_dataType, _isMany);
    return _type;
  }
  
  protected String _toType(final Parameter it) {
    Type _type = it.getType();
    boolean _isMany = it.isMany();
    return this.toType(_type, _isMany);
  }
  
  protected String _toType(final DaoOperation it) {
    Type _type = it.getType();
    boolean _equals = Objects.equal(_type, null);
    if (_equals) {
      return "void";
    }
    Type _type_1 = it.getType();
    boolean _isMany = it.isMany();
    return this.toType(_type_1, _isMany);
  }
  
  public String toType(final Type it, final boolean isCollectionType) {
    String _xblockexpression = null;
    {
      final String qualifiedName = this._names.getQualifiedName(it);
      String _xifexpression = null;
      if (isCollectionType) {
        String _addImport = this._names.addImport(it, "java.util.Collection");
        String _plus = (_addImport + "<");
        String _plus_1 = (_plus + qualifiedName);
        String _plus_2 = (_plus_1 + ">");
        _xifexpression = _plus_2;
      } else {
        _xifexpression = qualifiedName;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  public String getAddMethodName(final Attribute it) {
    String _name = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    String _plus = ("add" + _firstUpper);
    return _plus;
  }
  
  public String getHasMethodName(final Attribute it) {
    String _name = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    String _plus = ("has" + _firstUpper);
    return _plus;
  }
  
  public String getRemoveMethodName(final Attribute it) {
    String _name = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    String _plus = ("remove" + _firstUpper);
    return _plus;
  }
  
  public String getRemoveAllMethodName(final Attribute it) {
    String _name = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    String _plus = ("removeAll" + _firstUpper);
    return _plus;
  }
  
  protected String _getAccessorMethodName(final FeatureReference it) {
    Attribute _attribute = it.getAttribute();
    String _accessorMethodName = this.getAccessorMethodName(_attribute);
    return _accessorMethodName;
  }
  
  protected String _getAccessorMethodName(final EObject it) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Attribute) {
        final Attribute _attribute = (Attribute)it;
        _matched=true;
        String _name = _attribute.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        String _plus = ("get" + _firstUpper);
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (it instanceof AttributeGroup) {
        final AttributeGroup _attributeGroup = (AttributeGroup)it;
        _matched=true;
        String _name = _attributeGroup.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        String _plus = ("get" + _firstUpper);
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      String _xblockexpression = null;
      {
        String name = SimpleAttributeResolver.NAME_RESOLVER.apply(it);
        String _xifexpression = null;
        boolean _isBooleanTyped = this.isBooleanTyped(it);
        if (_isBooleanTyped) {
          String _firstUpper = StringExtensions.toFirstUpper(name);
          String _plus = ("is" + _firstUpper);
          _xifexpression = _plus;
        } else {
          String _firstUpper_1 = StringExtensions.toFirstUpper(name);
          String _plus_1 = ("get" + _firstUpper_1);
          _xifexpression = _plus_1;
        }
        _xblockexpression = (_xifexpression);
      }
      _switchResult = _xblockexpression;
    }
    return _switchResult;
  }
  
  private boolean isBooleanTyped(final EObject it) {
    boolean _switchResult = false;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Property) {
        final Property _property = (Property)it;
        _matched=true;
        SimpleType _type = _property.getType();
        boolean _isBooleanTyped = this.isBooleanTyped(_type);
        _switchResult = _isBooleanTyped;
      }
    }
    if (!_matched) {
      if (it instanceof Function) {
        final Function _function = (Function)it;
        _matched=true;
        SimpleType _resultType = _function.getResultType();
        boolean _isBooleanTyped = this.isBooleanTyped(_resultType);
        _switchResult = _isBooleanTyped;
      }
    }
    if (!_matched) {
      if (it instanceof Attribute) {
        final Attribute _attribute = (Attribute)it;
        _matched=true;
        Type _dataType = _attribute.getDataType();
        boolean _isBooleanTyped = this.isBooleanTyped(_dataType);
        _switchResult = _isBooleanTyped;
      }
    }
    if (!_matched) {
      if (it instanceof SimpleType) {
        final SimpleType _simpleType = (SimpleType)it;
        _matched=true;
        String _javaType = SimpleTypeExtension.getJavaType(_simpleType);
        boolean _equalsIgnoreCase = "boolean".equalsIgnoreCase(_javaType);
        _switchResult = _equalsIgnoreCase;
      }
    }
    if (!_matched) {
      _switchResult = false;
    }
    return _switchResult;
  }
  
  public String getMethodName(final DataBaseConstraint it) {
    String _switchResult = null;
    DataBaseConstraintType _type = it.getType();
    final DataBaseConstraintType getType = _type;
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(getType,DataBaseConstraintType.NATURAL)) {
        _matched=true;
        String _name = it.getName();
        String _plus = ("loadBy" + _name);
        _switchResult = _plus;
      }
    }
    if (!_matched) {
      if (Objects.equal(getType,DataBaseConstraintType.UNIQUE)) {
        _matched=true;
        String _name_1 = it.getName();
        String _plus_1 = ("loadBy" + _name_1);
        _switchResult = _plus_1;
      }
    }
    if (!_matched) {
      if (Objects.equal(getType,DataBaseConstraintType.INDEX)) {
        _matched=true;
        String _name_2 = it.getName();
        String _plus_2 = ("findBy" + _name_2);
        _switchResult = _plus_2;
      }
    }
    if (!_matched) {
      _switchResult = "NOT IMPLEMENTED";
    }
    return _switchResult;
  }
  
  public String getMutatorMethodName(final Attribute it) {
    String _name = it.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    String _plus = ("set" + _firstUpper);
    return _plus;
  }
  
  public boolean isIdOrVersionAttribute(final Attribute it) {
    boolean _or = false;
    boolean _isVersion = it.isVersion();
    if (_isVersion) {
      _or = true;
    } else {
      boolean _isIdentifier = it.isIdentifier();
      _or = (_isVersion || _isIdentifier);
    }
    return _or;
  }
  
  public CharSequence generateAttributes(final ComplexType it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Attribute> _attributes = it.getAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            boolean _isDerived = it.isDerived();
            boolean _not = (!_isDerived);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_attributes, _function);
      for(final Attribute attribute : _filter) {
        CharSequence _field = this.toField(attribute);
        _builder.append(_field, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toField(final Attribute it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _documentationIndented = this.getDocumentationIndented(it);
    _builder.append(_documentationIndented, "");
    _builder.newLineIfNotEmpty();
    CharSequence _fieldExtension = this._featureExtensions.toFieldExtension(it);
    _builder.append(_fieldExtension, "");
    _builder.newLineIfNotEmpty();
    _builder.append("protected ");
    String _type = this.toType(it);
    _builder.append(_type, "");
    _builder.append(" ");
    String _name = it.getName();
    _builder.append(_name, "");
    {
      DataTypeAndTypeParameter _type_1 = it.getType();
      Type _dataType = _type_1.getDataType();
      if ((_dataType instanceof ValueObject)) {
        _builder.append(" = new ");
        {
          boolean _isMany = it.isMany();
          if (_isMany) {
            String _addImport = this._names.addImport(it, "java.util.ArrayList");
            _builder.append(_addImport, "");
            _builder.append("<");
            DataTypeAndTypeParameter _type_2 = it.getType();
            String _qualifiedName = this._names.getQualifiedName(_type_2);
            _builder.append(_qualifiedName, "");
            _builder.append(">");
          } else {
            DataTypeAndTypeParameter _type_3 = it.getType();
            String _qualifiedName_1 = this._names.getQualifiedName(_type_3);
            _builder.append(_qualifiedName_1, "");
          }
        }
        _builder.append("()");
      }
    }
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _generateProperties(final DataView it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Attribute> _attributes = it.getAttributes();
      for(final Attribute attribute : _attributes) {
        _builder.newLine();
        {
          boolean _isDerived = attribute.isDerived();
          boolean _not = (!_isDerived);
          if (_not) {
            _builder.append("public ");
            {
              boolean _isMany = attribute.isMany();
              if (_isMany) {
                _builder.append("List<");
                DataTypeAndTypeParameter _type = attribute.getType();
                String _qualifiedName = this._names.getQualifiedName(_type);
                _builder.append(_qualifiedName, "");
                _builder.append(">");
              } else {
                DataTypeAndTypeParameter _type_1 = attribute.getType();
                String _qualifiedName_1 = this._names.getQualifiedName(_type_1);
                _builder.append(_qualifiedName_1, "");
              }
            }
            _builder.append(" ");
            String _accessorMethodName = this.getAccessorMethodName(attribute);
            _builder.append(_accessorMethodName, "");
            _builder.append("() {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("return this.");
            String _name = attribute.getName();
            _builder.append(_name, "	");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
            _builder.append("}");
            _builder.newLine();
            _builder.newLine();
            _builder.append("public void ");
            String _mutatorMethodName = this.getMutatorMethodName(attribute);
            _builder.append(_mutatorMethodName, "");
            _builder.append("(");
            {
              boolean _isMany_1 = attribute.isMany();
              if (_isMany_1) {
                String _addImport = this._names.addImport(it, "java.util.List");
                _builder.append(_addImport, "");
                _builder.append("<");
                DataTypeAndTypeParameter _type_2 = attribute.getType();
                String _qualifiedName_2 = this._names.getQualifiedName(_type_2);
                _builder.append(_qualifiedName_2, "");
                _builder.append(">");
              } else {
                DataTypeAndTypeParameter _type_3 = attribute.getType();
                String _qualifiedName_3 = this._names.getQualifiedName(_type_3);
                _builder.append(_qualifiedName_3, "");
              }
            }
            _builder.append(" ");
            String _name_1 = attribute.getName();
            _builder.append(_name_1, "");
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("this.");
            String _name_2 = attribute.getName();
            _builder.append(_name_2, "	");
            _builder.append("=");
            String _name_3 = attribute.getName();
            _builder.append(_name_3, "	");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
            _builder.append("}");
            _builder.newLine();
            {
              boolean _isMany_2 = attribute.isMany();
              if (_isMany_2) {
                _builder.newLine();
                _builder.append("public void ");
                String _addMethodName = this.getAddMethodName(attribute);
                _builder.append(_addMethodName, "");
                _builder.append("(");
                DataTypeAndTypeParameter _type_4 = attribute.getType();
                String _qualifiedName_4 = this._names.getQualifiedName(_type_4);
                _builder.append(_qualifiedName_4, "");
                _builder.append(" ");
                String _variableName = this._names.toVariableName(attribute);
                _builder.append(_variableName, "");
                _builder.append(") {");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("if (");
                String _variableName_1 = this._names.toVariableName(attribute);
                _builder.append(_variableName_1, "	");
                _builder.append(" == null) {");
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t");
                _builder.append("throw new IllegalArgumentException(\"parameter \'");
                String _variableName_2 = this._names.toVariableName(attribute);
                _builder.append(_variableName_2, "		");
                _builder.append("\' must not be null\");");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("}");
                _builder.newLine();
                _builder.append("\t");
                _builder.append("if (");
                String _accessorMethodName_1 = this.getAccessorMethodName(attribute);
                _builder.append(_accessorMethodName_1, "	");
                _builder.append("() == null) {");
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t");
                _builder.append("this.");
                String _variableName_3 = this._names.toVariableName(attribute);
                _builder.append(_variableName_3, "		");
                _builder.append(" = new ");
                String _addImport_1 = this._names.addImport(it, "java.util.ArrayList");
                _builder.append(_addImport_1, "		");
                _builder.append("<");
                DataTypeAndTypeParameter _type_5 = attribute.getType();
                String _qualifiedName_5 = this._names.getQualifiedName(_type_5);
                _builder.append(_qualifiedName_5, "		");
                _builder.append(">();");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                _builder.append("}");
                _builder.newLine();
                _builder.append("\t");
                _builder.append("this.");
                String _variableName_4 = this._names.toVariableName(attribute);
                _builder.append(_variableName_4, "	");
                _builder.append(".add(");
                String _variableName_5 = this._names.toVariableName(attribute);
                _builder.append(_variableName_5, "	");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
                _builder.append("}");
                _builder.newLine();
              }
            }
          } else {
            _builder.append("public abstract ");
            {
              boolean _isMany_3 = attribute.isMany();
              if (_isMany_3) {
                _builder.append("List<");
                DataTypeAndTypeParameter _type_6 = attribute.getType();
                String _qualifiedName_6 = this._names.getQualifiedName(_type_6);
                _builder.append(_qualifiedName_6, "");
                _builder.append(">");
              } else {
                DataTypeAndTypeParameter _type_7 = attribute.getType();
                String _qualifiedName_7 = this._names.getQualifiedName(_type_7);
                _builder.append(_qualifiedName_7, "");
              }
            }
            _builder.append(" ");
            String _accessorMethodName_2 = this.getAccessorMethodName(attribute);
            _builder.append(_accessorMethodName_2, "");
            _builder.append("();");
            _builder.newLineIfNotEmpty();
            {
              boolean _isReadOnly = attribute.isReadOnly();
              boolean _not_1 = (!_isReadOnly);
              if (_not_1) {
                _builder.append("public abstract void ");
                String _mutatorMethodName_1 = this.getMutatorMethodName(attribute);
                _builder.append(_mutatorMethodName_1, "");
                _builder.append("(");
                {
                  boolean _isMany_4 = attribute.isMany();
                  if (_isMany_4) {
                    String _addImport_2 = this._names.addImport(it, "java.util.List");
                    _builder.append(_addImport_2, "");
                    _builder.append("<");
                    DataTypeAndTypeParameter _type_8 = attribute.getType();
                    String _qualifiedName_8 = this._names.getQualifiedName(_type_8);
                    _builder.append(_qualifiedName_8, "");
                    _builder.append(">");
                  } else {
                    DataTypeAndTypeParameter _type_9 = attribute.getType();
                    String _qualifiedName_9 = this._names.getQualifiedName(_type_9);
                    _builder.append(_qualifiedName_9, "");
                  }
                }
                _builder.append(" ");
                String _variableName_6 = this._names.toVariableName(attribute);
                _builder.append(_variableName_6, "");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
              }
            }
          }
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _generateProperties(final ComplexType it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<Attribute> _attributes = it.getAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            boolean _isDerived = it.isDerived();
            boolean _not = (!_isDerived);
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_attributes, _function);
      for(final Attribute attribute : _filter) {
        _builder.newLine();
        CharSequence _getterMethod = this.toGetterMethod(attribute);
        _builder.append(_getterMethod, "");
        _builder.newLineIfNotEmpty();
        {
          boolean _isReadOnly = attribute.isReadOnly();
          boolean _not = (!_isReadOnly);
          if (_not) {
            CharSequence _setterMethod = this.toSetterMethod(attribute);
            _builder.append(_setterMethod, "");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  public CharSequence toGetterMethod(final Attribute it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isMany = it.isMany();
      if (_isMany) {
        {
          boolean _isReference = it.isReference();
          if (_isReference) {
            _builder.append("public Set<");
            DataTypeAndTypeParameter _type = it.getType();
            String _qualifiedName = this._names.getQualifiedName(_type);
            _builder.append(_qualifiedName, "");
            _builder.append("> ");
            String _accessorMethodName = this.getAccessorMethodName(it);
            _builder.append(_accessorMethodName, "");
            _builder.append("() {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("if (this.");
            String _name = it.getName();
            _builder.append(_name, "	");
            _builder.append(" == null) {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t\t");
            _builder.append("this.");
            String _name_1 = it.getName();
            _builder.append(_name_1, "		");
            _builder.append(" = new ");
            String _addImport = this._names.addImport(it, "java.util.HashSet");
            _builder.append(_addImport, "		");
            _builder.append("<");
            DataTypeAndTypeParameter _type_1 = it.getType();
            String _qualifiedName_1 = this._names.getQualifiedName(_type_1);
            _builder.append(_qualifiedName_1, "		");
            _builder.append(">();");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
            _builder.append("\t");
            _builder.append("return ");
            String _addImport_1 = this._names.addImport(it, "java.util.Collections");
            _builder.append(_addImport_1, "	");
            _builder.append(".unmodifiableSet(this.");
            String _name_2 = it.getName();
            _builder.append(_name_2, "	");
            _builder.append(");");
            _builder.newLineIfNotEmpty();
            _builder.append("}");
            _builder.newLine();
            _builder.newLine();
            _builder.append("public boolean ");
            String _hasMethodName = this.getHasMethodName(it);
            _builder.append(_hasMethodName, "");
            _builder.append("() {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("return ");
            String _name_3 = it.getName();
            _builder.append(_name_3, "	");
            _builder.append(" != null && !");
            String _name_4 = it.getName();
            _builder.append(_name_4, "	");
            _builder.append(".isEmpty();");
            _builder.newLineIfNotEmpty();
            _builder.append("}");
            _builder.newLine();
            _builder.newLine();
            _builder.append("public void ");
            String _addMethodName = this.getAddMethodName(it);
            _builder.append(_addMethodName, "");
            _builder.append("(");
            DataTypeAndTypeParameter _type_2 = it.getType();
            String _qualifiedName_2 = this._names.getQualifiedName(_type_2);
            _builder.append(_qualifiedName_2, "");
            _builder.append(" ");
            String _name_5 = it.getName();
            _builder.append(_name_5, "");
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("if (");
            String _name_6 = it.getName();
            String _firstLower = StringExtensions.toFirstLower(_name_6);
            _builder.append(_firstLower, "	");
            _builder.append(" == null) {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t\t");
            _builder.append("throw new IllegalArgumentException(\"parameter \'");
            String _name_7 = it.getName();
            _builder.append(_name_7, "		");
            _builder.append("\' must not be null\");");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
            _builder.append("\t");
            _builder.append("if (!");
            String _accessorMethodName_1 = this.getAccessorMethodName(it);
            _builder.append(_accessorMethodName_1, "	");
            _builder.append("().contains(");
            String _name_8 = it.getName();
            _builder.append(_name_8, "	");
            _builder.append(")) {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t\t");
            _builder.append("this.");
            String _name_9 = it.getName();
            _builder.append(_name_9, "		");
            _builder.append(".add(");
            String _name_10 = it.getName();
            _builder.append(_name_10, "		");
            _builder.append(");");
            _builder.newLineIfNotEmpty();
            {
              Attribute _oppositeReference = it.getOppositeReference();
              boolean _notEquals = (!Objects.equal(_oppositeReference, null));
              if (_notEquals) {
                _builder.append("\t\t");
                String _name_11 = it.getName();
                _builder.append(_name_11, "		");
                _builder.append(".");
                {
                  Attribute _oppositeReference_1 = it.getOppositeReference();
                  boolean _isMany_1 = _oppositeReference_1.isMany();
                  boolean _not = (!_isMany_1);
                  if (_not) {
                    Attribute _oppositeReference_2 = it.getOppositeReference();
                    String _mutatorMethodName = this.getMutatorMethodName(_oppositeReference_2);
                    _builder.append(_mutatorMethodName, "		");
                  } else {
                    Attribute _oppositeReference_3 = it.getOppositeReference();
                    String _addMethodName_1 = this.getAddMethodName(_oppositeReference_3);
                    _builder.append(_addMethodName_1, "		");
                  }
                }
                _builder.append("(this);");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
            _builder.append("}");
            _builder.newLine();
            _builder.newLine();
            _builder.append("public void ");
            String _removeMethodName = this.getRemoveMethodName(it);
            _builder.append(_removeMethodName, "");
            _builder.append("(");
            DataTypeAndTypeParameter _type_3 = it.getType();
            String _qualifiedName_3 = this._names.getQualifiedName(_type_3);
            _builder.append(_qualifiedName_3, "");
            _builder.append(" ");
            String _name_12 = it.getName();
            _builder.append(_name_12, "");
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("if (");
            String _name_13 = it.getName();
            _builder.append(_name_13, "	");
            _builder.append(" == null) {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t\t");
            _builder.append("throw new IllegalArgumentException(\"parameter \'");
            String _name_14 = it.getName();
            _builder.append(_name_14, "		");
            _builder.append("\' must not be null\");");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
            _builder.append("\t");
            _builder.append("if (");
            String _accessorMethodName_2 = this.getAccessorMethodName(it);
            _builder.append(_accessorMethodName_2, "	");
            _builder.append("().contains(");
            String _name_15 = it.getName();
            _builder.append(_name_15, "	");
            _builder.append(")) {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t\t");
            _builder.append("this.");
            String _name_16 = it.getName();
            _builder.append(_name_16, "		");
            _builder.append(".remove(");
            String _name_17 = it.getName();
            _builder.append(_name_17, "		");
            _builder.append(");");
            _builder.newLineIfNotEmpty();
            {
              Attribute _oppositeReference_4 = it.getOppositeReference();
              boolean _notEquals_1 = (!Objects.equal(_oppositeReference_4, null));
              if (_notEquals_1) {
                _builder.append("\t\t");
                String _name_18 = it.getName();
                _builder.append(_name_18, "		");
                _builder.append(".");
                {
                  Attribute _oppositeReference_5 = it.getOppositeReference();
                  boolean _isMany_2 = _oppositeReference_5.isMany();
                  boolean _not_1 = (!_isMany_2);
                  if (_not_1) {
                    Attribute _oppositeReference_6 = it.getOppositeReference();
                    String _mutatorMethodName_1 = this.getMutatorMethodName(_oppositeReference_6);
                    _builder.append(_mutatorMethodName_1, "		");
                    _builder.append("(null)");
                  } else {
                    Attribute _oppositeReference_7 = it.getOppositeReference();
                    String _removeMethodName_1 = this.getRemoveMethodName(_oppositeReference_7);
                    _builder.append(_removeMethodName_1, "		");
                    _builder.append("(this)");
                  }
                }
                _builder.append(";");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
            _builder.append("}");
            _builder.newLine();
            _builder.newLine();
            _builder.append("public void ");
            String _removeAllMethodName = this.getRemoveAllMethodName(it);
            _builder.append(_removeAllMethodName, "");
            _builder.append("() {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("if (this.");
            String _name_19 = it.getName();
            _builder.append(_name_19, "	");
            _builder.append(" != null) {");
            _builder.newLineIfNotEmpty();
            {
              boolean _and = false;
              boolean _isInverseSide = EntityExtension.isInverseSide(it);
              if (!_isInverseSide) {
                _and = false;
              } else {
                boolean _isComposition = it.isComposition();
                boolean _not_2 = (!_isComposition);
                _and = (_isInverseSide && _not_2);
              }
              if (_and) {
                _builder.append("\t\t");
                _builder.append("for (");
                DataTypeAndTypeParameter _type_4 = it.getType();
                String _qualifiedName_4 = this._names.getQualifiedName(_type_4);
                _builder.append(_qualifiedName_4, "		");
                _builder.append(" ");
                DataTypeAndTypeParameter _type_5 = it.getType();
                String _qualifiedName_5 = this._names.getQualifiedName(_type_5);
                String _firstLower_1 = StringExtensions.toFirstLower(_qualifiedName_5);
                _builder.append(_firstLower_1, "		");
                _builder.append(" : new HashSet<");
                DataTypeAndTypeParameter _type_6 = it.getType();
                String _qualifiedName_6 = this._names.getQualifiedName(_type_6);
                _builder.append(_qualifiedName_6, "		");
                _builder.append(">(this.");
                String _name_20 = it.getName();
                _builder.append(_name_20, "		");
                _builder.append(")) {");
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t");
                _builder.append("\t");
                String _removeMethodName_2 = this.getRemoveMethodName(it);
                _builder.append(_removeMethodName_2, "			");
                _builder.append("(");
                DataTypeAndTypeParameter _type_7 = it.getType();
                String _qualifiedName_7 = this._names.getQualifiedName(_type_7);
                String _firstLower_2 = StringExtensions.toFirstLower(_qualifiedName_7);
                _builder.append(_firstLower_2, "			");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t");
                _builder.append("}");
                _builder.newLine();
              } else {
                _builder.append("\t\t");
                _builder.append("this.");
                String _name_21 = it.getName();
                _builder.append(_name_21, "		");
                _builder.append(".clear();");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
            _builder.append("}");
            _builder.newLine();
          } else {
            _builder.append("public List<");
            DataTypeAndTypeParameter _type_8 = it.getType();
            String _qualifiedName_8 = this._names.getQualifiedName(_type_8);
            _builder.append(_qualifiedName_8, "");
            _builder.append("> get");
            String _name_22 = it.getName();
            String _firstUpper = StringExtensions.toFirstUpper(_name_22);
            _builder.append(_firstUpper, "");
            _builder.append("() {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("return this.");
            String _name_23 = it.getName();
            String _firstLower_3 = StringExtensions.toFirstLower(_name_23);
            _builder.append(_firstLower_3, "	");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
            _builder.append("}");
            _builder.newLine();
          }
        }
        _builder.newLine();
      } else {
        _builder.append("public ");
        DataTypeAndTypeParameter _type_9 = it.getType();
        String _qualifiedName_9 = this._names.getQualifiedName(_type_9);
        _builder.append(_qualifiedName_9, "");
        _builder.append(" get");
        String _name_24 = it.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_24);
        _builder.append(_firstUpper_1, "");
        _builder.append("() {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("return this.");
        String _name_25 = it.getName();
        String _firstLower_4 = StringExtensions.toFirstLower(_name_25);
        _builder.append(_firstLower_4, "	");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("}");
        _builder.newLine();
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence toSetterMethod(final Attribute it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _isMany = it.isMany();
      if (_isMany) {
        {
          boolean _isReference = it.isReference();
          if (_isReference) {
          } else {
            _builder.append("public void ");
            String _mutatorMethodName = this.getMutatorMethodName(it);
            _builder.append(_mutatorMethodName, "");
            _builder.append("(List<");
            DataTypeAndTypeParameter _type = it.getType();
            String _qualifiedName = this._names.getQualifiedName(_type);
            _builder.append(_qualifiedName, "");
            _builder.append("> ");
            String _name = it.getName();
            _builder.append(_name, "");
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("this.");
            String _name_1 = it.getName();
            _builder.append(_name_1, "	");
            _builder.append(" = ");
            String _name_2 = it.getName();
            _builder.append(_name_2, "	");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
            _builder.append("}");
            _builder.newLine();
          }
        }
      } else {
        {
          boolean _isReference_1 = it.isReference();
          if (_isReference_1) {
            _builder.append("public void ");
            String _mutatorMethodName_1 = this.getMutatorMethodName(it);
            _builder.append(_mutatorMethodName_1, "");
            _builder.append("(");
            DataTypeAndTypeParameter _type_1 = it.getType();
            String _qualifiedName_1 = this._names.getQualifiedName(_type_1);
            _builder.append(_qualifiedName_1, "");
            _builder.append(" ");
            String _name_3 = it.getName();
            _builder.append(_name_3, "");
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("if (this.");
            String _name_4 = it.getName();
            _builder.append(_name_4, "	");
            _builder.append(" != ");
            String _name_5 = it.getName();
            _builder.append(_name_5, "	");
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            {
              boolean _and = false;
              Attribute _oppositeReference = it.getOppositeReference();
              boolean _notEquals = (!Objects.equal(_oppositeReference, null));
              if (!_notEquals) {
                _and = false;
              } else {
                Attribute _oppositeReference_1 = it.getOppositeReference();
                boolean _isMany_1 = _oppositeReference_1.isMany();
                _and = (_notEquals && _isMany_1);
              }
              if (_and) {
                _builder.append("\t\t");
                _builder.append("if (this.");
                String _name_6 = it.getName();
                _builder.append(_name_6, "		");
                _builder.append(" != null) {");
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t");
                _builder.append("\t");
                _builder.append("this.");
                String _name_7 = it.getName();
                _builder.append(_name_7, "			");
                _builder.append(".");
                Attribute _oppositeReference_2 = it.getOppositeReference();
                String _removeMethodName = this.getRemoveMethodName(_oppositeReference_2);
                _builder.append(_removeMethodName, "			");
                _builder.append("(this);");
                _builder.newLineIfNotEmpty();
                _builder.append("\t\t");
                _builder.append("}");
                _builder.newLine();
              }
            }
            _builder.append("\t\t");
            _builder.append("this.");
            String _name_8 = it.getName();
            _builder.append(_name_8, "		");
            _builder.append(" = ");
            String _name_9 = it.getName();
            String _firstLower = StringExtensions.toFirstLower(_name_9);
            _builder.append(_firstLower, "		");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
            {
              Attribute _oppositeReference_3 = it.getOppositeReference();
              boolean _notEquals_1 = (!Objects.equal(_oppositeReference_3, null));
              if (_notEquals_1) {
                _builder.append("\t\t");
                _builder.append("if (");
                String _name_10 = it.getName();
                String _firstLower_1 = StringExtensions.toFirstLower(_name_10);
                _builder.append(_firstLower_1, "		");
                _builder.append(" != null) {");
                _builder.newLineIfNotEmpty();
                {
                  Attribute _oppositeReference_4 = it.getOppositeReference();
                  boolean _isMany_2 = _oppositeReference_4.isMany();
                  boolean _not = (!_isMany_2);
                  if (_not) {
                    _builder.append("\t\t");
                    _builder.append("\t");
                    String _name_11 = it.getName();
                    String _firstLower_2 = StringExtensions.toFirstLower(_name_11);
                    _builder.append(_firstLower_2, "			");
                    _builder.append(".");
                    Attribute _oppositeReference_5 = it.getOppositeReference();
                    String _mutatorMethodName_2 = this.getMutatorMethodName(_oppositeReference_5);
                    _builder.append(_mutatorMethodName_2, "			");
                    _builder.append("(this);");
                    _builder.newLineIfNotEmpty();
                  } else {
                    _builder.append("\t\t");
                    _builder.append("\t");
                    String _name_12 = it.getName();
                    String _firstLower_3 = StringExtensions.toFirstLower(_name_12);
                    _builder.append(_firstLower_3, "			");
                    _builder.append(".");
                    Attribute _oppositeReference_6 = it.getOppositeReference();
                    String _addMethodName = this.getAddMethodName(_oppositeReference_6);
                    _builder.append(_addMethodName, "			");
                    _builder.append("(this);");
                    _builder.newLineIfNotEmpty();
                  }
                }
                _builder.append("\t\t");
                _builder.append("}");
                _builder.newLine();
              }
            }
            _builder.append("\t");
            _builder.append("}");
            _builder.newLine();
            _builder.append("}");
            _builder.newLine();
          } else {
            _builder.append("public void ");
            String _mutatorMethodName_3 = this.getMutatorMethodName(it);
            _builder.append(_mutatorMethodName_3, "");
            _builder.append("(");
            DataTypeAndTypeParameter _type_2 = it.getType();
            String _qualifiedName_2 = this._names.getQualifiedName(_type_2);
            _builder.append(_qualifiedName_2, "");
            _builder.append(" ");
            String _name_13 = it.getName();
            _builder.append(_name_13, "");
            _builder.append(") {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("this.");
            String _name_14 = it.getName();
            _builder.append(_name_14, "	");
            _builder.append(" = ");
            String _name_15 = it.getName();
            _builder.append(_name_15, "	");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
            _builder.append("}");
            _builder.newLine();
          }
        }
      }
    }
    return _builder;
  }
  
  public CharSequence generateToString(final ComplexType complexType) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@Override");
    _builder.newLine();
    _builder.append("public String toString() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("StringBuilder toStringBuilder = new StringBuilder(\"");
    String _name = complexType.getName();
    _builder.append(_name, "	");
    _builder.append(" [\");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (complexType instanceof Entity) {
        final Entity _entity = (Entity)complexType;
        Entity _superType = _entity.getSuperType();
        boolean _notEquals = (!Objects.equal(_superType, null));
        if (_notEquals) {
          _matched=true;
          StringConcatenation _builder_1 = new StringConcatenation();
          _builder_1.append("toStringBuilder.append(super.toString())");
          {
            EList<Attribute> _attributes = _entity.getAttributes();
            boolean _isEmpty = _attributes.isEmpty();
            boolean _not = (!_isEmpty);
            if (_not) {
              _builder_1.append(".append(\",\")");
            }
          }
          _builder_1.append(";");
          _switchResult = _builder_1;
        }
      }
    }
    if (!_matched) {
      if (complexType instanceof DataView) {
        final DataView _dataView = (DataView)complexType;
        DataView _superType = _dataView.getSuperType();
        boolean _notEquals = (!Objects.equal(_superType, null));
        if (_notEquals) {
          _matched=true;
          StringConcatenation _builder_1 = new StringConcatenation();
          _builder_1.append("toStringBuilder.append(super.toString())");
          {
            EList<Attribute> _attributes = _dataView.getAttributes();
            boolean _isEmpty = _attributes.isEmpty();
            boolean _not = (!_isEmpty);
            if (_not) {
              _builder_1.append(".append(\",\")");
            }
          }
          _builder_1.append(";");
          _switchResult = _builder_1;
        }
      }
    }
    _builder.append(_switchResult, "	");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      EList<Attribute> _attributes = complexType.getAttributes();
      final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
          public Boolean apply(final Attribute it) {
            Type _dataType = it.getDataType();
            boolean _not = (!(_dataType instanceof ComplexType));
            return Boolean.valueOf(_not);
          }
        };
      Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_attributes, _function);
      boolean _hasElements = false;
      for(final Attribute attribute : _filter) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(".append(\",\");\n", "	");
        }
        _builder.append("toStringBuilder.append(\"");
        String _name_1 = attribute.getName();
        _builder.append(_name_1, "	");
        _builder.append("=\").append(");
        String _accessorMethodName = this.getAccessorMethodName(attribute);
        _builder.append(_accessorMethodName, "	");
        _builder.append("())");
      }
      if (_hasElements) {
        _builder.append(";\n", "	");
      }
    }
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return toStringBuilder.append(\"]\").toString();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public String toSerialVersionUID(final ComplexType it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("private static final long serialVersionUID = ");
    String _serialVersionUID = this.serialVersionUID(it);
    _builder.append(_serialVersionUID, "");
    _builder.append(";");
    return _builder.toString();
  }
  
  public String serialVersionUID(final ComplexType it) {
    String _serialVersionUID = JavaUtilExtension.serialVersionUID(it);
    return _serialVersionUID;
  }
  
  public CharSequence toOperationSignature(final Operation it, final String visibility) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _documentationIndented = this.getDocumentationIndented(it);
    _builder.append(_documentationIndented, "");
    _builder.newLineIfNotEmpty();
    {
      boolean _notEquals = (!Objects.equal(visibility, null));
      if (_notEquals) {
        _builder.append("public ");
      }
    }
    String _type = this.toType(it);
    _builder.append(_type, "");
    _builder.append(" ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.append("(");
    {
      EList<Parameter> _parameters = it.getParameters();
      boolean _hasElements = false;
      for(final Parameter paramameter : _parameters) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        CharSequence _methodParameter = this.methodParameter(paramameter);
        _builder.append(_methodParameter, "");
      }
    }
    _builder.append(")");
    return _builder;
  }
  
  public CharSequence toOperationSignature(final DelegateOperation it, final String visibility) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _documentationIndented = this.getDocumentationIndented(it);
    _builder.append(_documentationIndented, "");
    _builder.newLineIfNotEmpty();
    {
      boolean _notEquals = (!Objects.equal(visibility, null));
      if (_notEquals) {
        _builder.append("public ");
      }
    }
    String _type = this.toType(it);
    _builder.append(_type, "");
    _builder.append(" ");
    DaoOperation _operation = it.getOperation();
    String _name = _operation.getName();
    _builder.append(_name, "");
    _builder.append("(");
    {
      DaoOperation _operation_1 = it.getOperation();
      EList<Parameter> _parameters = ((Operation) _operation_1).getParameters();
      boolean _hasElements = false;
      for(final Parameter paramameter : _parameters) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        CharSequence _methodParameter = this.methodParameter(paramameter);
        _builder.append(_methodParameter, "");
      }
    }
    _builder.append(")");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected String _methodParameterJavaDoc(final Attribute it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("* @param ");
    String _resolvedName = this.getResolvedName(it);
    _builder.append(_resolvedName, "");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }
  
  protected String _methodParameterJavaDoc(final Parameter it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("* @param ");
    String _name = it.getName();
    _builder.append(_name, "");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }
  
  protected String _methodParameterJavaDoc(final QueryParameter it) {
    Attribute _attribute = it.getAttribute();
    String _methodParameterJavaDoc = this.methodParameterJavaDoc(_attribute);
    return _methodParameterJavaDoc;
  }
  
  protected CharSequence _methodParameter(final Attribute it) {
    StringConcatenation _builder = new StringConcatenation();
    String _type = this.toType(it);
    _builder.append(_type, "");
    _builder.append(" ");
    String _resolvedName = this.getResolvedName(it);
    _builder.append(_resolvedName, "");
    return _builder;
  }
  
  protected CharSequence _methodParameter(final QueryParameter it) {
    StringConcatenation _builder = new StringConcatenation();
    String _type = this.toType(it);
    _builder.append(_type, "");
    _builder.append(" ");
    Attribute _attribute = it.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    return _builder;
  }
  
  protected CharSequence _methodParameter(final Parameter it) {
    StringConcatenation _builder = new StringConcatenation();
    String _type = this.toType(it);
    _builder.append(_type, "");
    _builder.append(" ");
    String _name = it.getName();
    _builder.append(_name, "");
    return _builder;
  }
  
  public CharSequence getDocumentation(final IDocumentable it) {
    CharSequence _xifexpression = null;
    String _txtDescription = it.getTxtDescription();
    boolean _notEquals = (!Objects.equal(_txtDescription, null));
    if (_notEquals) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("/**");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("* ");
      String _txtDescription_1 = it.getTxtDescription();
      String _replaceAll = _txtDescription_1.replaceAll("\n", "\n* ");
      _builder.append(_replaceAll, " ");
      _builder.newLineIfNotEmpty();
      _builder.append(" ");
      _builder.append("*/");
      _builder.newLine();
      _xifexpression = _builder;
    }
    return _xifexpression;
  }
  
  public CharSequence getDocumentationLines(final IDocumentable it) {
    CharSequence _xifexpression = null;
    String _txtDescription = it.getTxtDescription();
    boolean _notEquals = (!Objects.equal(_txtDescription, null));
    if (_notEquals) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("* ");
      String _txtDescription_1 = it.getTxtDescription();
      String _replaceAll = _txtDescription_1.replaceAll("\n", "\n* ");
      _builder.append(_replaceAll, "");
      _builder.newLineIfNotEmpty();
      _builder.append("*");
      _builder.newLine();
      _xifexpression = _builder;
    }
    return _xifexpression;
  }
  
  public CharSequence getDocumentationIndented(final IDocumentable it) {
    CharSequence _xifexpression = null;
    String _txtDescription = it.getTxtDescription();
    boolean _notEquals = (!Objects.equal(_txtDescription, null));
    if (_notEquals) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("/** ");
      String _txtDescription_1 = it.getTxtDescription();
      String _replaceAll = _txtDescription_1.replaceAll("\n", "  ");
      _builder.append(_replaceAll, "");
      _builder.append(" */");
      _builder.newLineIfNotEmpty();
      _xifexpression = _builder;
    }
    return _xifexpression;
  }
  
  public CharSequence getDocumentationLinesIndented(final IDocumentable it) {
    CharSequence _xifexpression = null;
    String _txtDescription = it.getTxtDescription();
    boolean _notEquals = (!Objects.equal(_txtDescription, null));
    if (_notEquals) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("* ");
      String _txtDescription_1 = it.getTxtDescription();
      String _replaceAll = _txtDescription_1.replaceAll("\n", "\n\t * ");
      _builder.append(_replaceAll, "");
      _builder.newLineIfNotEmpty();
      _builder.append("*");
      _builder.newLine();
      _xifexpression = _builder;
    }
    return _xifexpression;
  }
  
  public String getQualifiedQueryName(final QueryOperation it) {
    EObject _eContainer = it.eContainer();
    Entity _entity = ((Dao) _eContainer).getEntity();
    String _name = _entity.getName();
    String _plus = (_name + ".");
    String _queryName = this.getQueryName(it);
    String _plus_1 = (_plus + _queryName);
    return _plus_1;
  }
  
  public String getQualifiedQueryConstant(final QueryOperation it) {
    EObject _eContainer = it.eContainer();
    String _name = ((Dao) _eContainer).getName();
    String _plus = (_name + ".");
    String _queryConstant = this.getQueryConstant(it);
    String _plus_1 = (_plus + _queryConstant);
    return _plus_1;
  }
  
  public String getQueryConstant(final QueryOperation it) {
    String _queryName = this.getQueryName(it);
    String _underScoreString = Strings.toUnderScoreString(_queryName);
    return _underScoreString.toUpperCase();
  }
  
  public String getQueryName(final QueryOperation queryOperation) {
    String _xblockexpression = null;
    {
      EObject _eContainer = queryOperation.eContainer();
      EList<QueryOperation> _queryOperation = ((Dao) _eContainer).getQueryOperation();
      final Function1<QueryOperation,Boolean> _function = new Function1<QueryOperation,Boolean>() {
          public Boolean apply(final QueryOperation it) {
            String _name = queryOperation.getName();
            String _name_1 = it.getName();
            boolean _equals = _name.equals(_name_1);
            return Boolean.valueOf(_equals);
          }
        };
      final Iterable<QueryOperation> queryOperationsWithSameName = IterableExtensions.<QueryOperation>filter(_queryOperation, _function);
      String _xifexpression = null;
      int _size = IterableExtensions.size(queryOperationsWithSameName);
      boolean _greaterThan = (_size > 1);
      if (_greaterThan) {
        String _name = queryOperation.getName();
        String _firstUpper = StringExtensions.toFirstUpper(_name);
        List<QueryOperation> _list = IterableExtensions.<QueryOperation>toList(queryOperationsWithSameName);
        int _indexOf = _list.indexOf(queryOperation);
        String _plus = (_firstUpper + Integer.valueOf(_indexOf));
        _xifexpression = _plus;
      } else {
        String _name_1 = queryOperation.getName();
        String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
        _xifexpression = _firstUpper_1;
      }
      _xblockexpression = (_xifexpression);
    }
    return _xblockexpression;
  }
  
  public Type getType(final QueryParameter it) {
    Type _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Parameter) {
        final Parameter _parameter = (Parameter)it;
        _matched=true;
        Type _type = _parameter.getType();
        _switchResult = _type;
      }
    }
    if (!_matched) {
      Attribute _attribute = it.getAttribute();
      Type _dataType = _attribute.getDataType();
      _switchResult = _dataType;
    }
    return _switchResult;
  }
  
  public String getName(final QueryParameter it) {
    String _xifexpression = null;
    Attribute _attribute = it.getAttribute();
    boolean _notEquals = (!Objects.equal(_attribute, null));
    if (_notEquals) {
      Attribute _attribute_1 = it.getAttribute();
      return _attribute_1.getName();
    } else {
      String _name = ((Parameter) it).getName();
      _xifexpression = _name;
    }
    return _xifexpression;
  }
  
  public String getQualifiedKeyClassName(final AttributeGroup it) {
    EObject _eContainer = it.eContainer();
    String _classPackageName = this._names.getClassPackageName(((Entity) _eContainer));
    String _plus = (_classPackageName + ".");
    String _keyClassName = this.getKeyClassName(it);
    String _plus_1 = (_plus + _keyClassName);
    return _plus_1;
  }
  
  public String getKeyClassName(final AttributeGroup it) {
    String _name = it.getName();
    return _name;
  }
  
  public String toType(final EObject it) {
    if (it instanceof Attribute) {
      return _toType((Attribute)it);
    } else if (it instanceof DelegateOperation) {
      return _toType((DelegateOperation)it);
    } else if (it instanceof Operation) {
      return _toType((Operation)it);
    } else if (it instanceof Parameter) {
      return _toType((Parameter)it);
    } else if (it instanceof DaoOperation) {
      return _toType((DaoOperation)it);
    } else if (it instanceof QueryParameter) {
      return _toType((QueryParameter)it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
  
  public String getAccessorMethodName(final EObject it) {
    if (it instanceof FeatureReference) {
      return _getAccessorMethodName((FeatureReference)it);
    } else if (it != null) {
      return _getAccessorMethodName(it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
  
  public CharSequence generateProperties(final ComplexType it) {
    if (it instanceof DataView) {
      return _generateProperties((DataView)it);
    } else if (it != null) {
      return _generateProperties(it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
  
  public String methodParameterJavaDoc(final EObject it) {
    if (it instanceof Attribute) {
      return _methodParameterJavaDoc((Attribute)it);
    } else if (it instanceof Parameter) {
      return _methodParameterJavaDoc((Parameter)it);
    } else if (it instanceof QueryParameter) {
      return _methodParameterJavaDoc((QueryParameter)it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
  
  public CharSequence methodParameter(final EObject it) {
    if (it instanceof Attribute) {
      return _methodParameter((Attribute)it);
    } else if (it instanceof Parameter) {
      return _methodParameter((Parameter)it);
    } else if (it instanceof QueryParameter) {
      return _methodParameter((QueryParameter)it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
}
