package org.openxma.dsl.generator.impl;

import com.google.common.collect.Iterables;
import com.google.inject.Inject;
import java.util.Arrays;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.ParameterValue;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.core.model.TypeDefinition;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.core.model.impl.StringParameterValueImpl;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.PresentableFeature;
import org.openxma.dsl.dom.model.Service;
import org.openxma.dsl.generator.impl.JsfData;
import org.openxma.dsl.generator.impl.JsfNames;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.impl.Types;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.ListBox;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableCombo;
import org.openxma.dsl.pom.model.Text;
import org.openxma.dsl.pom.model.Tree;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.model.XmadslPage;

@SuppressWarnings("all")
public class JsfDataImpl implements JsfData {
  @Inject
  private Names names;
  
  @Inject
  private JsfNames jsfNames;
  
  @Inject
  private Types type;
  
  public CharSequence toDataObjectVars(final EList<DataObjectVariable> variables) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final DataObjectVariable dataObject : variables) {
        {
          boolean _isIsCollection = dataObject.isIsCollection();
          if (_isIsCollection) {
            _builder.append("protected ");
            String _addImport = this.names.addImport(dataObject, "java.util.Collection");
            _builder.append(_addImport, "");
            _builder.append("<");
            ComplexType _type = dataObject.getType();
            Type _pomType = this.type.pomType(_type);
            String _qualifiedName = this.names.getQualifiedName(_pomType);
            _builder.append(_qualifiedName, "");
            _builder.append("> ");
            String _name = dataObject.getName();
            _builder.append(_name, "");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append("protected ");
            ComplexType _type_1 = dataObject.getType();
            Type _pomType_1 = this.type.pomType(_type_1);
            String _qualifiedName_1 = this.names.getQualifiedName(_pomType_1);
            _builder.append(_qualifiedName_1, "");
            _builder.append(" ");
            String _name_1 = dataObject.getName();
            _builder.append(_name_1, "");
            _builder.append(";");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  public CharSequence toObjectPropertyVars(final EList<ObjectProperty> properties) {
    StringConcatenation _builder = new StringConcatenation();
    {
      for(final ObjectProperty objProperty : properties) {
        _builder.append("protected ");
        SimpleType _type = objProperty.getType();
        String _qualifiedName = this.names.getQualifiedName(_type);
        _builder.append(_qualifiedName, "");
        _builder.append(" ");
        String _name = objProperty.getName();
        _builder.append(_name, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toInjectedDependencies(final EList dependencies) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Iterable<Service> _filter = Iterables.<Service>filter(dependencies, Service.class);
      for(final Service service : _filter) {
        _builder.append("\t");
        _builder.append("@");
        String _addImport = this.names.addImport(service, "org.springframework.beans.factory.annotation.Autowired");
        _builder.append(_addImport, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("protected ");
        String _qualifiedName = this.names.getQualifiedName(service);
        _builder.append(_qualifiedName, "");
        _builder.append(" ");
        String _name = service.getName();
        String _firstLower = StringExtensions.toFirstLower(_name);
        _builder.append(_firstLower, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toGetterAndSetter(final ObjectProperty objProperty) {
    SimpleType _type = objProperty.getType();
    String _qualifiedName = this.names.getQualifiedName(_type);
    String _name = objProperty.getName();
    CharSequence _getterAndSetter = this.toGetterAndSetter(_qualifiedName, _name);
    return _getterAndSetter;
  }
  
  public CharSequence toGetterAndSetter(final CharSequence fullyQualifiedName, final String propertyName) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("public ");
    _builder.append(fullyQualifiedName, "");
    _builder.append(" get");
    String _firstUpper = StringExtensions.toFirstUpper(propertyName);
    _builder.append(_firstUpper, "");
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.append("return ");
    _builder.append(propertyName, "    ");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    _builder.append("public void set");
    String _firstUpper_1 = StringExtensions.toFirstUpper(propertyName);
    _builder.append(_firstUpper_1, "");
    _builder.append("(");
    _builder.append(fullyQualifiedName, "");
    _builder.append(" ");
    _builder.append(propertyName, "");
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("    ");
    _builder.append("this.");
    _builder.append(propertyName, "    ");
    _builder.append(" = ");
    _builder.append(propertyName, "    ");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _toGetterAndSetter(final DataObjectVariable dataObject) {
    CharSequence _xifexpression = null;
    boolean _isIsCollection = dataObject.isIsCollection();
    if (_isIsCollection) {
      String _addImport = this.names.addImport(dataObject, "java.util.Collection");
      String _name = dataObject.getName();
      CharSequence _getterAndSetter = this.toGetterAndSetter(_addImport, _name);
      _xifexpression = _getterAndSetter;
    } else {
      ComplexType _type = dataObject.getType();
      Type _pomType = this.type.pomType(_type);
      String _qualifiedName = this.names.getQualifiedName(_pomType);
      String _name_1 = dataObject.getName();
      CharSequence _getterAndSetter_1 = this.toGetterAndSetter(_qualifiedName, _name_1);
      _xifexpression = _getterAndSetter_1;
    }
    return _xifexpression;
  }
  
  protected CharSequence _toGetterAndSetter(final StatusFlag flag) {
    String _name = flag.getName();
    CharSequence _getterAndSetter = this.toGetterAndSetter("Boolean", _name);
    return _getterAndSetter;
  }
  
  protected CharSequence _getModelPropertyName(final Table table) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = table.getName();
    _builder.append(_name, "");
    _builder.append("TableModel");
    return _builder;
  }
  
  protected CharSequence _getRootNodePropertyName(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = tree.getName();
    _builder.append(_name, "");
    _builder.append("TreeRootNode");
    return _builder;
  }
  
  protected CharSequence _getSelectedNodePropertyName(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = tree.getName();
    _builder.append(_name, "");
    _builder.append("TreeSelectedNode");
    return _builder;
  }
  
  protected CharSequence _getModelPropertyNameFirstUpper(final Table table) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = table.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("TableModel");
    return _builder;
  }
  
  protected CharSequence _getRootNodePropertyNameFirstUpper(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = tree.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("TreeRootNode");
    return _builder;
  }
  
  protected CharSequence _getSelectedNodePropertyNameFirstUpper(final Tree tree) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = tree.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("TreeSelectedNode");
    return _builder;
  }
  
  protected CharSequence _getSelectItemsPropertyName(final EObject eobject) {
    Class<? extends EObject> _class = eobject.getClass();
    String _name = _class.getName();
    String _plus = ("getSelectItemsPropertyName: Not supported class " + _name);
    RuntimeException _runtimeException = new RuntimeException(_plus);
    throw _runtimeException;
  }
  
  protected CharSequence _getSelectItemsPropertyName(final CustomizeableField field) {
    StringConcatenation _builder = new StringConcatenation();
    Attribute _attribute = field.getAttribute();
    String _name = _attribute.getName();
    _builder.append(_name, "");
    _builder.append("1SelectItems");
    return _builder;
  }
  
  protected CharSequence _getSelectItemsPropertyName(final Combo combo) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = combo.getName();
    _builder.append(_name, "");
    _builder.append("1SelectItems");
    return _builder;
  }
  
  protected CharSequence _getSelectItemsPropertyName(final ListBox listbox) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = listbox.getName();
    _builder.append(_name, "");
    _builder.append("1SelectItems");
    return _builder;
  }
  
  protected CharSequence _getSelectItemsPropertyName(final TableCombo combo) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = combo.getName();
    _builder.append(_name, "");
    _builder.append("1SelectItems");
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final Tree it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    String _addImport = this.names.addImport(it, "org.primefaces.model.TreeNode");
    _builder.append(_addImport, "");
    _builder.append(" ");
    CharSequence _rootNodePropertyName = this.getRootNodePropertyName(it);
    _builder.append(_rootNodePropertyName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("protected ");
    String _addImport_1 = this.names.addImport(it, "org.primefaces.model.TreeNode");
    _builder.append(_addImport_1, "");
    _builder.append(" ");
    CharSequence _selectedNodePropertyName = this.getSelectedNodePropertyName(it);
    _builder.append(_selectedNodePropertyName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImport_2 = this.names.addImport(it, "org.primefaces.model.TreeNode");
    CharSequence _rootNodePropertyName_1 = this.getRootNodePropertyName(it);
    String _string = _rootNodePropertyName_1.toString();
    CharSequence _getterAndSetter = this.toGetterAndSetter(_addImport_2, _string);
    _builder.append(_getterAndSetter, "");
    _builder.newLineIfNotEmpty();
    String _addImport_3 = this.names.addImport(it, "org.primefaces.model.TreeNode");
    CharSequence _selectedNodePropertyName_1 = this.getSelectedNodePropertyName(it);
    String _string_1 = _selectedNodePropertyName_1.toString();
    CharSequence _getterAndSetter_1 = this.toGetterAndSetter(_addImport_3, _string_1);
    _builder.append(_getterAndSetter_1, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final Table it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    String _addImport = this.names.addImport(it, "org.openxma.dsl.platform.jsf.model.MapDataModel");
    _builder.append(_addImport, "");
    _builder.append("<");
    PresentableFeature _key = it.getKey();
    Attribute _attribute = _key.getAttribute();
    Type _dataType = _attribute.getDataType();
    String _qualifiedName = this.names.getQualifiedName(_dataType);
    _builder.append(_qualifiedName, "");
    _builder.append(",");
    DataObjectVariable _object = it.getObject();
    ComplexType _type = _object.getType();
    Type _pomType = this.type.pomType(_type);
    String _qualifiedName_1 = this.names.getQualifiedName(_pomType);
    _builder.append(_qualifiedName_1, "");
    _builder.append("> ");
    CharSequence _modelPropertyName = this.getModelPropertyName(it);
    _builder.append(_modelPropertyName, "");
    _builder.append(" = new MapDataModel<");
    PresentableFeature _key_1 = it.getKey();
    Attribute _attribute_1 = _key_1.getAttribute();
    Type _dataType_1 = _attribute_1.getDataType();
    String _qualifiedName_2 = this.names.getQualifiedName(_dataType_1);
    _builder.append(_qualifiedName_2, "");
    _builder.append(",");
    DataObjectVariable _object_1 = it.getObject();
    ComplexType _type_1 = _object_1.getType();
    Type _pomType_1 = this.type.pomType(_type_1);
    String _qualifiedName_3 = this.names.getQualifiedName(_pomType_1);
    _builder.append(_qualifiedName_3, "");
    _builder.append(">() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public ");
    PresentableFeature _key_2 = it.getKey();
    Attribute _attribute_2 = _key_2.getAttribute();
    Type _dataType_2 = _attribute_2.getDataType();
    String _qualifiedName_4 = this.names.getQualifiedName(_dataType_2);
    _builder.append(_qualifiedName_4, "	");
    _builder.append(" getRowKey(");
    DataObjectVariable _object_2 = it.getObject();
    ComplexType _type_2 = _object_2.getType();
    Type _pomType_2 = this.type.pomType(_type_2);
    String _qualifiedName_5 = this.names.getQualifiedName(_pomType_2);
    _builder.append(_qualifiedName_5, "	");
    _builder.append(" row) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return row.get");
    PresentableFeature _key_3 = it.getKey();
    Attribute _attribute_3 = _key_3.getAttribute();
    String _name = _attribute_3.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "		");
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("};");
    _builder.newLine();
    _builder.newLine();
    String _addImport_1 = this.names.addImport(it, "org.openxma.dsl.platform.jsf.model.MapDataModel");
    CharSequence _modelPropertyName_1 = this.getModelPropertyName(it);
    String _string = _modelPropertyName_1.toString();
    CharSequence _getterAndSetter = this.toGetterAndSetter(_addImport_1, _string);
    _builder.append(_getterAndSetter, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final CustomizeableField it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Attribute _attribute = it.getAttribute();
      DataTypeAndTypeParameter _type = _attribute.getType();
      Type _dataType = _type.getDataType();
      if ((_dataType instanceof EnumType)) {
        _builder.append("protected ");
        String _addImport = this.names.addImport(it, "java.util.List");
        _builder.append(_addImport, "");
        _builder.append("<");
        String _addImport_1 = this.names.addImport(it, "javax.faces.model.SelectItem");
        _builder.append(_addImport_1, "");
        _builder.append("> ");
        CharSequence _selectItemsPropertyName = this.getSelectItemsPropertyName(it);
        _builder.append(_selectItemsPropertyName, "");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.newLine();
        String _addImport_2 = this.names.addImport(it, "java.util.List");
        CharSequence _selectItemsPropertyName_1 = this.getSelectItemsPropertyName(it);
        String _string = _selectItemsPropertyName_1.toString();
        CharSequence _getterAndSetter = this.toGetterAndSetter(_addImport_2, _string);
        _builder.append(_getterAndSetter, "");
        _builder.newLineIfNotEmpty();
      } else {
        Attribute _attribute_1 = it.getAttribute();
        DataTypeAndTypeParameter _type_1 = _attribute_1.getType();
        ValidatorReference _validatorReferenceDefinition = _type_1.getValidatorReferenceDefinition();
        boolean _isI18nValidator = _validatorReferenceDefinition.isI18nValidator();
        if (_isI18nValidator) {
          _builder.append("protected ");
          String _addImport_3 = this.names.addImport(it, "java.util.List");
          _builder.append(_addImport_3, "");
          _builder.append("<");
          String _addImport_4 = this.names.addImport(it, "javax.faces.model.SelectItem");
          _builder.append(_addImport_4, "");
          _builder.append("> ");
          CharSequence _selectItemsPropertyName_2 = this.getSelectItemsPropertyName(it);
          _builder.append(_selectItemsPropertyName_2, "");
          _builder.append(";");
          _builder.newLineIfNotEmpty();
          _builder.newLine();
          String _addImport_5 = this.names.addImport(it, "java.util.List");
          CharSequence _selectItemsPropertyName_3 = this.getSelectItemsPropertyName(it);
          String _string_1 = _selectItemsPropertyName_3.toString();
          CharSequence _getterAndSetter_1 = this.toGetterAndSetter(_addImport_5, _string_1);
          _builder.append(_getterAndSetter_1, "");
          _builder.newLineIfNotEmpty();
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final CheckBox checkBox) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected Boolean ");
    String _backingBeanVar = this.jsfNames.getBackingBeanVar(checkBox);
    _builder.append(_backingBeanVar, "");
    _builder.append(" = false;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _backingBeanVar_1 = this.jsfNames.getBackingBeanVar(checkBox);
    CharSequence _getterAndSetter = this.toGetterAndSetter("Boolean", _backingBeanVar_1);
    _builder.append(_getterAndSetter, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final Text text) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected String ");
    String _backingBeanVar = this.jsfNames.getBackingBeanVar(text);
    _builder.append(_backingBeanVar, "");
    _builder.append(" = \"\";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _backingBeanVar_1 = this.jsfNames.getBackingBeanVar(text);
    CharSequence _getterAndSetter = this.toGetterAndSetter("String", _backingBeanVar_1);
    _builder.append(_getterAndSetter, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final Combo it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    String _addImport = this.names.addImport(it, "java.util.List");
    _builder.append(_addImport, "");
    _builder.append("<");
    String _addImport_1 = this.names.addImport(it, "javax.faces.model.SelectItem");
    _builder.append(_addImport_1, "");
    _builder.append("> ");
    CharSequence _selectItemsPropertyName = this.getSelectItemsPropertyName(it);
    _builder.append(_selectItemsPropertyName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.newLine();
    String _addImport_2 = this.names.addImport(it, "java.util.List");
    CharSequence _selectItemsPropertyName_1 = this.getSelectItemsPropertyName(it);
    String _string = _selectItemsPropertyName_1.toString();
    CharSequence _getterAndSetter = this.toGetterAndSetter(_addImport_2, _string);
    _builder.append(_getterAndSetter, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("protected Object ");
    String _backingBeanVar = this.jsfNames.getBackingBeanVar(it);
    _builder.append(_backingBeanVar, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _backingBeanVar_1 = this.jsfNames.getBackingBeanVar(it);
    CharSequence _getterAndSetter_1 = this.toGetterAndSetter("Object", _backingBeanVar_1);
    _builder.append(_getterAndSetter_1, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final TableCombo it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected Object ");
    String _backingBeanVar = this.jsfNames.getBackingBeanVar(it);
    _builder.append(_backingBeanVar, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _backingBeanVar_1 = this.jsfNames.getBackingBeanVar(it);
    CharSequence _getterAndSetter = this.toGetterAndSetter("Object", _backingBeanVar_1);
    _builder.append(_getterAndSetter, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final ListBox it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("protected ");
    String _addImport = this.names.addImport(it, "java.util.List");
    _builder.append(_addImport, "");
    _builder.append("<");
    String _addImport_1 = this.names.addImport(it, "javax.faces.model.SelectItem");
    _builder.append(_addImport_1, "");
    _builder.append("> ");
    CharSequence _selectItemsPropertyName = this.getSelectItemsPropertyName(it);
    _builder.append(_selectItemsPropertyName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.newLine();
    String _addImport_2 = this.names.addImport(it, "java.util.List");
    CharSequence _selectItemsPropertyName_1 = this.getSelectItemsPropertyName(it);
    String _string = _selectItemsPropertyName_1.toString();
    CharSequence _getterAndSetter = this.toGetterAndSetter(_addImport_2, _string);
    _builder.append(_getterAndSetter, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("protected Object ");
    String _backingBeanVar = this.jsfNames.getBackingBeanVar(it);
    _builder.append(_backingBeanVar, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _backingBeanVar_1 = this.jsfNames.getBackingBeanVar(it);
    CharSequence _getterAndSetter_1 = this.toGetterAndSetter("Object", _backingBeanVar_1);
    _builder.append(_getterAndSetter_1, "");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final EObject o) {
    return null;
  }
  
  protected CharSequence _toJsfDataModel(final XmadslPage page) {
    Content _content = page.getContent();
    CharSequence _jsfDataModel = _content==null?(CharSequence)null:this.toJsfDataModel(_content);
    return _jsfDataModel;
  }
  
  protected CharSequence _toJsfDataModel(final XmadslComposite composite) {
    Content _content = composite.getContent();
    CharSequence _jsfDataModel = _content==null?(CharSequence)null:this.toJsfDataModel(_content);
    return _jsfDataModel;
  }
  
  protected CharSequence _toJsfDataModel(final ReferencedXMAComposite composite) {
    Content _content = composite.getContent();
    CharSequence _jsfDataModel = this.toJsfDataModel(_content);
    return _jsfDataModel;
  }
  
  protected CharSequence _toJsfDataModel(final SetOfGuiElements composite) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = composite.eContents();
      for(final EObject item : _eContents) {
        CharSequence _jsfDataModel = this.toJsfDataModel(item);
        _builder.append(_jsfDataModel, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final TabFolder tabFolder) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = tabFolder.eContents();
      for(final EObject item : _eContents) {
        CharSequence _jsfDataModel = this.toJsfDataModel(item);
        _builder.append(_jsfDataModel, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final TabPage page) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = page.eContents();
      for(final EObject item : _eContents) {
        CharSequence _jsfDataModel = this.toJsfDataModel(item);
        _builder.append(_jsfDataModel, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toJsfDataModel(final Content content) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = content.eContents();
      for(final EObject item : _eContents) {
        CharSequence _jsfDataModel = this.toJsfDataModel(item);
        _builder.append(_jsfDataModel, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toInitCode(final EObject it) {
    return null;
  }
  
  protected CharSequence _toInitCode(final CustomizeableField it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      Attribute _attribute = it.getAttribute();
      DataTypeAndTypeParameter _type = _attribute.getType();
      Type _dataType = _type.getDataType();
      if ((_dataType instanceof EnumType)) {
      } else {
        Attribute _attribute_1 = it.getAttribute();
        DataTypeAndTypeParameter _type_1 = _attribute_1.getType();
        ValidatorReference _validatorReferenceDefinition = _type_1.getValidatorReferenceDefinition();
        boolean _isI18nValidator = _validatorReferenceDefinition.isI18nValidator();
        if (_isI18nValidator) {
          CharSequence _selectItemsPropertyName = this.getSelectItemsPropertyName(it);
          _builder.append(_selectItemsPropertyName, "");
          _builder.append(" = ");
          String _addImport = this.names.addImport(it, "org.openxma.dsl.platform.jsf.util.DomainValuesUtil");
          _builder.append(_addImport, "");
          _builder.append(".getSelectItems(");
          String _addImport_1 = this.names.addImport(it, "at.spardat.xma.datasource.Domain");
          _builder.append(_addImport_1, "");
          _builder.append(".getInstance(\"");
          Attribute _attribute_2 = it.getAttribute();
          DataTypeAndTypeParameter _type_2 = _attribute_2.getType();
          CharSequence _domainSpec = this.getDomainSpec(_type_2);
          _builder.append(_domainSpec, "");
          _builder.append("\", xmaSession).getDomRows(false));");
          _builder.newLineIfNotEmpty();
        }
      }
    }
    return _builder;
  }
  
  public CharSequence getDomainSpec(final DataTypeAndTypeParameter type) {
    CharSequence _xblockexpression = null;
    {
      Type _dataType = type.getDataType();
      final SimpleType simpleType = ((SimpleType) _dataType);
      TypeDefinition _typeDefinition = simpleType.getTypeDefinition();
      final DataTypeAndTypeParameter definition = _typeDefinition.getDefinition();
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("TODO:testspec ");
      EList<ParameterValue> _parameterValues = definition.getParameterValues();
      ParameterValue _get = _parameterValues.get(0);
      String _stringValue = ((StringParameterValueImpl) _get).getStringValue();
      _builder.append(_stringValue, "");
      EList<ParameterValue> _parameterValues_1 = definition.getParameterValues();
      ParameterValue _get_1 = _parameterValues_1.get(1);
      String _stringValue_1 = ((StringParameterValueImpl) _get_1).getStringValue();
      _builder.append(_stringValue_1, "");
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  protected CharSequence _toInitCode(final XmadslPage page) {
    Content _content = page.getContent();
    CharSequence _initCode = _content==null?(CharSequence)null:this.toInitCode(_content);
    return _initCode;
  }
  
  protected CharSequence _toInitCode(final XmadslComposite composite) {
    Content _content = composite.getContent();
    CharSequence _initCode = _content==null?(CharSequence)null:this.toInitCode(_content);
    return _initCode;
  }
  
  protected CharSequence _toInitCode(final ReferencedXMAComposite composite) {
    Content _content = composite.getContent();
    CharSequence _initCode = this.toInitCode(_content);
    return _initCode;
  }
  
  protected CharSequence _toInitCode(final SetOfGuiElements composite) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = composite.eContents();
      for(final EObject item : _eContents) {
        CharSequence _initCode = this.toInitCode(item);
        _builder.append(_initCode, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toInitCode(final TabFolder tabFolder) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = tabFolder.eContents();
      for(final EObject item : _eContents) {
        CharSequence _initCode = this.toInitCode(item);
        _builder.append(_initCode, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toInitCode(final TabPage page) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = page.eContents();
      for(final EObject item : _eContents) {
        CharSequence _initCode = this.toInitCode(item);
        _builder.append(_initCode, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _toInitCode(final Content content) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EObject> _eContents = content.eContents();
      for(final EObject item : _eContents) {
        CharSequence _initCode = this.toInitCode(item);
        _builder.append(_initCode, "");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence toGetterAndSetter(final EObject flag) {
    if (flag instanceof StatusFlag) {
      return _toGetterAndSetter((StatusFlag)flag);
    } else if (flag instanceof DataObjectVariable) {
      return _toGetterAndSetter((DataObjectVariable)flag);
    } else if (flag instanceof ObjectProperty) {
      return _toGetterAndSetter((ObjectProperty)flag);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(flag).toString());
    }
  }
  
  public CharSequence getModelPropertyName(final Table table) {
    {
      return _getModelPropertyName(table);
    }
  }
  
  public CharSequence getRootNodePropertyName(final Tree tree) {
    {
      return _getRootNodePropertyName(tree);
    }
  }
  
  public CharSequence getSelectedNodePropertyName(final Tree tree) {
    {
      return _getSelectedNodePropertyName(tree);
    }
  }
  
  public CharSequence getModelPropertyNameFirstUpper(final Table table) {
    {
      return _getModelPropertyNameFirstUpper(table);
    }
  }
  
  public CharSequence getRootNodePropertyNameFirstUpper(final Tree tree) {
    {
      return _getRootNodePropertyNameFirstUpper(tree);
    }
  }
  
  public CharSequence getSelectedNodePropertyNameFirstUpper(final Tree tree) {
    {
      return _getSelectedNodePropertyNameFirstUpper(tree);
    }
  }
  
  public CharSequence getSelectItemsPropertyName(final EObject combo) {
    if (combo instanceof Combo) {
      return _getSelectItemsPropertyName((Combo)combo);
    } else if (combo instanceof CustomizeableField) {
      return _getSelectItemsPropertyName((CustomizeableField)combo);
    } else if (combo instanceof ListBox) {
      return _getSelectItemsPropertyName((ListBox)combo);
    } else if (combo instanceof TableCombo) {
      return _getSelectItemsPropertyName((TableCombo)combo);
    } else if (combo != null) {
      return _getSelectItemsPropertyName(combo);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(combo).toString());
    }
  }
  
  public CharSequence toJsfDataModel(final EObject composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _toJsfDataModel((ReferencedXMAComposite)composite);
    } else if (composite instanceof TabFolder) {
      return _toJsfDataModel((TabFolder)composite);
    } else if (composite instanceof TabPage) {
      return _toJsfDataModel((TabPage)composite);
    } else if (composite instanceof XmadslComposite) {
      return _toJsfDataModel((XmadslComposite)composite);
    } else if (composite instanceof CheckBox) {
      return _toJsfDataModel((CheckBox)composite);
    } else if (composite instanceof Combo) {
      return _toJsfDataModel((Combo)composite);
    } else if (composite instanceof CustomizeableField) {
      return _toJsfDataModel((CustomizeableField)composite);
    } else if (composite instanceof ListBox) {
      return _toJsfDataModel((ListBox)composite);
    } else if (composite instanceof Table) {
      return _toJsfDataModel((Table)composite);
    } else if (composite instanceof TableCombo) {
      return _toJsfDataModel((TableCombo)composite);
    } else if (composite instanceof Text) {
      return _toJsfDataModel((Text)composite);
    } else if (composite instanceof Tree) {
      return _toJsfDataModel((Tree)composite);
    } else if (composite instanceof XmadslPage) {
      return _toJsfDataModel((XmadslPage)composite);
    } else if (composite instanceof Content) {
      return _toJsfDataModel((Content)composite);
    } else if (composite instanceof SetOfGuiElements) {
      return _toJsfDataModel((SetOfGuiElements)composite);
    } else if (composite != null) {
      return _toJsfDataModel(composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
  
  public CharSequence toInitCode(final EObject composite) {
    if (composite instanceof ReferencedXMAComposite) {
      return _toInitCode((ReferencedXMAComposite)composite);
    } else if (composite instanceof TabFolder) {
      return _toInitCode((TabFolder)composite);
    } else if (composite instanceof TabPage) {
      return _toInitCode((TabPage)composite);
    } else if (composite instanceof XmadslComposite) {
      return _toInitCode((XmadslComposite)composite);
    } else if (composite instanceof CustomizeableField) {
      return _toInitCode((CustomizeableField)composite);
    } else if (composite instanceof XmadslPage) {
      return _toInitCode((XmadslPage)composite);
    } else if (composite instanceof Content) {
      return _toInitCode((Content)composite);
    } else if (composite instanceof SetOfGuiElements) {
      return _toInitCode((SetOfGuiElements)composite);
    } else if (composite != null) {
      return _toInitCode(composite);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(composite).toString());
    }
  }
}
