package org.openxma.dsl.generator.impl;

import com.google.common.collect.Iterables;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.openxma.dsl.generator.impl.JsfEvents;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.EnterEventMapping;
import org.openxma.dsl.pom.model.EventFunction;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.InitEventMapping;
import org.openxma.dsl.pom.model.LeaveEventMapping;
import org.openxma.dsl.pom.model.XmadslPage;

@Singleton
@SuppressWarnings("all")
public class JsfEventsImpl implements JsfEvents {
  @Inject
  private Names names;
  
  public CharSequence toLifecycleMethods(final XmadslPage it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@");
    String _addImport = this.names.addImport(it, "javax.annotation.PostConstruct");
    _builder.append(_addImport, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public void beanInit() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("dataModelInit();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("onInit();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("onEnter();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("public void invoke() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("onInit();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("onEnter();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toLifecycleMethods(final EventMappingList it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* PML onLoad event");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("protected void onEnter() {");
    _builder.newLine();
    {
      EList<EventMapping> _mapping = it.getMapping();
      Iterable<EnterEventMapping> _filter = Iterables.<EnterEventMapping>filter(_mapping, EnterEventMapping.class);
      for(final EnterEventMapping mapping : _filter) {
        {
          EList<EventFunction> _eventFunctions = mapping.getEventFunctions();
          for(final EventFunction function : _eventFunctions) {
            {
              Command _command = function.getCommand();
              boolean _isClientOnly = _command.isClientOnly();
              boolean _not = (!_isClientOnly);
              if (_not) {
                _builder.append("        ");
                Command _command_1 = function.getCommand();
                String _name = _command_1.getName();
                _builder.append(_name, "        ");
                _builder.append("();");
                _builder.newLineIfNotEmpty();
              }
            }
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* PML onLeave event");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("protected void onLeave() {");
    _builder.newLine();
    {
      EList<EventMapping> _mapping_1 = it.getMapping();
      Iterable<LeaveEventMapping> _filter_1 = Iterables.<LeaveEventMapping>filter(_mapping_1, LeaveEventMapping.class);
      for(final LeaveEventMapping mapping_1 : _filter_1) {
        {
          EList<EventFunction> _eventFunctions_1 = mapping_1.getEventFunctions();
          for(final EventFunction function_1 : _eventFunctions_1) {
            {
              Command _command_2 = function_1.getCommand();
              boolean _isClientOnly_1 = _command_2.isClientOnly();
              boolean _not_1 = (!_isClientOnly_1);
              if (_not_1) {
                _builder.append("        ");
                Command _command_3 = function_1.getCommand();
                String _name_1 = _command_3.getName();
                _builder.append(_name_1, "        ");
                _builder.append("();");
                _builder.newLineIfNotEmpty();
              }
            }
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("/**");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* PML onInit event");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("*/");
    _builder.newLine();
    _builder.append("protected void onInit() {");
    _builder.newLine();
    {
      EList<EventMapping> _mapping_2 = it.getMapping();
      Iterable<InitEventMapping> _filter_2 = Iterables.<InitEventMapping>filter(_mapping_2, InitEventMapping.class);
      for(final InitEventMapping mapping_2 : _filter_2) {
        {
          EList<EventFunction> _eventFunctions_2 = mapping_2.getEventFunctions();
          for(final EventFunction function_2 : _eventFunctions_2) {
            {
              Command _command_4 = function_2.getCommand();
              boolean _isClientOnly_2 = _command_4.isClientOnly();
              boolean _not_2 = (!_isClientOnly_2);
              if (_not_2) {
                _builder.append("        ");
                Command _command_5 = function_2.getCommand();
                String _name_2 = _command_5.getName();
                _builder.append(_name_2, "        ");
                _builder.append("();");
                _builder.newLineIfNotEmpty();
              }
            }
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
