package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import java.util.List;
import java.util.Map;
import javax.inject.Inject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.mwe.core.monitor.ProgressMonitor;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.Functions.Function0;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.MapExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure2;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.generator.Generator;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.IFileSystemAccessProvider;
import org.openxma.dsl.generator.ModelModifier;
import org.openxma.dsl.generator.helper.GeneratorConfigurationExtension;
import org.openxma.dsl.generator.impl.AutomationTestCaseGenerator;
import org.openxma.dsl.generator.impl.BackingBean;
import org.openxma.dsl.generator.impl.Facelet;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.IGeneratorDelegate;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.XmadslPage;

@SuppressWarnings("all")
public class JsfGenerator implements Generator, IGenerator {
  private final Log logger = new Function0<Log>() {
    public Log apply() {
      Class<? extends JsfGenerator> _class = JsfGenerator.this.getClass();
      String _name = _class.getName();
      Log _log = LogFactory.getLog(_name);
      return _log;
    }
  }.apply();
  
  private GeneratorConfiguration generatorConfiguration;
  
  @Inject
  private Facelet facelet;
  
  @Inject
  private BackingBean backingBean;
  
  @Inject
  private Files files;
  
  @Inject
  private IFileSystemAccessProvider fileSystemAccessProvider;
  
  @Inject
  private AutomationTestCaseGenerator automationTestCaseGenerator;
  
  public void init(final GeneratorConfiguration generatorConfiguration) {
    this.generatorConfiguration = generatorConfiguration;
  }
  
  public void generate(final Resource resource, final ProgressMonitor progressMonitor) {
    ResourceSet _resourceSet = resource.getResourceSet();
    this.modify(_resourceSet);
    IFileSystemAccess _get = this.fileSystemAccessProvider.get(this.generatorConfiguration);
    ResourceSet _resourceSet_1 = resource.getResourceSet();
    IGeneratorDelegate _iGeneratorDelegate = new IGeneratorDelegate(_get, _resourceSet_1);
    final IGeneratorDelegate fsa = _iGeneratorDelegate;
    this.doGenerate(resource, fsa);
  }
  
  public void doGenerate(final Resource input, final IFileSystemAccess fsa) {
    EObject _eObject = input.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    final Function1<ModelElement,Boolean> _function = new Function1<ModelElement,Boolean>() {
        public Boolean apply(final ModelElement it) {
          return Boolean.valueOf((it instanceof Component));
        }
      };
    ModelElement _findFirst = IterableExtensions.<ModelElement>findFirst(_elements, _function);
    final Component component = ((Component) _findFirst);
    String _backingBeanFileName = this.files.backingBeanFileName(component, true);
    String _name = model.getName();
    CharSequence _abstractComponentBackingBean = this.backingBean.toAbstractComponentBackingBean(_name, component);
    fsa.generateFile(_backingBeanFileName, GeneratorConstants.OUTLET_GENERATED_JAVA, _abstractComponentBackingBean);
    String _backingBeanFileName_1 = this.files.backingBeanFileName(component, true);
    String _plus = ("Generate " + _backingBeanFileName_1);
    this.logger.info(_plus);
    String _backingBeanFileName_2 = this.files.backingBeanFileName(component, false);
    String _name_1 = model.getName();
    CharSequence _componentBackingBean = this.backingBean.toComponentBackingBean(_name_1, component);
    fsa.generateFile(_backingBeanFileName_2, GeneratorConstants.OUTLET_JAVA, _componentBackingBean);
    String _backingBeanFileName_3 = this.files.backingBeanFileName(component, false);
    String _plus_1 = ("Generate " + _backingBeanFileName_3);
    this.logger.info(_plus_1);
    StringBuffer _stringBuffer = new StringBuffer();
    final StringBuffer resourcePropertyContent = _stringBuffer;
    EList<Page> _pages = component.getPages();
    final Iterable<XmadslPage> xmaDslPages = Iterables.<XmadslPage>filter(_pages, XmadslPage.class);
    final Procedure1<XmadslPage> _function_1 = new Procedure1<XmadslPage>() {
        public void apply(final XmadslPage page) {
          CharSequence pageContents = JsfGenerator.this.facelet.createContents(page);
          String _faceletPageFileName = JsfGenerator.this.files.faceletPageFileName(page, true);
          fsa.generateFile(_faceletPageFileName, GeneratorConstants.OUTLET_GENERATED_WEBAPP, pageContents);
          String _faceletPageFileName_1 = JsfGenerator.this.files.faceletPageFileName(page, true);
          String _plus = ("Generate " + _faceletPageFileName_1);
          JsfGenerator.this.logger.info(_plus);
          CharSequence _generateResourceBundleForPage = JsfGenerator.this.generateResourceBundleForPage(page);
          resourcePropertyContent.append(_generateResourceBundleForPage);
          String _faceletPageFileName_2 = JsfGenerator.this.files.faceletPageFileName(page, false);
          CharSequence _faceletPage = JsfGenerator.this.toFaceletPage(page);
          fsa.generateFile(_faceletPageFileName_2, GeneratorConstants.OUTLET_WEBAPP, _faceletPage);
          String _faceletPageFileName_3 = JsfGenerator.this.files.faceletPageFileName(page, false);
          String _plus_1 = ("Generate " + _faceletPageFileName_3);
          JsfGenerator.this.logger.info(_plus_1);
          String _backingBeanFileName = JsfGenerator.this.files.backingBeanFileName(page, true);
          String _name = model.getName();
          CharSequence _abstractBackingBean = JsfGenerator.this.backingBean.toAbstractBackingBean(_name, page, component);
          fsa.generateFile(_backingBeanFileName, GeneratorConstants.OUTLET_GENERATED_JAVA, _abstractBackingBean);
          String _backingBeanFileName_1 = JsfGenerator.this.files.backingBeanFileName(page, true);
          String _plus_2 = ("Generate " + _backingBeanFileName_1);
          JsfGenerator.this.logger.info(_plus_2);
          String _backingBeanFileName_2 = JsfGenerator.this.files.backingBeanFileName(page, false);
          String _name_1 = model.getName();
          CharSequence _backingBean = JsfGenerator.this.backingBean.toBackingBean(_name_1, page);
          fsa.generateFile(_backingBeanFileName_2, GeneratorConstants.OUTLET_JAVA, _backingBean);
          String _backingBeanFileName_3 = JsfGenerator.this.files.backingBeanFileName(page, false);
          String _plus_3 = ("Generate " + _backingBeanFileName_3);
          JsfGenerator.this.logger.info(_plus_3);
          boolean _isAutomationTestingEnabled = GeneratorConfigurationExtension.isAutomationTestingEnabled();
          if (_isAutomationTestingEnabled) {
            String _iTCaseName = JsfGenerator.this.automationTestCaseGenerator.getITCaseName(page);
            String _name_2 = model.getName();
            CharSequence _generateTestCase = JsfGenerator.this.automationTestCaseGenerator.generateTestCase(page, _name_2);
            fsa.generateFile(_iTCaseName, GeneratorConstants.OUTLET_TEST, _generateTestCase);
            String _iTCaseName_1 = JsfGenerator.this.automationTestCaseGenerator.getITCaseName(page);
            String _plus_4 = ("Generate " + _iTCaseName_1);
            JsfGenerator.this.logger.info(_plus_4);
            String _pageObjectName = JsfGenerator.this.automationTestCaseGenerator.getPageObjectName(page, true);
            String _name_3 = model.getName();
            Map<CharSequence,CharSequence> _fieldClientIds = JsfGenerator.this.facelet.getFieldClientIds(page);
            CharSequence _abstractPageObject = JsfGenerator.this.automationTestCaseGenerator.getAbstractPageObject(page, _name_3, _fieldClientIds);
            fsa.generateFile(_pageObjectName, GeneratorConstants.OUTLET_GENERATED_TEST, _abstractPageObject);
            String _pageObjectName_1 = JsfGenerator.this.automationTestCaseGenerator.getPageObjectName(page, true);
            String _plus_5 = ("Generate " + _pageObjectName_1);
            JsfGenerator.this.logger.info(_plus_5);
            String _pageObjectName_2 = JsfGenerator.this.automationTestCaseGenerator.getPageObjectName(page, false);
            String _name_4 = model.getName();
            CharSequence _pageObject = JsfGenerator.this.automationTestCaseGenerator.getPageObject(page, _name_4);
            fsa.generateFile(_pageObjectName_2, GeneratorConstants.OUTLET_TEST, _pageObject);
            String _pageObjectName_3 = JsfGenerator.this.automationTestCaseGenerator.getPageObjectName(page, false);
            String _plus_6 = ("Generate " + _pageObjectName_3);
            JsfGenerator.this.logger.info(_plus_6);
          }
        }
      };
    IterableExtensions.<XmadslPage>forEach(xmaDslPages, _function_1);
    String _resourceBundleFilePath = this.files.toResourceBundleFilePath(component);
    String _string = resourcePropertyContent.toString();
    fsa.generateFile(_resourceBundleFilePath, GeneratorConstants.OUTLET_GENERATED_RESOURCES, _string);
    String _resourceBundleFilePath_1 = this.files.toResourceBundleFilePath(component);
    String _plus_2 = ("Generate " + _resourceBundleFilePath_1);
    this.logger.info(_plus_2);
  }
  
  public CharSequence toResourceBundle() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("#Enter you own property messages here, property declared here can be referred using msgs variable\t");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence generateResourceBundleForPage(final XmadslPage page) {
    Map<CharSequence,CharSequence> resourceProperties = this.facelet.getPageResourceProperties(page);
    String content = "";
    boolean _and = false;
    boolean _notEquals = (!Objects.equal(resourceProperties, null));
    if (!_notEquals) {
      _and = false;
    } else {
      int _size = resourceProperties.size();
      boolean _greaterThan = (_size > 0);
      _and = (_notEquals && _greaterThan);
    }
    if (_and) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("#*************************** Resources for ");
      String _name = page.getName();
      _builder.append(_name, "");
      _builder.append(" *******************************");
      _builder.newLineIfNotEmpty();
      CharSequence _generateResourceBundleFromMap = this.generateResourceBundleFromMap(resourceProperties);
      _builder.append(_generateResourceBundleFromMap, "");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      content = _builder.toString();
    }
    resourceProperties.clear();
    return content;
  }
  
  public CharSequence generateResourceBundleFromMap(final Map<CharSequence,CharSequence> resourceProperties) {
    StringBuffer _stringBuffer = new StringBuffer();
    final StringBuffer sb = _stringBuffer;
    final Procedure2<CharSequence,CharSequence> _function = new Procedure2<CharSequence,CharSequence>() {
        public void apply(final CharSequence key, final CharSequence value) {
          String _plus = (key + "=");
          String _plus_1 = (_plus + value);
          String _plus_2 = (_plus_1 + "\n");
          sb.append(_plus_2);
        }
      };
    MapExtensions.<CharSequence, CharSequence>forEach(resourceProperties, _function);
    return sb;
  }
  
  public void modify(final ResourceSet resourceSet) {
    EList<Resource> _resources = resourceSet.getResources();
    final List<Resource> resources = Lists.<Resource>newArrayList(_resources);
    for (final Resource resource : resources) {
      boolean _and = false;
      EObject _eObject = resource.getEObject("/");
      if (!(_eObject instanceof Model)) {
        _and = false;
      } else {
        boolean _hasProperty = EObjectPropertiesAdapter.hasProperty(resource, GeneratorConstants.MODIFIED_MODEL);
        boolean _not = (!_hasProperty);
        _and = ((_eObject instanceof Model) && _not);
      }
      if (_and) {
        EObjectPropertiesAdapter.setProperty(resource, GeneratorConstants.MODIFIED_MODEL, Boolean.TRUE);
        URI _uRI = resource.getURI();
        String _lastSegment = _uRI.lastSegment();
        String _plus = ("Pre-process model \'" + _lastSegment);
        String _plus_1 = (_plus + "\'");
        this.logger.info(_plus_1);
        ModelModifier _modelModifier = this.generatorConfiguration.getModelModifier();
        EObject _eObject_1 = resource.getEObject("/");
        _modelModifier.modifyModel(((Model) _eObject_1));
      }
    }
  }
  
  public CharSequence toFaceletPage(final XmadslPage page) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    _builder.newLine();
    _builder.append("<!DOCTYPE html");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"");
    _builder.newLine();
    _builder.append("        ");
    _builder.append("\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
    _builder.newLine();
    _builder.append("<html xmlns=\"http://www.w3.org/1999/xhtml\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:f=\"http://java.sun.com/jsf/core\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:h=\"http://java.sun.com/jsf/html\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:ui=\"http://java.sun.com/jsf/facelets\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:p=\"http://primefaces.org/ui\"");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("xmlns:fn=\"http://java.sun.com/jsp/jstl/functions\">");
    _builder.newLine();
    _builder.append("<head>");
    _builder.newLine();
    _builder.newLine();
    _builder.append("</head>");
    _builder.newLine();
    _builder.append("<body>");
    _builder.newLine();
    _builder.append("<ui:composition template=\"");
    String _name = page.getName();
    String _firstLower = StringExtensions.toFirstLower(_name);
    _builder.append(_firstLower, "");
    _builder.append(".gen.xhtml\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("<ui:param name=\"template\" value=\"/templates/template.xhtml\" />");
    _builder.newLine();
    _builder.append("</ui:composition>");
    _builder.newLine();
    _builder.append("</body>");
    _builder.newLine();
    _builder.append("</html>");
    _builder.newLine();
    return _builder;
  }
}
