package org.openxma.dsl.generator.impl;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.common.collect.Iterators;
import com.google.common.collect.Sets;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.inject.Inject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.IGenerator;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.IteratorExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.AttributeGroup;
import org.openxma.dsl.dom.model.AttributeSortOrder;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.CrudOperationType;
import org.openxma.dsl.dom.model.Dao;
import org.openxma.dsl.dom.model.DaoOperation;
import org.openxma.dsl.dom.model.DataBaseConstraint;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.DelegateOperation;
import org.openxma.dsl.dom.model.Dependant;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.Expression;
import org.openxma.dsl.dom.model.FeatureReference;
import org.openxma.dsl.dom.model.Operation;
import org.openxma.dsl.dom.model.Parameter;
import org.openxma.dsl.dom.model.QlStatement;
import org.openxma.dsl.dom.model.QueryOperation;
import org.openxma.dsl.dom.model.QueryParameter;
import org.openxma.dsl.dom.model.QueryParameterReference;
import org.openxma.dsl.dom.model.QueryParameterValue;
import org.openxma.dsl.dom.model.SelectStatement;
import org.openxma.dsl.dom.model.Service;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.helper.EntityExtension;
import org.openxma.dsl.generator.helper.ServiceExtension;
import org.openxma.dsl.generator.impl.Features;
import org.openxma.dsl.generator.impl.Files;
import org.openxma.dsl.generator.impl.Names;
import org.openxma.dsl.generator.lib.ImportAdapter;
import org.openxma.dsl.generator.lib.IterableExtensions2;

@SuppressWarnings("all")
public class ServiceGenerator implements IGenerator {
  @Inject
  private Names _names;
  
  @Inject
  private Files _files;
  
  @Inject
  private Features _features;
  
  public void doGenerate(final Resource resource, final IFileSystemAccess fsa) {
    EObject _eObject = resource.getEObject("/");
    final Model model = ((Model) _eObject);
    EList<ModelElement> _elements = model.getElements();
    Iterable<Service> _filter = Iterables.<Service>filter(_elements, Service.class);
    final Procedure1<Service> _function = new Procedure1<Service>() {
        public void apply(final Service it) {
          String _qualifiedGenInterfaceName = ServiceGenerator.this._names.getQualifiedGenInterfaceName(it);
          String _javaFilePath = ServiceGenerator.this._files.toJavaFilePath(_qualifiedGenInterfaceName);
          CharSequence _genInterface = ServiceGenerator.this.toGenInterface(it);
          fsa.generateFile(_javaFilePath, GeneratorConstants.OUTLET_GENERATED_JAVA, _genInterface);
          String _qualifiedInterfaceName = ServiceGenerator.this._names.getQualifiedInterfaceName(it);
          String _javaFilePath_1 = ServiceGenerator.this._files.toJavaFilePath(_qualifiedInterfaceName);
          CharSequence _interface = ServiceGenerator.this.toInterface(it);
          fsa.generateFile(_javaFilePath_1, GeneratorConstants.OUTLET_JAVA, _interface);
          String _qualifiedGenClassName = ServiceGenerator.this._names.getQualifiedGenClassName(it);
          String _javaFilePath_2 = ServiceGenerator.this._files.toJavaFilePath(_qualifiedGenClassName);
          CharSequence _genClass = ServiceGenerator.this.toGenClass(it);
          fsa.generateFile(_javaFilePath_2, GeneratorConstants.OUTLET_GENERATED_JAVA, _genClass);
          String _qualifiedClassName = ServiceGenerator.this._names.getQualifiedClassName(it);
          String _javaFilePath_3 = ServiceGenerator.this._files.toJavaFilePath(_qualifiedClassName);
          CharSequence _class = ServiceGenerator.this.toClass(it);
          fsa.generateFile(_javaFilePath_3, GeneratorConstants.OUTLET_JAVA, _class);
        }
      };
    IterableExtensions.<Service>forEach(_filter, _function);
  }
  
  public CharSequence toGenInterface(final Service it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public interface ");
    String _genInterfaceName = this._names.getGenInterfaceName(it);
    _builder.append(_genInterfaceName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    {
      EList<Operation> _operations = it.getOperations();
      for(final Operation operation : _operations) {
        _builder.append("\t");
        CharSequence _operationSignature = this._features.toOperationSignature(operation, null);
        _builder.append(_operationSignature, "	");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Operation> _operations_1 = it.getOperations();
      List<Operation> _createUpdateOperations = ServiceExtension.getCreateUpdateOperations(_operations_1);
      for(final Operation operation_1 : _createUpdateOperations) {
        _builder.append("\t");
        DelegateOperation _delegate = operation_1.getDelegate();
        DataView _view = _delegate.getView();
        String _name = _view.getName();
        _builder.append(_name, "	");
        _builder.append(" saveOrUpdate(");
        DelegateOperation _delegate_1 = operation_1.getDelegate();
        DataView _viewParameter = _delegate_1.getViewParameter();
        String _name_1 = _viewParameter.getName();
        _builder.append(_name_1, "	");
        _builder.append(" ");
        DelegateOperation _delegate_2 = operation_1.getDelegate();
        DataView _viewParameter_1 = _delegate_2.getViewParameter();
        String _variableName = this._names.toVariableName(_viewParameter_1);
        _builder.append(_variableName, "	");
        _builder.append(");");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toInterface(final Service it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._names.getPackageName(it);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public interface ");
    String _interfaceName = this._names.getInterfaceName(it);
    _builder.append(_interfaceName, "");
    _builder.append(" extends ");
    String _genInterfaceName = this._names.getGenInterfaceName(it);
    _builder.append(_genInterfaceName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toClass(final Service it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import org.springframework.stereotype.Service;");
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("@Service(\"");
    String _name = it.getName();
    String _firstLower = StringExtensions.toFirstLower(_name);
    _builder.append(_firstLower, "");
    _builder.append("\")");
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    String _className = this._names.getClassName(it);
    _builder.append(_className, "");
    _builder.append(" extends ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    {
      EList<Operation> _operations = it.getOperations();
      final Function1<Operation,Boolean> _function = new Function1<Operation,Boolean>() {
          public Boolean apply(final Operation e) {
            DelegateOperation _delegate = e.getDelegate();
            boolean _equals = Objects.equal(_delegate, null);
            return Boolean.valueOf(_equals);
          }
        };
      Iterable<Operation> _filter = IterableExtensions.<Operation>filter(_operations, _function);
      for(final Operation operation : _filter) {
        _builder.append("\t");
        CharSequence _operationSignature = this._features.toOperationSignature(operation, "public");
        _builder.append(_operationSignature, "	");
        _builder.append(" {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("// TODO");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("throw new IllegalStateException(\"not yet implemented\");");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence toGenClass(final Service it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _classPackageName = this._names.getClassPackageName(it);
    _builder.append(_classPackageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import org.apache.commons.logging.Log;");
    _builder.newLine();
    _builder.append("import org.apache.commons.logging.LogFactory;");
    _builder.newLine();
    _builder.append("import org.openxma.dsl.platform.service.Mapper;");
    _builder.newLine();
    _builder.append("import org.springframework.beans.factory.annotation.Autowired;");
    _builder.newLine();
    String _addImportsHere = ImportAdapter.addImportsHere(it);
    _builder.append(_addImportsHere, "");
    _builder.newLineIfNotEmpty();
    CharSequence _documentation = this._features.getDocumentation(it);
    _builder.append(_documentation, "");
    _builder.newLineIfNotEmpty();
    _builder.append("public abstract class ");
    String _genClassName = this._names.getGenClassName(it);
    _builder.append(_genClassName, "");
    _builder.append(" implements ");
    String _qualifiedName = this._names.getQualifiedName(it);
    _builder.append(_qualifiedName, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("protected final Log logger = LogFactory.getLog(getClass());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("@Autowired(required=false)");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("protected Mapper mapper;");
    _builder.newLine();
    {
      EList<Dependant> _dependencies = it.getDependencies();
      for(final Dependant dependency : _dependencies) {
        _builder.append("\t");
        _builder.append("protected ");
        String _name = this.getName(dependency);
        _builder.append(_name, "	");
        _builder.append(" ");
        String _qualifiedName_1 = this._names.getQualifiedName(dependency);
        String _firstLower = StringExtensions.toFirstLower(_qualifiedName_1);
        _builder.append(_firstLower, "	");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Dependant> _dependencies_1 = it.getDependencies();
      for(final Dependant dependency_1 : _dependencies_1) {
        _builder.append("\t");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("@Autowired");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("public void set");
        String _name_1 = this.getName(dependency_1);
        _builder.append(_name_1, "	");
        _builder.append("(");
        String _qualifiedName_2 = this._names.getQualifiedName(dependency_1);
        _builder.append(_qualifiedName_2, "	");
        _builder.append(" ");
        String _variableName = this._names.toVariableName(dependency_1);
        _builder.append(_variableName, "	");
        _builder.append(") {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("this.");
        String _variableName_1 = this._names.toVariableName(dependency_1);
        _builder.append(_variableName_1, "		");
        _builder.append(" = ");
        String _variableName_2 = this._names.toVariableName(dependency_1);
        _builder.append(_variableName_2, "		");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    {
      Iterable<Operation> _crudOperations = this.getCrudOperations(it);
      for(final Operation crudOperation : _crudOperations) {
        _builder.append("\t");
        _builder.newLine();
        _builder.append("\t");
        CharSequence _crudOperationImpl = this.toCrudOperationImpl(crudOperation);
        _builder.append(_crudOperationImpl, "	");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      EList<Operation> _operations = it.getOperations();
      List<Operation> _createUpdateOperations = ServiceExtension.getCreateUpdateOperations(_operations);
      for(final Operation op : _createUpdateOperations) {
        {
          boolean _or = false;
          DelegateOperation _delegate = op.getDelegate();
          Dao _repository = _delegate.getRepository();
          Entity _entity = _repository.getEntity();
          Attribute _version = _entity.getVersion();
          boolean _notEquals = (!Objects.equal(_version, null));
          if (_notEquals) {
            _or = true;
          } else {
            DelegateOperation _delegate_1 = op.getDelegate();
            Dao _repository_1 = _delegate_1.getRepository();
            Entity _entity_1 = _repository_1.getEntity();
            Attribute _identifier = _entity_1.getIdentifier();
            boolean _notEquals_1 = (!Objects.equal(_identifier, null));
            _or = (_notEquals || _notEquals_1);
          }
          if (_or) {
            _builder.append("\t");
            _builder.newLine();
            _builder.append("\t");
            CharSequence _crudSaveOrUpdateImplementation = this.toCrudSaveOrUpdateImplementation(op);
            _builder.append(_crudSaveOrUpdateImplementation, "	");
            _builder.newLineIfNotEmpty();
          }
        }
        _builder.append("\t");
        _builder.newLine();
      }
    }
    {
      EList<Operation> _operations_1 = it.getOperations();
      final Function1<Operation,Boolean> _function = new Function1<Operation,Boolean>() {
          public Boolean apply(final Operation e) {
            DelegateOperation _delegate = e.getDelegate();
            DaoOperation _operation = _delegate==null?(DaoOperation)null:_delegate.getOperation();
            boolean _notEquals = (!Objects.equal(_operation, null));
            return Boolean.valueOf(_notEquals);
          }
        };
      Iterable<Operation> _filter = IterableExtensions.<Operation>filter(_operations_1, _function);
      for(final Operation queryOperation : _filter) {
        _builder.append("\t");
        CharSequence _queryOperationImplementation = this.toQueryOperationImplementation(queryOperation);
        _builder.append(_queryOperationImplementation, "	");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.newLine();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  private CharSequence toCrudOperationImpl(final Operation it) {
    CharSequence _xblockexpression = null;
    {
      DelegateOperation _delegate = it.getDelegate();
      final CrudOperationType crudOperationType = _delegate==null?(CrudOperationType)null:_delegate.getCrudOperationType();
      DelegateOperation _delegate_1 = it.getDelegate();
      final Dao repository = _delegate_1==null?(Dao)null:_delegate_1.getRepository();
      final Entity entity = repository==null?(Entity)null:repository.getEntity();
      DelegateOperation _delegate_2 = it.getDelegate();
      final DataView viewParameter = _delegate_2==null?(DataView)null:_delegate_2.getViewParameter();
      CharSequence _switchResult = null;
      boolean _matched = false;
      if (!_matched) {
        if (Objects.equal(crudOperationType,CrudOperationType.CREATE)) {
          _matched=true;
          CharSequence _xblockexpression_1 = null;
          {
            EList<FeatureReference> _featureReferences = viewParameter.getFeatureReferences();
            final Function1<FeatureReference,Boolean> _function = new Function1<FeatureReference,Boolean>() {
                public Boolean apply(final FeatureReference e) {
                  boolean _and = false;
                  boolean _and_1 = false;
                  boolean _and_2 = false;
                  Entity _source = e.getSource();
                  boolean _equals = Objects.equal(_source, entity);
                  if (!_equals) {
                    _and_2 = false;
                  } else {
                    Attribute _attribute = e.getAttribute();
                    boolean _isReference = _attribute==null?false:_attribute.isReference();
                    _and_2 = (_equals && _isReference);
                  }
                  if (!_and_2) {
                    _and_1 = false;
                  } else {
                    Attribute _attribute_1 = e.getAttribute();
                    boolean _isMany = _attribute_1==null?false:_attribute_1.isMany();
                    _and_1 = (_and_2 && _isMany);
                  }
                  if (!_and_1) {
                    _and = false;
                  } else {
                    DataView _view = e.getView();
                    Attribute _attribute_2 = e.getAttribute();
                    Type _dataType = _attribute_2==null?(Type)null:_attribute_2.getDataType();
                    boolean _includesAllRequiredFeaturesFor = _view==null?false:_view.includesAllRequiredFeaturesFor(((Entity) _dataType));
                    _and = (_and_1 && _includesAllRequiredFeaturesFor);
                  }
                  return Boolean.valueOf(_and);
                }
              };
            final Iterable<FeatureReference> toManyReferences = IterableExtensions.<FeatureReference>filter(_featureReferences, _function);
            final EList<Attribute> optionalReferences = entity==null?(EList<Attribute>)null:EntityExtension.getAllNonRequiredReferenceNonManyAttributesRecursive(entity);
            StringConcatenation _builder = new StringConcatenation();
            _builder.append("@");
            String _addImport = this._names.addImport(it, "org.springframework.transaction.annotation.Transactional");
            _builder.append(_addImport, "");
            _builder.newLineIfNotEmpty();
            CharSequence _operationSignature = this._features.toOperationSignature(it, "public");
            _builder.append(_operationSignature, "");
            _builder.append(" {");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            String _addImport_1 = this._names.addImport(it, "org.springframework.util.Assert");
            _builder.append(_addImport_1, "	");
            _builder.append(".notNull(");
            String _variableName = this._names.toVariableName(viewParameter);
            _builder.append(_variableName, "	");
            _builder.append(", \"parameter \'");
            String _variableName_1 = this._names.toVariableName(viewParameter);
            _builder.append(_variableName_1, "	");
            _builder.append("\' must not be null\");");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            String _qualifiedName = this._names.getQualifiedName(entity);
            _builder.append(_qualifiedName, "	");
            _builder.append(" ");
            String _variableName_2 = this._names.toVariableName(entity);
            _builder.append(_variableName_2, "	");
            _builder.append(" = this.mapper.createAndMapOne(");
            String _variableName_3 = this._names.toVariableName(viewParameter);
            _builder.append(_variableName_3, "	");
            _builder.append(", ");
            String _qualifiedName_1 = this._names.getQualifiedName(entity);
            _builder.append(_qualifiedName_1, "	");
            _builder.append(".class,\"save");
            String _name = entity.getName();
            _builder.append(_name, "	");
            _builder.append("\");");
            _builder.newLineIfNotEmpty();
            {
              EList<Attribute> _requiredReferences = entity.getRequiredReferences();
              for(final Attribute requiredReference : _requiredReferences) {
                _builder.append("\t");
                Type _dataType = requiredReference.getDataType();
                String _qualifiedName_2 = this._names.getQualifiedName(_dataType);
                _builder.append(_qualifiedName_2, "	");
                _builder.append(" ");
                String _variableName_4 = this._names.toVariableName(requiredReference);
                _builder.append(_variableName_4, "	");
                _builder.append(" = this.mapper.createAndMapOne(");
                String _variableName_5 = this._names.toVariableName(viewParameter);
                _builder.append(_variableName_5, "	");
                _builder.append(", ");
                Type _dataType_1 = requiredReference.getDataType();
                String _name_1 = _dataType_1.getName();
                _builder.append(_name_1, "	");
                _builder.append(".class,\"");
                String _name_2 = requiredReference.getName();
                _builder.append(_name_2, "	");
                _builder.append("\");");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                String _variableName_6 = this._names.toVariableName(requiredReference);
                _builder.append(_variableName_6, "	");
                _builder.append(" = ");
                Type _dataType_2 = requiredReference.getDataType();
                Dao _repository = ((Entity) _dataType_2).getRepository();
                String _variableName_7 = this._names.toVariableName(_repository);
                _builder.append(_variableName_7, "	");
                _builder.append(".load(");
                String _variableName_8 = this._names.toVariableName(requiredReference);
                _builder.append(_variableName_8, "	");
                _builder.append(".");
                String _idOrKeyAccessorMethodName = this.getIdOrKeyAccessorMethodName(requiredReference);
                _builder.append(_idOrKeyAccessorMethodName, "	");
                _builder.append("());");
                _builder.newLineIfNotEmpty();
                _builder.append("\t");
                String _variableName_9 = this._names.toVariableName(entity);
                _builder.append(_variableName_9, "	");
                _builder.append(".");
                String _mutatorMethodName = this._features.getMutatorMethodName(requiredReference);
                _builder.append(_mutatorMethodName, "	");
                _builder.append("(");
                String _variableName_10 = this._names.toVariableName(requiredReference);
                _builder.append(_variableName_10, "	");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
              }
            }
            {
              for(final Attribute optionalReference : optionalReferences) {
                _builder.append("\t");
                EList<FeatureReference> _featureReferences_1 = viewParameter.getFeatureReferences();
                final Function1<FeatureReference,Boolean> _function_1 = new Function1<FeatureReference,Boolean>() {
                    public Boolean apply(final FeatureReference e) {
                      Attribute _attribute = e.getAttribute();
                      boolean _equals = Objects.equal(_attribute, optionalReference);
                      return Boolean.valueOf(_equals);
                    }
                  };
                final FeatureReference target = IterableExtensions.<FeatureReference>findFirst(_featureReferences_1, _function_1);
                _builder.newLineIfNotEmpty();
                {
                  boolean _notEquals = (!Objects.equal(target, null));
                  if (_notEquals) {
                    _builder.append("\t");
                    _builder.append("if (null != ");
                    String _variableName_11 = this._names.toVariableName(viewParameter);
                    _builder.append(_variableName_11, "	");
                    _builder.append(".");
                    String _accessorMethodName = this._features.getAccessorMethodName(target);
                    _builder.append(_accessorMethodName, "	");
                    _builder.append("()) {");
                    _builder.newLineIfNotEmpty();
                    _builder.append("\t");
                    _builder.append("\t");
                    Type _dataType_3 = optionalReference.getDataType();
                    String _qualifiedName_3 = this._names.getQualifiedName(_dataType_3);
                    _builder.append(_qualifiedName_3, "		");
                    _builder.append(" optional");
                    String _name_3 = optionalReference.getName();
                    String _firstUpper = StringExtensions.toFirstUpper(_name_3);
                    _builder.append(_firstUpper, "		");
                    _builder.append(" = this.mapper.createAndMapOne(");
                    String _variableName_12 = this._names.toVariableName(viewParameter);
                    _builder.append(_variableName_12, "		");
                    _builder.append(".");
                    String _accessorMethodName_1 = this._features.getAccessorMethodName(target);
                    _builder.append(_accessorMethodName_1, "		");
                    _builder.append("(), ");
                    Type _dataType_4 = optionalReference.getDataType();
                    String _name_4 = _dataType_4.getName();
                    _builder.append(_name_4, "		");
                    _builder.append(".class,\"");
                    String _name_5 = optionalReference.getName();
                    _builder.append(_name_5, "		");
                    _builder.append("\");");
                    _builder.newLineIfNotEmpty();
                    _builder.append("\t");
                    _builder.append("\t");
                    _builder.append("optional");
                    String _name_6 = optionalReference.getName();
                    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_6);
                    _builder.append(_firstUpper_1, "		");
                    _builder.append(" = ");
                    Type _dataType_5 = optionalReference.getDataType();
                    Dao _repository_1 = ((Entity) _dataType_5).getRepository();
                    String _variableName_13 = this._names.toVariableName(_repository_1);
                    _builder.append(_variableName_13, "		");
                    _builder.append(".load(optional");
                    String _name_7 = optionalReference.getName();
                    String _firstUpper_2 = StringExtensions.toFirstUpper(_name_7);
                    _builder.append(_firstUpper_2, "		");
                    _builder.append(".");
                    String _idOrKeyAccessorMethodName_1 = this.getIdOrKeyAccessorMethodName(optionalReference);
                    _builder.append(_idOrKeyAccessorMethodName_1, "		");
                    _builder.append("());");
                    _builder.newLineIfNotEmpty();
                    _builder.append("\t");
                    _builder.append("\t");
                    String _variableName_14 = this._names.toVariableName(entity);
                    _builder.append(_variableName_14, "		");
                    _builder.append(".");
                    String _mutatorMethodName_1 = this._features.getMutatorMethodName(optionalReference);
                    _builder.append(_mutatorMethodName_1, "		");
                    _builder.append("(optional");
                    String _name_8 = optionalReference.getName();
                    String _firstUpper_3 = StringExtensions.toFirstUpper(_name_8);
                    _builder.append(_firstUpper_3, "		");
                    _builder.append(");");
                    _builder.newLineIfNotEmpty();
                    _builder.append("\t");
                    _builder.append("}");
                    _builder.newLine();
                  }
                }
              }
            }
            {
              for(final FeatureReference includedFeature : toManyReferences) {
                _builder.append("\t");
                _builder.append("add");
                Attribute _attribute = includedFeature.getAttribute();
                String _name_9 = _attribute.getName();
                String _firstUpper_4 = StringExtensions.toFirstUpper(_name_9);
                _builder.append(_firstUpper_4, "	");
                _builder.append("(");
                String _variableName_15 = this._names.toVariableName(entity);
                _builder.append(_variableName_15, "	");
                _builder.append(",");
                String _variableName_16 = this._names.toVariableName(viewParameter);
                _builder.append(_variableName_16, "	");
                _builder.append(");");
                _builder.newLineIfNotEmpty();
              }
            }
            _builder.append("\t");
            _builder.append("this.");
            String _variableName_17 = this._names.toVariableName(repository);
            _builder.append(_variableName_17, "	");
            _builder.append(".saveOrUpdate(");
            String _variableName_18 = this._names.toVariableName(entity);
            _builder.append(_variableName_18, "	");
            _builder.append(");");
            _builder.newLineIfNotEmpty();
            _builder.append("\t");
            _builder.append("return this.mapper.mapOne(");
            String _variableName_19 = this._names.toVariableName(entity);
            _builder.append(_variableName_19, "	");
            _builder.append(",new ");
            Type _type = it.getType();
            String _name_10 = _type.getName();
            _builder.append(_name_10, "	");
            _builder.append("());");
            _builder.newLineIfNotEmpty();
            _builder.append("}");
            _builder.newLine();
            final Set<Attribute> processedAttributes = Sets.<Attribute>newHashSet();
            _builder.newLineIfNotEmpty();
            {
              for(final FeatureReference includedFeature_1 : toManyReferences) {
                String _addReferenceMethod = this.toAddReferenceMethod(includedFeature_1, viewParameter, processedAttributes);
                _builder.append(_addReferenceMethod, "");
                _builder.newLineIfNotEmpty();
              }
            }
            _xblockexpression_1 = (_builder);
          }
          _switchResult = _xblockexpression_1;
        }
      }
      if (!_matched) {
        if (Objects.equal(crudOperationType,CrudOperationType.READ)) {
          _matched=true;
          StringConcatenation _builder = new StringConcatenation();
          _builder.append("@");
          String _addImport = this._names.addImport(it, "org.springframework.transaction.annotation.Transactional");
          _builder.append(_addImport, "");
          _builder.append("(readOnly = true)");
          _builder.newLineIfNotEmpty();
          CharSequence _operationSignature = this._features.toOperationSignature(it, "public");
          _builder.append(_operationSignature, "");
          _builder.append(" {");
          _builder.newLineIfNotEmpty();
          {
            Attribute _identifier = entity==null?(Attribute)null:entity.getIdentifier();
            boolean _notEquals = (!Objects.equal(_identifier, null));
            if (_notEquals) {
              _builder.append("\t");
              String _addImport_1 = this._names.addImport(it, "org.springframework.util.Assert");
              _builder.append(_addImport_1, "	");
              _builder.append(".notNull(");
              Attribute _identifier_1 = entity.getIdentifier();
              String _name = _identifier_1.getName();
              _builder.append(_name, "	");
              _builder.append(", \"parameter \'");
              Attribute _identifier_2 = entity.getIdentifier();
              String _name_1 = _identifier_2.getName();
              _builder.append(_name_1, "	");
              _builder.append("\' must not be null\");");
              _builder.newLineIfNotEmpty();
              _builder.append("\t");
              _builder.append("return this.mapper.createAndMapOne(this.");
              String _variableName = this._names.toVariableName(repository);
              _builder.append(_variableName, "	");
              _builder.append(".load(");
              Attribute _identifier_3 = entity.getIdentifier();
              String _name_2 = _identifier_3.getName();
              _builder.append(_name_2, "	");
              _builder.append("),");
              Type _type = it.getType();
              String _name_3 = _type.getName();
              _builder.append(_name_3, "	");
              _builder.append(".class);");
              _builder.newLineIfNotEmpty();
            } else {
              AttributeGroup _key = entity==null?(AttributeGroup)null:entity.getKey();
              boolean _notEquals_1 = (!Objects.equal(_key, null));
              if (_notEquals_1) {
                {
                  AttributeGroup _key_1 = entity.getKey();
                  EList<AttributeSortOrder> _attributes = _key_1.getAttributes();
                  final Function1<AttributeSortOrder,Attribute> _function = new Function1<AttributeSortOrder,Attribute>() {
                      public Attribute apply(final AttributeSortOrder e) {
                        Attribute _attribute = e.getAttribute();
                        return _attribute;
                      }
                    };
                  List<Attribute> _map = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes, _function);
                  Set<Attribute> _set = IterableExtensions.<Attribute>toSet(_map);
                  for(final Attribute attribute : _set) {
                    _builder.append("\t");
                    String _addImport_2 = this._names.addImport(it, "org.springframework.util.Assert");
                    _builder.append(_addImport_2, "	");
                    _builder.append(".notNull(");
                    String _name_4 = attribute.getName();
                    _builder.append(_name_4, "	");
                    _builder.append(", \"parameter \'");
                    String _name_5 = attribute.getName();
                    _builder.append(_name_5, "	");
                    _builder.append("\' must not be null\");");
                    _builder.newLineIfNotEmpty();
                  }
                }
                _builder.append("\t");
                _builder.append("return this.mapper.createAndMapOne(this.");
                String _variableName_1 = this._names.toVariableName(repository);
                _builder.append(_variableName_1, "	");
                _builder.append(".load(new ");
                AttributeGroup _key_2 = entity.getKey();
                String _qualifiedKeyClassName = this._features.getQualifiedKeyClassName(_key_2);
                String _addImport_3 = this._names.addImport(it, _qualifiedKeyClassName);
                _builder.append(_addImport_3, "	");
                _builder.append("(");
                {
                  AttributeGroup _key_3 = entity.getKey();
                  EList<AttributeSortOrder> _attributes_1 = _key_3.getAttributes();
                  final Function1<AttributeSortOrder,Attribute> _function_1 = new Function1<AttributeSortOrder,Attribute>() {
                      public Attribute apply(final AttributeSortOrder e) {
                        Attribute _attribute = e.getAttribute();
                        return _attribute;
                      }
                    };
                  List<Attribute> _map_1 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_1, _function_1);
                  Set<Attribute> _set_1 = IterableExtensions.<Attribute>toSet(_map_1);
                  boolean _hasElements = false;
                  for(final Attribute field : _set_1) {
                    if (!_hasElements) {
                      _hasElements = true;
                    } else {
                      _builder.appendImmediate(",", "	");
                    }
                    String _name_6 = field.getName();
                    _builder.append(_name_6, "	");
                  }
                }
                _builder.append(")),");
                Type _type_1 = it.getType();
                String _name_7 = _type_1.getName();
                _builder.append(_name_7, "	");
                _builder.append(".class);");
                _builder.newLineIfNotEmpty();
              }
            }
          }
          _builder.append("}");
          _builder.newLine();
          _switchResult = _builder;
        }
      }
      if (!_matched) {
        if (Objects.equal(crudOperationType,CrudOperationType.UPDATE)) {
          _matched=true;
          StringConcatenation _builder_1 = new StringConcatenation();
          _builder_1.append("@");
          String _addImport_4 = this._names.addImport(it, "org.springframework.transaction.annotation.Transactional");
          _builder_1.append(_addImport_4, "");
          _builder_1.newLineIfNotEmpty();
          CharSequence _operationSignature_1 = this._features.toOperationSignature(it, "public");
          _builder_1.append(_operationSignature_1, "");
          _builder_1.append(" {");
          _builder_1.newLineIfNotEmpty();
          _builder_1.append("\t");
          String _addImport_5 = this._names.addImport(it, "org.springframework.util.Assert");
          _builder_1.append(_addImport_5, "	");
          _builder_1.append(".notNull(");
          String _variableName_2 = this._names.toVariableName(viewParameter);
          _builder_1.append(_variableName_2, "	");
          _builder_1.append(", \"parameter \'");
          String _variableName_3 = this._names.toVariableName(viewParameter);
          _builder_1.append(_variableName_3, "	");
          _builder_1.append("\' must not be null\");");
          _builder_1.newLineIfNotEmpty();
          {
            Attribute _identifier_4 = entity==null?(Attribute)null:entity.getIdentifier();
            boolean _notEquals_2 = (!Objects.equal(_identifier_4, null));
            if (_notEquals_2) {
              _builder_1.append("\t");
              String _qualifiedName = this._names.getQualifiedName(entity);
              _builder_1.append(_qualifiedName, "	");
              _builder_1.append(" mapped");
              String _name_8 = entity.getName();
              _builder_1.append(_name_8, "	");
              _builder_1.append(" = this.mapper.createAndMapOne(");
              String _variableName_4 = this._names.toVariableName(viewParameter);
              _builder_1.append(_variableName_4, "	");
              _builder_1.append(", ");
              String _name_9 = entity.getName();
              _builder_1.append(_name_9, "	");
              _builder_1.append(".class);");
              _builder_1.newLineIfNotEmpty();
              _builder_1.append("\t");
              String _name_10 = entity.getName();
              _builder_1.append(_name_10, "	");
              _builder_1.append(" ");
              String _variableName_5 = this._names.toVariableName(entity);
              _builder_1.append(_variableName_5, "	");
              _builder_1.append(" = this.");
              String _variableName_6 = this._names.toVariableName(repository);
              _builder_1.append(_variableName_6, "	");
              _builder_1.append(".load(mapped");
              String _name_11 = entity.getName();
              _builder_1.append(_name_11, "	");
              _builder_1.append(".");
              Attribute _identifier_5 = entity.getIdentifier();
              String _accessorMethodName = this._features.getAccessorMethodName(_identifier_5);
              _builder_1.append(_accessorMethodName, "	");
              _builder_1.append("());");
              _builder_1.newLineIfNotEmpty();
              {
                boolean _and = false;
                Attribute _version = entity.getVersion();
                boolean _notEquals_3 = (!Objects.equal(_version, null));
                if (!_notEquals_3) {
                  _and = false;
                } else {
                  boolean _includesVersionFor = viewParameter.includesVersionFor(entity);
                  _and = (_notEquals_3 && _includesVersionFor);
                }
                if (_and) {
                  _builder_1.append("\t");
                  _builder_1.append("if (!");
                  String _variableName_7 = this._names.toVariableName(entity);
                  _builder_1.append(_variableName_7, "	");
                  _builder_1.append(".");
                  Attribute _version_1 = entity.getVersion();
                  String _accessorMethodName_1 = this._features.getAccessorMethodName(_version_1);
                  _builder_1.append(_accessorMethodName_1, "	");
                  _builder_1.append("().equals(mapped");
                  String _name_12 = entity.getName();
                  _builder_1.append(_name_12, "	");
                  _builder_1.append(".");
                  Attribute _version_2 = entity.getVersion();
                  String _accessorMethodName_2 = this._features.getAccessorMethodName(_version_2);
                  _builder_1.append(_accessorMethodName_2, "	");
                  _builder_1.append("())) {");
                  _builder_1.newLineIfNotEmpty();
                  _builder_1.append("\t");
                  _builder_1.append("\t");
                  _builder_1.append("throw new ");
                  String _addImport_6 = this._names.addImport(it, "org.springframework.dao.OptimisticLockingFailureException");
                  _builder_1.append(_addImport_6, "		");
                  _builder_1.append("(\"Entity \'");
                  String _name_13 = entity.getName();
                  _builder_1.append(_name_13, "		");
                  _builder_1.append("\' was updated or deleted by another transaction.\");");
                  _builder_1.newLineIfNotEmpty();
                  _builder_1.append("\t");
                  _builder_1.append("}");
                  _builder_1.newLine();
                }
              }
            } else {
              _builder_1.append("\t");
              String _qualifiedName_1 = this._names.getQualifiedName(entity);
              _builder_1.append(_qualifiedName_1, "	");
              _builder_1.append(" ");
              String _variableName_8 = this._names.toVariableName(entity);
              _builder_1.append(_variableName_8, "	");
              _builder_1.append(" = this.");
              String _variableName_9 = this._names.toVariableName(repository);
              _builder_1.append(_variableName_9, "	");
              _builder_1.append(".");
              DataBaseConstraint _naturalKey = repository.getNaturalKey();
              String _methodName = this._features.getMethodName(_naturalKey);
              _builder_1.append(_methodName, "	");
              _builder_1.append("(");
              {
                AttributeGroup _key_4 = entity.getKey();
                EList<AttributeSortOrder> _attributes_2 = _key_4.getAttributes();
                final Function1<AttributeSortOrder,Attribute> _function_2 = new Function1<AttributeSortOrder,Attribute>() {
                    public Attribute apply(final AttributeSortOrder e) {
                      Attribute _attribute = e.getAttribute();
                      return _attribute;
                    }
                  };
                List<Attribute> _map_2 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_2, _function_2);
                Set<Attribute> _set_2 = IterableExtensions.<Attribute>toSet(_map_2);
                boolean _hasElements_1 = false;
                for(final Attribute field_1 : _set_2) {
                  if (!_hasElements_1) {
                    _hasElements_1 = true;
                  } else {
                    _builder_1.appendImmediate(",", "	");
                  }
                  String _variableName_10 = this._names.toVariableName(viewParameter);
                  _builder_1.append(_variableName_10, "	");
                  _builder_1.append(".");
                  String _accessorMethodName_3 = this._features.getAccessorMethodName(field_1);
                  _builder_1.append(_accessorMethodName_3, "	");
                  _builder_1.append("()");
                }
              }
              _builder_1.append(");");
              _builder_1.newLineIfNotEmpty();
            }
          }
          _builder_1.append("\t");
          _builder_1.append("this.mapper.mapOne(");
          String _variableName_11 = this._names.toVariableName(viewParameter);
          _builder_1.append(_variableName_11, "	");
          _builder_1.append(", ");
          String _variableName_12 = this._names.toVariableName(entity);
          _builder_1.append(_variableName_12, "	");
          _builder_1.append(",\"update");
          String _name_14 = entity.getName();
          _builder_1.append(_name_14, "	");
          _builder_1.append("\");");
          _builder_1.newLineIfNotEmpty();
          _builder_1.append("}");
          _builder_1.newLine();
          _switchResult = _builder_1;
        }
      }
      if (!_matched) {
        if (Objects.equal(crudOperationType,CrudOperationType.DELETE)) {
          _matched=true;
          StringConcatenation _builder_2 = new StringConcatenation();
          _builder_2.append("@");
          String _addImport_7 = this._names.addImport(it, "org.springframework.transaction.annotation.Transactional");
          _builder_2.append(_addImport_7, "");
          _builder_2.newLineIfNotEmpty();
          CharSequence _operationSignature_2 = this._features.toOperationSignature(it, "public");
          _builder_2.append(_operationSignature_2, "");
          _builder_2.append(" {");
          _builder_2.newLineIfNotEmpty();
          {
            Attribute _identifier_6 = entity==null?(Attribute)null:entity.getIdentifier();
            boolean _notEquals_4 = (!Objects.equal(_identifier_6, null));
            if (_notEquals_4) {
              _builder_2.append("\t");
              String _addImport_8 = this._names.addImport(it, "org.springframework.util.Assert");
              _builder_2.append(_addImport_8, "	");
              _builder_2.append(".notNull(");
              Attribute _identifier_7 = entity.getIdentifier();
              String _name_15 = _identifier_7.getName();
              _builder_2.append(_name_15, "	");
              _builder_2.append(", \"parameter \'");
              Attribute _identifier_8 = entity.getIdentifier();
              String _name_16 = _identifier_8.getName();
              _builder_2.append(_name_16, "	");
              _builder_2.append("\' must not be null\");");
              _builder_2.newLineIfNotEmpty();
              _builder_2.append("\t");
              _builder_2.append("this.");
              String _variableName_13 = this._names.toVariableName(repository);
              _builder_2.append(_variableName_13, "	");
              _builder_2.append(".delete(");
              Attribute _identifier_9 = entity.getIdentifier();
              String _name_17 = _identifier_9.getName();
              _builder_2.append(_name_17, "	");
              _builder_2.append(");");
              _builder_2.newLineIfNotEmpty();
            } else {
              AttributeGroup _key_5 = entity==null?(AttributeGroup)null:entity.getKey();
              boolean _notEquals_5 = (!Objects.equal(_key_5, null));
              if (_notEquals_5) {
                {
                  AttributeGroup _key_6 = entity.getKey();
                  EList<AttributeSortOrder> _attributes_3 = _key_6.getAttributes();
                  final Function1<AttributeSortOrder,Attribute> _function_3 = new Function1<AttributeSortOrder,Attribute>() {
                      public Attribute apply(final AttributeSortOrder e) {
                        Attribute _attribute = e.getAttribute();
                        return _attribute;
                      }
                    };
                  List<Attribute> _map_3 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_3, _function_3);
                  Set<Attribute> _set_3 = IterableExtensions.<Attribute>toSet(_map_3);
                  for(final Attribute field_2 : _set_3) {
                    _builder_2.append("\t");
                    String _addImport_9 = this._names.addImport(it, "org.springframework.util.Assert");
                    _builder_2.append(_addImport_9, "	");
                    _builder_2.append(".notNull(");
                    String _name_18 = field_2.getName();
                    _builder_2.append(_name_18, "	");
                    _builder_2.append(", \"parameter \'");
                    String _name_19 = field_2.getName();
                    _builder_2.append(_name_19, "	");
                    _builder_2.append("\' must not be null\");");
                    _builder_2.newLineIfNotEmpty();
                  }
                }
                _builder_2.append("\t");
                _builder_2.append("this.");
                String _variableName_14 = this._names.toVariableName(repository);
                _builder_2.append(_variableName_14, "	");
                _builder_2.append(".delete(new ");
                AttributeGroup _key_7 = entity.getKey();
                String _qualifiedKeyClassName_1 = this._features.getQualifiedKeyClassName(_key_7);
                String _addImport_10 = this._names.addImport(it, _qualifiedKeyClassName_1);
                _builder_2.append(_addImport_10, "	");
                _builder_2.append("(");
                {
                  AttributeGroup _key_8 = entity.getKey();
                  EList<AttributeSortOrder> _attributes_4 = _key_8.getAttributes();
                  final Function1<AttributeSortOrder,Attribute> _function_4 = new Function1<AttributeSortOrder,Attribute>() {
                      public Attribute apply(final AttributeSortOrder e) {
                        Attribute _attribute = e.getAttribute();
                        return _attribute;
                      }
                    };
                  List<Attribute> _map_4 = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes_4, _function_4);
                  Set<Attribute> _set_4 = IterableExtensions.<Attribute>toSet(_map_4);
                  boolean _hasElements_2 = false;
                  for(final Attribute field_3 : _set_4) {
                    if (!_hasElements_2) {
                      _hasElements_2 = true;
                    } else {
                      _builder_2.appendImmediate(",", "	");
                    }
                    String _name_20 = field_3.getName();
                    _builder_2.append(_name_20, "	");
                  }
                }
                _builder_2.append("));");
                _builder_2.newLineIfNotEmpty();
              }
            }
          }
          _builder_2.append("}");
          _builder_2.newLine();
          _switchResult = _builder_2;
        }
      }
      _xblockexpression = (_switchResult);
    }
    return _xblockexpression;
  }
  
  private CharSequence toCrudSaveOrUpdateImplementation(final Operation it) {
    CharSequence _xblockexpression = null;
    {
      DelegateOperation _delegate = it.getDelegate();
      final Dao repository = _delegate.getRepository();
      DelegateOperation _delegate_1 = it.getDelegate();
      final DataView view = _delegate_1.getView();
      DelegateOperation _delegate_2 = it.getDelegate();
      final DataView viewParameter = _delegate_2.getViewParameter();
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("@");
      String _addImport = this._names.addImport(it, "org.springframework.transaction.annotation.Transactional");
      _builder.append(_addImport, "");
      _builder.newLineIfNotEmpty();
      _builder.append("public ");
      String _name = view.getName();
      _builder.append(_name, "");
      _builder.append(" saveOrUpdate(");
      String _name_1 = viewParameter.getName();
      _builder.append(_name_1, "");
      _builder.append(" ");
      String _variableName = this._names.toVariableName(viewParameter);
      _builder.append(_variableName, "");
      _builder.append(") {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      String _addImport_1 = this._names.addImport(it, "org.springframework.util.Assert");
      _builder.append(_addImport_1, "	");
      _builder.append(".notNull(");
      String _variableName_1 = this._names.toVariableName(viewParameter);
      _builder.append(_variableName_1, "	");
      _builder.append(", \"parameter \'");
      String _variableName_2 = this._names.toVariableName(viewParameter);
      _builder.append(_variableName_2, "	");
      _builder.append("\' must not be null\");");
      _builder.newLineIfNotEmpty();
      {
        Entity _entity = repository.getEntity();
        Attribute _version = _entity.getVersion();
        boolean _notEquals = (!Objects.equal(_version, null));
        if (_notEquals) {
          _builder.append("\t");
          _builder.append("if ( ");
          String _variableName_3 = this._names.toVariableName(viewParameter);
          _builder.append(_variableName_3, "	");
          _builder.append(".");
          Entity _entity_1 = repository.getEntity();
          Attribute _version_1 = _entity_1.getVersion();
          String _accessorMethodName = this._features.getAccessorMethodName(_version_1);
          _builder.append(_accessorMethodName, "	");
          _builder.append("() == null ) {");
          _builder.newLineIfNotEmpty();
        } else {
          _builder.append("\t");
          _builder.append("if ( ");
          String _variableName_4 = this._names.toVariableName(viewParameter);
          _builder.append(_variableName_4, "	");
          _builder.append(".");
          Entity _entity_2 = repository.getEntity();
          Attribute _identifier = _entity_2.getIdentifier();
          String _accessorMethodName_1 = this._features.getAccessorMethodName(_identifier);
          _builder.append(_accessorMethodName_1, "	");
          _builder.append("() == null ) {");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("\t\t");
      _builder.append("return this.save(");
      String _variableName_5 = this._names.toVariableName(viewParameter);
      _builder.append(_variableName_5, "		");
      _builder.append(");");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("else {");
      _builder.newLine();
      _builder.append("\t\t");
      _builder.append("this.update(");
      String _variableName_6 = this._names.toVariableName(viewParameter);
      _builder.append(_variableName_6, "		");
      _builder.append(");");
      _builder.newLineIfNotEmpty();
      {
        boolean _equals = Objects.equal(view, viewParameter);
        if (_equals) {
          _builder.append("\t\t");
          _builder.append("return ");
          String _variableName_7 = this._names.toVariableName(viewParameter);
          _builder.append(_variableName_7, "		");
          _builder.append(";");
          _builder.newLineIfNotEmpty();
        } else {
          {
            Entity _entity_3 = repository.getEntity();
            Attribute _identifier_1 = _entity_3.getIdentifier();
            boolean _notEquals_1 = (!Objects.equal(_identifier_1, null));
            if (_notEquals_1) {
              _builder.append("\t\t");
              Entity _entity_4 = repository.getEntity();
              String _qualifiedName = this._names.getQualifiedName(_entity_4);
              _builder.append(_qualifiedName, "		");
              _builder.append(" mapped");
              Entity _entity_5 = repository.getEntity();
              String _name_2 = _entity_5.getName();
              _builder.append(_name_2, "		");
              _builder.append(" = this.mapper.createAndMapOne(");
              String _variableName_8 = this._names.toVariableName(viewParameter);
              _builder.append(_variableName_8, "		");
              _builder.append(", ");
              Entity _entity_6 = repository.getEntity();
              String _name_3 = _entity_6.getName();
              _builder.append(_name_3, "		");
              _builder.append(".class);");
              _builder.newLineIfNotEmpty();
              _builder.append("\t\t");
              Entity _entity_7 = repository.getEntity();
              String _name_4 = _entity_7.getName();
              _builder.append(_name_4, "		");
              _builder.append(" ");
              Entity _entity_8 = repository.getEntity();
              String _variableName_9 = this._names.toVariableName(_entity_8);
              _builder.append(_variableName_9, "		");
              _builder.append(" = this.");
              String _variableName_10 = this._names.toVariableName(repository);
              _builder.append(_variableName_10, "		");
              _builder.append(".load(mapped");
              Entity _entity_9 = repository.getEntity();
              String _name_5 = _entity_9.getName();
              _builder.append(_name_5, "		");
              _builder.append(".");
              Entity _entity_10 = repository.getEntity();
              Attribute _identifier_2 = _entity_10.getIdentifier();
              String _accessorMethodName_2 = this._features.getAccessorMethodName(_identifier_2);
              _builder.append(_accessorMethodName_2, "		");
              _builder.append("());");
              _builder.newLineIfNotEmpty();
            } else {
              _builder.append("\t\t");
              Entity _entity_11 = repository.getEntity();
              String _qualifiedName_1 = this._names.getQualifiedName(_entity_11);
              _builder.append(_qualifiedName_1, "		");
              _builder.append(" ");
              Entity _entity_12 = repository.getEntity();
              String _variableName_11 = this._names.toVariableName(_entity_12);
              _builder.append(_variableName_11, "		");
              _builder.append(" = this.");
              String _variableName_12 = this._names.toVariableName(repository);
              _builder.append(_variableName_12, "		");
              _builder.append(".");
              DataBaseConstraint _naturalKey = repository.getNaturalKey();
              String _methodName = this._features.getMethodName(_naturalKey);
              _builder.append(_methodName, "		");
              _builder.append("(");
              {
                Entity _entity_13 = repository.getEntity();
                AttributeGroup _key = _entity_13.getKey();
                EList<AttributeSortOrder> _attributes = _key.getAttributes();
                final Function1<AttributeSortOrder,Attribute> _function = new Function1<AttributeSortOrder,Attribute>() {
                    public Attribute apply(final AttributeSortOrder e) {
                      Attribute _attribute = e.getAttribute();
                      return _attribute;
                    }
                  };
                List<Attribute> _map = ListExtensions.<AttributeSortOrder, Attribute>map(_attributes, _function);
                Set<Attribute> _set = IterableExtensions.<Attribute>toSet(_map);
                boolean _hasElements = false;
                for(final Attribute field : _set) {
                  if (!_hasElements) {
                    _hasElements = true;
                  } else {
                    _builder.appendImmediate(",", "		");
                  }
                  String _variableName_13 = this._names.toVariableName(viewParameter);
                  _builder.append(_variableName_13, "		");
                  _builder.append(".");
                  String _accessorMethodName_3 = this._features.getAccessorMethodName(field);
                  _builder.append(_accessorMethodName_3, "		");
                  _builder.append("()");
                }
              }
              _builder.append(");");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t\t");
          _builder.append("this.mapper.mapOne(");
          String _variableName_14 = this._names.toVariableName(viewParameter);
          _builder.append(_variableName_14, "		");
          _builder.append(", ");
          Entity _entity_14 = repository.getEntity();
          String _variableName_15 = this._names.toVariableName(_entity_14);
          _builder.append(_variableName_15, "		");
          _builder.append(",\"update");
          Entity _entity_15 = repository.getEntity();
          String _name_6 = _entity_15.getName();
          _builder.append(_name_6, "		");
          _builder.append("\");");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t");
          _builder.append("return this.mapper.createAndMapOne(");
          Entity _entity_16 = repository.getEntity();
          String _variableName_16 = this._names.toVariableName(_entity_16);
          _builder.append(_variableName_16, "		");
          _builder.append(",");
          String _name_7 = view.getName();
          _builder.append(_name_7, "		");
          _builder.append(".class);");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      _xblockexpression = (_builder);
    }
    return _xblockexpression;
  }
  
  private CharSequence toQueryOperationImplementation(final Operation it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("@");
    String _addImport = this._names.addImport(it, "org.springframework.transaction.annotation.Transactional");
    _builder.append(_addImport, "");
    {
      boolean _and = false;
      boolean _and_1 = false;
      DelegateOperation _delegate = it.getDelegate();
      boolean _notEquals = (!Objects.equal(_delegate, null));
      if (!_notEquals) {
        _and_1 = false;
      } else {
        DelegateOperation _delegate_1 = it.getDelegate();
        DaoOperation _operation = _delegate_1.getOperation();
        _and_1 = (_notEquals && (_operation instanceof QueryOperation));
      }
      if (!_and_1) {
        _and = false;
      } else {
        DelegateOperation _delegate_2 = it.getDelegate();
        DaoOperation _operation_1 = _delegate_2.getOperation();
        QlStatement _statement = ((QueryOperation) _operation_1).getStatement();
        _and = (_and_1 && (_statement instanceof SelectStatement));
      }
      if (_and) {
        _builder.append("(readOnly=true)");
      }
    }
    _builder.newLineIfNotEmpty();
    CharSequence _operationSignature = this._features.toOperationSignature(it, "public");
    _builder.append(_operationSignature, "");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    {
      boolean _and_2 = false;
      boolean _and_3 = false;
      boolean _and_4 = false;
      DelegateOperation _delegate_3 = it.getDelegate();
      Expression _filter = _delegate_3.getFilter();
      boolean _notEquals_1 = (!Objects.equal(_filter, null));
      if (!_notEquals_1) {
        _and_4 = false;
      } else {
        DelegateOperation _delegate_4 = it.getDelegate();
        DaoOperation _operation_2 = _delegate_4.getOperation();
        _and_4 = (_notEquals_1 && (_operation_2 instanceof QueryOperation));
      }
      if (!_and_4) {
        _and_3 = false;
      } else {
        EList<Parameter> _parameters = it.getParameters();
        boolean _isEmpty = _parameters.isEmpty();
        boolean _not = (!_isEmpty);
        _and_3 = (_and_4 && _not);
      }
      if (!_and_3) {
        _and_2 = false;
      } else {
        EList<Parameter> _parameters_1 = it.getParameters();
        Parameter _last = IterableExtensions.<Parameter>last(_parameters_1);
        String _name = _last.getName();
        boolean _equalsIgnoreCase = _name.equalsIgnoreCase("_filter");
        _and_2 = (_and_3 && _equalsIgnoreCase);
      }
      if (_and_2) {
        _builder.append("\t");
        String _addImport_1 = this._names.addImport(it, "org.openxma.dsl.platform.query.QueryObject");
        _builder.append(_addImport_1, "	");
        _builder.append(" queryObject = QueryObject.createNamedQuery(");
        DelegateOperation _delegate_5 = it.getDelegate();
        DaoOperation _operation_3 = _delegate_5.getOperation();
        String _qualifiedQueryConstant = this._features.getQualifiedQueryConstant(((QueryOperation) _operation_3));
        _builder.append(_qualifiedQueryConstant, "	");
        _builder.append(", _filter);");
        _builder.newLineIfNotEmpty();
        {
          DelegateOperation _delegate_6 = it.getDelegate();
          DaoOperation _operation_4 = _delegate_6.getOperation();
          EList<QueryParameter> _queryParameters = ((QueryOperation) _operation_4).getQueryParameters();
          final Function1<QueryParameter,Boolean> _function = new Function1<QueryParameter,Boolean>() {
              public Boolean apply(final QueryParameter e) {
                boolean _and = false;
                if (!(e instanceof Parameter)) {
                  _and = false;
                } else {
                  boolean _or = false;
                  String _name = ((Parameter) e).getName();
                  boolean _equals = _name.equals("firstResult");
                  if (_equals) {
                    _or = true;
                  } else {
                    String _name_1 = ((Parameter) e).getName();
                    boolean _equals_1 = _name_1.equals("maxResults");
                    _or = (_equals || _equals_1);
                  }
                  _and = ((e instanceof Parameter) && _or);
                }
                return Boolean.valueOf(_and);
              }
            };
          Iterable<QueryParameter> _reject = IterableExtensions2.<QueryParameter>reject(_queryParameters, _function);
          for(final QueryParameter parameter : _reject) {
            _builder.append("\t");
            DelegateOperation _delegate_7 = it.getDelegate();
            DaoOperation _operation_5 = _delegate_7.getOperation();
            TreeIterator<EObject> _eAllContents = _operation_5.eAllContents();
            Iterator<QueryParameterValue> _filter_1 = Iterators.<QueryParameterValue>filter(_eAllContents, QueryParameterValue.class);
            final Function1<QueryParameterValue,Boolean> _function_1 = new Function1<QueryParameterValue,Boolean>() {
                public Boolean apply(final QueryParameterValue value) {
                  boolean _or = false;
                  QueryParameterReference _parameter = value.getParameter();
                  boolean _equals = Objects.equal(_parameter, parameter);
                  if (_equals) {
                    _or = true;
                  } else {
                    QueryParameterReference _parameter_1 = value.getParameter();
                    Attribute _attribute = parameter.getAttribute();
                    boolean _equals_1 = Objects.equal(_parameter_1, _attribute);
                    _or = (_equals || _equals_1);
                  }
                  return Boolean.valueOf(_or);
                }
              };
            final QueryParameterValue queryParameterValue = IteratorExtensions.<QueryParameterValue>findFirst(_filter_1, _function_1);
            _builder.newLineIfNotEmpty();
            {
              boolean _and_5 = false;
              Type _type = this._features.getType(parameter);
              if (!(_type instanceof ComplexType)) {
                _and_5 = false;
              } else {
                Attribute _attribute = queryParameterValue==null?(Attribute)null:queryParameterValue.getAttribute();
                boolean _notEquals_2 = (!Objects.equal(_attribute, null));
                _and_5 = ((_type instanceof ComplexType) && _notEquals_2);
              }
              if (_and_5) {
                {
                  DelegateOperation _delegate_8 = it.getDelegate();
                  DaoOperation _operation_6 = _delegate_8.getOperation();
                  TreeIterator<EObject> _eAllContents_1 = _operation_6.eAllContents();
                  Iterator<QueryParameterValue> _filter_2 = Iterators.<QueryParameterValue>filter(_eAllContents_1, QueryParameterValue.class);
                  final Function1<QueryParameterValue,Boolean> _function_2 = new Function1<QueryParameterValue,Boolean>() {
                      public Boolean apply(final QueryParameterValue e) {
                        QueryParameterReference _parameter = e.getParameter();
                        boolean _equals = Objects.equal(_parameter, parameter);
                        return Boolean.valueOf(_equals);
                      }
                    };
                  Iterator<QueryParameterValue> _filter_3 = IteratorExtensions.<QueryParameterValue>filter(_filter_2, _function_2);
                  final Function1<QueryParameterValue,Attribute> _function_3 = new Function1<QueryParameterValue,Attribute>() {
                      public Attribute apply(final QueryParameterValue e) {
                        Attribute _attribute = e.getAttribute();
                        return _attribute;
                      }
                    };
                  Iterator<Attribute> _map = IteratorExtensions.<QueryParameterValue, Attribute>map(_filter_3, _function_3);
                  Set<Attribute> _set = IteratorExtensions.<Attribute>toSet(_map);
                  final Function1<Attribute,String> _function_4 = new Function1<Attribute,String>() {
                      public String apply(final Attribute e) {
                        String _name = e.getName();
                        return _name;
                      }
                    };
                  List<Attribute> _sortBy = IterableExtensions.<Attribute, String>sortBy(_set, _function_4);
                  for(final Attribute attribute : _sortBy) {
                    _builder.append("\t");
                    _builder.append("queryObject.withArgs(\"");
                    String _name_1 = this._features.getName(parameter);
                    _builder.append(_name_1, "	");
                    _builder.append("_");
                    String _name_2 = attribute.getName();
                    _builder.append(_name_2, "	");
                    _builder.append("\", ");
                    String _name_3 = this._features.getName(parameter);
                    _builder.append(_name_3, "	");
                    _builder.append(".");
                    String _accessorMethodName = this._features.getAccessorMethodName(attribute);
                    _builder.append(_accessorMethodName, "	");
                    _builder.append("());");
                    _builder.newLineIfNotEmpty();
                  }
                }
              } else {
                boolean _notEquals_3 = (!Objects.equal(queryParameterValue, null));
                if (_notEquals_3) {
                  _builder.append("\t");
                  _builder.append("queryObject.withArgs(\"");
                  String _name_4 = this._features.getName(parameter);
                  _builder.append(_name_4, "	");
                  _builder.append("\", ");
                  String _name_5 = this._features.getName(parameter);
                  _builder.append(_name_5, "	");
                  _builder.append(");");
                  _builder.newLineIfNotEmpty();
                }
              }
            }
          }
        }
        _builder.append("\t");
        DelegateOperation _delegate_9 = it.getDelegate();
        DaoOperation _operation_7 = _delegate_9.getOperation();
        String _type_1 = this._features.toType(_operation_7);
        _builder.append(_type_1, "	");
        _builder.append(" _result =  this.");
        DelegateOperation _delegate_10 = it.getDelegate();
        Dao _repository = _delegate_10.getRepository();
        String _variableName = this._names.toVariableName(_repository);
        _builder.append(_variableName, "	");
        _builder.append(".query(queryObject);");
        _builder.newLineIfNotEmpty();
        {
          DelegateOperation _delegate_11 = it.getDelegate();
          DataView _view = _delegate_11.getView();
          boolean _notEquals_4 = (!Objects.equal(_view, null));
          if (_notEquals_4) {
            _builder.append("\t");
            _builder.append("return this.mapper.createAndMapMany(_result,");
            DelegateOperation _delegate_12 = it.getDelegate();
            DataView _view_1 = _delegate_12.getView();
            String _qualifiedName = this._names.getQualifiedName(_view_1);
            _builder.append(_qualifiedName, "	");
            _builder.append(".class);");
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append("\t");
            _builder.append("return _result;");
            _builder.newLine();
          }
        }
      } else {
        {
          DelegateOperation _delegate_13 = it.getDelegate();
          DataView _view_2 = _delegate_13.getView();
          boolean _notEquals_5 = (!Objects.equal(_view_2, null));
          if (_notEquals_5) {
            _builder.append("\t");
            _builder.append("return this.mapper.");
            {
              boolean _isMany = it.isMany();
              if (_isMany) {
                _builder.append("createAndMapMany");
              } else {
                _builder.append("createAndMapOne");
              }
            }
            _builder.append("(this.");
            DelegateOperation _delegate_14 = it.getDelegate();
            Dao _repository_1 = _delegate_14.getRepository();
            String _variableName_1 = this._names.toVariableName(_repository_1);
            _builder.append(_variableName_1, "	");
            _builder.append(".");
            DelegateOperation _delegate_15 = it.getDelegate();
            DaoOperation _operation_8 = _delegate_15.getOperation();
            String _name_6 = _operation_8.getName();
            _builder.append(_name_6, "	");
            _builder.append("(");
            {
              EList<Parameter> _parameters_2 = it.getParameters();
              boolean _hasElements = false;
              for(final Parameter parameter_1 : _parameters_2) {
                if (!_hasElements) {
                  _hasElements = true;
                } else {
                  _builder.appendImmediate(",", "	");
                }
                {
                  boolean _hasProperty = EObjectPropertiesAdapter.hasProperty(parameter_1, "entity");
                  if (_hasProperty) {
                    _builder.append("mapper.createAndMapOne(");
                    String _variableName_2 = this._names.toVariableName(parameter_1);
                    _builder.append(_variableName_2, "	");
                    _builder.append(",");
                    EObject _entityType = this.getEntityType(parameter_1);
                    String _qualifiedName_1 = this._names.getQualifiedName(_entityType);
                    _builder.append(_qualifiedName_1, "	");
                    _builder.append(".class)");
                  } else {
                    String _variableName_3 = this._names.toVariableName(parameter_1);
                    _builder.append(_variableName_3, "	");
                  }
                }
              }
            }
            _builder.append("),");
            DelegateOperation _delegate_16 = it.getDelegate();
            DataView _view_3 = _delegate_16.getView();
            String _qualifiedName_2 = this._names.getQualifiedName(_view_3);
            _builder.append(_qualifiedName_2, "	");
            _builder.append(".class);");
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append("\t");
            {
              Type _type_2 = it.getType();
              boolean _notEquals_6 = (!Objects.equal(_type_2, null));
              if (_notEquals_6) {
                _builder.append("return ");
              }
            }
            _builder.append("this.");
            DelegateOperation _delegate_17 = it.getDelegate();
            Dao _repository_2 = _delegate_17.getRepository();
            String _variableName_4 = this._names.toVariableName(_repository_2);
            _builder.append(_variableName_4, "	");
            _builder.append(".");
            DelegateOperation _delegate_18 = it.getDelegate();
            DaoOperation _operation_9 = _delegate_18.getOperation();
            String _name_7 = _operation_9.getName();
            _builder.append(_name_7, "	");
            _builder.append("(");
            {
              EList<Parameter> _parameters_3 = it.getParameters();
              boolean _hasElements_1 = false;
              for(final Parameter parameter_2 : _parameters_3) {
                if (!_hasElements_1) {
                  _hasElements_1 = true;
                } else {
                  _builder.appendImmediate(",", "	");
                }
                {
                  boolean _hasProperty_1 = EObjectPropertiesAdapter.hasProperty(parameter_2, "entity");
                  if (_hasProperty_1) {
                    _builder.append("mapper.createAndMap");
                    {
                      boolean _isMany_1 = parameter_2.isMany();
                      if (_isMany_1) {
                        _builder.append("Many");
                      } else {
                        _builder.append("One");
                      }
                    }
                    _builder.append("(");
                    String _variableName_5 = this._names.toVariableName(parameter_2);
                    _builder.append(_variableName_5, "	");
                    _builder.append(",");
                    EObject _entityType_1 = this.getEntityType(parameter_2);
                    String _qualifiedName_3 = this._names.getQualifiedName(_entityType_1);
                    _builder.append(_qualifiedName_3, "	");
                    _builder.append(".class)");
                  } else {
                    String _variableName_6 = this._names.toVariableName(parameter_2);
                    _builder.append(_variableName_6, "	");
                  }
                }
              }
            }
            _builder.append(");");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public EObject getEntityType(final Parameter it) {
    Object _property = EObjectPropertiesAdapter.getProperty(it, "entity");
    return ((EObject) _property);
  }
  
  private String getIdOrKeyAccessorMethodName(final Attribute it) {
    Type _dataType = it.getDataType();
    if ((_dataType instanceof Entity)) {
      Type _dataType_1 = it.getDataType();
      final Entity entity = ((Entity) _dataType_1);
      Attribute _identifier = entity.getIdentifier();
      boolean _notEquals = (!Objects.equal(_identifier, null));
      if (_notEquals) {
        Attribute _identifier_1 = entity.getIdentifier();
        return this._features.getAccessorMethodName(_identifier_1);
      } else {
        AttributeGroup _key = entity.getKey();
        boolean _notEquals_1 = (!Objects.equal(_key, null));
        if (_notEquals_1) {
          AttributeGroup _key_1 = entity.getKey();
          return this._features.getAccessorMethodName(_key_1);
        }
      }
    }
    return this._features.getAccessorMethodName(it);
  }
  
  private Iterable<Operation> getCrudOperations(final Service it) {
    EList<Operation> _operations = it.getOperations();
    final Function1<Operation,Boolean> _function = new Function1<Operation,Boolean>() {
        public Boolean apply(final Operation e) {
          boolean _and = false;
          boolean _and_1 = false;
          DelegateOperation _delegate = e.getDelegate();
          CrudOperationType _crudOperationType = _delegate==null?(CrudOperationType)null:_delegate.getCrudOperationType();
          boolean _notEquals = (!Objects.equal(_crudOperationType, null));
          if (!_notEquals) {
            _and_1 = false;
          } else {
            DelegateOperation _delegate_1 = e.getDelegate();
            CrudOperationType _crudOperationType_1 = _delegate_1.getCrudOperationType();
            boolean _notEquals_1 = (!Objects.equal(_crudOperationType_1, CrudOperationType.NULL));
            _and_1 = (_notEquals && _notEquals_1);
          }
          if (!_and_1) {
            _and = false;
          } else {
            DelegateOperation _delegate_2 = e.getDelegate();
            CrudOperationType _crudOperationType_2 = _delegate_2.getCrudOperationType();
            boolean _notEquals_2 = (!Objects.equal(_crudOperationType_2, CrudOperationType.ALL));
            _and = (_and_1 && _notEquals_2);
          }
          return Boolean.valueOf(_and);
        }
      };
    return IterableExtensions.<Operation>filter(_operations, _function);
  }
  
  private String toAddReferenceMethod(final FeatureReference it, final DataView dataView, final Set<Attribute> processedReferences) {
    String _xblockexpression = null;
    {
      Attribute _attribute = it.getAttribute();
      boolean _add = processedReferences.add(_attribute);
      boolean _not = (!_add);
      if (_not) {
        return "";
      }
      StringConcatenation _builder = new StringConcatenation();
      _builder.newLine();
      _builder.append("private void add");
      Attribute _attribute_1 = it.getAttribute();
      String _name = _attribute_1.getName();
      String _firstUpper = StringExtensions.toFirstUpper(_name);
      _builder.append(_firstUpper, "");
      _builder.append("(");
      Entity _source = it.getSource();
      String _qualifiedName = this._names.getQualifiedName(_source);
      _builder.append(_qualifiedName, "");
      _builder.append(" parent");
      Entity _source_1 = it.getSource();
      String _name_1 = _source_1.getName();
      _builder.append(_name_1, "");
      _builder.append(",");
      String _qualifiedName_1 = this._names.getQualifiedName(dataView);
      _builder.append(_qualifiedName_1, "");
      _builder.append(" ");
      String _variableName = this._names.toVariableName(dataView);
      _builder.append(_variableName, "");
      _builder.append(") {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t");
      _builder.append("if (");
      String _variableName_1 = this._names.toVariableName(dataView);
      _builder.append(_variableName_1, "	");
      _builder.append(".");
      Attribute _attribute_2 = it.getAttribute();
      String _accessorMethodName = this._features.getAccessorMethodName(_attribute_2);
      _builder.append(_accessorMethodName, "	");
      _builder.append("() != null) {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("for (");
      DataView _view = it.getView();
      String _qualifiedName_2 = this._names.getQualifiedName(_view);
      _builder.append(_qualifiedName_2, "		");
      _builder.append(" _");
      DataView _view_1 = it.getView();
      String _variableName_2 = this._names.toVariableName(_view_1);
      _builder.append(_variableName_2, "		");
      _builder.append(" : ");
      String _variableName_3 = this._names.toVariableName(dataView);
      _builder.append(_variableName_3, "		");
      _builder.append(".");
      Attribute _attribute_3 = it.getAttribute();
      String _accessorMethodName_1 = this._features.getAccessorMethodName(_attribute_3);
      _builder.append(_accessorMethodName_1, "		");
      _builder.append("()) {");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t\t");
      Attribute _attribute_4 = it.getAttribute();
      Type _dataType = _attribute_4.getDataType();
      String _qualifiedName_3 = this._names.getQualifiedName(_dataType);
      _builder.append(_qualifiedName_3, "			");
      _builder.append(" ");
      Attribute _attribute_5 = it.getAttribute();
      Type _dataType_1 = _attribute_5.getDataType();
      String _variableName_4 = this._names.toVariableName(_dataType_1);
      _builder.append(_variableName_4, "			");
      _builder.append(" = this.mapper.createAndMapOne(_");
      DataView _view_2 = it.getView();
      String _variableName_5 = this._names.toVariableName(_view_2);
      _builder.append(_variableName_5, "			");
      _builder.append(", ");
      Attribute _attribute_6 = it.getAttribute();
      Type _dataType_2 = _attribute_6.getDataType();
      String _qualifiedName_4 = this._names.getQualifiedName(_dataType_2);
      _builder.append(_qualifiedName_4, "			");
      _builder.append(".class,\"save");
      Attribute _attribute_7 = it.getAttribute();
      Type _dataType_3 = _attribute_7.getDataType();
      String _name_2 = _dataType_3.getName();
      _builder.append(_name_2, "			");
      _builder.append("\");");
      _builder.newLineIfNotEmpty();
      {
        Attribute _attribute_8 = it.getAttribute();
        Type _dataType_4 = _attribute_8.getDataType();
        EList<Attribute> _requiredReferences = ((Entity) _dataType_4).getRequiredReferences();
        final Function1<Attribute,Boolean> _function = new Function1<Attribute,Boolean>() {
            public Boolean apply(final Attribute e) {
              boolean _and = false;
              Attribute _opposite = e.getOpposite();
              Attribute _attribute = it.getAttribute();
              boolean _notEquals = (!Objects.equal(_opposite, _attribute));
              if (!_notEquals) {
                _and = false;
              } else {
                Attribute _attribute_1 = it.getAttribute();
                Attribute _opposite_1 = _attribute_1.getOpposite();
                boolean _notEquals_1 = (!Objects.equal(_opposite_1, e));
                _and = (_notEquals && _notEquals_1);
              }
              return Boolean.valueOf(_and);
            }
          };
        Iterable<Attribute> _filter = IterableExtensions.<Attribute>filter(_requiredReferences, _function);
        for(final Attribute requiredReference : _filter) {
          _builder.append("\t\t\t");
          Type _dataType_5 = requiredReference.getDataType();
          final Entity requiredEntity = ((Entity) _dataType_5);
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t\t");
          Type _dataType_6 = requiredReference.getDataType();
          String _qualifiedName_5 = this._names.getQualifiedName(_dataType_6);
          _builder.append(_qualifiedName_5, "			");
          _builder.append(" ");
          String _variableName_6 = this._names.toVariableName(requiredReference);
          _builder.append(_variableName_6, "			");
          _builder.append(" = this.mapper.createAndMapOne(_");
          DataView _view_3 = it.getView();
          String _variableName_7 = this._names.toVariableName(_view_3);
          _builder.append(_variableName_7, "			");
          _builder.append(", ");
          Type _dataType_7 = requiredReference.getDataType();
          String _qualifiedName_6 = this._names.getQualifiedName(_dataType_7);
          _builder.append(_qualifiedName_6, "			");
          _builder.append(".class,\"");
          String _name_3 = requiredReference.getName();
          _builder.append(_name_3, "			");
          _builder.append("\");");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t\t");
          _builder.append("if (");
          String _variableName_8 = this._names.toVariableName(requiredReference);
          _builder.append(_variableName_8, "			");
          _builder.append(".");
          Attribute _identifier = requiredEntity.getIdentifier();
          String _accessorMethodName_2 = this._features.getAccessorMethodName(_identifier);
          _builder.append(_accessorMethodName_2, "			");
          _builder.append("() != null) {");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t\t");
          _builder.append("\t");
          String _variableName_9 = this._names.toVariableName(requiredReference);
          _builder.append(_variableName_9, "				");
          _builder.append(" = ");
          Dao _repository = requiredEntity.getRepository();
          String _variableName_10 = this._names.toVariableName(_repository);
          _builder.append(_variableName_10, "				");
          _builder.append(".load(");
          String _variableName_11 = this._names.toVariableName(requiredReference);
          _builder.append(_variableName_11, "				");
          _builder.append(".");
          Attribute _identifier_1 = requiredEntity.getIdentifier();
          String _accessorMethodName_3 = this._features.getAccessorMethodName(_identifier_1);
          _builder.append(_accessorMethodName_3, "				");
          _builder.append("());");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t\t");
          _builder.append("\t");
          Attribute _attribute_9 = it.getAttribute();
          Type _dataType_8 = _attribute_9.getDataType();
          String _variableName_12 = this._names.toVariableName(_dataType_8);
          _builder.append(_variableName_12, "				");
          _builder.append(".");
          String _mutatorMethodName = this._features.getMutatorMethodName(requiredReference);
          _builder.append(_mutatorMethodName, "				");
          _builder.append("(");
          String _variableName_13 = this._names.toVariableName(requiredReference);
          _builder.append(_variableName_13, "				");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
          _builder.append("\t\t\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
      {
        DataView _view_4 = it.getView();
        EList<FeatureReference> _featureReferences = _view_4.getFeatureReferences();
        final Function1<FeatureReference,Boolean> _function_1 = new Function1<FeatureReference,Boolean>() {
            public Boolean apply(final FeatureReference e) {
              boolean _and = false;
              boolean _and_1 = false;
              boolean _and_2 = false;
              Attribute _attribute = e.getAttribute();
              boolean _isReference = _attribute==null?false:_attribute.isReference();
              if (!_isReference) {
                _and_2 = false;
              } else {
                Attribute _attribute_1 = e.getAttribute();
                boolean _isMany = _attribute_1==null?false:_attribute_1.isMany();
                _and_2 = (_isReference && _isMany);
              }
              if (!_and_2) {
                _and_1 = false;
              } else {
                Attribute _attribute_2 = e.getAttribute();
                boolean _contains = processedReferences.contains(_attribute_2);
                boolean _not = (!_contains);
                _and_1 = (_and_2 && _not);
              }
              if (!_and_1) {
                _and = false;
              } else {
                DataView _view = e.getView();
                Attribute _attribute_3 = e.getAttribute();
                Type _dataType = _attribute_3.getDataType();
                boolean _includesAllRequiredFeaturesFor = _view.includesAllRequiredFeaturesFor(((Entity) _dataType));
                _and = (_and_1 && _includesAllRequiredFeaturesFor);
              }
              return Boolean.valueOf(_and);
            }
          };
        Iterable<FeatureReference> _filter_1 = IterableExtensions.<FeatureReference>filter(_featureReferences, _function_1);
        for(final FeatureReference nestedIncludedFeature : _filter_1) {
          _builder.append("\t\t\t");
          _builder.append("add");
          Attribute _attribute_10 = nestedIncludedFeature.getAttribute();
          String _name_4 = _attribute_10.getName();
          String _firstUpper_1 = StringExtensions.toFirstUpper(_name_4);
          _builder.append(_firstUpper_1, "			");
          _builder.append("(");
          Attribute _attribute_11 = it.getAttribute();
          Type _dataType_9 = _attribute_11.getDataType();
          String _variableName_14 = this._names.toVariableName(_dataType_9);
          _builder.append(_variableName_14, "			");
          _builder.append(",_");
          DataView _view_5 = it.getView();
          String _variableName_15 = this._names.toVariableName(_view_5);
          _builder.append(_variableName_15, "			");
          _builder.append(");");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("\t\t\t");
      _builder.append("parent");
      Entity _source_2 = it.getSource();
      String _name_5 = _source_2.getName();
      _builder.append(_name_5, "			");
      _builder.append(".");
      Attribute _attribute_12 = it.getAttribute();
      String _addMethodName = this._features.getAddMethodName(_attribute_12);
      _builder.append(_addMethodName, "			");
      _builder.append("(");
      Attribute _attribute_13 = it.getAttribute();
      Type _dataType_10 = _attribute_13.getDataType();
      String _variableName_16 = this._names.toVariableName(_dataType_10);
      _builder.append(_variableName_16, "			");
      _builder.append(");");
      _builder.newLineIfNotEmpty();
      _builder.append("\t\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("\t");
      _builder.append("}");
      _builder.newLine();
      _builder.append("}");
      _builder.newLine();
      final String addReferenceMethod = _builder.toString();
      StringBuilder _stringBuilder = new StringBuilder(addReferenceMethod);
      final StringBuilder result = _stringBuilder;
      DataView _view_6 = it.getView();
      EList<FeatureReference> _featureReferences_1 = _view_6.getFeatureReferences();
      final Function1<FeatureReference,Boolean> _function_2 = new Function1<FeatureReference,Boolean>() {
          public Boolean apply(final FeatureReference e) {
            boolean _and = false;
            boolean _and_1 = false;
            DataView _view = e.getView();
            boolean _notEquals = (!Objects.equal(_view, null));
            if (!_notEquals) {
              _and_1 = false;
            } else {
              Attribute _attribute = e.getAttribute();
              boolean _isMany = _attribute==null?false:_attribute.isMany();
              _and_1 = (_notEquals && _isMany);
            }
            if (!_and_1) {
              _and = false;
            } else {
              DataView _view_1 = e.getView();
              Attribute _attribute_1 = e.getAttribute();
              Type _dataType = _attribute_1.getDataType();
              boolean _includesAllRequiredFeaturesFor = _view_1.includesAllRequiredFeaturesFor(((Entity) _dataType));
              _and = (_and_1 && _includesAllRequiredFeaturesFor);
            }
            return Boolean.valueOf(_and);
          }
        };
      Iterable<FeatureReference> _filter_2 = IterableExtensions.<FeatureReference>filter(_featureReferences_1, _function_2);
      for (final FeatureReference includedFeature : _filter_2) {
        DataView _view_7 = it.getView();
        String _addReferenceMethod = this.toAddReferenceMethod(includedFeature, _view_7, processedReferences);
        result.append(_addReferenceMethod);
      }
      String _string = result.toString();
      _xblockexpression = (_string);
    }
    return _xblockexpression;
  }
  
  private String getName(final Dependant it) {
    String _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (it instanceof Dao) {
        final Dao _dao = (Dao)it;
        _matched=true;
        String _name = _dao.getName();
        _switchResult = _name;
      }
    }
    if (!_matched) {
      if (it instanceof Service) {
        final Service _service = (Service)it;
        _matched=true;
        String _name = _service.getName();
        _switchResult = _name;
      }
    }
    return _switchResult;
  }
}
