package org.openxma.dsl.generator.impl;

import java.io.CharArrayWriter;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;

import org.openxma.dsl.generator.impl.substrategy.ComboMappingStrategy;


import at.spardat.xma.guidesign.BDAttribute;
import at.spardat.xma.guidesign.SimpleCombo;
import at.spardat.xma.guidesign.XMACombo;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.generate.DefaultAttributeMappingStrategy;
import at.spardat.xma.guidesign.impl.BDAttributeImpl;
import at.spardat.xma.guidesign.util.ProjectClassLoaderFactory;

/**
 * Represent a custom xma mapping strategy to generate the code to map between
 * ValueObject and XMA Widgets.
 *
 */
public class ValueObjectAttributeMappingStrategy extends DefaultAttributeMappingStrategy {

	private String getBasicType(Class<?> valueObjectClass) {
		Constructor<?>[] constructors = valueObjectClass.getConstructors();
		for (int i = 0; i < constructors.length; i++) {
			if (constructors[i].getParameterTypes().length == 1) {
				return constructors[i].getParameterTypes()[0].getName();
			}
		}
		throw new IllegalStateException("ValueObject " + valueObjectClass.getName()
				+ " has no correct 1param-Constructor ");
	}
	
	private boolean implementsI18nEnum(Class<?> enumClazz) {
		try {			
			for (Class<?> interf : enumClazz.getInterfaces()) {
				if (interf.getName().equals("org.openxma.dsl.platform.enums.I18nEnum")) {
					return true;
				}
			}
			return false;
		} catch (Exception e) {
			throw new RuntimeException("Can not find out whether the method 'valueOrNullOf' exists",e);
		} 
	}

	@Override
    public String genToBusinessData(BDAttribute attribute, XMAWidget widget) {

		BDAttributeImpl attributeImpl = (BDAttributeImpl) attribute;
		if (attributeImpl.isBasicType()) {
			/* Additional supported Combo mapping */
			if (widget instanceof XMACombo || widget instanceof SimpleCombo) {
				if (ComboMappingStrategy.isSupported(attributeImpl)) {
					return ComboMappingStrategy.genToBusinessData(widget, attributeImpl);
				}
        	}					
			return super.genToBusinessData(attribute, widget);
		}

		try {
			Class<?> valueObjectClass = ProjectClassLoaderFactory.getProjectClassLoader().loadClass(attribute.getNamPropertyType());
			if (valueObjectClass.isEnum()) {
				if (attributeImpl.getNamSetter() == null) {
					return "";
				}
				String accessMethod;
				String checkSource;
				if (widget instanceof XMACombo || widget instanceof SimpleCombo) {
					accessMethod = "getSelected()";
					checkSource =  "getSelected() != null";				
				} else {				
					checkSource = "hasValue()";					
					accessMethod = "toString()";					
				}				
				
				CharArrayWriter buffer = new CharArrayWriter();
				PrintWriter out = new PrintWriter(buffer);				
				if (implementsI18nEnum(valueObjectClass)) {
					out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
							+ attributeImpl.getNamSetter() + "("+widget.getNamModel()+"."+checkSource+" ? " + getQualifiedName(valueObjectClass) + ".valueOfKey(" + widget.getNamModel()
							+ "."+accessMethod+") : null);");			
					
				} else {
					out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
							+ attributeImpl.getNamSetter() + "("+widget.getNamModel()+"."+checkSource+" ? " + getQualifiedName(valueObjectClass) + ".valueOf(" + widget.getNamModel()
							+ "."+accessMethod+") : null);");
				}
				out.close();
				return buffer.toString();								
			}
			CharArrayWriter buffer = new CharArrayWriter();
			PrintWriter out = new PrintWriter(buffer);
			String basicType = getBasicType(valueObjectClass);
			if ("java.math.BigDecimal".equals(basicType)) {
				out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
						+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(" + widget.getNamModel()
						+ ".toBigDecimal()));");
			} else if ("java.lang.Boolean".equals(basicType)) {
				out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
						+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(Boolean.valueOf("
						+ widget.getNamModel() + ".isTrue())));");
			} else if ("java.util.Date".equals(basicType)) {
				out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
						+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(" + widget.getNamModel()
						+ ".toDate()));");
			} else if ("java.lang.Double".equals(basicType)) {
				out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
						+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(" + widget.getNamModel()
						+ ".toDouble()));");
			} else if ("java.lang.Float".equals(basicType)) {
				out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
						+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(" + widget.getNamModel()
						+ ".toFLOAT()));");
			} else if ("java.lang.Integer".equals(basicType)) {
				out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
						+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(" + widget.getNamModel()
						+ ".toINTEGER()));");
			} else if ("java.lang.Long".equals(basicType)) {
				out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
						+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(" + widget.getNamModel()
						+ ".toLONG()));");
			} else if ("java.lang.String".equals(basicType)) {
				if (widget instanceof XMACombo || widget instanceof SimpleCombo) {
					out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
							+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(" + widget.getNamModel()
							+ ".getSelected()));");
				} else {
					out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
							+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(" + widget.getNamModel()
							+ ".toString()));");
				}
			} else if ("java.sql.Timestamp".equals(basicType)) {
				out.println("        " + attributeImpl.getBdObject().getNamInstance() + "."
						+ attributeImpl.getNamSetter() + "(" + getQualifiedName(valueObjectClass) + ".newOrNull(new java.sql.Timestamp("
						+ widget.getNamModel() + ".toDate().getTime())));");
			} else {
				throw new IllegalStateException(" valueObject " + getQualifiedName(valueObjectClass)
						+ " is not parseable For Xma, extend " + this.getClass().getName());
			}
			out.close();
			return buffer.toString();
		} catch (ClassNotFoundException e) {
			throw new IllegalStateException("ValueObject-Class '" + attribute.getNamPropertyType() + "' not found");
		}
	}



	@Override
    public String genToModel(BDAttribute attribute, XMAWidget widget) {
		BDAttributeImpl attributeImpl = (BDAttributeImpl) attribute;
		if (attributeImpl.isBasicType()) {
			/* Additional supported Combo mapping */
			if (widget instanceof XMACombo || widget instanceof SimpleCombo) {
				if (ComboMappingStrategy.isSupported(attributeImpl)) {
					return ComboMappingStrategy.genToModel(widget, attributeImpl);
				}
        	}								
			return super.genToModel(attribute, widget);
		}
		Class<?> valueObjectClass = null;		
		try {
			valueObjectClass = ProjectClassLoaderFactory.getProjectClassLoader().loadClass(attribute.getNamPropertyType());
		} catch (ClassNotFoundException e) {
			throw new IllegalStateException("ValueObject-Class '" + attribute.getNamPropertyType() + "' not found");
		}		
		if (valueObjectClass.isEnum()) {		
			if (attributeImpl.getNamGetter() == null) {
				throw new RuntimeException("The getter of the enum-attribute "+attribute.getNamAttrVal()+" is not defined.");
			}
			CharArrayWriter buffer = new CharArrayWriter();
			PrintWriter out = new PrintWriter(buffer);
			if (implementsI18nEnum(valueObjectClass)) {
				out.println("        " + widget.getNamModel() + ".set("
						+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()+"() != null ? "
						+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()+"().getKey() : null"					
						+ ");");
			} else {
				out.println("        " + widget.getNamModel() + ".set("
						+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()+"() != null ? "
						+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()+"().toString() : null"					
						+ ");");
			}
			out.close();
			return buffer.toString();								
		}		
				
		CharArrayWriter buffer = new CharArrayWriter();
		PrintWriter out = new PrintWriter(buffer);
		if (attributeImpl.getNamGetter() != null) {
			out.println("          " + widget.getNamModel() + ".set(org.openxma.dsl.platform.valueobject.ValueObject.valueOrNull("
					+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()
					+ "()));");
		} else {
			out.println("        " + widget.getNamModel() + ".set(org.openxma.dsl.platform.valueobject.ValueObject.valueOrNull("
					+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetterAttrVal()
					+ "()));");
		}
		out.close();
		return buffer.toString();
	}

	@Override
    public String genTableCell(BDAttribute attribute, int index) {
		BDAttributeImpl attributeImpl = (BDAttributeImpl) attribute;
		if (attributeImpl.isBasicType()) {
			return super.genTableCell(attribute, index);
		}
		Class<?> valueObjectClass = null;		
		try {
			valueObjectClass = ProjectClassLoaderFactory.getProjectClassLoader().loadClass(attribute.getNamPropertyType());
		} catch (ClassNotFoundException e) {
			throw new IllegalStateException("ValueObject-Class '" + attribute.getNamPropertyType() + "' not found");
		}
		if (valueObjectClass.isEnum()) {
			if (implementsI18nEnum(valueObjectClass)) {
				return "        cells[" + index + "] = "
				+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()+"() != null ? "
				+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()+"().getKey() : null"	
				+ ";";
			} else {
				return "        cells[" + index + "] = "
				+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()+"() != null ? "
				+ attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()+"().toString() : null"	
				+ ";";
			}			
		}
		return "        cells[" + index + "] = org.openxma.dsl.platform.valueobject.ValueObject.valueOrNull(" + attributeImpl.getBdObject().getNamInstance() + "."
				+ attributeImpl.getNamGetter() + "());";
	}

	@Override
    public String genToString(BDAttribute attribute) {
		BDAttributeImpl attributeImpl = (BDAttributeImpl) attribute;
		if (attributeImpl.isBasicType()) {
			return super.genToString(attribute);
		}
		Class<?> valueObjectClass = null;		
		try {
			valueObjectClass = ProjectClassLoaderFactory.getProjectClassLoader().loadClass(attribute.getNamPropertyType());
		} catch (ClassNotFoundException e) {
			throw new IllegalStateException("ValueObject-Class '" + attribute.getNamPropertyType() + "' not found");
		}
		if (valueObjectClass.isEnum()) {
			return attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()
			+ "()	.toString()";			
		}
		return attributeImpl.getBdObject().getNamInstance() + "." + attributeImpl.getNamGetter()
				+ "().getValue().toString()";
	}
	
	/**
	 * Return the qualified name of the given class: usually simply
	 * the class name, but component type class name + "[]" for arrays.
	 * @param clazz the class
	 * @return the qualified name of the class
	 */
	public static String getQualifiedName(Class<?> clazz) {
		if (clazz.isArray()) {
			return getQualifiedNameForArray(clazz);
		}
		else {
			return clazz.getName();
		}
	}

	/**
	 * Build a nice qualified name for an array:
	 * component type class name + "[]".
	 * @param clazz the array class
	 * @return a qualified name for the array class
	 */
	private static String getQualifiedNameForArray(Class<?> clazz) {
		StringBuffer buffer = new StringBuffer();
		while (clazz.isArray()) {
			clazz = clazz.getComponentType();
			buffer.append("[]");
		}
		buffer.insert(0, clazz.getName());
		return buffer.toString();
	}
}
